SUBROUTINE atdath_nl
USE PARAMETERS
IMPLICIT NONE

!  Atomic data for H I 

!  A-values : A(nl;n'l') are calculated using expressions in
!             Brocklehurst, 1971, MNRAS, 153, p 471.
!             Eqs. used are originally from Gordon (1929).
!             For n.le.20 the A-values are checked (1) by comparing
!             calculated radial integrals (ri2) with those tabulated by
!             Green, Rush, & Chandler, 1957, ApJS, 3, p 37,
!             (2) by comparing the averaged A(n;n') with the expressions
!             given by Johnson, L.C. 1972, ApJ 174, 227.
!             In both tests the agreement was very good. 

!  nlev = number of levels included
!  e    = energy of the levels in eV
!  a    = transition probabilities
!  e00  = ionization potential in eV
!  g    = statistical weights of the levels

INTEGER, PARAMETER :: nm = 30, nmax=0.5*nm*(nm+1)
INTEGER, DIMENSION(nm,nm) :: ab
DOUBLE PRECISION, DIMENSION(nm) :: gn, en
DOUBLE PRECISION, DIMENSION(nmax,nmax) :: a, ups_mat, alfa_mat
INTEGER, DIMENSION(nmax,nmax) :: T_mat
DOUBLE PRECISION, DIMENSION(nmax) :: enlong
DOUBLE PRECISION :: e00
INTEGER :: nmn, nlev, nlm(nm,0:nm), n, l, i, j, nj, ni, lj, li, il, nj_lj, ni_li, counter
COMMON/catdath/gn, en, a, e00, nmn, nlev, nlm
DOUBLE PRECISION :: rn, gi, ei, const, c1, c2, xnj, xni, xnsum, xndiff, xxx, xlj, xli, x, hgfp, hgfm, hgfsum, xnf1, &
xnf2, xnf3, xnf4, cc1, cc2, cc3, cc4, x1, ri, ri2, asum, gnj, gnjlj
 
DOUBLE PRECISION, EXTERNAL :: hgf    
CHARACTER :: trash 
!  nmax = 30 => number of nl-states is 210      

T_mat = 0  ! Imp

alfa_mat = 0  ! Imp

ups_mat = 0   ! Imp

nmn = nm

nlm = 0

nlev = 0

enlong = 0
      
DO n = 1, nmn
   DO l = 0, n-1
          
      nlm(n,l) = nlev + 1 + l
      
   END DO

   nlev = nlev + n
    
ENDDO


e00 = 13.598434  ! NIST 2013

en(1) = 0.       ! NIST 2013 (ref to Kramida 2010, A critical compilation of experimental data on spectral lines and energy levels of hydrodgen..."
en(2) = 10.1988
en(3) = 12.0875
en(4) = 12.7485
en(5) = 13.0545
en(6) = 13.2207

gn(1) = 2.
gn(2) = 8.
gn(3) = 18.
gn(4) = 32.
gn(5) = 50.
gn(6) = 72.

DO i = 7, nmn  ! Compute energies and g-values for n>= 7, seems to be accurate also for n = 2-6

   rn = DBLE(i)

   gi = 2.*rn**2.

   gn(i) = gi

   ei = e00*(1.-1./rn**2.)
        
   en(i) = ei

END DO

a = 0

ab = 0

!   Two-photon transition 2 2S - 1 2S
      
a(nlm(2,0),nlm(1,0)) = 8.23

const = 2.6774d9

DO nj = 1, nm
   DO ni = 1, nj-1

      c1 = (1./DBLE(ni*ni)-1./DBLE(nj*nj))**3.
 
      xnj = DBLE(nj)

      xni = DBLE(ni)

      xnsum = xnj + xni

      xndiff = xnj - xni
      
      DO lj = 0, nj-1

         IF (nj.EQ.55.AND.lj.EQ.54) THEN

               xxx = 1.

         END IF

         xlj = DBLE(lj)

         li = lj-1

         xli = DBLE(li)
               
         IF (li.GE.0.AND.li.LE.(ni-1)) THEN
                  
            c2 = c1*DBLE(lj)/(2.*DBLE(lj)+1)

            x = -4.*xnj*xni/(xnj-xni)**2
                  
            hgfp = hgf(-nj+lj+1,-ni+lj,2*lj,x)

            hgfm = hgf(-nj+lj-1,-ni+lj,2*lj,x)

            hgfsum = hgfp-((xndiff*xndiff)/(xnsum*xnsum))*hgfm

            xnf1 = 1

            DO il = 2,2*lj-1

               xnf1 = xnf1*DBLE(il)

            END DO

            xnf1 = 4*xnf1

            cc1=1./xnf1

            xnf1 = 1
            xnf2 = 1
            xnf3 = 1
            xnf4 = 1

            DO il =2, nj+lj

               xnf1=xnf1*DBLE(il)

            ENDDO

            DO il = 2, ni+lj-1
                     
               xnf2 = xnf2*DBLE(il)
                  
            ENDDO

            DO il = 2, nj-lj-1

               xnf3 = xnf3*DBLE(il)
                  
            ENDDO

            DO il = 2, ni-lj

               xnf4 = xnf4*DBLE(il)
                  
            ENDDO

            cc2 = SQRT(xnf1/xnf3)*SQRT(xnf2/xnf4)

            cc3 = (((4.*xnj*xni)**(xlj+1.))/(xnsum**xnsum))

            x1 = xnsum - 2.*xlj-2.
             
            cc4 = ((xnj-xni)**x1)

            ri = cc1*cc2*cc3*cc4*hgfsum
                  
            ri2 = ri*ri

            nj_lj = nlm(nj,lj)

            ni_li = nlm(ni,li)
                  
            a(nj_lj,ni_li) = const*c2*ri2
               
         ENDIF

         li = lj + 1

         xli = DBLE(li)

         IF (li.GE.0.AND.li.LE.(ni-1)) THEN

            c2 = c1*DBLE(li)/(2.*DBLE(lj)+1)

            x = -4.*xnj*xni/(xnj-xni)**2

            hgfp = hgf(-ni+li+1,-nj+li,2*li,x)

            hgfm = hgf(-ni+li-1,-nj+li,2*li,x)

            hgfsum = hgfp-((xndiff*xndiff)/(xnsum*xnsum))*hgfm

            xnf1=1

            DO il = 2,2*li-1
            
               xnf1=xnf1*DBLE(il)
                  
            ENDDO
                  
            xnf1=4*xnf1
                  
            cc1=1./xnf1

            xnf1=1
            xnf2=1
            xnf3=1
            xnf4=1
                  
            DO il =2, ni+li
                     
               xnf1=xnf1*DBLE(il)
                  
            ENDDO
                  
            DO il = 2,nj+li-1
                     
               xnf2=xnf2*DBLE(il)
                  
            ENDDO
                  
            DO il=2,ni-li-1

               xnf3=xnf3*DBLE(il)
                  
            ENDDO
                  
            DO il=2,nj-li
                     
               xnf4=xnf4*DBLE(il)
                  
            ENDDO

            cc2 = SQRT(xnf1/xnf3)*SQRT(xnf2/xnf4)

            cc3 = (((4.*xnj*xni)**(xli+1.))/(xnsum**xnsum))

            x1 = xnsum-2.*xli-2.
                  
            cc4 = ((xni-xnj)**x1)

            ri = cc1*cc2*cc3*cc4*hgfsum
                  
            ri2 = ri*ri

            nj_lj = nlm(nj,lj)
                  
            ni_li = nlm(ni,li)
                  
            a(nj_lj,ni_li) = const*c2*ri2
               
         ENDIF
            
      ENDDO
            
      asum = 0

      gnj = 2.*DBLE(nj)*DBLE(nj)
            
      DO lj = 0,nj-1
               
         DO li = 0,ni-1
                  
            nj_lj = nlm(nj,lj)
                  
            ni_li = nlm(ni,li)

            gnjlj = 2.*(2.*DBLE(lj)+1.)
                  
            asum = asum+a(nj_lj,ni_li)*gnjlj/gnj
               
         ENDDO
            
      ENDDO
            
      ab(nj,ni) = asum
         
   ENDDO
      
ENDDO

OPEN(50, file='collstrengths_simple.dat',STATUS='OLD')

READ(50,*) trash
READ(50,*) trash

DO WHILE (1 .EQ. 1)

   READ(50,*,END=123) j, i, ups_mat(j,i), T_mat(j,i), alfa_mat(j,i)

END DO

123 close(50)

!OPEN(10,file='hI.dat',STATUS='REPLACE')
OPEN(10,file='temp.dat',STATUS='REPLACE')

WRITE(10,*) 'HI with specific nl states. Atom made by AJ 2009-01-10.'
WRITE(10,*) '         i          nE              g                n          l'

WRITE(10,99) 1,          0, 2, 1, 0

enlong(1) = 0

WRITE(10,99) 2, 82258.9544, 2, 2, 0

enlong(2) = h*c*82258.9544/eV  ! 2s

WRITE(10,99) 3, 82259.1630, 6, 2, 1
 
enlong(3) = h*c*82259.1630/eV   ! 2p

enlong(4) = h*c*97492.2217/eV  ! 3s

enlong(5) = h*c*97492.2835/eV  ! 3p

enlong(6) = h*c*97492.3411/eV  ! 3d

WRITE(10,99) 4, 97492.2217, 2, 3, 0
WRITE(10,99) 5, 97492.2835, 6, 3, 1
WRITE(10,99) 6, 97492.3411, 10, 3, 2

counter = 6

DO n = 4, nm

   DO l = 0,(n-1)

      counter = counter + 1

      WRITE(10,99) counter, en(n)*eV/(h*c), 2*(2*l+1), n, l

      enlong(counter) = en(n)

   END DO

END DO

99 FORMAT(I5, F15.4, I10, 2I6)

WRITE(10,*) 'Lines'
WRITE(10,*) '    j     i         lambda         A         Upsilon      T0        alfa'

DO j = 2, nmax
   DO i = 1, (j-1)

      IF (a(j,i) > 0 .OR. ups_mat(j,i) > 0) WRITE(10,'(2I6, F20.3, 2ES12.2, I10, ES12.2)') j, i, h*c/(enlong(j)*eV-enlong(i)*eV)*1e8, a(j,i), ups_mat(j,i), T_mat(j,i), alfa_mat(j,i)

   END DO
END DO

close(10)

RETURN
      
END

DOUBLE PRECISION FUNCTION hgf(a,b,c,x)
IMPLICIT REAL*8 (a-h,o-z)
INTEGER a,b,c
DIMENSION fact(100)

nmax = 100
      
IF (a.GT.0) THEN
         WRITE(*,*) 'a out of range in hgf !' 
         STOP
      
ENDIF
      
IF (b.GT.0) THEN
         WRITE(*,*) 'b out of range in hgf !' 
         STOP
      
ENDIF
      
IF (c.LE.0) THEN
         WRITE(*,*) 'c out of range in hgf !' 
         STOP
ENDIF

xa = DBLE(a)
xb = DBLE(b)
xc = DBLE(c)

DO n = 1,nmax
   nm = n
         
   xn = DBLE(n)
         
   fact(n) = ((xa+xn-1.)*(xb+xn-1.)/(xc+xn-1.))*x/xn
         
   IF (fact(n).EQ.0.) GOTO 9
      
ENDDO
      
IF (fact(nmax).NE.0) THEN
         WRITE(*,*) 'Insufficient summation in hgf'
         STOP
      
ENDIF
9    CONTINUE

f = 1.
      
DO n = nm,1,-1
         
   f = (1.+fact(n)*f)
      
ENDDO
      
hgf = f

      
RETURN
END

