import sys
import csv

import numpy as np
sys.path.append('scripts')

# =============================================================================
# This code takes the human readable NBINCODES, that contain what molecules and
# ions exist for each atom nuclei, and make a machine readable version.
# =============================================================================



data = []

# opens the abin code file
with open('ABINCODES','r') as fin:
    data=fin.readlines()

data = [d.split() for d in data]
abin = [int(d[0]) for d in data]
abin_names = [d[1] + d[2] for d in data]

data = []

# open the mbin cone file
with open('MBINCODES','r') as fin:
    data=fin.readlines()

data = [d.split() for d in data]
mbin = [int(d[0]) for d in data]
mbin_names = [d[1] + d[2] for d in data]

def return_abin(name):
    '''
    This function takes the name of an element and returns the 
    numerical code (ABIN), which is used internally as identifiers in the SUMO
    code. 
    
    Parameters
    ----------
    name: string
              The name of the element or molecule.
    Returns
    -------
    abin: int
              ABIN code.
    '''
    indi = abin_names.index(name)
    return abin[indi]

def return_mbin(name):
    '''
    This function takes the name of a molecule and returns the 
    numerical code (MBIN), which is used internally as identifiers in the SUMO
    code. 
    
    Parameters
    ----------
    name: string
              The name of the element or molecule.
    Returns
    -------
    mbin: int
              MBIN code.
    '''
    indi = mbin_names.index(name)
    return mbin[indi]
    

data = []

# open the table with nuclei conservation
with open('NBINCODES','r') as fin:
    data=fin.readlines()

data = [d.split(',') for d in data]
data = [[d.replace(' ', '') for d in dd] for dd in data]
data = [[d.rstrip() for d in dd] for dd in data]

out_array = np.array([[0]*10]*len(data))

for i in range(len(data)):
    dd = data[i]
    k = 1
    
    for j in range(1, len(dd), 1):
        d = dd[j]
        
        try:
            temp = return_abin(d)
            out_array[i,j-1] = temp
        except:
            temp = return_mbin(d)
            out_array[i][10-k] = temp
            k +=1


with open('nuccons_input.csv', mode='w') as out_file:
    out_write = csv.writer(out_file, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)

    [out_write.writerow(o) for o in out_array]


print('Done. ')














