import numpy as np
import matplotlib.pyplot as plt
import math as math
from scipy.optimize import curve_fit
import pickle

#----------------------------------------------------------
# picks out the data that we want to use
numax = 8
jmax = 100
delv = 8
#----------------------------------------------------------
# ----------------------------------------------------
# find nearest value
def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    
    if value == 0.0 or np.abs((array[idx] - value) / value) < 0.01:
        return idx
    else:
        print('error in ', value, idx)
        return 'NaN'
# ----------------------------------------------------

class el_class:
  def __init__(self, v1, j1):
    self.j1 = j1
    self.v1 = v1
    
    self.elev = E_levels(v1, j1)
    self.g = stat_weight(j1)


hfor = [0,2,1,12,10,10,5,5,10,4,8,15,15,15,15,6,12,1,7,7]
hfor = np.cumsum(hfor)


fields = '''Molecule number
Isotopologue number
Transition wavenumber (cm-1)
Line Intensity
Einstein A-coefficient
Air- broadened width
Self- broadened width
lower-state Energy
Temperature dependence (of air width)
Pressure shift
upper vibrational quanta
lower vibrational quanta
upper local quanta
lower local quanta
Error codes
Reference codes
Flag for line-mixing
upper statistical weight
lower statistical weight'''

fields = fields.split('\n')


# ---------------------
# ---------------------
#
# The HITRAN format
#
# 0   Molecule number
# 1   Isotopologue number
# 2   Transition wavenumber (cm-1)
# 3   Line Intensity
# 4   Einstein A-coefficient
# 5   Air- broadened width
# 6   Self- broadened width
# 7   lower-state Energy
# 8   Temperature dependence (of air width)
# 9   Pressure shift
# 10  upper vibrational quanta
# 11  lower vibrational quanta
# 12  upper local quanta
# 13  lower local quanta
# 14  Error codes
# 15  Reference codes
# 16  Flag for line-mixing
# 17  upper statistical weight
# 18  lower statistical weight
#
# ---------------------
# ---------------------

# opening the data from 2015 paper
#inp_data2 = []
#with open('apjs504015_data/Table_S5.par','r') as fin:
#    inp_data2=fin.readlines()

## picks out the data for O12C16, which is molecule 5:1
#inp_data = [d for d in inp_data2 if '51' in d[:3]]

inp_data = pickle.load( open( "line_data.p", "rb" ) )

data = []

# reads and split the data according to the format previously defined
for q in range(len(inp_data)):
    i = inp_data[q]
    temp = i[:160]
    temp2 = [i[hfor[j]:hfor[j+1]] for j in range(len(hfor)-1)]
    temp3 = i[160:]
    temp3 = temp3.split()

    temp4 = temp2 + temp3
    
    llj = float(temp4[13][6:])
    if 'R' in temp4[13]:
        ulj = llj + 1
    elif 'P' in temp4[13]:
        ulj = llj - 1
    j1 = llj
    j2 = ulj
    
    v1 = float(temp4[11])
    v2 = float(temp4[10])
    
    # makes a dictionary with all line information
    dict_temp = {'e1': float(temp4[7]), 'e2': float(temp4[7]) + float(temp4[2]), 
    'dele' : float(temp4[2]), 'delj':j2-j1, 'delv': v2-v1, 'v1': v1, 'v2': v2, 'j1':j1,
    'j2':j2, 'a-val':float(temp4[4]), 'indi':q, 'g1': float(temp4[18]), 'g2': float(temp4[17])}
    data.append(dict_temp)

data2 = []

# this is put into data2, a list of dictionaries
for d in data:
    if d['v2'] <numax:
        if d['j2'] < jmax and d['j1'] < jmax:
            if d['delv']>0 and d['delv']<=delv:
                data2.append(d)

# picks out the upper e2 and lower e1 energy levels
el1 = [d['e1'] for d in data2]
el2 = [d['e2'] for d in data2]


#data is from Young, LA 1968
we      = 2169.826
wexe    = 13.295
weye    = 0.0115
be      = 1.931285
ae      = 0.017535
ye      = 1.01*1e-5
de      = 6.12*1e-6
betae   = -1e-9

#this version of the energy levels are from Young, LA 1968 and used in Spyromilio+ 88
#returns the energy elevels of the thing
def E_levels(nu, j):
    bv = be - ae * (nu + 0.5) + ye * (nu + 0.5)**2
    dv = de + betae * (nu + 0.5)
    
    gv = we*(nu + 0.5) -wexe * (nu + 0.5)**2 + weye *(nu + 0.5)**3
    
    fvj = bv * j *(j+1) -dv * j**2 *(j+1)**2
    
    el1 = gv + fvj
    
    temp = el1 -(we*( 0.5) -wexe * ( 0.5)**2 + weye *( 0.5)**3)
    return temp


# statisitical weights calculated (i think!)
def stat_weight(j):
    return 2*j+1

all_lev = []

for d in data2:
    l1 = [d['v1'], d['j1']]
    l2 = [d['v2'], d['j2']]
    all_lev.append(l1)
    all_lev.append(l2)

energy_levels=[]

def unique(list1): 
  
    # intilize a null list 
    unique_list = [] 
      
    # traverse for all elements 
    for x in list1: 
        # check if exists in unique_list or not 
        if x not in unique_list: 
            unique_list.append(x) 
    # print list 
    return unique_list

all_lev = unique(all_lev)

for al in all_lev:
    nu = al[0]; j = al[1]
    energy_levels.append([nu, j, E_levels(nu, j), stat_weight(j)])


el_unsorted = [e[2] for e in energy_levels]

energy_levels = [x for _,x in sorted(zip(el_unsorted,energy_levels))]

el = [e[2] for e in energy_levels]

## ----------------------------------------------------
## plotting stuff
## ----------------------------------------------------

#plt.figure(1)
#for e in el:
#    plt.axhline(e, color = 'orange')

#plt.figure()

#for d in data2:
#    de = d['e2'] - d['e1']
#    aval = d['a-val']
#    plt.scatter(de, aval)
#plt.show()





temp_aval = []


for d in data2:
    if d['delv'] == 1. and d['v1']==0:
        temp_aval.append([d['j1'], d['j2'], d['a-val']])



temp_aval.sort()


# Calculating the output for rad lines
def out_calc_rad(inp_dict):

    v1 = inp_dict['v1']; j1 = inp_dict['j1']
    v2 = inp_dict['v2']; j2 = inp_dict['j2']
    
    e1 = E_levels(v1, j1)
    e2 = E_levels(v2, j2)
    
    a1 = inp_dict['a-val']
    
    indi1 = find_nearest(el, e1)
    indi2 = find_nearest(el, e2)
    
    wl = 1/(e2-e1) * 1e8
    
    if j2-j1 == 1:
        a0 = temp_aval[2*int(j1)][2]
    elif j2 - j1 == -1:
        a0 = temp_aval[2*int(j1)-1][2]
    
    upsi = 0.0; T0 = 0.0; beta = 0.0
    return [indi2, indi1, wl, a1, upsi, T0, beta]

def coe_coll(T):
    return 1.9 * 1e-11 * T**(1/3.) * ((2000 / T)**(1/3) + 50 * (1 + 7e4/T)**0.5 * np.exp(-10210/T)) * np.exp (-4.5 * 1e-5 * T)

def rate_const_nou(T, gu, del_e):
    temp = 8.629 * 1e-6 / (T**0.5)
    return temp / gu

def fit_fun(t, beta, u0, T0):
    return u0 * (t/T0)**beta

def upsi_fit(T0, gu, del_e, v1, v2):
    t_temp = np.linspace(1e3, 1e4, 100)
    u_temp = sca_chandra(v1, v2) *coe_coll(t_temp) / rate_const_nou(t_temp, gu, del_e)
    
    u0 = sca_chandra(v1, v2) * coe_coll(T0) / rate_const_nou(T0, gu, del_e)
#    print(v1, v2, sca_chandra(v1, v2))
    
#    popt, pcov = curve_fit(fit_fun, t_temp, u_temp)
    popt,pcov = curve_fit(lambda t, beta: fit_fun(t, beta, u0, T0), t_temp, u_temp)
    return t_temp, u_temp, popt[0], u0, T0

def sca_chandra(v1, v2):
    dv = v2 - v1
    temp = (2 * v1 + 1) / (2 * (v2 - 1) + 1) * v2 
    return temp

def sca_elitzur(v1, v2, dele, T):
    temp1 = (v2 - v1)
    temp2 = np.exp( -(v2 - v1 - 1) * (1.5 * dele / (kb*T)) / (1 + 1.5 * dele / (kb*T)))
    return temp1 * temp2

def sca_scoville(a10, aij):
    temp = aij / a10 

# Calculating the output for collision lines
def out_calc_coll(inp_arr, dv):

    v1 = inp_arr[0]; j1 = inp_arr[1]
    v2 = v1 + dv; j2 = j1
    
    e1 = E_levels(v1, j1)
    e2 = E_levels(v2, j2)
    
    a1 = 0.0
    
    indi1 = find_nearest(el, e1)
    indi2 = find_nearest(el, e2)
    
    wl = 1/(e2-e1) * 1e8
    
    gu = stat_weight(j1)
    
    x, y, beta, u0, T0 = upsi_fit(4000, gu, e2-e1, v1, v2)
    
#    plt.figure()
#    plt.plot(x, y, label = 'Real value')
#    plt.plot(x, u0 * (x / T0)**beta, label = 'Fit in SUMO')
#    plt.yscale('log')
#    plt.xscale('log')
#    plt.title('%i %i -> %i %i'%(v1, j1, v2, j2))
#    plt.xlabel('Temperature [K]')
#    plt.ylabel(r'$\Upsilon$')
#    plt.legend(loc = 'best')
#    plt.savefig('res_pic/img%i%i-%i%i.png'%(v1,j1,v2,j2), bbox_inches = 'tight')
#    plt.close('all')
    
    return [indi2, indi1, wl, a1, u0, T0, beta]

# Large collision rates for lines in same ban
def out_calc_boltz(inp_arr1, inp_arr2):

    v1 = inp_arr1[0]; j1 = inp_arr1[1]
    v2 = inp_arr2[0]; j2 = inp_arr2[1]
    
    e1 = E_levels(v1, j1)
    e2 = E_levels(v2, j2)
    
    a1 = 0.0
    
    indi1 = find_nearest(el, e1)
    indi2 = find_nearest(el, e2)
    
    wl = 1/(e2-e1) * 1e8
    
    gu = stat_weight(j1)

    
    return [indi2, indi1, wl, 0.0, 1e8, 4000., 0.0]




# creates the output list
outp = []

# -------------------------------------
# -------------------------------------
#Avalues
for d in data2:
    outp.append(out_calc_rad(d))
# -------------------------------------
# -------------------------------------
#e- collisions
for d in energy_levels:
    for dv in range(1, delv+1):
#        print(d[0], dv, d[0]+dv, numax)
        if d[0]+dv < numax:
#            print(dv, d[0], numax)
#            print(d[0], d[0] + dv, dv,  numax)
            outp.append(out_calc_coll(d, dv))
# -------------------------------------
# -------------------------------------
#LTE in bands
##for v in range(0, numax):
##    for j in range(1, jmax):
##        d2 = [v, j, E_levels(nu, j), stat_weight(j)]
##        d1 = [v, j-1, E_levels(nu, j-1), stat_weight(j-1)]
##        print(d1[0], d1[1], d2[0], d2[1])
##        outp.append(out_calc_boltz(d1, d2))
# -------------------------------------
for v in range(0, numax):
    for j in range(1, jmax):
        d2 = [v, j, E_levels(nu, j), stat_weight(j)]
        for k in range(0, j):
            d1 = [v, k, E_levels(nu, k), stat_weight(k)]
            print(d1[0], d1[1], d2[0], d2[1])
            outp.append(out_calc_boltz(d1, d2))
# -------------------------------------

outp = [x for _,x in sorted(zip([d[0] for d in outp],outp))]


# creates the output file
file = open('cmI-v%i-j%i.dat'%(numax, jmax),'w')

file.write('n       E[cm-1]    g\n')
file.write('number of lines: %i\n'%len(energy_levels))

for i in range(len(energy_levels)):
    file.write('%i %.3f %i\n'%(i+1, energy_levels[i][2], energy_levels[i][3]))

file.write('i j WL[A]           A[s-1]     Upsilon(T0)  T0[K]   Power-law\n')
file.write('number of transitions: %i\n'%len(outp))

for i in range(len(outp)):
    op = outp[i]
    file.write('%3i %3i %.2f %.2e %.2e %.2e %.2e\n'%(op[0]+1, op[1]+1, op[2], op[3], op[4], op[5], op[6]))

file.close()


## -------------------------------------
##checking the elevels

#elevels = np.ones(7*40).reshape(7, 40)

#for d in data2:
#    v1 = d['v1']
#    j1 = d['j1']
#    e1 = d['e1']
#    elevels[int(v1)][int(j1)] = e1
#    
#    e11 = E_levels(int(v1), int(j1))
#    
##    print(e1, e11)
#    
#    v2 = d['v2']
#    j2 = d['j2']
#    e2 = d['e2']
#    
#    e22 = E_levels(int(v2), int(j2))
#    
##    print(e2/e22)
#    
#    elevels[int(v2)][int(j2)] = e2
#    
#    if np.abs(e2 - e22) > 0.1:
#        print(e2 - e22, d)



#
#
#
#
# temp = []
# for j in range(1,len(energy_levels)):
#     for i in range(0,j):
#         ei = energy_levels[i]
#         ej = energy_levels[j]
#
#         gi = stat_weight(ei[1])
#         gj = stat_weight(ej[1])
#
#         dele = ej[2] - ei[2]
#
#         delg = gj - gi
#
#         if dele < gj * 40 and delg > 0:
#             if gj > stat_weight(32):
#
#                 derp = (gj-65) * 70
#
#                 if dele < derp and delg < 14:
#                                         #
#                     # print('--u---', derp)
#                     # print(ej)
#                     # print(ei)
#                     if ei[0] != ej[0]:
#                         print('--u---', derp)
#                         print(ej)
#                         print(ei)
#                 elif dele > derp and delg > dele / 80:
#                     if ei[0] != ej[0]:
#                         print('--l---', derp)
#                         print(ej)
#                         print(ei)
#             elif delg > dele / 80:
#                 if ei[0] != ej[0]:
#                     print('--o---', derp)
#                     print(ej)
#                     print(ei)
#
#
#
#
#
#
#
#
# diff = []
# for v in range(7):
#     j1= np.arange(1, 151)
#     j2= np.arange(0, 150)
#     elev1 = E_levels(v, j1)
#     elev2 =  E_levels(v, j2)
#     print(max(elev1 - elev2))


#i = 33
#int(i/33)*33






























