SUMO - Supernova MOnte Carlo code.
Papers : Jerkstrand, Fransson & Kozma 2011, see also
Jerkstrand et al. 2012, Jerkstrand et al. 2015 (A&A).

PREPARE THE ENVIRONMENT
-------------------------------------
  
1. The code needs access to the LAPACK and BLAS mathematical libraries. If you do not have these installed, download them from www.netlib.org and install. All clusters have these available, see later for how to link. 

2. Some environmental variables need to be set (normally in .bashrc):

	Code location:
```bash
    export SUMODIR='location-of-code-source' (replace with relevant path)
```
	A directory to store run results (with getfiles.sh and putfiles.sh):
```bash
    export DD='location-of-scratch-folder'
```
	BLAS/LAPACK:
```bash
    export SUMO_LIBS='location-of-blas-library location-of-lapack-library'(replace with relevant path) # custom installation
```
```bash
    export SUMO_LIBS='-llapack -lblas'  # BESKOW and laptop
```
```bash
    export SUMO_LIBS='-L/location-of-lapack-and-blas -llpack -lblas'(replace with relevant path) ??
```
	To use MKL library files for BLAS and LAPACK, instead use
```bash
    export SUMO_LIBS='$(MKLROOT)/lib/em64t/libmkl_lapack95_lp64.a -Wl,--start-group $(MKLROOT)/lib/em64t/libmkl_intel_lp64.a $(MKLROOT)/lib/em64t/libmkl_intel_thread.a $(MKLROOT)/lib/em64t/libmkl_core.a -Wl,--end-group -L$(MKLROOT)/lib/em64t -liomp5 -lpthread'
```

	Set compiler and MPI implementation:
```bash
    export SUMO_FC='gfortran'/'ifort'/'ftn'   # Use ftn on BESKOW
```
```bash
    export SUMO_FC_MPI='mpifort'/'mpiifort'/'ftn' (gfortran/ifort, respectively) #Use ftn on BESKOW
```

COMPILATION
-------------------------------------

The code compiles with a Makefile.

Compilation is by default with -O3 optimization, edit this in
`FFLAGS=...` if needed. Note that -O3 is not in all instances faster than -O2, this should be tested.

Go into the folder where you unpacked the code and run
```bash
     make all
```

Compilation creates a large set of executables, most of which are called sequentially in an iteration loop when running (the script iterate.sh handles this). The main code is made of the three executables

* "suse" : Calculates temperatures, excitation structure, and emissivities (radiative terms and ionization balance fixed)

* "sumo" : Runs Monte Carlo simulation of radiation field (temperatures, excitation and ionization structure fixed)

* "suib" : Calculates ionization structure (temperatures,  excitation structure, radiative terms fixed).

These three are iterated until convergence is achieved. Information on how to construct wraps and test for convergence to be added..

RUNNING
-------------------------------------

You can run the code from any directory : to be able to do multiple runs in parallell it is necessart that you position yourself in a unique directory for each run. If its the first time you run from that particular directory, do

```bash
$SUMODIR/dirmake.sh
```
This links the input directories (e.g. all the atomic data) from the code directory, and creates output directories attached to the run location.

Then, running involves:

### Ex suse: (NOTE don't run yet, see first preparation of input files below)

```bash
mpirun -np 19 $SUMODIR/suse example file_modelparameters_example y n 500 10000 1 > outfile_suse
```

Note "srun -n 19" instead if using srun rather than mpirun. 

This runs a model with name `example` and run-file named `file_modelparameters_example`. The `y` means compute the gamma-ray deposition (only needed in the first iteration), the `n` means this is not an iterative run (but the first one), `500` means bluest wavelength is 500 A, `10000` means longest wavelength is 10000 A, `1` means use resolution of 1 A (at 1000 A). The number of processes (`19` here) has to be the number of zones in the model plus one; so here the model must have 18 zones. In the first iteration one should use the combination `y n`, whereas later on `n y` should be used. Note that its possible to select a subset of the shells present in the explosion model, if you want to use only the first 2 shells put Nr to 2 in the runfile and run suse with -np 3 (always number of shells plus 1).

### Ex sumo (always after suse):

```bash
mpirun -np 100 $SUMODIR/sumo example file_modelparameters_example 500 10000 1 100000000 5 5 50 > outfile_sumo
```

Here, `example`, `file_modelparameters_example`, `500`, `10000`, and `1` mean same as before (keep the same as when running suse). `100000000` means use a (fiducial) total number of 100000000 packets (this is, in fact, not really the total number of packets but a basis number to calculate the number of packets emitted in a given zone/wavelength bin). `5 5 50` means, use at least 5 packet per optical/infrared energy bin, at least 5 packets in the UV, and never use more than 50 packets for a given zone/wavelength bin.

For sumo the number of processors can be anything (100 in the example). If running on your laptop use the number of cores you have (typically 2, 4 or 6).
Scaling should be relatively linear with number of processors if a large number of packets are sent.

Then run

```bash
$SUMODIR/gather.sh     
```
which is an intermediate step necessary between sumo and suib.

### Ex suib (always after sumo):
```bash
mpirun -np 19 $SUMODIR/suib file_modelparameters_example n > outfile_suib
```

The `n` means it not an iterative run : change this to `y` for subsequent iterations.

After suib completes run
```bash
$SUMODIR/adcoolcollector file_modelparameters_example
```

The total emergent flux can be checked by running 

```bash
$SUMODIR/process example file_modelparameters_example 1000 1000
```
(Ignore meaning of the two 1000 for now). Upon convergence, in steady state, the emergent flux should be close to the total deposited energy by radioactivity or other external power input.

PREPARING INPUT FILES
---------------------------------------
The code needs two input files to run, the explosion model file, and the model parameters file.

1) The explosion model file.

   An example file is provided (`DATA/expmodels/file_explmodel_example`).

   The format of these files should be as follows:
```
   Line
   1    Name of model and then name of various zones (not read by program so just for clarity)
   2	Mass of zone [Msun]
   3	Inner zone velocity [km/s]
   4	Outer zone velocity [km/s]
   5	Filling factor. Is normally 1. But the code contains an option to macroscopically mix the first Ncore (set in model parameters file) zones. The velocity limits for all these are then the inner velocity of zone 1 and the outer velocity of zone Ncore. The filling factor for each of the first Ncore zones can then be smaller than 1 (but should add up to 1).
   6	Density at 100 days [g cm^-3]. (not read by program so just for clarity)
   7	---------
   8	56Ni+56Co mass fraction (at t=0, before any decay has occurred).
   9	57Ni mass fraction (at t=0, before any decay has occurred).
   10	44Ti mass fraction (at t=0, before any decay has occurred).
   11	---------
   12	Z1   Mass fractions, in each zone, for element of atomic number Z1..
   13	Z2	 Mass fractions, in each zone, for element of atomic number Z2..
   ...
```

   Note that all entries Z1, Z2, ... refer to stable element masses.

2) The model parameters file.

   An example file (`file_modelparameters_example`) is provided.

   The philosophy of the code design is to maintain a core source code that should not have to be recompiled depending on which models or parameter space is to be investigated. If one wishes to run 10 different models with 10 different dust parameters, for example, create 10 different "run settings" files, and set up a script to process these in turn.  

   The format the model parameters file is as follows:

```
Row
1      	Time [days since explosion]
2      	Distance to SN [MPc]
3      	Dust optical depth [dim-less]
4      	Explosion model file (see above)
5      	Number of zones
6      	Ncore : Number of zones to macroscopically mix ("0" for no macroscopic mixing).
7      	Ndust : Number of zones to apply dust opacity to
8      	Nblobs : Number of clumps, per zone, in macroscopically mixed model
9      	--------------------------
10     	VOID
11     	VOID
12     	VOID
13     	VOID
14     	VOID
15     	VOID
16     	VOID
17     	VOID
18     	-----------------------
19     	e+ leakage : 1 => free streaming (no magnetic field), 0 => on-the-spot absorption (complete magnetic confinement). 
20     	Npis : 	Maximum number of levels to include photoionization from. 0 => no photoionizations from excited states. Typical value : 50
21     	PHOTOEXC : Include photoexcitation in NLTE solutions? 2 => Yes. 1 => Only deexcitations. 0: No. Typical value : 2.
22      beta_Lya : Probability of Lya branching into overlapping metal lines. Typical value : 5E-9.
23      beta_Lyb : Probability of Lyb branching into overlapping metal lines. Typical value : 2.1E-5.
24      tau_tresh : Ignore line optical depths below this value. Typical value : 1E-3.
25 	sf_tresh : Ignore computation of non-thermal excitation and ionization for elements with mass fractions below this value. Typical value 1E-3.
26  	CTON : .F. => Charge transfer reactions on. .T. => On, .F. => Off. Typical value : .F.
27      MOLSW : 0 => No molecules. 1 => Old molecule treatment 3 => New molecule treatment. 
28   	OLI : Allow only photoionization by local radiation field? .F. => No, .T.=> Yes. Typical value : 0.
29      REALRECCONT_H : Use real free-bound continua for HI? .T. => Yes. .F. => No (then uses box-shaped approximation as for other elements). Typical value : .T.
30      FORCECAII : Force Ca to be in the Ca II form everywhere? .F. => no, .T. => yes. Typical value : .F.
31     	VOID
32     	VOID
33      DOPRINT : Verbose print output? .T. => Yes, .F. => No. Typical value: .F. (.T. can slow down the code)
```

RUNNING WITH MOLECULES
--------------------------

runfile : MOLON = 3  (1 and 2 depreciated). 

m1.f90 : Nmol		Number of molecular species in the network (different ion stages count separately). Should match list size in DATA/mol_input/MBINCODES and DATA/mol_input/moldata_input.csv.
m1.f90 : Nmol_NLTE	Number of molecules (different ion states bundled) to include emission/absorption from.
m1.f90 : Nelem	    Should equal number of atomic elements plus Nmol_NLTE.

Current default is Nmol_NLTE = 4 (CO, SiO, SiS, and SO).

The ABIN slot range 126-145 is reserved for these 4 molecules, so NABIN has to be >= 145 to include them.

Make sure to have files DATA/mol_input/MBINCODES and DATA/mol_input/ABINCODES updated : a new network based on these can be created by
DATA/mol_input/func_dbfort.py. Network to use is specified (hardcoded) in molecules.f90.

EXAMINING THE OUTPUT
-------------------------------------------

   Spectrum
   -----------

   The main output of the code is the model spectrum, which is written to the file ./out/modelspectra/spectrum.dat*RUNID*.
   The columns in this file are as follows:
```
   1	Wavelength [A]
   2	F_lambda [erg s^-1 cm^-2 A^-1]
   3	F_lambda, smoothed with Vsmooth
   4    component1..
   5    component2..
   ..
   ..
```

   Contribution by element ABIN is in column 5 + ABIN

   Temperatures (from suse)
   -------------  
   Written to ./out/plts/teqs.dat*SHELLNUMBER*
 
   Excitation (from suse)
   --------------
   Written to ./out/calculatecubes/EXC_CUBE.dat*SHELLNUMBER*

   Ionization (from suib)
   --------------
   Written to ./out/ionization/ionfracs.dat*SHELLNUMBER*

