SUBROUTINE alfa_photoionization(lambda, Natoms, atoms, EXC_MAT, nmolvec, alfa_dust, T, ne, ndensvec_abin, alfa_tot, ALFA_MAT)    
USE M1, ONLY : Nabin, Nmol, Nelem, Npis, map2, atomm, lambdaion, ngvec, Zvec, minshellvec, maxshellvec, Nstages
USE INTERFACE, ONLY : phfit2
use PARAMETERS, ONLY : h, c, eV, k
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: lambda, alfa_dust, T, ne
INTEGER, INTENT(IN) :: Natoms
TYPE(atomm), DIMENSION(Natoms), INTENT(IN) :: atoms
REAL, DIMENSION(:,:), INTENT(IN) :: EXC_MAT  ! Dim is Nabin*Npis
DOUBLE PRECISION, DIMENSION(maxval([1,Nmol])), INTENT(IN) :: nmolvec
DOUBLE PRECISION, DIMENSION(Nabin), INTENT(IN) :: ndensvec_abin
DOUBLE PRECISION, INTENT(OUT) :: alfa_tot
DOUBLE PRECISION, DIMENSION(:, :), INTENT(OUT) :: ALFA_MAT  !Last one is the dust  2*Nelem+Nmol+3 * Npis
DOUBLE PRECISION, EXTERNAL :: gbf
! ===========================================================
! Task  : Computes the continuum absorption coefficient (photoionization + free-free + electron scattering + dust) per unit length (alfa_tot= [cm^-1]) at the specified wavelength.
! Input : lambda             Wavelength [�ngstr�ms]
!         Natoms             Number of atoms with excitation structure solutions (length of atoms array)
!         atoms              Array with atoms [type ATOMM, see m1 module] with solved excitation structures
!         EXC_MAT            Matrix with level populations [cm^-3]: first index is ABIN code, second excitation level number
!         alfa_dust          Dust absorption coeffient [cm^-1]
! Output: alfa_tot           Total continuum absorption coefficient [cm^-1]
!         ALFA_MAT           Matrix with individual absorption coefficients [cm^-1]
! ===========================================================
TYPE(atomm) :: atom
DOUBLE PRECISION, DIMENSION(100,Nstages-1,Npis) :: SIGMA0, BETA ! First index is atomic number
INTEGER :: i, j, m
DOUBLE PRECISION :: sigma, energy_eV, Etresh, energy_erg, Z, gff, nu, nplus, sig2s, sig2p, gauntfactor_2s, gauntfactor_2p, wll, gauntfactor, Zuse
INTEGER, DIMENSION((Nstages-1)*Nelem) :: Zvecdouble, nevec, shellvec
INTEGER, DIMENSION(Nelem) :: neutrals, singlyionized, doublyionized, triplyionized
LOGICAL :: FOUND
! -----------------------------------------------------------

ALFA_MAT = 0   ! Important
SIGMA0 = 0     ! Important
BETA = 0       ! Important
Zvecdouble = 0
nevec = 0      ! Important

! ------------------------------------------------------------

! Set the number of protons:  ! TEMP define this globally once in mak_atom instead?

DO i = 1, Nelem

	DO j = 1, Nstages-1

    	Zvecdouble((Nstages-1)*(i-1)+j) = Zvec(i)
        
    END DO

END DO

IF (Zvecdouble((Nstages-1)*Nelem) .EQ. 0) THEN; PRINT *, 'ERROR in alfa_photoionization. Last element is not allocated.'; stop 1; END IF  ! A check to see that newly added elements are also listed above

neutrals = [((Nstages-1)*(i-1)+1, i=1,Nelem)]  ! 1,3,5,...

singlyionized = [((Nstages-1)*(i-1)+2, i=1,Nelem)] ! 2,4,6,... Avoid HII TEMPFIX

if (Nstages >= 4) then

	doublyionized = [((Nstages-1)*(i-1)+3, i=1,Nelem)] ! No HIII or HeIII

    if (Nstages >= 5) then

		triplyionized = [((Nstages-1)*(i-1)+4,i=1,Nelem)] ! No HIII or HeIII or CIII

    end if

end if

nevec(neutrals) = Zvecdouble(neutrals)  ! Neutrals have as many electrons as protons

nevec(singlyionized) = Zvecdouble(singlyionized) - 1  ! Singly ionized have one less electron than protons

if (Nstages >= 4) then

	nevec(doublyionized) = Zvecdouble(doublyionized) - 2

    if (Nstages >= 5) then

		nevec(triplyionized) = Zvecdouble(triplyionized) - 3

    end if

end if

! ID   Number of  Innermost        Outermost ionizable shell
!      levels in  ionizable
!      ground     shell
!      state

!IF (minshellvec((Nstages-1)*Nelem) .EQ. 0) THEN; PRINT *, 'ERROR in alfa_photoionization. Last element has no minshell/maxshell';stop 1; END IF TEMPFix I think this is not needed

SIGMA0(:,1,:) = 7.9E-18; BETA = -3;   ! TEMP SET DEFAULT CROSS SECTION FOR EXC STATES , Jerkstrand2012  CARE

SIGMA0(:,2,:) = 7.9E-18/4.0  ! account for Z^-2 term

if (Nstages == 4) then

	SIGMA0(:,Nstages-1,:) = 7.9E-18/9.0

elseif (Nstages == 5) then

	SIGMA0(:,Nstages-2,:) = 7.9E-18/9.0

	SIGMA0(:,Nstages-1,:) = 7.9E-18/16.0

end if

! Set cross section for excited states: (ground state is done by routine later)

! HI -------------

!!!SIGMA0(1,2) = 1.5E-17; BETA(1,2) = -2.25     ! 2s 3647 �   THESE HAVE SPECIAL ROUTINES SEE BELOW
!!!SIGMA0(1,3) = 1.4E-17; BETA(1,3) = -3.3      ! 2p 3647 �

IF (Npis >= 2) THEN

   ! HeI ------------

   SIGMA0(2,1,2) = 6.0E-18; BETA(2,1,2) = -1.4      ! 2599 

   IF (Npis >=3) THEN;SIGMA0(2,1,3) = 1.0E-17; BETA(2,1,3) = -2;END IF        ! 3120    CARE there are strong resonances

   ! CI ------------

   !SIGMA0(5,4) = 2.0E-17; BETA(5,4) = -1.5   ! 1240 CARE complex
   !SIGMA0(5,5) = 4.0E-17; BETA(5,5) = -2     ! 1446 CARE complex
   !SIGMA0(5,6) = 2.5E-17; BETA(5,6) = -1.6   ! 1752
   ! J11 states TOPBASE but x-sections are marked out since 2012 in code because they are so complex, and this is also noted on the ARC pages.

   ! CII ----------------

   !SIGMA0(6,3) = 1.0E-17; BETA(6,3) = -2.3           ! Ok quite smooth
   !SIGMA0(6,4) = SIGMA0(6,3); BETA(6,4) = BETA(6,3)
   !SIGMA0(6,5) = SIGMA0(6,3); BETA(6,5) = BETA(6,3)

   ! NI ------------------

   SIGMA0(7,1,2) = 2.0E-17; BETA(7,1,2) = -1    ! 1021  CARE several resonances
   IF (Npis >=3) THEN;SIGMA0(7,1,3) = SIGMA0(7,1,2); BETA(7,1,3) = BETA(7,1,2); END IF ! 1021 

   ! OI ------------------

   IF (Npis >=4) THEN; SIGMA0(8,1,4) = 1D-50; BETA(8,1,4) = -2; END IF    ! The 1D --> 4S transition is forbidden (Delta-S = 1 cannot occur). Email discussion with Nahar, and also TOPBASE values start only at energies that can ionize to excited stated in OII. Could in principle put zero here but in case program would replace zero entries with some deafult better put to a low value.

   IF (Npis >=5) THEN; SIGMA0(8,1,5) = 1D-50; BETA(8,1,5) = -2; END IF   ! 1S --> 4S forbidden (Delta S = 1)

   IF (Npis >=6) THEN

      IF (lambda > 1600.) THEN ! Complex cross section, split in two power laws

         SIGMA0(8,1,6) = 0.35*1E-18;  BETA(8,1,6) = +0.4  ! slopes upwards from threshhold

      ELSE

         SIGMA0(8,1,6) = 1.3E-18; BETA(8,1,6) = -2   ! Then excited state in OII comes in

      END IF

   END IF

   IF (Npis >=7) THEN

      IF (lambda > 1600.) THEN

         SIGMA0(8,1,7) = 0.5*1E-18;  BETA(8,1,7) = 0  ! very many resonances : approximately constant if smoothed

      ELSE

         SIGMA0(8,1,7) = 1.3E-18; BETA(8,1,7) = -2   ! Then excited state in OII comes in. Similar as level 6 at cutoff.

      END IF

   END IF

   ! MGI -----------------

   SIGMA0(12,1,2) = 1.7E-17; BETA(12,1,2) = -4.0 ! 2511  CARE resonances
   IF (Npis >=3) THEN; SIGMA0(12,1,3) = SIGMA0(12,1,2); BETA(12,1,3) = BETA(12,1,2); END IF ! 2511
   IF (Npis >=4) THEN; SIGMA0(12,1,4) = SIGMA0(12,1,2); BETA(12,1,4) = BETA(12,1,2); END IF ! 2511

   IF (Npis >=5) THEN   ! TOPBASE

      IF (lambda > 3041.) THEN
         
         SIGMA0(12,1,5) = 7.7E-17; BETA(12,1,5) = 0

      ELSEIF (lambda < 2914.) THEN

         SIGMA0(12,1,5) = 3.5E-17; BETA(12,1,5) = -6

      ELSE

         SIGMA0(12,1,5) = 2E-16; BETA(12,1,5) = 0  ! Strong resonance

      END IF

   END IF

   ! MgII -----------------

   IF (lambda < 1599) THEN

      SIGMA0(12,2,2) = 5.5E-19; BETA(12,2,2) = -2.5   ! 1168 

   ELSE

      SIGMA0(12,2,2) = 3.2E-19; BETA(12,2,2) = -0.3

   END IF

   IF (Npis >=3) THEN; SIGMA0(12,2,3) = SIGMA0(12,2,2); BETA(12,2,3) = BETA(12,2,2); END IF

   ! SiI ----------

   !IF (lambda > 1193.) THEN

   !   SIGMA0(17,4) = 3E-17; BETA(17,4) = +3.5    ! 1682 �

   !ELSE

   !   SIGMA0(17,4) = 7E-16; BETA(17,4) = -7

   !END IF

   ! SI ------------------------------

   IF (Npis >=4) THEN; SIGMA0(16,1,4) = 6.0E-17; BETA(16,1,4) = -1; END IF  ! 1345  CARE complex

   ! CaII -----------------------------

   SIGMA0(20,2,2) = 6.0E-18; BETA(20,2,2) = -3   ! CaII  TOP-base own fit  

   IF (Npis >=3) THEN; SIGMA0(20,2,3) = 8.0E-18; BETA(20,2,3) = -3; END IF

   IF (Npis >=4) THEN; SIGMA0(20,2,4) = 1.5E-18; BETA(20,2,4) = -3; END IF

   IF (Npis >=5) THEN; SIGMA0(20,2,5) = SIGMA0(20,2,4); BETA(20,2,5) = BETA(20,2,4); END IF

   ! FeI --------------------------------

   IF (Npis >=6) THEN

      IF (lambda > 1030.) THEN          ! Nahar online

         SIGMA0(26,1,6) = 6.5E-18; BETA(26,1,6) = +0.7

      ELSE

         SIGMA0(26,1,6) = 1.5E-17; BETA(26,1,6) = -0.7

      END IF

   END IF

   IF (Npis >=7) THEN; SIGMA0(26,1,7) = SIGMA0(26,1,6); BETA(26,1,7) = BETA(26,1,6); END IF
   IF (Npis >=8) THEN; SIGMA0(26,1,8) = SIGMA0(26,1,6); BETA(26,1,8) = BETA(26,1,6); END IF
   IF (Npis >=9) THEN; SIGMA0(26,1,9) = SIGMA0(26,1,6); BETA(26,1,9) = BETA(26,1,6); END IF
   IF (Npis >=10) THEN; SIGMA0(26,1,10) = SIGMA0(26,1,6); BETA(26,1,10) = BETA(26,1,6); END IF

   ! FeII --------------------------------

   IF (Npis >=6) THEN

      IF (lambda > 573.) THEN          ! Nahar on-line

         SIGMA0(26,2,6) = 1.1E-17; BETA(26,2,6) = +0.5

      ELSE

         SIGMA0(26,2,6) = 1.4E-17; BETA(26,2,6) = -0.3

      END IF

   END IF

   IF (Npis >=7) THEN; SIGMA0(26,2,7) = SIGMA0(26,2,6); BETA(26,2,7) = BETA(26,2,6); END IF
   IF (Npis >=8) THEN; SIGMA0(26,2,8) = SIGMA0(26,2,6); BETA(26,2,8) = BETA(26,2,6); END IF
   IF (Npis >=9) THEN; SIGMA0(26,2,9) = SIGMA0(26,2,6); BETA(26,2,9) = BETA(26,2,6); END IF

END IF

! -----------------------------------------------------------

energy_erg = h*c/(lambda*1E-8)  ! Transform photon energy to ergs

energy_eV = energy_erg/eV    ! Transform to eV

! Add contributions from the ground state (multiplet)

DO i = 1, (Nstages-1)*Nelem ! CARE only neutral and singly ionized species can be ionized

   Zuse = (Zvecdouble(i)-nevec(i)+1)

   IF ((i .EQ. 2) .or. (ngvec(i) == 0)) CYCLE   ! i=2 is a free proton. Also cycle if element not assigned. This covers also minshell=0 cases.  

   IF (lambda < lambdaion(i)) THEN   
      
      DO j = minshellvec(i), maxshellvec(i)   ! Several of the electrons can add to the opacity
        ! FIXME TEMP sl: this is a quickfix to exclude molecules from this step
         if ((Zvecdouble(i) <= 30) .and. (map2(i) .lt. 106)) then ! Verner routines goes to Z=30 only

                CALL phfit2(Zvecdouble(i), nevec(i), j, energy_eV, sigma)  ! Returns the ground state cross section in cm^2

         else ! Heavier element

                sigma = 7.9E-18*(lambda/lambdaion(i))**(3)/Zuse**2  ! TEMP, per shell? Hydrogenic for heavy elements

         end if
         ALFA_MAT(i,1:ngvec(i)) = ALFA_MAT(i,1:ngvec(i)) + sigma*EXC_MAT(map2(i), 1:ngvec(i))        
  
      END DO

      IF (map2(i) == 50) THEN  ! TEMPFIX CARE Ca II has a weird cross section from the verner routine

         ALFA_MAT(i,1:ngvec(i)) = 6E-16*(h*c/(lambda*1e-8)/eV)**(-3)*EXC_MAT(map2(i), 1:ngvec(i))  ! Note nu^-3 not (nu/nu0)^-3...this approximation is actually bit on low side of true cross section (sigma is ~5E-19 at threshold)

      END IF

   END IF

END DO

! IONIZATION FROM EXCITED STATES -------------------------

DO i = 1, Natoms

   j = 0   ! find the map2 value of this atom

   FOUND = .FALSE.

   DO WHILE (FOUND .EQV. .FALSE.)

      IF (atoms(i)%ABIN .EQ. map2(j+1)) THEN

         FOUND = .TRUE.

      END IF

      j = j + 1

     IF (j+1 > (Nstages-1)*Nelem) GOTO 50  ! Not found in map2 array so much be highest ion state, skip

   END DO

   IF (MINVAL([Npis,atoms(i)%N_LEVELS]) >= ngvec(j) + 1 ) THEN  ! If more levels than the ground multiplet are to be solved for

      DO m = (ngvec(j)+1), MINVAL([Npis,atoms(i)%N_LEVELS])  ! For each excited state..

         Etresh = atoms(i)%E_ION_GS - atoms(i)%LEVELS_E(m)   ! Ionization treshold of exc. state (ergs)

         IF (energy_erg > Etresh) THEN   ! If photons have energy above ion. threshold..

            IF (atoms(i)%NAME == ' HI  ' .AND. m == 2) THEN  ! Use same cross section as when computing the rec emission from to 2s

               wll = 12398.41896/(energy_erg/eV)   ! Wavelength in �ngstr�ms (E_EV is in eV)

               CALL hy_2s2p(wll, gauntfactor_2s, gauntfactor_2p, sig2s, sig2p)

               sigma = sig2s

            ELSE IF (atoms(i)%NAME == ' HI  ' .AND. m == 3) THEN  ! Use same cross section as when computing the rec emission from to 2p
               wll = 12398.41896/(energy_erg/eV)   ! Wavelength in �ngstr�ms (E_EV is in eV)

               CALL hy_2s2p(wll, gauntfactor_2s, gauntfactor_2p, sig2s, sig2p)

               sigma = sig2p

            ELSE IF (atoms(i)%NAME == ' HI  ' .AND. (m >= 3 .AND. m <= 5)) THEN  ! n = 3

               gauntfactor = gbf(3,energy_erg/eV)

               sigma = 7.9e-18*3*gauntfactor*(Etresh/energy_erg)**3  !

            ELSE
      
               sigma = SIGMA0(atoms(i)%Z,atoms(i)%CHARGE+1,m)*(energy_erg/Etresh)**BETA(atoms(i)%Z,atoms(i)%CHARGE+1,m)

            END IF

            ALFA_MAT(j,m) = sigma*EXC_MAT(map2(j), m)

         END IF

      END DO

   END IF

50 continue

END DO

! MOLECULES :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

! CO

! TEMPFIX are these used anywhere?

IF (lambda < 1100.) THEN

   sigma = 1E-18  ! CARE canonical value used for CO

ELSE

   sigma = 0.

END IF

if (Nmol >= 1) ALFA_MAT((Nstages-1)*Nelem+1,1) = sigma*nmolvec(1)

IF (lambda < 1100.) THEN  ! SiO -----------------------------

   sigma = 1E-17  ! CARE canonical value used for SiO

ELSE

   sigma = 0.

END IF

if (Nmol >= 3) ALFA_MAT((Nstages-1)*Nelem+3,1) = sigma*nmolvec(3)

! DUST :::::::::::::::::::::::::::::::::::::::::::::::::::::

ALFA_MAT((Nstages-1)*Nelem+Nmol+1,1) = alfa_dust

! BREMSSTRAHLUNG OPACITY ::::::::::::::::::::::::::::::::::::

gff = 1.3  ! CARE  Gaunt factor   see fig 5.2 in Rybicki Lightman for a more detailed treatment (but then change also in the routines where the Bremsstrahlung emission is computed)

nu = c/(lambda*1e-8)  ! Frequency (lambda is in �)

DO i = 1, Natoms   ! For each atom, compute Bremsstrahlung opacity caused by the NEXT ion  (ie HI --> HII)

   atom = atoms(i)

   if (atom%SPECIES_TYPE == 'mol') CYCLE

   IF (atom%ABIN .EQ. 1) THEN   ! HI special case, protons are at abin code 70

      nplus = ndensvec_abin(70)

   ELSEIF (atom%ABIN .EQ. 3) THEN  ! He II

      nplus = 0  ! TEMP n(HeIII) not stored anywhere as it is now

   ELSE IF (atom%ABIN < Nabin) THEN  ! all other ions are one step above the abin code of the previous ion

      nplus = ndensvec_abin(atom%ABIN+1)  ! TEMPfix would break if we would have all 5 elements present

   ELSE

      nplus = 0

   END IF

   Z = DBLE(atom%CHARGE) + 1.
    
   ALFA_MAT((Nstages-1)*Nelem+Nmol+2,1) = ALFA_MAT((Nstages-1)*Nelem+Nmol+2,1) + 3.7E8*T**(-0.5)*Z**2*ne*nplus*nu**(-3)*(1. - exp(-h*nu/(k*T)))*gff      ! Rybicki Lightman eq 5.18b (care 5.19b is only for RJ limit)

END DO

ALFA_MAT((Nstages-1)*Nelem+Nmol+3,1) = 6E-25*ne    ! Thomson scattering

! TOTAL OPACITY ::::::::::::::::::::::::::::

alfa_tot = SUM(ALFA_MAT)

END
