SUBROUTINE ALLOC(atom)
USE FILES, ONLY: dir_atomic_data, dir_molecular_data
USE M1, ONLY : atomm, myid, sharedcomm, window
USE PARAMETERS, ONLY : eV
USE memalloc_shared
IMPLICIT NONE

TYPE (atomm), INTENT(INOUT) :: atom
! ===============================================
! Task   : Allocates fundamental properties of atom : charge, mass, abincode etc. Also allocates arrays to be filled with atomic data
! Input  : atom            Atom (with atom%name filled in)
! Output : atom
! =============================================

CHARACTER(len=5) :: name
INTEGER :: errsig, ierr
atom%RR_ALPHA_EXISTS = .FALSE.
atom%RR_BETA_EXISTS = .FALSE.

! ------------------------------------------------------------

CALL chargereader(atom, errsig)   ! Determine charge (protons - electrons) of this atom

IF (errsig .NE. 0) THEN

   PRINT *, 'ERROR in alloc: chargereader does not recognize charge.'; stop 1

END IF

name = atom%NAME

atom%MINSHELL = 1 ! TEMP
atom%MAXSHELL = 1 ! TEMP
atom%N_LEVELS = 0

SELECT CASE (name)

  CASE(' HI  ')

     atom%Z = 1     ! Proton number
     atom%MASS = 1.008  ! Mass number
     atom%N = 465    ! Number of levels
     atom%Nlines = 8610 ! Number of lines
     atom%N_LEVELS = 210 ! Number of levels to calculate level populations for
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/HI/hI.dat'             
     atom%ABIN = 1        ! Code for this ion
     atom%E_ION_GS = 13.595*eV   ! Ground state ionization energy in ergs
     atom%NGROUND = 1
     !atom%GGROUND = 2
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.   ! Do specific recombination rates exist in the data file?
     atom%UPSILON0_EXISTS = .TRUE.    ! Do collision strengths exist in the data file?
     atom%UPSILON0_BETA_EXISTS = .TRUE.  ! Do T-dependence of collision strengths exist in data file?

!CHECK ABIN numbers for H2, CMI, SiO2 and SiSi are not consistent with KNa abin numbers: is this an issue?

  CASE ('H2  ')

     atom%Z = 1
     atom%MASS = 2.016
     atom%N = 2182
     atom%Nlines = 26618
     atom%N_LEVELS = 313
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/H2/h2.dat'
     atom%ABIN = 111
     atom%E_ION_GS = 15.43*eV   ! 803 �
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .FALSE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.
     atom%SPECIES_TYPE = 'mol'


  CASE ('CMI  ')

     atom%Z = 1
     atom%MASS = 28.0
     atom%N = 800
     atom%Nlines = 42310
     atom%FILE_ADDRESS = TRIM(dir_molecular_data) // '/CMI/cmI.dat'
     atom%ABIN = 126
     atom%MBIN = 1
     atom%E_ION_GS = 14.014*eV   
     atom%NGROUND = 1
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .TRUE.
     atom%SPECIES_TYPE = 'mol'

! carbon monoxide COI
  CASE ('CMII ')  

     atom%Z = 1
     atom%MASS = 28.0
     atom%N = 2
     atom%Nlines = 1
     atom%FILE_ADDRESS = TRIM(dir_molecular_data) // '/CMII/cmII.dat'
     atom%ABIN = 127
     atom%MBIN = 2
     atom%E_ION_GS = 41.8*eV   
     atom%NGROUND = 1
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .FALSE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.
     atom%SPECIES_TYPE = 'mol'

! silicon monoxide SiO
  CASE ('SMI  ')  

     atom%Z = 1
     atom%MASS = 44.0
     atom%N = 800
     atom%Nlines = 42152
     atom%FILE_ADDRESS = TRIM(dir_molecular_data) // '/SiOI/sioI.dat'
     atom%ABIN = 131
     atom%MBIN = 3
     atom%E_ION_GS = 11.600*eV   
     atom%NGROUND = 1
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .TRUE.
     atom%SPECIES_TYPE = 'mol'

! silicon monoxide SiO
  CASE ('SMII ')  

     atom%Z = 1
     atom%MASS = 44.0
     atom%N = 2
     atom%Nlines = 1
     atom%FILE_ADDRESS = TRIM(dir_molecular_data) // '/SiOII/sioII.dat'
     atom%ABIN = 132
     atom%MBIN = 4
     atom%E_ION_GS = 23.200*eV   
     atom%NGROUND = 1
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .FALSE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.
     atom%SPECIES_TYPE = 'mol'

! silicon monosulphide SiS
  CASE ('SSI  ')  

     atom%Z = 1
     atom%MASS = 60.0
     atom%N = 800
     atom%Nlines = 40730
     atom%FILE_ADDRESS = TRIM(dir_molecular_data) // '/SiSI/sisI.dat'
     atom%ABIN = 136
     atom%MBIN = 17
     atom%E_ION_GS = 10.530*eV   
     atom%NGROUND = 1
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .TRUE.
     atom%SPECIES_TYPE = 'mol'

! silicon monosulphide SiS
  CASE ('SSII ')  

     atom%Z = 1
     atom%MASS = 60.0
     atom%N = 2
     atom%Nlines = 1
     atom%FILE_ADDRESS = TRIM(dir_molecular_data) // '/SiSII/sisII.dat'
     atom%ABIN = 137
     atom%MBIN = 18
     atom%E_ION_GS = 21.06*eV   
     atom%NGROUND = 1
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .FALSE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.
     atom%SPECIES_TYPE = 'mol'

! sulfur monoxide SO
  CASE ('UMI  ')  

     atom%Z = 1
     atom%MASS = 48.0
     atom%N = 800
     atom%Nlines = 42152
     atom%FILE_ADDRESS = TRIM(dir_molecular_data) // '/SOI/soI.dat'
     atom%ABIN = 141
     atom%MBIN = 19
     atom%E_ION_GS = 11.600*eV   
     atom%NGROUND = 1
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .TRUE.
     atom%SPECIES_TYPE = 'mol'

! sulfur monoxide SOII
  CASE ('UMII ')  

     atom%Z = 1
     atom%MASS = 48.0
     atom%N = 2
     atom%Nlines = 1
     atom%FILE_ADDRESS = TRIM(dir_molecular_data) // '/SOII/soII.dat'
     atom%ABIN = 142
     atom%MBIN = 20
     atom%E_ION_GS = 23.200*eV   
     atom%NGROUND = 1
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .FALSE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.
     atom%SPECIES_TYPE = 'mol'

  CASE('HeI  ')

     atom%Z = 2
     atom%MASS = 4.003
     atom%N = 29
     atom%Nlines = 202
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/HeI/heI.dat'             
     atom%ABIN = 2
     atom%E_ION_GS = 24.587*eV
     atom%NGROUND = 1
     !atom%GGROUND = 1
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .TRUE.
     atom%RR_BETA_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .TRUE.

   CASE('HeII ')

     atom%Z = 2
     atom%MASS = 4.003
     atom%N = 2
     atom%Nlines = 1
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/HeII/heII.dat'             
     atom%ABIN = 3
     atom%E_ION_GS = 54.418*eV
     atom%NGROUND = 1
     !atom%GGROUND = 2
     atom%MINSHELL = 1
     atom%MAXSHELL = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%RR_BETA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS=.FALSE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE(' CI  ')

     atom%Z = 6
     atom%MASS = 12.01
     atom%N = 91
     atom%Nlines = 793
     atom%N_LEVELS = 90 ! No A-values for level 91
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CI/cI.dat'
     atom%ABIN = 4
     atom%E_ION_GS = 11.260*eV
     atom%NGROUND = 3
     !atom%GGROUND = 9
     atom%MINSHELL = 3
     atom%MAXSHELL = 3
     atom%RR_ALPHA_EXISTS = .TRUE.
     atom%RR_BETA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE(' CII ')

     atom%Z = 6
     atom%MASS = 12.01
     atom%N = 77
     atom%Nlines = 284
     atom%N_LEVELS = 75 ! No A-values over 75
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CII/cII.dat'
     atom%ABIN = 5
     atom%E_ION_GS = 24.383*eV
     atom%NGROUND = 2
     !atom%GGROUND = 6
     atom%MINSHELL = 3
     atom%MAXSHELL = 3
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE(' CIII')

     atom%Z = 6
     atom%MASS = 12.01
     atom%N = 2
     atom%Nlines = 1
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CIII/cIII.dat'
     atom%ABIN = 6
     atom%E_ION_GS = 47.888*eV
     atom%NGROUND = 1
     !atom%GGROUND = 1
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE(' NI  ')

     atom%Z = 7
     atom%MASS = 14.007
     atom%N = 68
     atom%Nlines = 195
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/NI/nI.dat'
     atom%ABIN = 9
     atom%E_ION_GS = 14.534*eV
     atom%NGROUND = 1
     !atom%GGROUND = 4
     atom%MINSHELL = 3
     atom%MAXSHELL = 3
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.


  CASE(' N2 ') ! CARE

     atom%Z = 7
     atom%MASS = 14
     atom%N = 148
     atom%Nlines = 690
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/NII/nII.dat'
     atom%ABIN = 10
     atom%E_ION_GS = 29.601*eV
     atom%NGROUND = 3 !CARE in NiI the ground state have levels above levels in excited multiplets
     !atom%GGROUND = 9
     atom%MINSHELL = 3
     atom%MAXSHELL = 3
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE(' OI  ')

     atom%Z = 8
     atom%MASS = 15.999
     atom%N = 234
     atom%Nlines = 885
     atom%N_LEVELS = 135 ! No A-values over 135
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/OI/oI.dat' 
     atom%ABIN = 14
     atom%E_ION_GS = 13.618*eV
     atom%NGROUND = 3
     !atom%GGROUND = 9
     atom%MINSHELL = 3
     atom%MAXSHELL = 3
     atom%RR_ALPHA_EXISTS = .TRUE.
     atom%RR_BETA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .TRUE.

  CASE(' OII  ')

     atom%Z = 8
     atom%MASS = 15.999
     atom%N = 16
     atom%Nlines = 31	
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/OII/oII.dat' 
     atom%ABIN = 15
     atom%E_ION_GS = 35.121*eV
     atom%NGROUND = 1
     !atom%GGROUND = 4
     atom%MINSHELL = 3
     atom%MAXSHELL = 3
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE(' OIII')

     atom%Z = 8
     atom%MASS = 16
     atom%N = 6
     atom%Nlines = 15
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/OIII/oIII.dat'
     atom%ABIN = 16
     atom%E_ION_GS = 54.935*eV
     atom%NGROUND = 3
     !atom%GGROUND = 9
     !atom%MINSHELL = ..
     !atom%MAXSHELL = ..
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE('NeI  ')

     atom%Z = 10
     atom%MASS = 20.180
     atom%N = 5
     atom%Nlines = 2
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/NeI/neI.dat'
     atom%ABIN = 19
     atom%E_ION_GS = 21.565*eV
     atom%NGROUND = 1
     !atom%GGROUND = ..
     atom%MINSHELL = 3
     atom%MAXSHELL = 3
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .FALSE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.


  CASE('NeII ')

     atom%Z = 10
     atom%MASS = 20.180
     atom%N = 3
     atom%Nlines = 3
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/NeII/neII.dat' 
     atom%ABIN = 20
     atom%E_ION_GS = 40.963*eV
     atom%NGROUND = 2
     !atom%GGROUND = ..
     atom%MINSHELL = 3
     atom%MAXSHELL = 3
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE('NaI  ')

     atom%Z = 11
     atom%MASS = 22.990
     atom%N = 66
     atom%Nlines = 390
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/NaI/naI.dat' 
     atom%ABIN = 24
     atom%E_ION_GS = 5.139*eV
     atom%NGROUND = 1
     !atom%GGROUND = ..
     atom%MINSHELL = 4
     atom%MAXSHELL = 4
     atom%RR_ALPHA_EXISTS = .TRUE.
     atom%RR_BETA_EXISTS = .TRUE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .TRUE.

  CASE('NaII ')   ! First line at 380 �

     atom%Z = 11
     atom%MASS = 22.990
     atom%N = 2
     atom%Nlines = 1
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/NaII/naII.dat'
     atom%ABIN = 25
     atom%E_ION_GS = 47.286*eV
     atom%NGROUND = 1
     !atom%GGROUND = ..
     atom%MINSHELL = 3
     atom%MAXSHELL = 3
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%RR_BETA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .FALSE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE('MgI  ')

     atom%Z = 12
     atom%MASS = 24.305
     atom%N = 371
     atom%Nlines = 1898
     atom%N_LEVELS = 200
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/MgI/mgI.dat' 
     atom%ABIN = 29	
     atom%E_ION_GS = 7.646*eV 
     atom%NGROUND = 1
     !atom%GGROUND = ..
     atom%MINSHELL = 4
     atom%MAXSHELL = 4
     atom%RR_ALPHA_EXISTS = .TRUE.
     atom%RR_BETA_EXISTS = .TRUE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .TRUE.

  CASE('MgII ')

     atom%Z = 12
     atom%MASS = 24.305
     atom%N = 35
     atom%Nlines = 165
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/MgII/mgII.dat'
     atom%ABIN = 30	
     atom%E_ION_GS = 15.035*eV 
     atom%NGROUND = 1
     !atom%GGROUND = ..
     atom%MINSHELL = 4
     atom%MAXSHELL = 4
     atom%RR_ALPHA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .TRUE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

  CASE('SiI  ')

     atom%Z = 14
     atom%MASS= 28.086
     atom%N = 494
     atom%Nlines = 5252 !DC 2013-06
     atom%N_LEVELS = 200
     atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/Si_neutral/siI.dat'
     atom%ABIN = 34
     atom%E_ION_GS = 8.152*eV
     atom%NGROUND = 3
     !atom%GGROUND = ..
     atom%MINSHELL = 5
     atom%MAXSHELL = 5
     atom%RR_ALPHA_EXISTS = .TRUE.
     atom%RR_BETA_EXISTS = .FALSE.
     atom%UPSILON0_EXISTS = .FALSE.
     atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('SiII ')

      atom%Z = 14
      atom%MASS = 28.086
      atom%N = 77
      atom%Nlines = 376
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/SiII/siII.dat'
      atom%ABIN = 35
      atom%E_ION_GS = 16.349*eV
      atom%NGROUND = 2
      !atom%GGROUND = ..
      atom%MINSHELL = 5
      atom%MAXSHELL = 5
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%RR_BETA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('SiIII')

      atom%Z =  14
      atom%MASS = 28.086
      atom%N = 2
      atom%Nlines = 1
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/SiIII/siIII.dat'
      atom%ABIN = 36
      atom%E_ION_GS = 33.493*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      !atom%MINSHELL = ..
      !atom%MAXSHELL = ..
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE(' SI  ')

      atom%Z = 16
      atom%MASS = 32.065
      atom%N = 123
      atom%Nlines = 662
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/SI/sI.dat'
      atom%ABIN = 39
      atom%E_ION_GS = 10.360*eV
      atom%NGROUND = 3
      !atom%GGROUND = ..
      atom%MINSHELL = 5
      atom%MAXSHELL = 5
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

    CASE(' S2  ')

      atom%Z = 16
      atom%MASS = 32.065
      atom%N = 5
      atom%Nlines = 10
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/SII/sII.dat'
      atom%ABIN = 40
      atom%E_ION_GS = 23.338*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 5
      atom%MAXSHELL = 5
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE(' S3  ')

      atom%Z = 16
      atom%MASS = 32.065
      atom%N = 6
      atom%Nlines = 15
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/S_III/sIII.dat'
      atom%ABIN = 41
      atom%E_ION_GS = 34.86*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 5
      atom%MAXSHELL = 5
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('ArI  ')

      atom%Z = 18
      atom%MASS = 39.948
      atom%N = 5
      atom%Nlines = 2
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/ArI/arI.dat'
      atom%ABIN = 44
      atom%E_ION_GS = 15.759*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 5
      atom%MAXSHELL = 5
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('ArII ')

      atom%Z = 18
      atom%MASS = 39.948
      atom%N = 2
      atom%Nlines = 1
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/ArII/arII.dat'
      atom%ABIN = 45
      atom%E_ION_GS = 27.630*eV
      atom%NGROUND = 2
      !atom%GGROUND = ..
      atom%MINSHELL = 5
      atom%MAXSHELL = 5
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('ArIII')

      atom%Z = 18
      atom%MASS = 39.948
      atom%N = 3
      atom%Nlines = 3
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/ArIII/arIII.dat'
      atom%ABIN = 46
      atom%E_ION_GS = 40.735*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      !atom%MINSHELL = ..
      !atom%MAXSHELL = ..
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('CaI  ')

      atom%Z = 20
      atom%MASS = 40.078
      atom%N = 198
      atom%Nlines = 5583 !CARE 22 levels lost
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CaI/caI.dat'
      atom%ABIN = 49
      atom%E_ION_GS = 6.113*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 7
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .TRUE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('CaII ')

      atom%Z = 20
      atom%MASS = 40.078
      atom%N = 69
      atom%Nlines = 139
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CaII/caII.dat'
      atom%ABIN = 50
      atom%E_ION_GS = 11.872*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 7
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('FeI  ')

      atom%Z = 26
      atom%MASS = 55.845
      atom%N = 496
      atom%Nlines = 23771
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/FeI/feI.dat'
      atom%ABIN = 54
      atom%E_ION_GS = 7.902*eV
      atom%NGROUND = 5
      !atom%GGROUND = ..
      atom%MINSHELL = 7
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('FeII ')

      atom%Z = 26
      atom%MASS = 55.845
      atom%N = 578
      atom%Nlines = 28487
      atom%N_LEVELS = 533  ! 533 is highest Ly-alpha pumping
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/FeII/feII.dat'
      atom%ABIN = 55
      atom%E_ION_GS = 16.188*eV
      atom%NGROUND = 5
      !atom%GGROUND = ..
      atom%MINSHELL = 7
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('FeIII')

      atom%Z = 26
      atom%MASS = 56
      atom%N = 600
      atom%Nlines = 34896
      atom%N_LEVELS = 300
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/FeIII/feIII.dat'
      atom%ABIN = 56
      atom%E_ION_GS = 30.651*eV
      atom%NGROUND = 5
      !atom%GGROUND = ..
      !atom%MINSHELL = 7
      !atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('CoI  ')

      atom%Z = 27
      atom%MASS = 58.933
      atom%N = 317
      atom%Nlines = 9500
      atom%N_LEVELS = 300
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CoI/coI.dat'
      atom%ABIN = 60
      atom%E_ION_GS = 7.881*eV
      atom%NGROUND = 4
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('CoII ')

      atom%Z = 27
      atom%MASS = 58.933
      atom%N = 108
      atom%Nlines = 5778 !warning file has text at the end don't do wc -l
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CoII/coII.dat'
      atom%ABIN = 61
      atom%E_ION_GS = 17.084*eV
      atom%NGROUND = 3
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 6
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

    CASE('CoIII')

      atom%Z = 27
      atom%MASS = 58.933
      atom%N = 306
      atom%Nlines = 13971
      atom%N_LEVELS = 300
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CoIII/coIII.dat'
      atom%ABIN = 62
      atom%E_ION_GS = 33.50*eV
      atom%NGROUND = 4
      !atom%GGROUND = ..
      !atom%MINSHELL = ..
      !atom%MAXSHELL = ..
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('NiI  ')

      atom%Z = 28
      atom%MASS = 58.693
      atom%N = 136
      atom%Nlines = 483
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/Ni_neutral/niI.dat'  
      atom%ABIN = 65
      atom%E_ION_GS = 7.640*eV
      atom%NGROUND = 3
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('NiII ')

      atom%Z = 28
      atom%MASS = 58.693
      atom%N = 500
      atom%Nlines = 17094
      atom%N_LEVELS = 300
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/NiII/niII.dat'
      atom%ABIN = 66
      atom%E_ION_GS = 18.169*eV
      atom%NGROUND = 2
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 6
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('NiIII')

      atom%Z = 28
      atom%MASS = 58.693
      atom%N = 8
      atom%Nlines = 28
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/NiIII/niIII.dat'
      atom%ABIN = 67
      atom%E_ION_GS = 35.187*eV
      atom%NGROUND = 3
      !atom%GGROUND = ..
      !atom%MINSHELL = ..
      !atom%MAXSHELL = ..
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .TRUE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.


   CASE('TiI  ')

      atom%Z = 22
      atom%MASS = 47.867
      atom%N = 394
      atom%Nlines = 13214
      atom%N_LEVELS = 300
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/TiI/tiI.dat'
      atom%ABIN = 71
      atom%E_ION_GS = 6.828*eV
      atom%NGROUND = 3
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('TiII ')

      atom%Z = 22
      atom%MASS = 47.867
      atom%N = 208
      atom%Nlines = 5160
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/TiII/tiII.dat'
      atom%ABIN = 72
      atom%E_ION_GS = 13.576*eV
      atom%NGROUND = 4
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('TiIII')

      atom%Z = 22.
      atom%MASS = 44.
      atom%N = 179
      atom%Nlines = 2265
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/TiIII/tiIII.dat'
      atom%ABIN = 73
      atom%E_ION_GS = 27.492*eV
      atom%NGROUND = 3
      !atom%GGROUND = ..
      !atom%MINSHELL = ..
      !atom%MAXSHELL = ..
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('AlI  ')

      atom%Z = 13
      atom%MASS = 26.982
      atom%N = 86
      atom%Nlines = 585
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/AlI/alI.dat'
      atom%ABIN = 76
      atom%E_ION_GS = 5.986*eV
      atom%NGROUND = 2
      !atom%GGROUND = ..
      atom%MINSHELL = 5 ! TEMP..4?
      atom%MAXSHELL = 5
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('AlII ')

      atom%Z = 13
      atom%MASS = 26.982
      atom%N = 154
      atom%Nlines = 2225
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/AlII/alII.dat'
      atom%ABIN = 77
      atom%E_ION_GS = 18.829*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 4
      atom%MAXSHELL = 4
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('CrI  ')

      atom%Z = 24
      atom%MASS = 51.996
      atom%N = 392
      atom%Nlines = 9148
      atom%N_LEVELS = 300
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CrI/crI.dat'
      atom%ABIN = 81
      atom%E_ION_GS = 6.767*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('CrII ')

      atom%Z = 24
      atom%MASS = 51.996
      atom%N = 500
      atom%Nlines = 21573
      atom%N_LEVELS = 364 ! Ly alpha pumping to 364
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/CrII/crII.dat'
      atom%ABIN = 82
      atom%E_ION_GS = 16.486*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 6
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('MnI  ')

      atom%Z = 25
      atom%MASS = 54.938
      atom%N = 293
      atom%Nlines = 5461
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/MnI/mnI.dat'
      atom%ABIN = 86
      atom%E_ION_GS = 7.434*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('MnII ')

      atom%Z = 25
      atom%MASS = 54.938
      atom%N = 509
      atom%Nlines = 19656
      atom%N_LEVELS = 300
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/MnII/mnII.dat'
      atom%ABIN = 87
      atom%E_ION_GS = 15.640*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('ScI  ')

      atom%Z = 21
      atom%MASS = 44.956
      atom%N = 254
      atom%Nlines = 6924
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/ScI/scI.dat'
      atom%ABIN = 91
      atom%E_ION_GS = 6.561*eV
      atom%NGROUND = 2
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE('ScII ')

      atom%Z = 21
      atom%MASS = 44.956
      atom%N = 165
      atom%Nlines = 3859
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/ScII/scII.dat'
      atom%ABIN = 92
      atom%E_ION_GS = 12.780*eV
      atom%NGROUND = 3
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE(' VI  ')

      atom%Z = 23
      atom%MASS = 50.942
      atom%N = 502
      atom%Nlines = 17209
      atom%N_LEVELS = 300
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/VI/vI.dat'
      atom%ABIN = 96
      atom%E_ION_GS = 6.746*eV
      atom%NGROUND = 4
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE(' VII ')

      atom%Z = 23
      atom%MASS = 50.942
      atom%N = 323
      atom%Nlines = 11634
      atom%N_LEVELS = 300
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/VII/vII.dat'
      atom%ABIN = 97
      atom%E_ION_GS = 14.618*eV
      atom%NGROUND = 5
      !atom%GGROUND = ..
      atom%MINSHELL = 6
      atom%MAXSHELL = 6
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE(' KI  ')

      atom%Z = 19
      atom%MASS = 39.09
      atom%N = 3
      atom%Nlines = 2
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/KI/kI.dat'
      atom%ABIN = 101
      atom%E_ION_GS = 4.34066*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 4
      atom%MAXSHELL = 7
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.

   CASE(' KII ')

      atom%Z = 19
      atom%MASS = 39.09
      atom%N = 2
      atom%Nlines = 1
      atom%FILE_ADDRESS = TRIM(dir_atomic_data) // '/KII/kII.dat'
      atom%ABIN = 102
      atom%E_ION_GS = 31.625*eV
      atom%NGROUND = 1
      !atom%GGROUND = ..
      atom%MINSHELL = 4
      atom%MAXSHELL = 5
      atom%RR_ALPHA_EXISTS = .FALSE.
      atom%UPSILON0_EXISTS = .FALSE.
      atom%UPSILON0_BETA_EXISTS = .FALSE.


CASE DEFAULT

   PRINT *, 'ERROR in alloc. No basic info about atom : ', name; STOP 1

END SELECT

IF (atom%N_LEVELS .EQ. 0) atom%N_LEVELS = atom%N  ! 0 means use full atom (read in readrunparameters.f90)

atom%N_LEVELS_ABSORB = 0 ! WILL BE CHANGED LATER IN PROGRAM

CALL memshare(sharedcomm, myid, atom%LEVELS_E, window, (/ atom%N /)); atom%LEVELS_E = 0
CALL memshare(sharedcomm, myid, atom%LEVELS_G, window, (/ atom%N /)); atom%LEVELS_G = 0
CALL memshare(sharedcomm, myid, atom%RR_ALPHA0, window, (/ atom%N /)); atom%RR_ALPHA0 = 0
CALL memshare(sharedcomm, myid, atom%RR_T0, window, (/ atom%N /)); atom%RR_T0 = 0
CALL memshare(sharedcomm, myid, atom%RR_BETA, window, (/ atom%N /)); atom%RR_BETA = 0
CALL memshare(sharedcomm, myid, atom%LAMBDA, window, (/ atom%N, atom%N /)); atom%LAMBDA = 0
CALL memshare(sharedcomm, myid, atom%A, window, (/ atom%N, atom%N /)); atom%A = 0
CALL memshare(sharedcomm, myid, atom%UPSILON0, window, (/ atom%N, atom%N /)); atom%UPSILON0 = 0
CALL memshare(sharedcomm, myid, atom%UPSILON0_T0, window, (/ atom%N, atom%N /)); atom%UPSILON0_T0 = 0
CALL memshare(sharedcomm, myid, atom%UPSILON0_BETA, window, (/ atom%N, atom%N /)); atom%UPSILON0_BETA = 0

END 
