SUBROUTINE allocGAMMAEXC(atom, deposition_tot_pervol, nk_ground, Tuse, fracvec_channels, GAMMAMAT)
USE M1, ONLY : atomm, SFMAX
USE PARAMETERS, ONLY : h, c, k
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
DOUBLE PRECISION, INTENT(IN) :: deposition_tot_pervol, nk_ground, Tuse
DOUBLE PRECISION, DIMENSION(SFMAX), INTENT(IN) :: fracvec_channels  
DOUBLE PRECISION, DIMENSION(atom%N_levels, atom%N_levels), INTENT(OUT) :: GAMMAMAT
! ==========================================================================================
! Task  :  Computes the non-thermal excitation rates (GAMMA, s^-1) given the amount of energy per channel
! Input :  atom                       The atom (struct type ATOMM in module M1)
!          deposition_tot_pervol      Total nonthermal energy deposition [erg s^-1 cm^-3]
!          nk_ground                  Number density of atom in ground state [cm^-3]
!          Tuse                       Temperature [K]
!          fracvec_channels           Fraction of total deposited energy per excitation channel [dim-less]
! Output : GAMMAMAT                   Matrix with the nonthermal excitation rates per particle in the lower state [s^-1]
! =========================================================================================
CHARACTER(len=5) :: name
INTEGER :: Nchannels, uppLEV, lowLEV, i, j, q
DOUBLE PRECISION :: GAMMATOT, Echannel, gtot, PF, x1, x2, x3
INTEGER, DIMENSION(:,:), ALLOCATABLE :: LOWER, UPPER
INTEGER, DIMENSION(:), ALLOCATABLE :: Nl, Nu
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: Elower, Eupper
DOUBLE PRECISION, DIMENSION(SFMAX) :: fsc_dummy  
DOUBLE PRECISION, DIMENSION(SFMAX) :: x=0   

INTEGER, PARAMETER :: nlevR=262 ! Number of levels in Ramsbottoms FeII atom
INTEGER, PARAMETER :: msbin=350 ! ?
INTEGER, PARAMETER :: ntrans=1000  ! CARE change here if extending more levels in SF

COMMON/cfe2/EFE2, CRFE2, C_IPOL, i_CFvec, j_CFvec, fitvec, hecvec, deltaEvec, mtrans, Ebinsvec, ies1

REAL, DIMENSION(ntrans,msbin) :: EFE2, CRFE2, C_IPOL ! These are not used
REAL, DIMENSION(ntrans) :: deltaEvec, hecvec
INTEGER, DIMENSION(ntrans) :: i_CFvec, j_CFvec, fitvec, Ebinsvec, ies1
INTEGER :: mtrans, Ebins
DOUBLE PRECISION, DIMENSION(5) :: xv
LOGICAL :: CHECK
! ------------------------------------------------------------------

GAMMAMAT = 0                               

fsc_dummy = fracvec_channels      

! -------------------------------------------------------------------------

name = atom%NAME

SELECT CASE (name)

CASE(' HI  ')

   Nchannels = 9        ! Number of excitation channels

   x = 1.             ! Fraction of nk in each level (ground levels bunched together, i.e. dont put different x for them)
       
   ALLOCATE(Nl(Nchannels))     
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels)); Elower = 0
   ALLOCATE(Eupper(Nchannels)); Eupper = 0

   Nl(:) = 1      ! Maximum number of states in the lower level
   Nu(:) = 1      ! Maximum number of states in the upper level

   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0

   LOWER(:,1) = 1; 

   UPPER(1:9,1) = [2,3,4,5,6,7,8,9,10]     ! Upper states....2s 2p 3s 3p 3d 4s 4p 4d 4f

   Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))

CASE('HeI  ') 

   Nchannels = 7

   x = 1.

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels)); Elower = 0
   ALLOCATE(Eupper(Nchannels)); Elower = 0

   Nl(:) = 1
   Nu(:) = 1

   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0 
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0
    
   LOWER(:,1) = 1
   
   UPPER(1:7,1) = [2,3,4,5,6,8,9]   ! 2s3S, 2s1S, 2p3P, 2p1P, 3s3S, 3p3P, 3d3D, Note skips 7

   Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))

CASE (' CI  ')   ! BETHE APPROXIMATION

   Nchannels = 37   

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels)); Elower = 0
   ALLOCATE(Eupper(Nchannels)); Eupper = 0               

   PF = 1. + 3*EXP(-h*c*16.40/(k*Tuse)) + 5*EXP(-h*c*43.4/(k*Tuse))  ! Partition function for ground triplet..CARE have to use same T as in SF routine

   x(1:7) = 1./PF    ! Fraction of atoms in level 1 (g=1) assuming ground triplet in LTE
      
   x(8:24) = 3./PF*EXP(-h*c*16.40/(k*Tuse))  ! ...Level 2 (g=3)
        
   x(25:37) = 5./PF*EXP(-h*c*43.4/(k*Tuse))   ! ...Level 3 (g=5)
    
   Nl(:) = 1
   Nu(:) = 1
        
   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0 
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0

   ! From level 1
   i=  1;LOWER(i,1) = 1    ;UPPER(i,1) =    8 
   i=  2;LOWER(i,1) = 1    ;UPPER(i,1) =   12
   i=  3;LOWER(i,1) = 1    ;UPPER(i,1) =   24
   i=  4;LOWER(i,1) = 1    ;UPPER(i,1) =   29
   i=  5;LOWER(i,1) = 1    ;UPPER(i,1) =   34
   i=  6;LOWER(i,1) = 1    ;UPPER(i,1) =   37
   i=  7;LOWER(i,1) = 1    ;UPPER(i,1) =   41
   ! From level 2        
   i=  8;LOWER(i,1) = 2    ;UPPER(i,1) =    7
   i=  9;LOWER(i,1) = 2    ;UPPER(i,1) =    8
   i= 10;LOWER(i,1) = 2    ;UPPER(i,1) =    9
   i= 11;LOWER(i,1) = 2    ;UPPER(i,1) =   12
   i= 12;LOWER(i,1) = 2    ;UPPER(i,1) =   13
   i= 13;LOWER(i,1) = 2    ;UPPER(i,1) =   24
   i= 14;LOWER(i,1) = 2    ;UPPER(i,1) =   25
   i= 15;LOWER(i,1) = 2    ;UPPER(i,1) =   26
   i= 16;LOWER(i,1) = 2    ;UPPER(i,1) =   28
   i= 17;LOWER(i,1) = 2    ;UPPER(i,1) =   29
   i= 18;LOWER(i,1) = 2    ;UPPER(i,1) =   30
   i= 19;LOWER(i,1) = 2    ;UPPER(i,1) =   34
   i= 20;LOWER(i,1) = 2    ;UPPER(i,1) =   35
   i= 21;LOWER(i,1) = 2    ;UPPER(i,1) =   37
   i= 22;LOWER(i,1) = 2    ;UPPER(i,1) =   40
   i= 23;LOWER(i,1) = 2    ;UPPER(i,1) =   41
   i= 24;LOWER(i,1) = 2    ;UPPER(i,1) =   42 
   ! From level 3
   i= 25;LOWER(i,1) = 3    ;UPPER(i,1) =    8
   i= 26;LOWER(i,1) = 3    ;UPPER(i,1) =    9
   i= 27;LOWER(i,1) = 3    ;UPPER(i,1) =   11
   i= 28;LOWER(i,1) = 3    ;UPPER(i,1) =   13
   i= 29;LOWER(i,1) = 3    ;UPPER(i,1) =   24
   i= 30;LOWER(i,1) = 3    ;UPPER(i,1) =   25
   i= 31;LOWER(i,1) = 3    ;UPPER(i,1) =   29
   i= 32;LOWER(i,1) = 3    ;UPPER(i,1) =   30
   i= 33;LOWER(i,1) = 3    ;UPPER(i,1) =   32
   i= 34;LOWER(i,1) = 3    ;UPPER(i,1) =   35
   i= 35;LOWER(i,1) = 3    ;UPPER(i,1) =   36
   i= 36;LOWER(i,1) = 3    ;UPPER(i,1) =   40
   i= 37;LOWER(i,1) = 3    ;UPPER(i,1) =   41

   Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))

CASE(' CII ') 

   Nchannels = 10

   x = 1.

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels)); Elower = 0
   ALLOCATE(Eupper(Nchannels)); Eupper = 0

   Nl(:) = 2
   Nu(:) = 1
  
   ALLOCATE(LOWER( Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER( Nchannels, MAXVAL(Nu))); UPPER = 0

   LOWER(1,1:2) = [1,2]  ! 2p 2P*

   PF = 2. + 4.*EXP(-h*c*63.42/(k*Tuse))

   x1 = 2./PF

   x2 = 4./PF

   Elower(1:Nchannels) = h*c*(0*x1 + 63.42*x2)

   DO i = 2,Nchannels

      LOWER(i,:) = LOWER(1,:) 
        
   END DO

   UPPER(:,1) = [7,8,9,10,11,15,19,25,36,47]

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))

CASE (' NI  ')

   Nchannels = 2 

   x = 1. 

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels)); Elower=0
   ALLOCATE(Eupper(Nchannels)); Eupper=0

   Nl(:) = 1
   Nu = [3,3]

   ALLOCATE(LOWER(Nchannels,MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER(Nchannels,MAXVAL(Nu))); UPPER = 0

   LOWER(1,1) = 1
   LOWER(2,1) = 1

   UPPER(1,1:3) = [6,7,8]
   UPPER(2,1:3) = [11,12,13]

   DO i = 1, Nchannels

      Eupper(i) = 0

      gtot = sum(atom%LEVELS_G(UPPER(i,1:Nu(i))))

      DO j = 1, Nu(i)

         Eupper(i) = Eupper(i) + atom%LEVELS_E(UPPER(i,j))*atom%LEVELS_G(UPPER(i,j))/gtot

      END DO


   END DO

CASE(' OI  ') 

   Nchannels = 13

   x = 1.    ! CARE Assumes OI ground triplet according to stat weights

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels)); Elower = 0
   ALLOCATE(Eupper(Nchannels)); Eupper = 0

   Nl(:) = 3

   Nu = [1,1,1,1,3,3,1,3,5,1,3,3,3]
 
   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0

   LOWER(1,1:3) = [1,2,3]  ! 3P

   PF = 5. + 3.*EXP(-h*c*158.265/(k*Tuse)) + 1.*EXP(-h*c*226.977/(k*Tuse))

   x1 = 5./PF

   x2 = 3./PF

   x3 = 1./PF

   DO i = 1,Nchannels

      LOWER(i,:) = LOWER(1,:) 
        
   END DO
   
   Elower(1:Nchannels) = h*c*(0*x1 + 158.265*x2 + 226.977*x3)

   UPPER(1,1) = 4; ! 1D
   UPPER(2,1) = 5 ! 1S
   UPPER(3,1) = 6 ! 3s 5S
   UPPER(4,1) = 7 ! 3s 3S
   UPPER(5,1:3) = [8,9,10] ! 3p 5P
   UPPER(6,1:3) = [11,12,13] ! 3p 3P
   UPPER(7,1) = 15 ! 4s 3S
   UPPER(8,1:3) = [21,22,23] ! 3d 3D
   UPPER(9,1:5) = [16,17,18,19,20] ! 3d 5D
   UPPER(10,1) = 14 ! 4s 5S
   UPPER(11,1:3) = [24,25,26]
   UPPER(12,1:3) = [27,28,29]
   UPPER(13,1:3) = [30,31,32]  ! 3s 3D

   DO i = 1, Nchannels

      Eupper(i) = 0

      gtot = sum(atom%LEVELS_G(UPPER(i,1:Nu(i))))

      DO j = 1, Nu(i)

         Eupper(i) = Eupper(i) + atom%LEVELS_E(UPPER(i,j))*atom%LEVELS_G(UPPER(i,j))/gtot

      END DO


   END DO
 
CASE(' OII ') 

   Nchannels = 2

   x = 1.   ! ground state is single state

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   Nl(:) = 1
   Nu(:) = 3
 
   ALLOCATE(LOWER( Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER( Nchannels, MAXVAL(Nu))); UPPER = 0

   LOWER(:,1) = 1

   Elower(1:Nchannels) = 0
   
   UPPER(1,:) = [6,7,8]
   UPPER(2,:) = [11,12,13] 

   DO i = 1, Nchannels

      Eupper(i) = 0

      gtot = sum(atom%LEVELS_G(UPPER(i,1:Nu(i))))

      DO j = 1, Nu(i)

         Eupper(i) = Eupper(i) + atom%LEVELS_E(UPPER(i,j))*atom%LEVELS_G(UPPER(i,j))/gtot

      END DO

   END DO

CASE('NeI  ')

   Nchannels = 2

   x = 1.  ! Ground state is single state

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels)); Elower = 0
   ALLOCATE(Eupper(Nchannels)); Eupper = 0

   Nl(:) = 1
   Nu(:) = 1

   ALLOCATE(LOWER( Nchannels, MAXVAL(Nl))); LOWER = 0 
   ALLOCATE(UPPER( Nchannels, MAXVAL(Nu))); UPPER = 0
   
   LOWER(:,1) = 1

   UPPER(1,1) = 3
   UPPER(2,1) = 5

   Elower(1:Nchannels) = 0

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))

CASE('NaI  ') 

   Nchannels = 8

   x = 1.   ! GS is single state

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   Nl(:) = 1
   Nu = [2,1,2,2,2,2,2,2]
 
   ALLOCATE(LOWER( Nchannels, MAXVAL(Nl))); LOWER = 0 
   ALLOCATE(UPPER( Nchannels, MAXVAL(Nu))); UPPER = 0

   LOWER(:,1) = 1

   Elower(1:Nchannels) = 0
      
   UPPER(1,1:2) = [2,3]
   UPPER(2,1) = 4
   UPPER(3,1:2) = [5,6]
   UPPER(4,1:2) = [7,8]
   UPPER(5,1:2) = [10,11]
   UPPER(6,1:2) = [14,15]
   UPPER(7,1:2) = [23,24]
   UPPER(8,1:2) = [34,35]

   DO i = 1, Nchannels

      Eupper(i) = 0

      gtot = sum(atom%LEVELS_G(UPPER(i,1:Nu(i))))
      
      DO j = 1, Nu(i)

         Eupper(i) = Eupper(i) + atom%LEVELS_E(UPPER(i,j))*atom%LEVELS_G(UPPER(i,j))/gtot

      END DO

   END DO

CASE('SiI  ') 

   Nchannels = 59    ! Bethe approximations

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))
 
   PF = 1. + 3*EXP(-h*c*77.115/(k*Tuse)) + 5*EXP(-h*c*223.157/(k*Tuse))

   x(1:12)  = 1./PF
   x(13:38) = 3./PF*EXP(-h*c*77.115/(k*Tuse))
   x(39:59) = 5./PF*EXP(-h*c*223.157/(k*Tuse)) 
  
   Nl(:) = 1
   Nu(:) = 1
 
   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0

   ! From level 1
   i=  1;LOWER(i,1) = 1    ;UPPER(i,1) =    8 
   i=  2;LOWER(i,1) = 1    ;UPPER(i,1) =   11
   i=  3;LOWER(i,1) = 1    ;UPPER(i,1) =   28
   i=  4;LOWER(i,1) = 1    ;UPPER(i,1) =   33
   i=  5;LOWER(i,1) = 1    ;UPPER(i,1) =   37
   i=  6;LOWER(i,1) = 1    ;UPPER(i,1) =   42
   i=  7;LOWER(i,1) = 1    ;UPPER(i,1) =   63
   i=  8;LOWER(i,1) = 1    ;UPPER(i,1) =   68
   i=  9;LOWER(i,1) = 1    ;UPPER(i,1) =   81
   i= 10;LOWER(i,1) = 1    ;UPPER(i,1) =  101
   i= 11;LOWER(i,1) = 1    ;UPPER(i,1) =  105
   i= 12;LOWER(i,1) = 1    ;UPPER(i,1) =  120
   ! From level = 2
   i= 13;LOWER(i,1) = 2    ;UPPER(i,1) =    7
   i= 14;LOWER(i,1) = 2    ;UPPER(i,1) =    8
   i= 15;LOWER(i,1) = 2    ;UPPER(i,1) =    9
   i= 16;LOWER(i,1) = 2    ;UPPER(i,1) =   11
   i= 17;LOWER(i,1) = 2    ;UPPER(i,1) =   12
   i= 18;LOWER(i,1) = 2    ;UPPER(i,1) =   27 
   i= 19;LOWER(i,1) = 2    ;UPPER(i,1) =   28
   i= 20;LOWER(i,1) = 2    ;UPPER(i,1) =   29
   i= 21;LOWER(i,1) = 2    ;UPPER(i,1) =   33
   i= 22;LOWER(i,1) = 2    ;UPPER(i,1) =   34
   i= 23;LOWER(i,1) = 2    ;UPPER(i,1) =   38
   i= 24;LOWER(i,1) = 2    ;UPPER(i,1) =   41   
   i= 25;LOWER(i,1) = 2    ;UPPER(i,1) =   42
   i= 26;LOWER(i,1) = 2    ;UPPER(i,1) =   43
   i= 27;LOWER(i,1) = 2    ;UPPER(i,1) =   63
   i= 28;LOWER(i,1) = 2    ;UPPER(i,1) =   65
   i= 29;LOWER(i,1) = 2    ;UPPER(i,1) =   68
   i= 30;LOWER(i,1) = 2    ;UPPER(i,1) =   80
   i= 31;LOWER(i,1) = 2    ;UPPER(i,1) =   81
   i= 32;LOWER(i,1) = 2    ;UPPER(i,1) =   82
   i= 33;LOWER(i,1) = 2    ;UPPER(i,1) =  101
   i= 34;LOWER(i,1) = 2    ;UPPER(i,1) =  104
   i= 35;LOWER(i,1) = 2    ;UPPER(i,1) =  105
   i= 36;LOWER(i,1) = 2    ;UPPER(i,1) =  120
   i= 37;LOWER(i,1) = 2    ;UPPER(i,1) =  121
   i= 38;LOWER(i,1) = 2    ;UPPER(i,1) =  127
   ! From level 3
   i= 39;LOWER(i,1) = 3    ;UPPER(i,1) =   8
   i= 40;LOWER(i,1) = 3    ;UPPER(i,1) =   9 
   i= 41;LOWER(i,1) = 3    ;UPPER(i,1) =  12 
   i= 42;LOWER(i,1) = 3    ;UPPER(i,1) =  13 
   i= 43;LOWER(i,1) = 3    ;UPPER(i,1) =  27 
   i= 44;LOWER(i,1) = 3    ;UPPER(i,1) =  28 
   i= 45;LOWER(i,1) = 3    ;UPPER(i,1) =  34 
   i= 46;LOWER(i,1) = 3    ;UPPER(i,1) =  36 
   i= 47;LOWER(i,1) = 3    ;UPPER(i,1) =  38 
   i= 48;LOWER(i,1) = 3    ;UPPER(i,1) =  41 
   i= 49;LOWER(i,1) = 3    ;UPPER(i,1) =  42 
   i= 50;LOWER(i,1) = 3    ;UPPER(i,1) =  64 
   i= 51;LOWER(i,1) = 3    ;UPPER(i,1) =  65 
   i= 52;LOWER(i,1) = 3    ;UPPER(i,1) =  71 
   i= 53;LOWER(i,1) = 3    ;UPPER(i,1) =  80 
   i= 54;LOWER(i,1) = 2    ;UPPER(i,1) =  81
   i= 55;LOWER(i,1) = 3    ;UPPER(i,1) = 103
   i= 56;LOWER(i,1) = 3    ;UPPER(i,1) = 104
   i= 57;LOWER(i,1) = 3    ;UPPER(i,1) = 109
   i= 58;LOWER(i,1) = 3    ;UPPER(i,1) = 118
   i= 59;LOWER(i,1) = 3    ;UPPER(i,1) = 120

   Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))

CASE('SiII')

  Nchannels = 16    ! Bethe approximations
 
   PF = 2. + 4*EXP(-h*c*287.24/(k*Tuse))

   x(1:7) = 2./PF

   x(8:16) = 4*exp(-h*c*287.24/(k*Tuse))/PF

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   Nl(:) = 1
   Nu(:) = 1
 
   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0

   i=  1;LOWER(i,1) = 1    ;UPPER(i,1) =  8  
   i=  2;LOWER(i,1) = 1    ;UPPER(i,1) =  9 
   i=  3;LOWER(i,1) = 1    ;UPPER(i,1) = 10 
   i=  4;LOWER(i,1) = 1    ;UPPER(i,1) = 14 
   i=  5;LOWER(i,1) = 1    ;UPPER(i,1) = 15 
   i=  6;LOWER(i,1) = 1    ;UPPER(i,1) = 16 
   i=  7;LOWER(i,1) = 1    ;UPPER(i,1) = 17 
                                            
   i=  8;LOWER(i,1) = 2    ;UPPER(i,1) =  8 
   i=  9;LOWER(i,1) = 2    ;UPPER(i,1) =  9 
   i= 10;LOWER(i,1) = 2    ;UPPER(i,1) = 10 
   i= 11;LOWER(i,1) = 2    ;UPPER(i,1) = 11 
   i= 12;LOWER(i,1) = 2    ;UPPER(i,1) = 14 
   i= 13;LOWER(i,1) = 2    ;UPPER(i,1) = 15 
   i= 14;LOWER(i,1) = 2    ;UPPER(i,1) = 16 
   i= 15;LOWER(i,1) = 2    ;UPPER(i,1) = 17 
   i= 16;LOWER(i,1) = 2    ;UPPER(i,1) = 18   ! DC 2013-06

   Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))

CASE('CaI  ') 

   Nchannels = 4             

   x = 1.   ! GS is single state

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   Nl = 1
   Nu = [3,3,1,1]       

   Elower(1:Nchannels) = 0
        
   ALLOCATE(LOWER( Nchannels, MAXVAL(Nl))); LOWER = 0 
   ALLOCATE(UPPER( Nchannels, MAXVAL(Nu))); UPPER = 0

   i=  1;LOWER(i,1) = 1    ;UPPER(i,1:3) =    [2,3,4] 
   i=  2;LOWER(i,1) = 1    ;UPPER(i,1:3) =    [5,6,7]
   i = 3;LOWER(i,1) = 1    ;UPPER(i,1) =      8
   i=  4;LOWER(i,1) = 1    ;UPPER(i,1) =      9         ! DC 2013-06

   DO i = 1, Nchannels

      Eupper(i) = 0

      gtot = sum(atom%LEVELS_G(UPPER(i,1:Nu(i))))

      DO j = 1, Nu(i)

         Eupper(i) = Eupper(i) + atom%LEVELS_E(UPPER(i,j))*atom%LEVELS_G(UPPER(i,j))/gtot

      END DO

   END DO
  
CASE('CaII ') 

   Nchannels = 5                 !5 Cross sections available

   x = 1.   ! GS is single state

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   Nl(:) = 1
   Nu = [2,2,1,2,2]       
       
   ALLOCATE(LOWER( Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER( Nchannels, MAXVAL(Nu))); UPPER = 0

   LOWER(:,1) = 1

   Elower(1:Nchannels) = 0
   
   UPPER(1,1:2) = [2,3]     !3d2D
   UPPER(2,1:2) = [4,5]     !4p2P
   UPPER(3,1) = 6           !5s2S
   UPPER(4,1:2) = [7,8]     !4d2D
   UPPER(5,1:2) = [9,10]    !5p2P  ! DC 2013-06

   DO i = 1, Nchannels

      Eupper(i) = 0

      gtot = sum(atom%LEVELS_G(UPPER(i,1:Nu(i))))

      DO j = 1, Nu(i)

         Eupper(i) = Eupper(i) + atom%LEVELS_E(UPPER(i,j))*atom%LEVELS_G(UPPER(i,j))/gtot

      END DO

   END DO

CASE('MgI  ') 

   Nchannels = 4

   x = 1.   ! Singlet ground state

   ALLOCATE(Nl(Nchannels) )
   ALLOCATE(Nu(Nchannels) )
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   Nl(:) = 1    
   
   Nu(1) = 3
   Nu(2:4) = 1
 
   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER( Nchannels, MAXVAL(Nu))); UPPER = 0

   LOWER(:,1) = 1
   
   Elower(1:Nchannels) = 0

   UPPER(1,1:3) = [2,3,4]  ! 3p3P
   UPPER(2,1) = 5  !  3p1P
   UPPER(3,1) = 15   ! 4p1P
   UPPER(4,1) = 29   ! 5p1P    DC 2013-06

   DO i = 1, Nchannels

      Eupper(i) = 0

      gtot = sum(atom%LEVELS_G(UPPER(i,1:Nu(i))))

      DO j = 1, Nu(i)

         Eupper(i) = Eupper(i) + atom%LEVELS_E(UPPER(i,j))*atom%LEVELS_G(UPPER(i,j))/gtot

      END DO

   END DO

	
CASE('MgII ')

   Nchannels = 5         ! 5 cross sections available 

   x = 1.   ! Single ground state
       
   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   Nl(:) = 1

   Nu = [2,1,2,2,2]

   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0 
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0

   LOWER(:,1) = 1

   Elower(1:Nchannels) = 0
   
   UPPER(1,1:2) = [2,3]  ! 3p2P
   UPPER(2,1) = 4    ! 4s2S
   UPPER(3,1:2) = [5,6]  ! 3d2D
   UPPER(4,1:2) = [7,8]  ! 4p2P
   UPPER(5,1:2) = [14,15]  ! 5p2P  DC 2013-06

   DO i = 1, Nchannels

      Eupper(i) = 0

      gtot = sum(atom%LEVELS_G(UPPER(i,1:Nu(i))))

      DO j = 1, Nu(i)

         Eupper(i) = Eupper(i) + atom%LEVELS_E(UPPER(i,j))*atom%LEVELS_G(UPPER(i,j))/gtot
         
      END DO
      
   END DO

CASE(' SI  ') 

   Nchannels = 53   ! Bethe approximation

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   PF = 5. + 3*EXP(-h*c*396./(k*Tuse)) + 1*EXP(-h*c*573./(k*Tuse))  ! Partition function for ground triplet

   x(1:21) = 5./PF    ! Fraction of atoms in level 1 assuming ground triplet in LTE
        
   x(22:41) = 3./PF*EXP(-h*c*396./(k*Tuse))  ! ...
        
   x(42:53) = 1./PF*EXP(-h*c*573./(k*Tuse))   ! ...

   Nl(:) = 1
   Nu(:) = 1

   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0

   i=  1;LOWER(i,1) = 1    ;UPPER(i,1) =   7  
   i=  2;LOWER(i,1) = 1    ;UPPER(i,1) =  14 
   i=  3;LOWER(i,1) = 1    ;UPPER(i,1) =  15 
   i=  4;LOWER(i,1) = 1    ;UPPER(i,1) =  16 
   i=  5;LOWER(i,1) = 1    ;UPPER(i,1) =  23 
   i=  6;LOWER(i,1) = 1    ;UPPER(i,1) =  24 
   i=  7;LOWER(i,1) = 1    ;UPPER(i,1) =  25 
   i=  8;LOWER(i,1) = 1    ;UPPER(i,1) =  27 
   i=  9;LOWER(i,1) = 1    ;UPPER(i,1) =  44 
   i= 10;LOWER(i,1) = 1    ;UPPER(i,1) =  45 
   i= 11;LOWER(i,1) = 1    ;UPPER(i,1) =  55 
   i= 12;LOWER(i,1) = 1    ;UPPER(i,1) =  58 
   i= 13;LOWER(i,1) = 1    ;UPPER(i,1) =  59 
   i= 14;LOWER(i,1) = 1    ;UPPER(i,1) =  81 
   i= 15;LOWER(i,1) = 1    ;UPPER(i,1) =  82 
   i= 16;LOWER(i,1) = 1    ;UPPER(i,1) = 100 
   i= 17;LOWER(i,1) = 1    ;UPPER(i,1) = 101 
   i= 18;LOWER(i,1) = 1    ;UPPER(i,1) = 110 
   i= 19;LOWER(i,1) = 1    ;UPPER(i,1) = 111 
   i= 20;LOWER(i,1) = 1    ;UPPER(i,1) = 121 
   i= 21;LOWER(i,1) = 1    ;UPPER(i,1) = 122 

   i= 22;LOWER(i,1) = 2    ;UPPER(i,1) =   7
   i= 23;LOWER(i,1) = 2    ;UPPER(i,1) =  14
   i= 24;LOWER(i,1) = 2    ;UPPER(i,1) =  15
   i= 25;LOWER(i,1) = 2    ;UPPER(i,1) =  23
   i= 26;LOWER(i,1) = 2    ;UPPER(i,1) =  24
   i= 27;LOWER(i,1) = 2    ;UPPER(i,1) =  27
   i= 28;LOWER(i,1) = 2    ;UPPER(i,1) =  43
   i= 29;LOWER(i,1) = 2    ;UPPER(i,1) =  44
   i= 30;LOWER(i,1) = 2    ;UPPER(i,1) =  55
   i= 31;LOWER(i,1) = 2    ;UPPER(i,1) =  57
   i= 32;LOWER(i,1) = 2    ;UPPER(i,1) =  58
   i= 33;LOWER(i,1) = 2    ;UPPER(i,1) =  59
   i= 34;LOWER(i,1) = 2    ;UPPER(i,1) =  81
   i= 35;LOWER(i,1) = 2    ;UPPER(i,1) =  83
   i= 36;LOWER(i,1) = 2    ;UPPER(i,1) = 101
   i= 37;LOWER(i,1) = 2    ;UPPER(i,1) = 102
   i= 38;LOWER(i,1) = 2    ;UPPER(i,1) = 111
   i= 39;LOWER(i,1) = 2    ;UPPER(i,1) = 112
   i= 40;LOWER(i,1) = 2    ;UPPER(i,1) = 122
   i= 41;LOWER(i,1) = 2    ;UPPER(i,1) = 123

   i= 42;LOWER(i,1) = 3    ;UPPER(i,1) =    7
   i= 43;LOWER(i,1) = 3    ;UPPER(i,1) =   14
   i= 44;LOWER(i,1) = 3    ;UPPER(i,1) =   19
   i= 45;LOWER(i,1) = 3    ;UPPER(i,1) =   23
   i= 46;LOWER(i,1) = 3    ;UPPER(i,1) =   27
   i= 47;LOWER(i,1) = 3    ;UPPER(i,1) =   43
   i= 48;LOWER(i,1) = 3    ;UPPER(i,1) =   55
   i= 49;LOWER(i,1) = 3    ;UPPER(i,1) =   58
   i= 50;LOWER(i,1) = 3    ;UPPER(i,1) =   83
   i= 51;LOWER(i,1) = 3    ;UPPER(i,1) =  102
   i= 52;LOWER(i,1) = 3    ;UPPER(i,1) =  112
   i= 53;LOWER(i,1) = 3    ;UPPER(i,1) =  123  ! DC 2013-06

   Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))
   
CASE ('ArI  ')

  Nchannels = 2

  x = 1.

  ALLOCATE(Nl(Nchannels))
  ALLOCATE(Nu(Nchannels))
  ALLOCATE(Elower(Nchannels))
  ALLOCATE(Eupper(Nchannels))

  Nl(:) = 1
  Nu(:) = 1
   
  ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
  ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0

  LOWER(:,1) = 1
  UPPER(1,1) = 3
  UPPER(2,1) = 5

  Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

  Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))

CASE('FeI  ') 

   Nchannels = 124     ! Bethe approximation, fosc > 1e-2

   PF = 9. + 7*EXP(-h*c*415.93/(k*Tuse)) + 5*EXP(-h*c*704.00/(k*Tuse)) + 3*EXP(-h*c*888.129/(k*Tuse)) + 1*EXP(-h*c*978.072/(k*Tuse))  

   x(1:19)  = 9./PF

   x(20:44) = 7./PF*EXP(-h*c*415.93/(k*Tuse))

   x(45:72) = 5./PF*EXP(-h*c*704.00/(k*Tuse)) 

   x(73:102) = 3./PF*EXP(-h*c*888.129/(k*Tuse)) 

   x(103:124) = 1./PF*EXP(-h*c*978.072/(k*Tuse))

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))
	
   Nl(:) = 1              
   Nu(:) = 1     
          	
   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0
        
   i=  1;LOWER(i,1) = 1    ;UPPER(i,1) =    54
   i=  2;LOWER(i,1) = 1    ;UPPER(i,1) =    65
   i=  3;LOWER(i,1) = 1    ;UPPER(i,1) =    73
   i=  4;LOWER(i,1) = 1    ;UPPER(i,1) =    88
   i=  5;LOWER(i,1) = 1    ;UPPER(i,1) =    90
   i=  6;LOWER(i,1) = 1    ;UPPER(i,1) =    91
   i=  7;LOWER(i,1) = 1    ;UPPER(i,1) =    96
   i=  8;LOWER(i,1) = 1    ;UPPER(i,1) =   113
   i=  9;LOWER(i,1) = 1    ;UPPER(i,1) =   124
   i= 10;LOWER(i,1) = 1    ;UPPER(i,1) =   125
   i= 11;LOWER(i,1) = 1    ;UPPER(i,1) =   129
   i= 12;LOWER(i,1) = 1    ;UPPER(i,1) =   133
   i= 13;LOWER(i,1) = 1    ;UPPER(i,1) =   158
   i= 14;LOWER(i,1) = 1    ;UPPER(i,1) =   161
   i= 15;LOWER(i,1) = 1    ;UPPER(i,1) =   195
   i= 16;LOWER(i,1) = 1    ;UPPER(i,1) =   200
   i= 17;LOWER(i,1) = 1    ;UPPER(i,1) =   233
   i= 18;LOWER(i,1) = 1    ;UPPER(i,1) =   338
   i= 19;LOWER(i,1) = 1    ;UPPER(i,1) =   412
  
   i= 20;LOWER(i,1) = 2    ;UPPER(i,1) =   56
   i= 21;LOWER(i,1) = 2    ;UPPER(i,1) =   66
   i= 22;LOWER(i,1) = 2    ;UPPER(i,1) =   78
   i= 23;LOWER(i,1) = 2    ;UPPER(i,1) =   88
   i= 24;LOWER(i,1) = 2    ;UPPER(i,1) =   90
   i= 25;LOWER(i,1) = 2    ;UPPER(i,1) =   93
   i= 26;LOWER(i,1) = 2    ;UPPER(i,1) =   96
   i= 27;LOWER(i,1) = 2    ;UPPER(i,1) =   98
   i= 28;LOWER(i,1) = 2    ;UPPER(i,1) =  113
   i= 29;LOWER(i,1) = 2    ;UPPER(i,1) =  114
   i= 30;LOWER(i,1) = 2    ;UPPER(i,1) =  117
   i= 31;LOWER(i,1) = 2    ;UPPER(i,1) =  124
   i= 32;LOWER(i,1) = 2    ;UPPER(i,1) =  125
   i= 33;LOWER(i,1) = 2    ;UPPER(i,1) =  128
   i= 34;LOWER(i,1) = 2    ;UPPER(i,1) =  133
   i= 35;LOWER(i,1) = 2    ;UPPER(i,1) =  134
   i= 36;LOWER(i,1) = 2    ;UPPER(i,1) =  158
   i= 37;LOWER(i,1) = 2    ;UPPER(i,1) =  161
   i= 38;LOWER(i,1) = 2    ;UPPER(i,1) =  195
   i= 39;LOWER(i,1) = 2    ;UPPER(i,1) =  196
   i= 40;LOWER(i,1) = 2    ;UPPER(i,1) =  202
   i= 41;LOWER(i,1) = 2    ;UPPER(i,1) =  236
   i= 42;LOWER(i,1) = 2    ;UPPER(i,1) =  241
   i= 43;LOWER(i,1) = 2    ;UPPER(i,1) =  350
   i= 44;LOWER(i,1) = 2    ;UPPER(i,1) =  418

   i= 45;LOWER(i,1) = 3    ;UPPER(i,1) =   67
   i= 46;LOWER(i,1) = 3    ;UPPER(i,1) =   69 
   i= 47;LOWER(i,1) = 3    ;UPPER(i,1) =   78 
   i= 48;LOWER(i,1) = 3    ;UPPER(i,1) =   90 
   i= 49;LOWER(i,1) = 3    ;UPPER(i,1) =   93 
   i= 50;LOWER(i,1) = 3    ;UPPER(i,1) =   95 
   i= 51;LOWER(i,1) = 3    ;UPPER(i,1) =   98 
   i= 52;LOWER(i,1) = 3    ;UPPER(i,1) =  100 
   i= 53;LOWER(i,1) = 3    ;UPPER(i,1) =  114 
   i= 54;LOWER(i,1) = 3    ;UPPER(i,1) =  116 
   i= 55;LOWER(i,1) = 3    ;UPPER(i,1) =  117 
   i= 56;LOWER(i,1) = 3    ;UPPER(i,1) =  119 
   i= 57;LOWER(i,1) = 3    ;UPPER(i,1) =  125 
   i= 58;LOWER(i,1) = 3    ;UPPER(i,1) =  128 
   i= 59;LOWER(i,1) = 3    ;UPPER(i,1) =  130 
   i= 60;LOWER(i,1) = 3    ;UPPER(i,1) =  134 
   i= 61;LOWER(i,1) = 3    ;UPPER(i,1) =  137 
   i= 62;LOWER(i,1) = 3    ;UPPER(i,1) =  138 
   i= 63;LOWER(i,1) = 3    ;UPPER(i,1) =  161 
   i= 64;LOWER(i,1) = 3    ;UPPER(i,1) =  168 
   i= 65;LOWER(i,1) = 3    ;UPPER(i,1) =  196 
   i= 66;LOWER(i,1) = 3    ;UPPER(i,1) =  197 
   i= 67;LOWER(i,1) = 3    ;UPPER(i,1) =  233 
   i= 68;LOWER(i,1) = 3    ;UPPER(i,1) =  236 
   i= 69;LOWER(i,1) = 3    ;UPPER(i,1) =  240 
   i= 70;LOWER(i,1) = 3    ;UPPER(i,1) =  241 
   i= 71;LOWER(i,1) = 3    ;UPPER(i,1) =  247 
   i= 72;LOWER(i,1) = 3    ;UPPER(i,1) =  418 

   i= 73;LOWER(i,1) = 4    ;UPPER(i,1) =   69   
   i= 74;LOWER(i,1) = 4    ;UPPER(i,1) =   70  
   i= 75;LOWER(i,1) = 4    ;UPPER(i,1) =   79  
   i= 76;LOWER(i,1) = 4    ;UPPER(i,1) =   93  
   i= 77;LOWER(i,1) = 4    ;UPPER(i,1) =   94  
   i= 78;LOWER(i,1) = 4    ;UPPER(i,1) =   97  
   i= 79;LOWER(i,1) = 4    ;UPPER(i,1) =  100  
   i= 80;LOWER(i,1) = 4    ;UPPER(i,1) =  103  
   i= 81;LOWER(i,1) = 4    ;UPPER(i,1) =  114  
   i= 82;LOWER(i,1) = 4    ;UPPER(i,1) =  116  
   i= 83;LOWER(i,1) = 4    ;UPPER(i,1) =  117  
   i= 84;LOWER(i,1) = 4    ;UPPER(i,1) =  119  
   i= 85;LOWER(i,1) = 4    ;UPPER(i,1) =  128  
   i= 86;LOWER(i,1) = 4    ;UPPER(i,1) =  130  
   i= 87;LOWER(i,1) = 4    ;UPPER(i,1) =  132  
   i= 88;LOWER(i,1) = 4    ;UPPER(i,1) =  137  
   i= 89;LOWER(i,1) = 4    ;UPPER(i,1) =  138  
   i= 90;LOWER(i,1) = 4    ;UPPER(i,1) =  164  
   i= 91;LOWER(i,1) = 4    ;UPPER(i,1) =  166  
   i= 92;LOWER(i,1) = 4    ;UPPER(i,1) =  170  
   i= 93;LOWER(i,1) = 4    ;UPPER(i,1) =  173  
   i= 94;LOWER(i,1) = 4    ;UPPER(i,1) =  196  
   i= 95;LOWER(i,1) = 4    ;UPPER(i,1) =  197  
   i= 96;LOWER(i,1) = 4    ;UPPER(i,1) =  205  
   i= 97;LOWER(i,1) = 4    ;UPPER(i,1) =  236  
   i= 98;LOWER(i,1) = 4    ;UPPER(i,1) =  240  
   i= 99;LOWER(i,1) = 4    ;UPPER(i,1) =  241  
   i=100;LOWER(i,1) = 4    ;UPPER(i,1) =  247 
   i=101;LOWER(i,1) = 4    ;UPPER(i,1) =  357 
   i=102;LOWER(i,1) = 4    ;UPPER(i,1) =  427 

   i=103;LOWER(i,1) = 5    ;UPPER(i,1) =    61
   i=104;LOWER(i,1) = 5    ;UPPER(i,1) =    70
   i=105;LOWER(i,1) = 5    ;UPPER(i,1) =    79
   i=106;LOWER(i,1) = 5    ;UPPER(i,1) =    95
   i=107;LOWER(i,1) = 5    ;UPPER(i,1) =   103
   i=108;LOWER(i,1) = 5    ;UPPER(i,1) =   116
   i=109;LOWER(i,1) = 5    ;UPPER(i,1) =   119 
   i=110;LOWER(i,1) = 5    ;UPPER(i,1) =   130 
   i=111;LOWER(i,1) = 5    ;UPPER(i,1) =   138 
   i=112;LOWER(i,1) = 5    ;UPPER(i,1) =   148 
   i=113;LOWER(i,1) = 5    ;UPPER(i,1) =   167 
   i=114;LOWER(i,1) = 5    ;UPPER(i,1) =   168 
   i=115;LOWER(i,1) = 5    ;UPPER(i,1) =   175 
   i=116;LOWER(i,1) = 5    ;UPPER(i,1) =   197 
   i=117;LOWER(i,1) = 5    ;UPPER(i,1) =   198 
   i=118;LOWER(i,1) = 5    ;UPPER(i,1) =   205 
   i=119;LOWER(i,1) = 5    ;UPPER(i,1) =   214 
   i=120;LOWER(i,1) = 5    ;UPPER(i,1) =   240 
   i=121;LOWER(i,1) = 5    ;UPPER(i,1) =   242 
   i=122;LOWER(i,1) = 5    ;UPPER(i,1) =   247 
   i=123;LOWER(i,1) = 5    ;UPPER(i,1) =   357 
   i=124;LOWER(i,1) = 5    ;UPPER(i,1) =   427 

   Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))
              
CASE('FeII ') 

   !ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))) 
   !ALLOCATE(UPPER(Nchannels, MAXVAL(Nu)))

   !LOWER = 0
   !UPPER = 0
        
   !DO j = 1, 10
   !   LOWER(j,1:5) = (/(i, i=1,5)/)
   !END DO
      
   !UPPER(1,:) = (/(i, i=6,9)/)            ! a4Fe   Note levels the same in 191 and 773 atom
   !UPPER(2,:) = (/(i, i=10,13)/)          ! a4De
   !UPPER(3,:) = (/(i, i=14,16)/)          ! a4Pe
   !UPPER(4,:) = (/ 24, 30, 31 /)          ! b4Pe
   !UPPER(5,:) = (/(i, i=32,35)/)          ! b4Fe
   !UPPER(6,1) = 36                        ! a6Se
   !UPPER(7,:) = (/(i, i=64,68)/)          ! z6Do
   !UPPER(8,:) = (/(i, i=69,74)/)          ! z6Fo
   !UPPER(9,:) = (/(i, i=75,77)/)          ! z6Po
   !UPPER(10,:) = (/(i, i=88,90)/)         ! z4Po


   ! ---- NEW ----------------------
  
   PF = 10. + 8*EXP(-h*c*384.79/(k*Tuse)) + 6*EXP(-h*c*667.68/(k*Tuse)) + 4*EXP(-h*c*862.613/(k*Tuse)) + 2*EXP(-h*c*977.053/(k*Tuse))

   xv(1) = 10./PF      ! Assume LTE

   xv(2) = 8./PF*EXP(-h*c*384.79/(k*Tuse))

   xv(3) = 6./PF*EXP(-h*c*667.68/(k*Tuse))

   xv(4) = 4./PF*EXP(-h*c*862.61/(k*Tuse))

   xv(5) = 2./PF*EXP(-h*c*977.05/(k*Tuse))

  
   Nchannels = mtrans

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   Nl(:) = 1
   Nu(:) = 1
                       
   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0 
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0
 
   LOWER(:,1) = i_CFvec(1:Nchannels)

   DO i = 1, Nchannels
      x(i) = xv(LOWER(i,1))
      Elower(i) = atom%LEVELS_E(i_CFvec(i))
      Eupper(i) = atom%LEVELS_E(j_CFvec(i))
     
   END DO
  
   UPPER(:,1) = j_CFvec(1:Nchannels)
        
   !i=  1;LOWER(i,1) = 1    ;UPPER(i,1) =   64 
   !i=  2;LOWER(i,1) = 1    ;UPPER(i,1) =   65
   !i=  3;LOWER(i,1) = 1    ;UPPER(i,1) =   69
   !i=  4;LOWER(i,1) = 1    ;UPPER(i,1) =   70
   !i=  5;LOWER(i,1) = 1    ;UPPER(i,1) =   75
   !i=  6;LOWER(i,1) = 1    ;UPPER(i,1) =   78
   !i=  7;LOWER(i,1) = 1    ;UPPER(i,1) =   79
   !i=  8;LOWER(i,1) = 1    ;UPPER(i,1) =  140
   !i=  9;LOWER(i,1) = 1    ;UPPER(i,1) =  145
   !i= 10;LOWER(i,1) = 1    ;UPPER(i,1) =  245
   !i= 11;LOWER(i,1) = 1    ;UPPER(i,1) =  296
   !i= 12;LOWER(i,1) = 1    ;UPPER(i,1) =  297
   !i= 13;LOWER(i,1) = 1    ;UPPER(i,1) =  298
   !i= 14;LOWER(i,1) = 1    ;UPPER(i,1) =  308
   !i= 15;LOWER(i,1) = 1    ;UPPER(i,1) =  309
   !i= 16;LOWER(i,1) = 1    ;UPPER(i,1) =  310
   !i= 17;LOWER(i,1) = 1    ;UPPER(i,1) =  311
   !i= 18;LOWER(i,1) = 1    ;UPPER(i,1) =  314
   !i= 19;LOWER(i,1) = 1    ;UPPER(i,1) =  321
   !i= 20;LOWER(i,1) = 1    ;UPPER(i,1) =  323
   !i= 21;LOWER(i,1) = 1    ;UPPER(i,1) =  339
   !i= 22;LOWER(i,1) = 1    ;UPPER(i,1) =  357
   !i= 23;LOWER(i,1) = 1    ;UPPER(i,1) =  362
   !i= 24;LOWER(i,1) = 1    ;UPPER(i,1) =  375
   !i= 25;LOWER(i,1) = 1    ;UPPER(i,1) =  376
   !i= 26;LOWER(i,1) = 1    ;UPPER(i,1) =  378
   !i= 27;LOWER(i,1) = 1    ;UPPER(i,1) =  386

   !i= 28 ;LOWER(i,1)= 2    ;UPPER(i,1) =  64    
   !i= 29 ;LOWER(i,1)= 2    ;UPPER(i,1) =  65 
   !i= 30 ;LOWER(i,1)= 2    ;UPPER(i,1) =  66 
   !i= 31 ;LOWER(i,1)= 2    ;UPPER(i,1) =  70 
   !i= 32 ;LOWER(i,1)= 2    ;UPPER(i,1) =  71 
   !i= 33 ;LOWER(i,1)= 2    ;UPPER(i,1) =  72 
   !i= 34 ;LOWER(i,1)= 2    ;UPPER(i,1) =  75 
   !i= 35 ;LOWER(i,1)= 2    ;UPPER(i,1) =  76 
   !i= 36 ;LOWER(i,1)= 2    ;UPPER(i,1) =  78 
   !i= 37 ;LOWER(i,1)= 2    ;UPPER(i,1) =  80 
   !i= 38 ;LOWER(i,1)= 2    ;UPPER(i,1) = 139 
   !i= 39 ;LOWER(i,1)= 2    ;UPPER(i,1) = 143 
   !i= 40 ;LOWER(i,1)= 2    ;UPPER(i,1) = 145 
   !i= 41 ;LOWER(i,1)= 2    ;UPPER(i,1) = 244 
   !i= 42 ;LOWER(i,1)= 2    ;UPPER(i,1) = 245 
   !i= 43 ;LOWER(i,1)= 2    ;UPPER(i,1) = 297
   !i= 44 ;LOWER(i,1)= 2    ;UPPER(i,1) = 298
   !i= 45 ;LOWER(i,1)= 2    ;UPPER(i,1) = 299
   !i= 46 ;LOWER(i,1)= 2    ;UPPER(i,1) = 308
   !i= 47 ;LOWER(i,1)= 2    ;UPPER(i,1) = 309
   !i= 48 ;LOWER(i,1)= 2    ;UPPER(i,1) = 310
   !i= 49 ;LOWER(i,1)= 2    ;UPPER(i,1) = 313
   !i= 50 ;LOWER(i,1)= 2    ;UPPER(i,1) = 314
   !i= 51 ;LOWER(i,1)= 2    ;UPPER(i,1) = 316
   !i= 52 ;LOWER(i,1)= 2    ;UPPER(i,1) = 323
   !i= 53 ;LOWER(i,1)= 2    ;UPPER(i,1) = 325
   !i= 54 ;LOWER(i,1)= 2    ;UPPER(i,1) = 326
   !i= 55 ;LOWER(i,1)= 2    ;UPPER(i,1) = 339
   !i= 56 ;LOWER(i,1)= 2    ;UPPER(i,1) = 342
   !i= 57 ;LOWER(i,1)= 2    ;UPPER(i,1) = 349
   !i= 58 ;LOWER(i,1)= 2    ;UPPER(i,1) = 357
   !i= 59 ;LOWER(i,1)= 2    ;UPPER(i,1) = 373
   !i= 60 ;LOWER(i,1)= 2    ;UPPER(i,1) = 375
   !i= 61 ;LOWER(i,1)= 2    ;UPPER(i,1) = 376
   !i= 62 ;LOWER(i,1)= 2    ;UPPER(i,1) = 378
   !i= 63 ;LOWER(i,1)= 2    ;UPPER(i,1) = 382
   !i= 64 ;LOWER(i,1)= 2    ;UPPER(i,1) = 383
   !i= 65 ;LOWER(i,1)= 2    ;UPPER(i,1) = 386

   !i= 66 ;LOWER(i,1)= 3    ;UPPER(i,1) =  65 
   !i= 67 ;LOWER(i,1)= 3    ;UPPER(i,1) =  66 
   !i= 68 ;LOWER(i,1)= 3    ;UPPER(i,1) =  67 
   !i= 69 ;LOWER(i,1)= 3    ;UPPER(i,1) =  71 
   !i= 70 ;LOWER(i,1)= 3    ;UPPER(i,1) =  72 
   !i= 71 ;LOWER(i,1)= 3    ;UPPER(i,1) =  73 
   !i= 72 ;LOWER(i,1)= 3    ;UPPER(i,1) =  75 
   !i= 73 ;LOWER(i,1)= 3    ;UPPER(i,1) =  76 
   !i= 74 ;LOWER(i,1)= 3    ;UPPER(i,1) =  77 
   !i= 75 ;LOWER(i,1)= 3    ;UPPER(i,1) =  80 
   !i= 76 ;LOWER(i,1)= 3    ;UPPER(i,1) =  85 
   !i= 77 ;LOWER(i,1)= 3    ;UPPER(i,1) = 138 
   !i= 78 ;LOWER(i,1)= 3    ;UPPER(i,1) = 139 
   !i= 79 ;LOWER(i,1)= 3    ;UPPER(i,1) = 143 
   !i= 80 ;LOWER(i,1)= 3    ;UPPER(i,1) = 145 
   !i= 81 ;LOWER(i,1)= 3    ;UPPER(i,1) = 242 
   !i= 82 ;LOWER(i,1)= 3    ;UPPER(i,1) = 243 
   !i= 83 ;LOWER(i,1)= 3    ;UPPER(i,1) = 244 
   !i= 84 ;LOWER(i,1)= 3    ;UPPER(i,1) = 245 
   !i= 85 ;LOWER(i,1)= 3    ;UPPER(i,1) = 298
   !i= 86 ;LOWER(i,1)= 3    ;UPPER(i,1) = 299
   !i= 87 ;LOWER(i,1)= 3    ;UPPER(i,1) = 300
   !i= 88 ;LOWER(i,1)= 3    ;UPPER(i,1) = 305
   !i= 89 ;LOWER(i,1)= 3    ;UPPER(i,1) = 308
   !i= 90 ;LOWER(i,1)= 3    ;UPPER(i,1) = 313
   !i= 91 ;LOWER(i,1)= 3    ;UPPER(i,1) = 314
   !i= 92 ;LOWER(i,1)= 3    ;UPPER(i,1) = 315
   !i= 93 ;LOWER(i,1)= 3    ;UPPER(i,1) = 316
   !i= 94 ;LOWER(i,1)= 3    ;UPPER(i,1) = 318
   !i= 95 ;LOWER(i,1)= 3    ;UPPER(i,1) = 325
   !i= 96 ;LOWER(i,1)= 3    ;UPPER(i,1) = 328
   !i= 97 ;LOWER(i,1)= 3    ;UPPER(i,1) = 339
   !i= 98 ;LOWER(i,1)= 3    ;UPPER(i,1) = 342
   !i= 99 ;LOWER(i,1)= 3    ;UPPER(i,1) = 343
   !i= 100;LOWER(i,1)= 3    ;UPPER(i,1) = 346
   !i= 101;LOWER(i,1)= 3    ;UPPER(i,1) = 349
   !i= 102;LOWER(i,1)= 3    ;UPPER(i,1) = 373
   !i= 103;LOWER(i,1)= 3    ;UPPER(i,1) = 374
   !i= 104;LOWER(i,1)= 3    ;UPPER(i,1) = 375
   !i= 105;LOWER(i,1)= 3    ;UPPER(i,1) = 376
   !i= 106;LOWER(i,1)= 3    ;UPPER(i,1) = 385
   !i= 107;LOWER(i,1)= 3    ;UPPER(i,1) = 386

   !i= 108;LOWER(i,1)= 4   ;UPPER(i,1) =  66 
   !i= 109;LOWER(i,1)= 4   ;UPPER(i,1) =  67 
   !i= 110;LOWER(i,1)= 4   ;UPPER(i,1) =  68 
   !i= 111;LOWER(i,1)= 4   ;UPPER(i,1) =  72 
   !i= 112;LOWER(i,1)= 4   ;UPPER(i,1) =  73 
   !i= 113;LOWER(i,1)= 4   ;UPPER(i,1) =  74 
   !i= 114;LOWER(i,1)= 4   ;UPPER(i,1) =  76 
   !i= 115;LOWER(i,1)= 4   ;UPPER(i,1) =  77 
   !i= 116;LOWER(i,1)= 4   ;UPPER(i,1) =  84 
   !i= 117;LOWER(i,1)= 4   ;UPPER(i,1) =  85 
   !i= 118;LOWER(i,1)= 4   ;UPPER(i,1) =  87 
   !i= 119;LOWER(i,1)= 4   ;UPPER(i,1) =  138 
   !i= 120;LOWER(i,1)= 4   ;UPPER(i,1) =  139 
   !i= 121;LOWER(i,1)= 4   ;UPPER(i,1) =  143 
   !i= 122;LOWER(i,1)= 4   ;UPPER(i,1) =  243 
   !i= 123;LOWER(i,1)= 4   ;UPPER(i,1) =  244 
   !i= 124;LOWER(i,1)= 4   ;UPPER(i,1) =  299
   !i= 125;LOWER(i,1)= 4   ;UPPER(i,1) =  300
   !i= 126;LOWER(i,1)= 4   ;UPPER(i,1) =  301
   !i= 127;LOWER(i,1)= 4   ;UPPER(i,1) =  302
   !i= 128;LOWER(i,1)= 4   ;UPPER(i,1) =  303
   !i= 129;LOWER(i,1)= 4   ;UPPER(i,1) =  305
   !i= 130;LOWER(i,1)= 4   ;UPPER(i,1) =  315
   !i= 131;LOWER(i,1)= 4   ;UPPER(i,1) =  316
   !i= 132;LOWER(i,1)= 4   ;UPPER(i,1) =  317
   !i= 133;LOWER(i,1)= 4   ;UPPER(i,1) =  318
   !i= 134;LOWER(i,1)= 4   ;UPPER(i,1) =  329
   !i= 135;LOWER(i,1)= 4   ;UPPER(i,1) =  342
   !i= 136;LOWER(i,1)= 4   ;UPPER(i,1) =  343
   !i= 137;LOWER(i,1)= 4   ;UPPER(i,1) =  344
   !i= 138;LOWER(i,1)= 4   ;UPPER(i,1) =  345
   !i= 139;LOWER(i,1)= 4   ;UPPER(i,1) =  346
   !i= 140;LOWER(i,1)= 4   ;UPPER(i,1) =  373
   !i= 141;LOWER(i,1)= 4   ;UPPER(i,1) =  374
   !i= 142;LOWER(i,1)= 4   ;UPPER(i,1) =  377
   !i= 143;LOWER(i,1)= 4   ;UPPER(i,1) =  383
   !i= 144;LOWER(i,1)= 4   ;UPPER(i,1) =  387

   !i= 145;LOWER(i,1) = 5  ;UPPER(i,1) =  67 
   !i= 146;LOWER(i,1) = 5  ;UPPER(i,1) =  68 
   !i= 147;LOWER(i,1) = 5  ;UPPER(i,1) =  73 
   !i= 148;LOWER(i,1) = 5  ;UPPER(i,1) =  74 
   !i= 149;LOWER(i,1) = 5  ;UPPER(i,1) =  77 
   !i= 150;LOWER(i,1) = 5  ;UPPER(i,1) =  85 
   !i= 151;LOWER(i,1) = 5  ;UPPER(i,1) =  86 
   !i= 152;LOWER(i,1) = 5  ;UPPER(i,1) =  138 
   !i= 153;LOWER(i,1) = 5  ;UPPER(i,1) =  243 
   !i= 154;LOWER(i,1) = 5  ;UPPER(i,1) =  300
   !i= 155;LOWER(i,1) = 5  ;UPPER(i,1) =  301
   !i= 156;LOWER(i,1) = 5  ;UPPER(i,1) =  302
   !i= 157;LOWER(i,1) = 5  ;UPPER(i,1) =  303
   !i= 158;LOWER(i,1) = 5  ;UPPER(i,1) =  317
   !i= 159;LOWER(i,1) = 5  ;UPPER(i,1) =  318
   !i= 160;LOWER(i,1) = 5  ;UPPER(i,1) =  343
   !i= 161;LOWER(i,1) = 5  ;UPPER(i,1) =  344
   !i= 162;LOWER(i,1) = 5  ;UPPER(i,1) =  374
   !i= 163;LOWER(i,1) = 5  ;UPPER(i,1) =  377
   !i= 164;LOWER(i,1) = 5  ;UPPER(i,1) =  385
   !i= 165;LOWER(i,1) = 5  ;UPPER(i,1) =  387
    
CASE('FeIII')

   Nchannels = 65   ! All with f_osc > 1E-2

   PF = 9. + 7*EXP(-h*c*435.79/(k*Tuse)) + 5*EXP(-h*c*738.55/(k*Tuse)) + 3*EXP(-h*c*932.06/(k*Tuse)) + 1*EXP(-h*c*1027.00/(k*Tuse))

   x(1:8) = 9./PF      ! Assume LTE

   x(9:23) = 7./PF*exp(-h*c*435.79/(k*Tuse))

   x(24:38) = 5./PF*exp(-h*c*738.55/(k*Tuse))

   x(39:53) = 3./PF*exp(-h*c*932.06/(k*Tuse))

   x(54:65) = 1./PF*exp(-h*c*1027./(k*Tuse))

   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))

   Nl(:) = 1
   Nu(:) = 1
 
   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0

   i=  1;LOWER(i,1) = 1    ;UPPER(i,1) =   81
   i=  2;LOWER(i,1) = 1    ;UPPER(i,1) =  116
   i=  3;LOWER(i,1) = 1    ;UPPER(i,1) =  120
   i=  4;LOWER(i,1) = 1    ;UPPER(i,1) =  132
   i=  5;LOWER(i,1) = 1    ;UPPER(i,1) =  144
   i=  6;LOWER(i,1) = 1    ;UPPER(i,1) =  154
   i=  7;LOWER(i,1) = 1    ;UPPER(i,1) =  208
   i=  8;LOWER(i,1) = 1    ;UPPER(i,1) =  216

   i=  9;LOWER(i,1) = 2    ;UPPER(i,1) =   81                                             
   i= 10;LOWER(i,1) = 2    ;UPPER(i,1) =   82
   i= 11;LOWER(i,1) = 2    ;UPPER(i,1) =  120
   i= 12;LOWER(i,1) = 2    ;UPPER(i,1) =  121
   i= 13;LOWER(i,1) = 2    ;UPPER(i,1) =  125
   i= 14;LOWER(i,1) = 2    ;UPPER(i,1) =  126
   i= 15;LOWER(i,1) = 2    ;UPPER(i,1) =  132
   i= 16;LOWER(i,1) = 2    ;UPPER(i,1) =  135
   i= 17;LOWER(i,1) = 2    ;UPPER(i,1) =  143
   i= 18;LOWER(i,1) = 2    ;UPPER(i,1) =  150
   i= 19;LOWER(i,1) = 2    ;UPPER(i,1) =  154
   i= 20;LOWER(i,1) = 2    ;UPPER(i,1) =  157
   i= 21;LOWER(i,1) = 2    ;UPPER(i,1) =  205
   i= 22;LOWER(i,1) = 2    ;UPPER(i,1) =  217
   i= 23;LOWER(i,1) = 2    ;UPPER(i,1) =  219

   i= 24;LOWER(i,1) = 3    ;UPPER(i,1) =    82   
   i= 25;LOWER(i,1) = 3    ;UPPER(i,1) =    83
   i= 26;LOWER(i,1) = 3    ;UPPER(i,1) =   121                                            
   i= 27;LOWER(i,1) = 3    ;UPPER(i,1) =   124
   i= 28;LOWER(i,1) = 3    ;UPPER(i,1) =   125
   i= 29;LOWER(i,1) = 3    ;UPPER(i,1) =   135
   i= 30;LOWER(i,1) = 3    ;UPPER(i,1) =   136
   i= 31;LOWER(i,1) = 3    ;UPPER(i,1) =   142
   i= 32;LOWER(i,1) = 3    ;UPPER(i,1) =   148
   i= 33;LOWER(i,1) = 3    ;UPPER(i,1) =   150
   i= 34;LOWER(i,1) = 3    ;UPPER(i,1) =   152
   i= 35;LOWER(i,1) = 3    ;UPPER(i,1) =   156
   i= 36;LOWER(i,1) = 3    ;UPPER(i,1) =   209
   i= 37;LOWER(i,1) = 3    ;UPPER(i,1) =   219
   i= 38;LOWER(i,1) = 3    ;UPPER(i,1) =   220

   i= 39;LOWER(i,1) = 4    ;UPPER(i,1) =    82
   i= 40;LOWER(i,1) = 4    ;UPPER(i,1) =    83
   i= 41;LOWER(i,1) = 4    ;UPPER(i,1) =   119
   i= 42;LOWER(i,1) = 4    ;UPPER(i,1) =   123
   i= 43;LOWER(i,1) = 4    ;UPPER(i,1) =   124                                            
   i= 44;LOWER(i,1) = 4    ;UPPER(i,1) =   135
   i= 45;LOWER(i,1) = 4    ;UPPER(i,1) =   136
   i= 46;LOWER(i,1) = 4    ;UPPER(i,1) =   141
   i= 47;LOWER(i,1) = 4    ;UPPER(i,1) =   149
   i= 48;LOWER(i,1) = 4    ;UPPER(i,1) =   152
   i= 49;LOWER(i,1) = 4    ;UPPER(i,1) =   155
   i= 50;LOWER(i,1) = 4    ;UPPER(i,1) =   157
   i= 51;LOWER(i,1) = 4    ;UPPER(i,1) =   207
   i= 52;LOWER(i,1) = 4    ;UPPER(i,1) =   219
   i= 53;LOWER(i,1) = 4    ;UPPER(i,1) =   221

   i= 54;LOWER(i,1) = 5    ;UPPER(i,1) =   83
   i= 55;LOWER(i,1) = 5    ;UPPER(i,1) =  118
   i= 56;LOWER(i,1) = 5    ;UPPER(i,1) =  123
   i= 57;LOWER(i,1) = 5    ;UPPER(i,1) =  136
   i= 58;LOWER(i,1) = 5    ;UPPER(i,1) =  138
   i= 59;LOWER(i,1) = 5    ;UPPER(i,1) =  140
   i= 60;LOWER(i,1) = 5    ;UPPER(i,1) =  151                                             
   i= 61;LOWER(i,1) = 5    ;UPPER(i,1) =  153
   i= 62;LOWER(i,1) = 5    ;UPPER(i,1) =  156
   i= 63;LOWER(i,1) = 5    ;UPPER(i,1) =  210
   i= 64;LOWER(i,1) = 5    ;UPPER(i,1) =  211
   i= 65;LOWER(i,1) = 5    ;UPPER(i,1) =  220

   Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))

   ! DC 2013-06

          
CASE('NiII ') 

   !Nchannels = 69
   Nchannels = 42  ! Up to level 300

   PF = 6 + 4*EXP(-h*c*1506.94/(k*Tuse))   ! 1506 = 2200 K

   x(1:42) = 6./PF
	
   ALLOCATE(Nl(Nchannels))
   ALLOCATE(Nu(Nchannels))
   ALLOCATE(Elower(Nchannels))
   ALLOCATE(Eupper(Nchannels))
	
   Nl(:) = 1              
   Nu(:) = 1     
          	
   ALLOCATE(LOWER(Nchannels, MAXVAL(Nl))); LOWER = 0
   ALLOCATE(UPPER(Nchannels, MAXVAL(Nu))); UPPER = 0
        
   i=  1;LOWER(i,1) = 1    ;UPPER(i,1) =  32  
   i=  2;LOWER(i,1) = 1    ;UPPER(i,1) =  33 
   i=  3;LOWER(i,1) = 1    ;UPPER(i,1) =  34 
   i=  4;LOWER(i,1) = 1    ;UPPER(i,1) =  36 
   i=  5;LOWER(i,1) = 1    ;UPPER(i,1) =  37 
   i=  6;LOWER(i,1) = 1    ;UPPER(i,1) =  38 
   i=  7;LOWER(i,1) = 1    ;UPPER(i,1) =  39 
   i=  8;LOWER(i,1) = 1    ;UPPER(i,1) =  40 
   i=  9;LOWER(i,1) = 1    ;UPPER(i,1) =  43 
   i= 10;LOWER(i,1) = 1    ;UPPER(i,1) =  45 
   i= 11;LOWER(i,1) = 1    ;UPPER(i,1) =  46 
   i= 12;LOWER(i,1) = 1    ;UPPER(i,1) =  50 
   i= 13;LOWER(i,1) = 1    ;UPPER(i,1) =  51 
   i= 14;LOWER(i,1) = 1    ;UPPER(i,1) =  53 
   i= 15;LOWER(i,1) = 1    ;UPPER(i,1) =  54 
   i= 16;LOWER(i,1) = 1    ;UPPER(i,1) =  56 
   i= 17;LOWER(i,1) = 1    ;UPPER(i,1) =  58 
   i= 18;LOWER(i,1) = 1    ;UPPER(i,1) =  59 
   i= 19;LOWER(i,1) = 1    ;UPPER(i,1) =  60 
   i= 20;LOWER(i,1) = 1    ;UPPER(i,1) =  64 
   i= 21;LOWER(i,1) = 1    ;UPPER(i,1) =  67 
   i= 22;LOWER(i,1) = 1    ;UPPER(i,1) =  68 
   i= 23;LOWER(i,1) = 1    ;UPPER(i,1) = 153 
   i= 24;LOWER(i,1) = 1    ;UPPER(i,1) = 162 
   i= 25;LOWER(i,1) = 1    ;UPPER(i,1) = 223 
   i= 26;LOWER(i,1) = 1    ;UPPER(i,1) = 236 
   i= 27;LOWER(i,1) = 1    ;UPPER(i,1) = 237 
   i= 28;LOWER(i,1) = 1    ;UPPER(i,1) = 239 
   i= 29;LOWER(i,1) = 1    ;UPPER(i,1) = 240 
   i= 30;LOWER(i,1) = 1    ;UPPER(i,1) = 243 
   i= 31;LOWER(i,1) = 1    ;UPPER(i,1) = 245 
   i= 32;LOWER(i,1) = 1    ;UPPER(i,1) = 264 
   i= 33;LOWER(i,1) = 1    ;UPPER(i,1) = 265 
   i= 34;LOWER(i,1) = 1    ;UPPER(i,1) = 268 
   i= 35;LOWER(i,1) = 1    ;UPPER(i,1) = 269 
   i= 36;LOWER(i,1) = 1    ;UPPER(i,1) = 270 
   i= 37;LOWER(i,1) = 1    ;UPPER(i,1) = 272 
   i= 38;LOWER(i,1) = 1    ;UPPER(i,1) = 278 
   i= 39;LOWER(i,1) = 1    ;UPPER(i,1) = 285 
   i= 40;LOWER(i,1) = 1    ;UPPER(i,1) = 287 
   i= 41;LOWER(i,1) = 1    ;UPPER(i,1) = 291 
   i= 42;LOWER(i,1) = 1    ;UPPER(i,1) = 292 
   !i= 43;LOWER(i,1) = 1    ;UPPER(i,1) = 305 
   !i= 44;LOWER(i,1) = 1    ;UPPER(i,1) = 330 
   !i= 45;LOWER(i,1) = 1    ;UPPER(i,1) = 336 
   !i= 46;LOWER(i,1) = 1    ;UPPER(i,1) = 338 
   !i= 47;LOWER(i,1) = 1    ;UPPER(i,1) = 340 
   !i= 48;LOWER(i,1) = 1    ;UPPER(i,1) = 342 
   !i= 49;LOWER(i,1) = 1    ;UPPER(i,1) = 344 
   !i= 50;LOWER(i,1) = 1    ;UPPER(i,1) = 348 
   !i= 51;LOWER(i,1) = 1    ;UPPER(i,1) = 379 
   !i= 52;LOWER(i,1) = 1    ;UPPER(i,1) = 380 
   !i= 53;LOWER(i,1) = 1    ;UPPER(i,1) = 383 
   !i= 54;LOWER(i,1) = 1    ;UPPER(i,1) = 388 
   !i= 55;LOWER(i,1) = 1    ;UPPER(i,1) = 389 
   !i= 56;LOWER(i,1) = 1    ;UPPER(i,1) = 399 
   !i= 57;LOWER(i,1) = 1    ;UPPER(i,1) = 420 
   !i= 58;LOWER(i,1) = 1    ;UPPER(i,1) = 421 
   !i= 59;LOWER(i,1) = 1    ;UPPER(i,1) = 423 
   !i= 60;LOWER(i,1) = 1    ;UPPER(i,1) = 424 
   !i= 61;LOWER(i,1) = 1    ;UPPER(i,1) = 425 
   !i= 62;LOWER(i,1) = 1    ;UPPER(i,1) = 444 
   !i= 63;LOWER(i,1) = 1    ;UPPER(i,1) = 446 
   !i= 64;LOWER(i,1) = 1    ;UPPER(i,1) = 448 
   !i= 65;LOWER(i,1) = 1    ;UPPER(i,1) = 450 
   !i= 66;LOWER(i,1) = 1    ;UPPER(i,1) = 454 
   !i= 67;LOWER(i,1) = 1    ;UPPER(i,1) = 481 
   !i= 68;LOWER(i,1) = 1    ;UPPER(i,1) = 483 
   !i= 69;LOWER(i,1) = 1    ;UPPER(i,1) = 493 
   !i= 70;LOWER(i,1) = 1    ;UPPER(i,1) = 520 
   !i= 71;LOWER(i,1) = 1    ;UPPER(i,1) = 536 
   !i= 72;LOWER(i,1) = 1    ;UPPER(i,1) = 540 
   !i= 73;LOWER(i,1) = 1    ;UPPER(i,1) = 542 
   !i= 74;LOWER(i,1) = 1    ;UPPER(i,1) = 543 
   !i= 75;LOWER(i,1) = 1    ;UPPER(i,1) = 544 
   !i= 76;LOWER(i,1) = 1    ;UPPER(i,1) = 546 
   !i= 77;LOWER(i,1) = 1    ;UPPER(i,1) = 555 
   !i= 78;LOWER(i,1) = 1    ;UPPER(i,1) = 556 
   !i= 79;LOWER(i,1) = 1    ;UPPER(i,1) = 558 
   !i= 80;LOWER(i,1) = 1    ;UPPER(i,1) = 559 
   !i= 81;LOWER(i,1) = 1    ;UPPER(i,1) = 562 
   !i= 82;LOWER(i,1) = 1    ;UPPER(i,1) = 584 
   !i= 83;LOWER(i,1) = 1    ;UPPER(i,1) = 586 
   !i= 84;LOWER(i,1) = 1    ;UPPER(i,1) = 590 
   !i= 85;LOWER(i,1) = 1    ;UPPER(i,1) = 600 
   !i= 86;LOWER(i,1) = 1    ;UPPER(i,1) = 614 
   !i= 87;LOWER(i,1) = 1    ;UPPER(i,1) = 615 
   !i= 88;LOWER(i,1) = 1    ;UPPER(i,1) = 646 
   !i= 89;LOWER(i,1) = 1    ;UPPER(i,1) = 653 

   Elower(1:Nchannels) = atom%LEVELS_E(LOWER(1:Nchannels,1))

   Eupper(1:Nchannels) = atom%LEVELS_E(UPPER(1:Nchannels,1))
        
CASE DEFAULT

    print *, 'WARNING. Lacking non-thermal allocations for atom ', atom%NAME

    RETURN

END SELECT

DO i = 1, Nchannels

   Echannel = Eupper(i) - Elower(i)

   IF (nk_ground*x(i) > 0) THEN
      
      GAMMATOT = deposition_tot_pervol*fsc_dummy(i)/(nk_ground*x(i)*Echannel)    ! Assume all atoms in ground multiplet

      gtot = 0
        
      DO j = 1, Nu(i)
                        
         gtot = gtot + atom%LEVELS_G(UPPER(i,j))

      END DO
                      
      DO j = 1, Nl(i)            ! If upper level is multiplet, divide rate according to statistical weights of upper level
                                
         DO q = 1, Nu(i) 
        
            lowLEV = LOWER(i,j)

            uppLEV = UPPER(i,q)
              
            IF (uppLEV .LE. atom%N_levels) THEN   ! check if this upper level is included in NLTE solutions

               GAMMAMAT(lowLEV, uppLEV) = GAMMATOT*atom%LEVELS_G(uppLEV)/gtot

            ELSE

               print *, 'WARNING, nonthermal excitation outside atom size', atom%NAME, uppLEV , fsc_dummy(i)
                       
            END IF

         END DO

      END DO

   END IF

END DO

END SUBROUTINE allocGAMMAEXC


