SUBROUTINE boltzmann(T, atom, N_levels, boltzmannsol, Zout)                   ! Calculates the normalized Boltzmann distribution for given T
USE M1, ONLY : atomm
USE PARAMETERS, ONLY : k
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: N_levels
DOUBLE PRECISION, DIMENSION(N_levels), INTENT(OUT)  :: boltzmannsol
DOUBLE PRECISION, OPTIONAL :: Zout
! ======================================================
! Task : Compute Boltzmann distribution for given temperature
! Input :   T            Temperature [K]
!           atom         Atom to be computes (struct type ATOMM (module m1))
!           N_levels     Number of levels to compute
! Output   boltzmannsol  Solution (normalized to unity)
! ===================================================
double precision, dimension(N_levels) :: U_vec
double precision :: Z

! -----------------------------------------
boltzmannsol = 0

U_vec = 0

Z = 0
! --------------------------------------------

U_vec = atom%levels_g(1:N_levels)*exp(-atom%levels_E(1:N_levels)/(k*T))

Z = sum(U_vec)

if (present(Zout)) Zout = Z

boltzmannsol = U_vec/Z     ! normalize                      


END
