SUBROUTINE BREMSSTRAHLUNG(Z, nplus, ne, T, lambdastart, lambdaend, stepatstart, N, lambdavec, emissvec_integrated)
USE M1, ONLY : DOPRINT, myid, IDP
USE PARAMETERS, ONLY : c, h, k
IMPLICIT NONE
INTEGER, INTENT(IN) :: Z
DOUBLE PRECISION, INTENT(IN) :: nplus, ne, T, lambdastart, lambdaend, stepatstart
INTEGER, INTENT(OUT) :: N
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE,  INTENT(OUT) :: lambdavec, emissvec_integrated
! =======================================================
! TASK :  Compute Bremsstrahlung emission spectrum given n+, ne, and T
! Input : Z           Charge of the ion
!         nplus       Number density of ion [cm^-3]
!         ne          Electron number density [cm^-3]
!         T           Temperature [K]
!         lambdastart Shortest wavelength desired []
!         lambdaend   Longest wavelength desired []
!         stepatstart First bin size []
! Output: N           Number of wavelengths in spectrum
!         lambdavec   Vector with wavelengths []
!         emissvec_integraded  Flux [erg s^-1 cm^-3] in each bin
! =========================================================
INTEGER :: i
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: emissvec_perA
DOUBLE PRECISION :: gaunt, nu, jnu, jlambda
! ==============================================

N = INT(LOG(lambdaend/lambdastart)/LOG(1.+stepatstart/lambdastart))  ! Number of elements, constant relative step size

ALLOCATE(lambdavec(N)); lambdavec = 0
ALLOCATE(emissvec_integrated(N)); emissvec_integrated = 0
ALLOCATE(emissvec_perA(N)); emissvec_perA = 0

lambdavec(1) = lambdastart

DO i = 2, N     ! Create wavelength vector
   
   lambdavec(i) = lambdavec(i-1)*(1. + stepatstart/lambdastart)

END DO

gaunt = 1.3  ! NEEDSFIX From deKool1998  Make sure to be the same as in alfa_photoionization...Is there a better way to do this?

DO i = 2, N  ! For each wavelength...

   nu = c/(lambdavec(i)*1E-8)   ! Frequency

   ! Rybicki Lightman eq 5.14b  see also Shu Radiation eq 15.28:

   jnu = 6.81D-38*nplus*ne*Z**2*gaunt*T**(-0.5)*EXP(-h*nu/(k*T))   ! erg /s /cm3 / Hz  NOT per steradian

   jlambda = jnu*c/(lambdavec(i)*1E-8)**2*1E-8   ! jlambda = c/lambda^2 jnu and then per 

   emissvec_integrated(i) = jlambda*(lambdavec(i)-lambdavec(i-1))


END DO



END
