SUBROUTINE calculatecubes(R_index, N_atoms, atoms, EXCMAX, nmolvec, EXC_MAT_ABIN, tau_tresh, DIMS, x, ndensvec_abin, alfa_dust, LINESTOT, COMPUTESHELL, REALRECCONT, n_co, Nlines, TAUMAT, N_abs, EMISS_MAT, molcool, lum)

USE M1, ONLY : atomm, R_index_str, Nelem, Nmol, Npis, time, myid, IDP, Nabin, lambdacut, ngvec, Nstages
USE PARAMETERS, ONLY : pi, h, c, eV
USE INTERFACE, ONLY : alfa_photoionization, co_photodiss, createtau_s, createbeta_s, createbeta_c, createbeta_l, findbin2, twophoton, hI_freebound, reclines_smooth, boltzmann, bremsstrahlung
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index, N_atoms, EXCMAX, DIMS
TYPE(atomm), DIMENSION(N_atoms) :: atoms
DOUBLE PRECISION, DIMENSION(Nmol), INTENT(IN) :: nmolvec
DOUBLE PRECISION, DIMENSION(Nabin, EXCMAX),  INTENT(IN) :: EXC_MAT_ABIN
DOUBLE PRECISION, INTENT(IN) :: tau_tresh
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION(Nabin), INTENT(INOUT) :: ndensvec_abin
DOUBLE PRECISION, INTENT(IN) :: alfa_dust, molcool
INTEGER, INTENT(IN) :: LINESTOT, Nlines
LOGICAL, INTENT(IN) :: COMPUTESHELL
LOGICAL, INTENT(IN) :: REALRECCONT
DOUBLE PRECISION, INTENT(IN):: n_co
DOUBLE PRECISION, DIMENSION(Nlines,6), INTENT(OUT) :: TAUMAT
INTEGER, INTENT(OUT) :: N_abs
DOUBLE PRECISION, DIMENSION(LINESTOT,6), INTENT(OUT) :: EMISS_MAT
DOUBLE PRECISION, INTENT(OUT) :: lum

! ========================================================
! Task :  Computes line optical depths (TAUMAT) and emissivities (EMISS_MAT) given the population solution.
! Input : R_index         Shell index
!         N_atoms         Number of atoms
!         atoms           Array of atoms (struct type atomm (module m1))
!         EXCMAX          Size of biggest atomic solution (=max(N_levels))
!         nmolvec         Array (size Nmol, set in m1 module)  with number densities of molecules [cm^-3]
!         EXC_MAT_ABIN    Population solutions in this zone, indexed with the ABIN scheme [cm^-3]
!         tau_tresh       Ignore optical depths below this threshold
!         DIMS            Dimension of x array
!         x               Array with shell parameters (see modelreader)
!         ndensvec_abin   Atom densities [cm^-3]
!         alfa_dust       Dust absorption coefficient [cm^-1]
!         LINESTOT        Total number of lines
!         COMPUTESHELL    Boolean : compute this zone?
!         REALRECCONT     Boolean : Compute real HI rec. continua? (or use alfa*(chi + 0.8kT) spread over box)
!         n_CO            Number density of CO [cm^-3]
!         Nlines          Number of lines
!         molcool         Molecular cooling [erg s^-1 cm^-3]
!Output:  TAUMAT          Matrix (Nlines*6) with line optical depths
!         N_abs           Number of absorbing lines in this zone
!         lum             Total power emitted from this zone [erg s^-1]
! Globals:Nr              Number of zones
!         Nelem           Number of elements
!         Npis            Number of excitation states ionization occurs from
!         file_penning
!         file_EXCCUBE
! ===========================================================

TYPE(atomm) :: atom
INTEGER :: i,j,n, jj,counter, cc=0,Nbins_twophoton, Nbins_freebound, N_LEVELS, N_LOWER, N_UPPER, Nsplit, Nbins_brems, ireadph, lineind
DOUBLE PRECISION :: TWOPHOTONEMISS, LINEEMISS, FREEBOUNDEMISS, emissivity, fluoremiss, xshift, BREMSEMISS, nk, tday, fir=0, fircont=0
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: population, fluxvec, wlvec, lambdavec, ratevec_integrated, emissvec_integrated, BM_sol_norm
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: TAU_S, BETA_S, BETA_C, BETA_L, EREC, LAMBDAREC
DOUBLE PRECISION :: PP, beta_eff, volemiss, T, ne, nplus, F, frac_send, frac_preion, frac_linefluor, Nphotons_preion, lineenergy, gammapenning, flow, branch1, beta_re, lambdastart, lambdaend, stepatstart, neII12mu, naI5890, siI1p64, eexc
INTEGER :: wl, shell, m, iseq, posin30, bin, q, ind, i2
INTEGER, PARAMETER :: Nlambda = 10000
INTEGER, DIMENSION(:), ALLOCATABLE :: indvec
DOUBLE PRECISION  :: totemiss, totlines, tottwo, totrec, totpreion, preheating, totbrems
DOUBLE PRECISION, DIMENSION(Nlambda) :: lambdavec_alfa, alfa_c
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: ALFA_MAT, ALFA_MAT_NORM, IONEDGES, NION, EXC_MAT_ABIN_ALLGROUND
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: ALFA_NORM_CUBE
DOUBLE PRECISION, DIMENSION(40) :: lambda_COvec, tau_COvec
DOUBLE PRECISION :: oIdoublet, feii26mu, feii7155, Halpha, ca7300, CHI_EV, LEVELG, alfa, hIextra=0
DOUBLE PRECISION, DIMENSION(100) :: bftot=0
CHARACTER :: trash
DOUBLE PRECISION, DIMENSION(Nabin) :: emisstot=0

ALLOCATE(ALFA_MAT((Nstages-1)*Nelem+Nmol+3,maxval([Npis,ngvec]))); ALFA_MAT = 0
ALLOCATE(ALFA_MAT_NORM((Nstages-1)*Nelem+Nmol+3,maxval([Npis,ngvec]))); ALFA_MAT_NORM = 0
ALLOCATE(ALFA_NORM_CUBE(Nlambda, (Nstages-1)*Nelem+Nmol+3,maxval([Npis,ngvec]))); ALFA_NORM_CUBE = 0
ALLOCATE(IONEDGES((Nstages-1)*Nelem,maxval([Npis,ngvec]))); IONEDGES=0
ALLOCATE(NION((Nstages-1)*Nelem,maxval([Npis,ngvec]))); NION=0
ALLOCATE(EXC_MAT_ABIN_ALLGROUND(Nabin,maxval([Npis,ngvec]))); EXC_MAT_ABIN_ALLGROUND = 0
! -----------------------------------------------------------------
TWOPHOTONEMISS = 0
LINEEMISS = 0
FREEBOUNDEMISS = 0
BREMSEMISS = 0
ALFA_MAT = 0
alfa_c = 0
EMISS_MAT = 0
totemiss = 0          ! Imp
totrec = 0
totlines = 0
tottwo = 0
NION = 0
preheating = 0
totpreion = 0
oIdoublet = 0.  ! Important
feii26mu = 0
feii7155 = 0
neII12mu = 0. ! Imp
naI5890 = 0.  ! Imp
siI1p64 = 0. ! Imp
Halpha = 0
ca7300 = 0
lineind = 0

Nsplit = 100  ! CARE how many bins to split free-bound continuum when using box approximation?

! ---------------------------------------------------------------------

OPEN(8,file=file_ionedges, STATUS='OLD')  ! Read ionization edges of first Npis excited states

DO i = 1, (Nstages-1)*Nelem

   READ(8,*) trash, IONEDGES(i,1:maxval([Npis,ngvec]))  ! values in �

END DO

CLOSE(8)

lambdavec_alfa = [(100. + 5000*i/DBLE(Nlambda),i=1,Nlambda)]   ! Wl vector for continuum purposes

OPEN(30,file=TRIM(file_TAU)//R_index_str(R_index), STATUS='REPLACE')  ! Write all line optical depths to this file

lineind = 0

IF (COMPUTESHELL) THEN       ! If zone is to be computed..

   counter = 1     ! Important

   T = x(DIMS)    ! Get zone temperature

   ne = x(DIMS-3)*x(DIMS-2)   ! Get zone electron density  ne=ntot*xe

   DO i = 1, Nabin

      EXC_MAT_ABIN_ALLGROUND(i,1) = sum(EXC_MAT_ABIN(i,:))

      EXC_MAT_ABIN_ALLGROUND(i,2:maxval([Npis,ngvec])) = 0

   END DO

   DO i = 1, Nlambda   ! Compute the continuum opacity for all wavelengths in the wl-vector (is needed only for the effective escape probability)

      CALL alfa_photoionization(lambdavec_alfa(i), N_atoms, atoms, REAL(EXC_MAT_ABIN_ALLGROUND), nmolvec, alfa_dust, T, ne, ndensvec_abin, alfa_c(i), ALFA_MAT)

      IF (SUM(ALFA_MAT) > 0) THEN

         ALFA_NORM_CUBE(i,:,:) = ALFA_MAT/SUM(ALFA_MAT)   ! Normalize the continuum opacities

      ELSE

         ALFA_NORM_CUBE(i,:,:) = 0.

      END IF

   END DO

   IF (n_co > 0) THEN

      tday = time/(3600.*24.)

      ireadph = 1

      CALL co_photodiss(ireadph, n_co, tday, lambda_COvec, tau_COvec)

      IF (myid == IDP) PRINT *, 'lambda_COvec', lambda_COvec

      DO i = 1, 37  ! There are 37 CO lines..

         lineind = lineind + 1

         TAUMAT(lineind,1) = 102        ! CO
         TAUMAT(lineind,2) = 0
         TAUMAT(lineind,3) = 0
         TAUMAT(lineind,4) = 0
         TAUMAT(lineind,5) = lambda_COvec(i)
         TAUMAT(lineind,6) = tau_COvec(i)

         WRITE (30,'(5I5,2ES15.5E3)') R_index, 0, 102, 0, 0, lambda_COvec(i), tau_COvec(i)

      END DO

   END IF

   DO n = 1, N_atoms  ! For all atoms..

      atom = atoms(n)

      nk = x(atom%POS)

      N_LEVELS = atom%N_LEVELS         ! Number of levels solved for

      N_LOWER = atom%N_LEVELS_ABSORB   ! Number of absorbing levels

      N_UPPER = atom%N                 ! Total number of levels

      ALLOCATE(TAU_S(N_UPPER, N_LEVELS)); TAU_S = 0         ! Important Rectangular TAU, need to high levels for opacities
      ALLOCATE(BETA_S(N_LEVELS, N_LEVELS)); BETA_S = 0       ! Quadratic BETA_S, need only for emissivities

      ALLOCATE(BETA_C(N_LEVELS, N_LEVELS)); BETA_C = 0      ! Quadratic BETA_C, need only beta_c for emissivities

      ALLOCATE(BETA_L(N_LEVELS, N_LEVELS)); BETA_L = 0

      ALLOCATE(population(N_LEVELS)); population = 0                                ! Important

      population = EXC_MAT_ABIN(atom%ABIN, 1:N_LEVELS)    ! Extract population solution

      CALL createtau_s(atom, N_UPPER, N_LEVELS, population, TAU_S)      ! Create Sobolev taus



      IF (nk > 0) THEN

         eexc=0.
         DO jj = 1, atom%N_LEVELS

           eexc = eexc + population(jj)/sum(population)*atom%LEVELS_E(jj)

         END DO

         CALL createbeta_s(atom%name, N_LEVELS, N_LEVELS, TAU_S(1:N_LEVELS,:), BETA_S)    ! Creates Sovolev escape probabilities

         CALL createbeta_c(atom, N_levels, TAU_S(1:N_LEVELS,:), T, Nlambda, lambdavec_alfa, alfa_c, BETA_C) ! Create continuum destruction probabilities



         CALL createbeta_l(atom, N_levels, TAU_S(1:N_LEVELS,:), T, BETA_L)  ! Create line destruction probabilities..

         ! Sort opacities into TAU_HYPERCUBE

         IF (N_LOWER > 0) THEN

            DO i = 1, N_LOWER    ! Only N_LOWER levels absorb..

               DO j = (i+1), N_UPPER   ! But can do so to total size of atom..

                  IF (ABS(TAU_S(j,i)) > tau_tresh) THEN   !  Ignore lines with small influence on transfer...with ABS negative optical depths are allowed for (lasering)

                     lineind = lineind + 1

                     IF (lineind > Nlines) THEN

                        print *, 'ERROR : lines > Nlines, stopping.'

                        stop 1

                     END IF

                     TAUMAT(lineind,1) = atom%ABIN
                     TAUMAT(lineind,2) = atom%SEQ
                     TAUMAT(lineind,3) = i
                     TAUMAT(lineind,4) = j
                     TAUMAT(lineind,5) = atom%LAMBDA(j,i)
                     TAUMAT(lineind,6) = TAU_S(j,i)

                     WRITE (30,'(5I5,2ES15.5E3)') R_index, atom%SEQ, atom%ABIN, i, j, atom%LAMBDA(j,i), TAU_S(j,i)

                     IF (i .EQ. atom%N_LEVELS) PRINT *, 'WARNING: Atom', atom%NAME,  'has thick lines beyond i = N_LEVELS!'
                  END IF

               END DO

            END DO

         END IF

         ! CO taus

         ! Compute emissivities

         IF (atom%SENDS) THEN

            DO j = 2, N_LEVELS   ! Levels 2 to N_LEVELS are allowed to send

               DO i = 1, (j-1)  ! transition to all levels below..

                  lineenergy = atom%LEVELS_E(j) - atom%LEVELS_E(i)  ! Line energy in ergs

                  !beta_re = (1. - BETA_S(j,i))/(1. + BETA_C(j,i) + BETA_L(j,i))

                  !PRINT *, 'beta_re', beta_re

                  beta_eff = (BETA_S(j,i) + BETA_C(j,i) + BETA_L(j,i)) / (1. + BETA_C(j,i) + BETA_L(j,i))  ! CARE has to be same as in stateq.f90

                  !frac_send = BETA_S(j,i)/beta_eff  ! Fraction of non-reabsorbed photons escaping

                  !frac_send = (BETA_S(j,i) + (1-BETA_S(j,i))*BETA_C(j,i))/beta_eff ! CARE sends off also continuum ion. photons

                  !frac_preion =  BETA_C(j,i)/beta_eff     ! Fraction of on-reabsorbed photons being continuum absorbed

                  frac_preion = 0.  ! CARE

                  frac_linefluor =  (1.-BETA_S(j,i))*BETA_L(j,i)/beta_eff   ! Ly-alpha. this energy should be bookkept elsewhere

                  frac_send = 1. - frac_preion - frac_linefluor

                  emissivity= 1/(4.*pi)*population(j)*atom%A(j,i)*beta_eff*lineenergy  ! erg s^-1 cm^-3 ster^-1 �^-1

                  volemiss = emissivity*x(DIMS-8)

                  totemiss = totemiss + 4*pi*volemiss*frac_send   ! Bookkeeping only

                  totlines = totlines + 4*pi*volemiss*frac_send    ! Bookkeeping only

                  IF (frac_preion > 0) THEN    ! Bookkeep preionization rates and heating

                     totpreion = totpreion +  4*pi*volemiss*frac_preion ! Total energy going into pre-ionizations

                     Nphotons_preion = totpreion/(h*c/(atom%LAMBDA(j,i)*1E-8))  ! number of photons

                     CALL findbin2(Nlambda-1, lambdavec_alfa(1:Nlambda-1), atom%LAMBDA(j,i), bin)

                     ALFA_MAT_NORM = ALFA_NORM_CUBE(bin,:,:)   ! Get normalized continuum opacities at this wl

                     DO m = 1, (Nstages-1)*Nelem  ! Go though all elemements..(also the ones that are not solved for)

                        DO q = 1, maxval([Npis,ngvec])  ! And all levels that ionization is permitted from..

                           NION(m, q) = NION(m, q) + Nphotons_preion*ALFA_MAT_NORM(m,q)   ! Add correct number of preionized photons to each element.

                           IF (IONEDGES(m,q) > 0) THEN    ! Compute heating by preionizations

                              preheating = preheating +  totpreion*ALFA_MAT_NORM(m,q)*(1. - atom%LAMBDA(j,i)/IONEDGES(m,q))

                           END IF

                        END DO

                     END DO

                  END IF

                  xshift = 0    ! If the photon escaped at a redshift due to high optical depths change here

                  IF ((atom%NAME .EQ. ' HI  ') .AND. (j == 3) .AND. (i .EQ. 1)) THEN  ! Ly-alpha

                     xshift = 0.005  ! TEMP redshift for Ly-alpha

                  END IF

                  IF (emissivity .GT. 0) THEN       ! CARE put better limit here

                     EMISS_MAT(counter, 1) = DBLE(atom%ABIN)   ! Element ABIN code
                     EMISS_MAT(counter, 2) = DBLE(j)           ! Upper level
                     EMISS_MAT(counter, 3) = DBLE(i)           ! Lower level
                     EMISS_MAT(counter, 4) = atom%LAMBDA(j,i)*(1+xshift)   ! Wavelength
                     EMISS_MAT(counter, 5) = 4*pi*emissivity*frac_send   ! escaping flux erg s^-1 cm^-3 �^-1
                     EMISS_MAT(counter, 6) = 0                 ! ID for line emission is 0


                     LINEEMISS = LINEEMISS + EMISS_MAT(counter, 5)   ! Bookkeeping

                     IF (atom%ABIN .EQ. 14 .AND. j .EQ. 4 .AND. (i .LE. 2)) THEN

                        oIdoublet = oIdoublet  + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 20 .AND. j .EQ. 2 .AND. i .EQ. 1) THEN

                        neII12mu = neII12mu + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 24 .AND. j .EQ. 2 .AND. i .EQ. 1) THEN

                        naI5890 = naI5890 + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 34 .AND. j .EQ. 4 .AND. i .EQ. 3) THEN

                        siI1p64 = siI1p64 + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 55 .AND. j .eq. 2 .and. i .eq. 1) THEN

                        feii26mu = feii26mu + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 55 .and. j .eq. 17 .and. i .eq. 6) THEN

                        feii7155 = feii7155 + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 55 .and. j .eq. 18 .and. i .eq. 7) THEN

                        feii7155 = feii7155 + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 1 .and. j .eq. 4 .and. i .eq. 3) THEN

                        Halpha = Halpha + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 1 .and. j .eq. 5 .and. i .eq. 2) THEN

                        Halpha = Halpha + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 1 .and. j .eq. 6 .and. i .eq. 3) THEN

                        Halpha = Halpha + 4*pi*emissivity*frac_send*x(DIMS-8)

                     ELSEIF (atom%ABIN .EQ. 50 .and. i .eq. 1 .and. (j .eq. 2 .or. j .eq. 3)) THEN

                        ca7300 = ca7300 + 4*pi*emissivity*frac_send*x(DIMS-8)


                     END IF

                     IF (atom%LAMBDA(j,i) > 50*1e4 .AND. atom%LAMBDA(j,i) < 200*1e4) THEN

                        fir = fir +  4*pi*emissivity*frac_send*x(DIMS-8)

                     END IF


                     counter = counter + 1

                     IF (counter > LINESTOT) THEN

                        PRINT *, 'ERROR in calculatecubes: more lines than storage space.', R_index; STOP 1;

                     END IF

                  END IF

               END DO

            END DO

            IF (atom%NAME .EQ. ' HI  ') THEN  ! Add HI twophoton continuum

               CALL twophoton("HI ", DBLE(1215.57), lambdacut, DBLE(0.5), Nbins_twophoton, lambdavec, ratevec_integrated)   ! Between 1215.57 and 10000 � with resolution 0.5 � at the start
               DO i = 2, Nbins_twophoton

                  emissivity = ratevec_integrated(i)*population(2)         !ratevec_integrated has 1/4pi factor

                  totemiss = totemiss + 4*pi*emissivity*x(DIMS-8)  ! Bookkeeping

                  tottwo = tottwo + 4*pi*emissivity*x(DIMS-8)   ! Bookkeeping

                  IF (emissivity > 0) THEN

                     EMISS_MAT(counter, 1) = DBLE(atom%ABIN)
                     EMISS_MAT(counter, 2) = DBLE(0)                 ! Note a line emission
                     EMISS_MAT(counter, 3) = DBLE(0)                 !  == " ==
                     EMISS_MAT(counter, 4) = lambdavec(i)
                     EMISS_MAT(counter, 5) = 4*pi*emissivity
                     EMISS_MAT(counter, 6) = DBLE(2)                  ! Label is 2 for two-photon emission

                     TWOPHOTONEMISS = TWOPHOTONEMISS + EMISS_MAT(counter, 5)  ! Bookkeeping only

                     counter = counter + 1

                  END IF

                  IF (lambdavec(i) > 50*1e4 .AND. lambdavec(i) < 200*1e4) THEN

                     fircont = fircont +  4*pi*emissivity*frac_send*x(DIMS-8)

                  END IF

               END DO

               DEALLOCATE(lambdavec)
               DEALLOCATE(ratevec_integrated)

               !CALL hI_freebound(atom, x_MAT(R_index,DIMS), Nbins_freebound, lambdavec, ratevec_integrated)

               ! CARE HI freebound continuum dones as lines now.

               !DO i = 1, Nbins_freebound

               !       emissivity = ratevec_integrated(i)*x_MAT(R_index,atom%POS+1)*x_MAT(R_index,DIMS-3)*x_MAT(R_index,DIMS-2)

               ! ratevec_integrated has 1/4pi: multiply with n+ne

               !       IF (emissivity > 0) THEN

               !               EMISS_CUBE(R_index, counter, 1) = DBLE(atom%ABIN)
               !               EMISS_CUBE(R_index, counter, 2) = 0
               !               EMISS_CUBE(R_index, counter, 3) = 0
               !               EMISS_CUBE(R_index, counter, 4) = lambdavec(i)
               !		EMISS_CUBE(R_index, counter, 5) = 4*pi*emissivity
               !                EMISS_CUBE(R_index, counter, 6) = 1

               !                FREEBOUNDEMISS = FREEBOUNDEMISS + EMISS_CUBE(R_index, counter, 5)

               !                counter = counter + 1

               !        END IF

               !END DO

               !lambdavec => null()
               !ratevec_integrated => null()

               !nplus = x_MAT(R_index, atom%POS+1)

               !ALLOCATE(LAMBDAREC(atom%N_LEVELS,Nsplit)); lambdarec = 0
               !ALLOCATE(EREC(atom%N_LEVELS,Nsplit)); Erec = 0

               !CALL reclines(atom, T, ne, nplus, atom%N_LEVELS, lambdarec, Erec)

               !CALL reclines_smooth(atom, T, ne, nplus, atom%N_LEVELS, Nsplit, LAMBDAREC, EREC)

               !totemiss(R_index) = totemiss(R_index) + SUM(EREC)*x_MAT(R_index,DIMS-8)

               !totrec(R_index) = totrec(R_index) + SUM(EREC)*x_MAT(R_index,DIMS-8)

               !DO i = 1, atom%N_LEVELS

               !   DO j = 1, Nsplit

               !      IF (EREC(i,j) > 0) THEN

               !         EMISS_CUBE(R_index, counter, 1) = DBLE(atom%ABIN)
               !         EMISS_CUBE(R_index, counter, 2) = 0
               !         EMISS_CUBE(R_index, counter, 3) = 0
               !         EMISS_CUBE(R_index, counter, 4) = LAMBDAREC(i,j)
               !         EMISS_CUBE(R_index, counter, 5) = EREC(i,j)
               !         EMISS_CUBE(R_index, counter, 6) = 4                ! Label 4 for recombination continua

               !         counter = counter + 1
               !
               !      END IF

               !   END DO

               !END DO

               !DEALLOCATE(LAMBDAREC)
               !DEALLOCATE(EREC)

            ELSE IF (atom%NAME .EQ. 'HeI  ') THEN    ! HeI 2-photon emission

               CALL twophoton("HeI", DBLE(601.40), lambdacut, DBLE(0.5), Nbins_twophoton, lambdavec, ratevec_integrated)

               DO i = 1, Nbins_twophoton

                  emissivity = ratevec_integrated(i)*population(3)     ! CARE HeI 2 1S level

                  totemiss = totemiss + 4*pi*emissivity*x(DIMS-8)   ! Bookkeeping

                  tottwo = tottwo + 4*pi*emissivity*x(DIMS-8)    ! Bookkeeping

                  IF (emissivity > 0) THEN

                     EMISS_MAT(counter, 1) = DBLE(atom%ABIN)
                     EMISS_MAT(counter, 2) = DBLE(0)
                     EMISS_MAT(counter, 3) = DBLE(0)
                     EMISS_MAT(counter, 4) = lambdavec(i)
                     EMISS_MAT(counter, 5) = 4*pi*emissivity
                     EMISS_MAT(counter, 6) = DBLE(2)             ! Label is 2 for 2-photon emission

                     !TWOPHOTONEMISS = TWOPHOTONEMISS + EMISS_MAT(counter, 5)   ! Bookkeeping

                     counter = counter + 1

                  END IF

                  IF (lambdavec(i) > 50*1e4 .AND. lambdavec(i) < 200*1e4) THEN

                     fircont = fircont +  4*pi*emissivity*frac_send*x(DIMS-8)

                  END IF

               END DO

               gammapenning = 7.5E-10*SQRT(T/300.)*population(2)    ! Penning ionization rate Gamma*n(HeI,23s)  s^-1

               OPEN(75,file=TRIM(file_penning)//R_index_str(R_index), STATUS='REPLACE')

               WRITE(75,*) R_index, gammapenning      ! Add rate for each shell

               close(75)

            END IF

            ! RECOMBINATION CONTINUUM ------------------------------------------------

            IF (REALRECCONT .AND. atom%NAME .EQ. ' HI  ') THEN

               !CALL reccontinuum(atom, T, ne, nplus, LAMBDAREC, EREC)

               CALL hI_freebound(atom, T, Nbins_freebound, lambdavec, ratevec_integrated)  ! ratevecintegraded should be per steradian

               nplus =  x(atom%POS+1)  ! Number density of the ion..

               ne = x(DIMS-3)*x(DIMS-2)

               DO i = 1, Nbins_freebound

                  emissivity = ratevec_integrated(i)*nplus*ne

                  totemiss = totemiss + 4*pi*emissivity*x(DIMS-8)

                  ! ratevec_integrated has 1/4pi: multiply with n+ne

                  IF (emissivity > 0) THEN

                     EMISS_MAT(counter, 1) = DBLE(atom%ABIN)
                     EMISS_MAT(counter, 2) = DBLE(0)
                     EMISS_MAT(counter, 3) = DBLE(0)
                     EMISS_MAT(counter, 4) = lambdavec(i)
                     EMISS_MAT(counter, 5) = 4*pi*emissivity
                     EMISS_MAT(counter, 6) = DBLE(4)

                     FREEBOUNDEMISS = FREEBOUNDEMISS + EMISS_MAT(counter, 5)

                     counter = counter + 1

                  END IF

                  IF (lambdavec(i) > 50*1e4 .AND. lambdavec(i) < 200*1e4) THEN

                     fircont = fircont +  4*pi*emissivity*frac_send*x(DIMS-8)

                  END IF

               END DO

               DEALLOCATE(lambdavec)
               DEALLOCATE(ratevec_integrated)

               !DO i = atom%N_LEVELS+1, atom%N   ! rec to levels above N_LEVELS are put directlt into N_LEVELS and the flux between is missing

               !   CHI_EV = 13.595 - atom%LEVELS_E(i)/eV

               !   LEVELG = DBLE(atom%LEVELS_G(i))

               !   call HI_RECOMBSPECTRUM_RECRATES(i, CHI_EV, LEVELG, T, alfa)

               !   hIextra = hIextra + (atom%LEVELS_E(i)-atom%LEVELS_E(atom%N_LEVELS))*alfa*ne*nplus*x(DIMS-8)

               !END DO  !

               hIextra = 0


            ELSE  ! For all other atoms OLD rec continua

               nplus = x(atom%POS+1)  ! Number density of the ion..

               ALLOCATE(LAMBDAREC(atom%N_LEVELS,Nsplit)); lambdarec = 0

               ALLOCATE(EREC(atom%N_LEVELS,Nsplit)); Erec = 0

               !CALL reclines(atom, T, ne, nplus, atom%N_LEVELS, lambdarec, Erec)

               CALL reclines_smooth(atom, T, ne, nplus, eexc, atom%N_LEVELS, Nsplit, LAMBDAREC, EREC)

               IF (atom%NAME == ' HI  ') THEN  ! may want to normalize

                  CALL hI_freebound(atom, T, Nbins_freebound, lambdavec, ratevec_integrated)  !

                  DEALLOCATE(lambdavec)
                  DEALLOCATE(ratevec_integrated)

                  DO i = 1, 20

                     DO j = 1, i

                        cc = cc + 1

                        bftot(i) = bftot(i) + sum(EREC(cc,:))/(ne*nplus)

                     END DO

                  END DO

               END IF

               totemiss = totemiss + SUM(EREC)*x(DIMS-8)   ! Bookkeeping

               totrec = totrec + SUM(EREC)*x(DIMS-8)    ! Bookkeeping

               DO i = 1, atom%N_LEVELS          ! Store fluxes in recombination continua

                  DO j = 1, Nsplit   ! Split continuum into Nsplit lines

                     IF (EREC(i,j) > 0) THEN

                        EMISS_MAT(counter, 1) = DBLE(atom%ABIN)
                        EMISS_MAT(counter, 2) = DBLE(0)
                        EMISS_MAT(counter, 3) = DBLE(0)
                        EMISS_MAT(counter, 4) = LAMBDAREC(i,j)
                        EMISS_MAT(counter, 5) = EREC(i,j)        ! EREC is already integrated over 4pi
                        EMISS_MAT(counter, 6) = DBLE(4)           ! Code is 4 for recombination continuum

                        IF (atom%NAME .EQ. ' HI  ') FREEBOUNDEMISS = FREEBOUNDEMISS + EMISS_MAT(counter, 5)

                        counter = counter + 1

                     END IF

                     IF (lambdarec(i,j) > 50*1e4 .AND. lambdarec(i,j) < 200*1e4) THEN

                        fircont = fircont +  4*pi*emissivity*frac_send*x(DIMS-8)  ! Bookkeeping

                     END IF

                  END DO

               END DO

               DEALLOCATE(lambdarec)
               DEALLOCATE(Erec)

            END IF

            ! THERMAL BREMSSTRAHLUNG  ----------------------------------------------------

            lambdastart = 1000.  ! CARE  There is very little flux at short wl..at a few 1000 K the peak is at ~5 microns

            lambdaend = 1E6   !  ! CARE

            stepatstart = 10.  ! CARE

            CALL bremsstrahlung(atom%CHARGE+1, nplus, ne, T, lambdastart, lambdaend, stepatstart, Nbins_brems, lambdavec, emissvec_integrated)

            totemiss = totemiss + SUM(emissvec_integrated)*x(DIMS-8)

            DO i = 1, Nbins_brems

               IF (emissvec_integrated(i) > 0) THEN

                  EMISS_MAT(counter, 1) = DBLE(atom%ABIN)
                  EMISS_MAT(counter, 2) = DBLE(0)
                  EMISS_MAT(counter, 3) = DBLE(0)
                  EMISS_MAT(counter, 4) = lambdavec(i)
                  EMISS_MAT(counter, 5) = emissvec_integrated(i) ! CARE should NOT be times 4pi here
                  EMISS_MAT(counter, 6) = DBLE(5)             ! Label is 5 for bremsstrahlung

                  BREMSEMISS = BREMSEMISS + EMISS_MAT(counter, 5)   ! Bookkeeping

                  counter = counter + 1

                  IF (lambdavec(i) > 50*1e4 .AND. lambdavec(i) < 200*1e4) THEN

                     fircont = fircont +  4*pi*emissivity*frac_send*x(DIMS-8)

                  END IF

               END IF

            END DO

            DEALLOCATE(lambdavec)
            DEALLOCATE(emissvec_integrated)

         END IF

      END IF

      DEALLOCATE(population)
      DEALLOCATE(TAU_S)
      DEALLOCATE(BETA_S)
      DEALLOCATE(BETA_C)
      DEALLOCATE(BETA_L)

   END DO

END IF


CLOSE(75)

CLOSE(30)

totemiss = totemiss + molcool

lum = totemiss

OPEN(10, file=TRIM(file_EXCCUBE)//R_index_str(R_index), STATUS='REPLACE')

! write populations (needed for the ionization routine)

T = x(DIMS)

DO j = 1, N_atoms  ! For all elements..

   ALLOCATE(BM_sol_norm(atoms(j)%N_levels))

   CALL boltzmann(T, atoms(j), atoms(j)%N_levels, BM_sol_norm)  ! Get LTE popuulations to write NLTE/LTE ratios

   DO m = 1, atoms(j)%N_LEVELS   ! CARE changes Npis to EXCMAX chaged EXCMAX to atoms(j)%N_LEVELS

      IF (EXC_MAT_ABIN(atoms(j)%ABIN,m) .NE. 0) THEN    ! Have passed atom size, skip to outer loop

         WRITE(10,'(4I6,2ES15.5E3)') R_index, j, atoms(j)%ABIN, m, EXC_MAT_ABIN(atoms(j)%ABIN,m), EXC_MAT_ABIN(atoms(j)%ABIN,m)/(ndensvec_abin(atoms(j)%ABIN)*BM_sol_norm(m)+1D-50) ! TEMP

      END IF

   END DO

   DEALLOCATE(BM_sol_norm)

END DO

close(10)

OPEN(10,file=TRIM(file_Eemit)//R_index_str(R_index),STATUS='REPLACE')

   WRITE(10,'(ES10.2)') totemiss

close(10)


OPEN(50,file=TRIM(file_totpreion)//R_index_str(R_index),STATUS='REPLACE')

  WRITE(50,'(ES12.2)') totpreion

close(50)

OPEN(50,file=TRIM(file_preheating)//R_index_str(R_index),STATUS='REPLACE')

  WRITE(50,'(1000ES12.2)') preheating

close(50)

OPEN(10,file=TRIM(file_nabspreion)//R_index_str(R_index),STATUS='REPLACE')   ! Used by ionization solver

DO i = 1, (Nstages-1)*Nelem

   DO j = 1, maxval([Npis,ngvec])

      IF (NION(i,j) > 0) WRITE(10,*) R_index, i, j, NION(i,j)

   END DO

END DO

close(10)

OPEN(10,file=TRIM(file_nlines)//R_index_str(R_index),STATUS='REPLACE')

   WRITE(10,*) lineind  ! Write number of absorbing lines for each zone

CLOSE(10)



N_abs = lineind

DO i = 1, counter-1

   emisstot(int(EMISS_MAT(INT(i),1))) = emisstot(int(EMISS_MAT(INT(i),1))) + EMISS_MAT(i,5)*x(DIMS-8)

END DO

OPEN(30,file=TRIM(file_emisstot)//R_index_str(R_index), STATUS='REPLACE')

DO i = 1, Nabin

   write(30,*) i, emisstot(i), emisstot(i)/sum(emisstot)

END DO

close(30)

open(30,file=TRIM(file_hIextra)//R_index_str(R_index), STATUS='REPLACE')

  write(30,*) hIextra

close(30)

!print *, 'fir', R_index, fir+fircont, fir/(fir+fircont)



END
