SUBROUTINE CEXH(NLOWER, NUPPER, T, nmax, Evec, CE)
USE PARAMETERS, ONLY : k, pi
IMPLICIT NONE

!     COLLISIONAL EXCITATION RATES FROM JOHNSON 1972 (AP J 174:227)
! INPUT 
! nlower       Principal quantum number of lower state
! nupper       Principal quantum number of upper state
! T            Temperature
! Evec         ?
! OUTPUT :::::::::
! CE           Collision rate

INTEGER, INTENT(IN) :: NLOWER, NUPPER, nmax
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(nmax), INTENT(IN) :: Evec
DOUBLE PRECISION, INTENT(OUT) :: CE

DOUBLE PRECISION :: CON, QP, Q, EIJ, X, YN, G0,G1,G2,BN,RN,GN, ANN,FNN,BNN,RNN,ZN,A1,A2
DOUBLE PRECISION, EXTERNAL :: EXP1, E2

CON = 1.092E-10   ! The combination (8k/(pi me))^0.5*2pi a0^2  in eq 36

QP = DBLE(NUPPER)

Q = DBLE(NLOWER)

EIJ = ABS(Evec(NLOWER)-Evec(NUPPER))

X = 1.-(Q/QP)**2        ! eq 2

YN = EIJ/(k*T)     ! eq 37

IF (NLOWER .EQ. 1) THEN   ! n=1

   G0 = 1.133       ! Table 1
   G1 = -.4059
   G2 = .07014
   BN = -.603         ! eq 25
   RN = .45           ! eq 31

ELSE IF (NLOWER .EQ. 2) THEN

   G0 = 1.0785       ! table 1
   G1 = -.2319
   G2 = .02947
   BN = (4.-18.63/Q+36.24/Q**2-28.09/Q**3)/Q   ! eq 26
   RN = 1.94/Q**1.57                           ! eq 32
      
ELSE

   G0 = .9935+.2328/Q-.1296/Q**2          ! table 1
   G1 = -(.6282-.5598/Q+.5299/Q**2)/Q
   G2 = (.3887-1.181/Q+1.470/Q**2)/Q**2
   BN = (4.-18.63/Q+36.24/Q**2-28.09/Q**3)/Q   ! eq 26
   RN = 1.94/Q**1.57                           ! eq 32

END IF
 
GN = G0 + G1/X + G2/X**2     ! eq 4

ANN = 64.*Q**3*GN/(3.*SQRT(3.)*PI*X**4*QP**3)   ! eq 11 with FNN from eq 3 inserted

FNN = ANN*X/(2.*Q**2)  ! eq 3   ok, given definition of Ann

BNN = 4.*Q**4*(1. + 4./(3.*X) + BN/X**2)/(QP**3*X**2)   ! eq 23 

RNN = X*RN    !   eq 30

ZN = RNN + YN   ! eq 41

A1 = (1./YN+.5)*EXP1(YN) - (1./ZN + .5)*EXP1(ZN)   ! The first [] bracket in eq 36

A2 = E2(YN)/YN - E2(ZN)/ZN    ! The second [] bracket in eq 36

CE = CON*SQRT(T)*Q**2*YN**2*(ANN*A1 + (BNN-ANN*LOG(2.*Q**2/X))*A2)/X     ! eq 36
    

END SUBROUTINE CEXH
