SUBROUTINE CHARGETRANSFER(T, nlev, ELEVELS, CREATE_SMALL, DESTROY_SMALL, CTH, CTABIN, CTLEVEL, XTOEXC)
use M1, ONLY : Nabin, Nelem, Nmol, map2, myid, map3, DOPRINT, massvec, ievec, ngvec, minshellvec, maxshellvec, map_abin2elem2, map_abin2elem3, gval, Nstages
USE INTERFACE, ONLY : findbin2, molecules, paguesses, findresonance
use PARAMETERS, ONLY : eV, k
IMPLICIT NONE

DOUBLE PRECISION, INTENT(IN) :: T  ! Temperature [K]
INTEGER, DIMENSION(Nabin) :: nlev  ! Number of atoms
DOUBLE PRECISION, DIMENSION(Nabin,1000) :: ELEVELS  ! Matrix of energy levels
!LOGICAL, INTENT(IN) :: MOLON, CTON
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(OUT) :: CREATE_SMALL  ! Creation rates i + j --> k (cm^3 s^-1)
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(OUT) :: DESTROY_SMALL, CTH  ! Destruction rates
INTEGER, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(OUT) :: CTABIN, CTLEVEL
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(OUT) :: XTOEXC

! --------------------------------
! TASK :   Stores all charge transfer and molecular reation/destruction rates in creation and destruction matrices
! INPUT :  T              : Temperature [K]
!          nlev           : 1D Array with number of excitation energies (ELEVELS) for each ABIN element
!          ELEVELS        : 2D array with the energies of the excited states [eV]
!          MOLON          : Boolean switch to tell if molecules should be allowed to form
!          CTON           : Boolean switch to tell if charge transfer rates should be included
! OUTPUT : CREATE_SMALL   : 3D array with creation rates [A,B,C] of species A by reaction of B with C ( B + C --> A). [cm^3 s^{-1}]
!          DESTROY_SMALL  : 2D array with destruction rates [A,B] of species A by collision with species B (A + B --> ..) [cm^3 s^{-1}]
!          CTH            : 2D array with heating contribution [A,B] from reactions between A and B  ( A + B --> ? + heat) [erg/reaction]
! GLOBALS: Nmol           : Number of molecules to include, set in m1
! -----------------------------------

! SECURE CODES
! 0     Guess
! 1     Calculated
! 2     Very probable high rate (still guess)
! ----------------------------------

!INTEGER, DIMENSION(Nabin+Nmol,Nabin+Nmol) :: CTABINBIG, CTLEVELBIG
!DOUBLE PRECISION, DIMENSION(Nabin+Nmol,Nabin+Nmol) :: XTOEXCBIG

DOUBLE PRECISION :: ct, t4, CO_radass, tmin, tmax
!INTEGER, DIMENSION(2*Nelem+Nmol) :: idvec2
!INTEGER, DIMENSION(3*Nelem+Nmol) :: idvec3

INTEGER :: ind, m, n, i, j, bin, irec, iion, itoion, itorec, PAlim, ctabin_best, ctlevel_best, ionrev, recrev, R1, R2, P1, P2
!INTEGER, PARAMETER :: Ntoion=40, Ntorec=40
INTEGER, DIMENSION(4) :: avec_in
character*5 :: FS 
DOUBLE PRECISION, DIMENSION(4) :: funcVar
DOUBLE PRECISION, DIMENSION(1000) :: securevec

!DOUBLE PRECISION :: gHI, gHII, gCI, gCII, gOI, gOII, gMgI, gMgII, gS1, gS2, gCaI, gCaII, gCaIII, gCoI, gCoII, gAlI, gAlII, gNaI, gNaII, gSi1, gSi2, gFeI, gFeII, GNiI, gNiII, gCrI, gCrII, gMnI, gMnII, gVI, gVII, gTiI, gTiII, gTiIII, gScI, gScII, gKI, gKII, ED, smallestED_1, bestED_2, E0, ED_ground, x_ground, ctrev, sc, temp_ct
DOUBLE PRECISION :: smallestED_1, bestED_2, ED_ground, ED, x_ground, ctrev, sc, temp_ct

!DOUBLE PRECISION, DIMENSION(Nabin) :: ie
!INTEGER, DIMENSION(Nabin) :: gval
!INTEGER, DIMENSION(Nabin+Nmol,Nabin+Nmol) :: SECURE
CHARACTER :: trash
!INTEGER, DIMENSION(Ntoion) :: abintoion
!INTEGER, DIMENSION(Ntorec) :: abintorec
LOGICAL :: PA86, ALLOWTWOELECTRON
DOUBLE PRECISION, PARAMETER :: x_groundmin=1.0 !  TEMP IF there is an excited state with small energy defect, use this fraction of the total rate to it

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
!temp stuff

CHARACTER (LEN=100):: fname
CHARACTER (LEN=3) :: i_str

INTEGER, DIMENSION(Nabin+Nmol,Nabin+Nmol) :: CTABINBIG, CTLEVELBIG
DOUBLE PRECISION, DIMENSION(Nabin+Nmol,Nabin+Nmol) :: XTOEXCBIG

DOUBLE PRECISION, DIMENSION(Nabin+Nmol,Nabin+Nmol) :: DESTROY, DE
DOUBLE PRECISION, DIMENSION(Nabin+Nmol,Nabin+Nmol,Nabin+Nmol) :: CREATE

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------



! PARAMETERS ------------

PA86 = .FALSE.  ! should the Pequignot Alvodrandi 1986 thumb rules be implemented for reactions without known rates?

! ---------------------------------------------------------

! Initiating variables

t4 = T/1.e4

CREATE_SMALL = 0  ! Important
DESTROY_SMALL = 0  ! Important
CTH = 0
CTLEVEL = 0

i = 0

! CHARGE TRANSFER :::::::::::::::::::::::::::::::::::
! :::::::::::::::::::::::::::::::::::::::::::::::::::

!opens the input file containing the charge transfer rates
open(12, file="DATA/chemicalreactions/chemrec_input.csv")

!the number of rows and columns in the input file
read(12, *) n, m

!goes through the input files, that contains n number of lines
i = 0
do j=1,n,1
!     reads in the values for the abin numbers (avec_in), the function switch, the function variables, and securevec
	read(12, *) avec_in, FS, funcVar, tmin, tmax, sc
    
!     check if the temperature is within the temperature limits for the reactions
    if (T .GE. tmin .AND. T .LE. tmax) then
        
        i = i + 1
        
!         check if the FK96 function form is used
        if (FS .EQ. "F") then
            ct = funcVar(1) * (t4)**funcVar(2) * (1 + funcVar(3) * EXP(funcVar(4) * t4))
!             since these values are extrapolated, this checks so that no rate is below zero
            if (ct < 0) then
                ct = 0.
                end if
                
!         else use the constant rate
        else
            ct = funcVar(1)
        end if
        
!         fills the arrays with charge transfer rates and securevec, for avec_in reactions
!        ctvec(i) = ct
        securevec(i) = sc

        DESTROY_SMALL(map_abin2elem2(avec_in(1)),map_abin2elem3(avec_in(2))) = ct
        
        if (map_abin2elem2(avec_in(2)) .NE. 0) then
            DESTROY_SMALL(map_abin2elem2(avec_in(2)),map_abin2elem3(avec_in(1))) = ct
        end if
        
        if (avec_in(3) .eq. avec_in(4)) then
            temp_ct = 2*ct
        else
            temp_ct = ct
        end if

        if (map_abin2elem2(avec_in(3)) .ne. 0) then
        CREATE_SMALL(map_abin2elem2(avec_in(3)), map_abin2elem3(avec_in(1)), map_abin2elem3(avec_in(2))) = temp_ct
        end if        

        if (map_abin2elem2(avec_in(4)) .NE. 0) then
            CREATE_SMALL(map_abin2elem2(avec_in(4)), map_abin2elem3(avec_in(1)), map_abin2elem3(avec_in(2))) = temp_ct
        end if

    end if
end do


close(12)


! AUTOMATIC THUMB RULES IMPLEMENTATION -----------------
! ------------------------------------------------------
! FIXME what is this?
!E0 = 4   ! Best energy defect for I + III reactions...1.8 - 5.5 eV according to Steigman

! ::::: MOLECULES ::::::::::::::::::::::::::::::::

! --Process transitions to identify possible resonances to exc. states, possibly add a PA86 guess, compute heating per transition... -------------------------------

print*, '------'

!IF (CTON .EQV. .False.) then   ! If CT-rates should not be on, replace everything by zero 

!   DESTROY_SMALL = 0

!   CREATE_SMALL = 0

!   CTH = 0

!ELSE

! The purpose of the following section is to find resonance levels
! and/or do rate estimates with paguesses()
DO i = 1, (Nstages-1)*Nelem  ! For each element to ionize..
    !    skips HII and HeII (2 and 4) becuse they cant be ionized
    IF(i .EQ. 2 .OR. i .EQ. 4) CYCLE ! TEMPFIX

    DO j = 1, Nstages*Nelem  ! For each element to recombine
        
        !      every single and dobule ionized elements
        !      XI XII XIII YI YII YIII ZI ...
        !      1   2   3   4   5   6   7
        !      I   x   x   I   x   x   I
        !      mod(j-1, 3) = 0. This loop will skip all neutral elements. 

        !      Special rules for 1, 3 (double ionized H) 
        !      and 6 (double ionized He)

        IF (j .EQ. 1 .OR. j .EQ. 6 .OR. j .EQ. 3 .OR. mod(j-1, 3) .EQ. 0 ) CYCLE ! TEMPFIX

        !      gets the abin codes for element i and element j
        itoion=map2(i)
        itorec=map3(j)


        IF (itoion .EQ. 1) THEN  ! If the element to be ionized is HI...
            iion = 70  ! .. HII is at abin=70 (special location)
        ELSE
            iion = itoion + 1   ! ..otherwise the ion is at location itoion + 1
        END IF

        IF (itorec .EQ. 70) THEN   ! if the element to recombine is HII..
            irec = 1           ! the recombined element is 1 (HI)
        ELSE  ! ..otherwise its one step down the abin list
            irec = itorec - 1
        END IF
        
        ED = ievec(map_abin2elem2(irec)) - ievec(map_abin2elem2(itoion))   ! The energy defect for the reaction to the ground states  (*ie* is in eV)
        ED_ground = ED

        ! print *, 'ddd', ED, ievec(map_abin2elem2(itoion)), ievec(map_abin2elem2(irec))

        IF (ED <= 0.) CYCLE     ! endothermic reaction, skip..(they are handled as inverse of exothermic reactions further down)
        
        
        call findresonance(iion, irec, ED, nlev(iion), nlev(irec), ELEVELS(iion,1:nlev(iion)), ELEVELS(irec,1:nlev(irec)), ctabin_best, ctlevel_best, smallestED_1) ! Find which is the best resonance transition

        IF (ctabin_best > 0) THEN ! Resonance found
            x_ground = x_groundmin
        ELSE  ! No resonances found, all to GS
            x_ground = 1.0
        END IF

        ct = DESTROY_SMALL(i,j)

        ! print *, 'yyy', i,j,DESTROY_SMALL(i,j)

        IF (PA86 .AND. ct == 0) THEN    ! If there is no rate (otherwise dont implement these guessing rules), and the thumb rules of PA86 should be implemented

            call paguesses(itoion, itorec, smallestED_1, ct) ! Guess rate with PA86 rules

            DESTROY_SMALL(i,j) = ct

            if (map_abin2elem2(itorec) .NE. 0) then
                DESTROY_SMALL(map_abin2elem2(itorec),map_abin2elem3(itoion)) = ct
            end if

            IF (iion == irec) THEN   ! ex FeI + FeIII ---> FeII + FeII
                temp_ct = 2*ct
            ELSE
                temp_ct = ct
            END IF

            if (map_abin2elem2(iion) .NE. 0) then
                CREATE_SMALL(map_abin2elem2(iion),map_abin2elem3(itoion),map_abin2elem3(itorec)) = ct
            end if

            CREATE_SMALL(map_abin2elem2(irec),map_abin2elem3(itoion),map_abin2elem3(itorec)) = ct

        END IF

        CTH(i,j) = (x_ground*ED_ground + (1.-x_ground)*smallestED_1)*eV  ! this energy in erg If best resonance in ground state, this equals 1*ED_ground*eV

        ! print *, 'bbb', CTH(i,j), ED_ground

        ! NLTE population rates to exc state

        CTABIN(i,j) = ctabin_best  ! Abin code of element that is left in an excited state

        CTLEVEL(i,j) = ctlevel_best  ! The excited level number

        XTOEXC(i,j) = 1.0 - x_ground  ! Fraction going to exc state

        ! REVERSE RATES (apply detailed balance): ===================

        ctrev = ct*gval(itoion)*gval(itorec)/(gval(iion)*gval(irec))*exp(-ED_ground*eV/(k*T))*x_ground  ! Only reactions from ground state for reverse

        ! print *, 'ctrev', ctrev, ED_ground*eV, k*T, T

        if (map_abin2elem2(iion) .NE. 0) then
            DESTROY_SMALL(map_abin2elem2(iion),map_abin2elem3(irec)) = ctrev  ! Nabin, Nabin
        end if

        DESTROY_SMALL(map_abin2elem2(irec),map_abin2elem3(iion)) = ctrev 

        IF (itoion == itorec) THEN
            temp_ct = 2*ctrev
        ELSE
            temp_ct = ctrev
        END IF

        CREATE_SMALL(i,map_abin2elem3(irec),map_abin2elem3(iion)) = temp_ct  ! shouyld be order avec(3), avec(4) in last two slots
        ! note i is the same as map_abin2elem2(itorec)

        if (map_abin2elem2(itorec) .NE. 0) then
            CREATE_SMALL(map_abin2elem2(itorec),map_abin2elem3(irec),map_abin2elem3(iion)) = ctrev
        end if

        CTH(map_abin2elem2(irec),map_abin2elem3(iion)) = -ED_ground*eV ! Cooling per reaction, endothermic direction 


    END DO

END DO


!  DO i = 1, 2*Nelem+Nmol    ! Only need destruction and creation terms for neutrals and singly ionized, the doubly ionized eq is replaced by the number cons. equation

!      DO j = 1, 3*Nelem+Nmol   ! But need triply ionized for creation and destruction of neutrals and singly ionized..

!         IF ((j .NE. 5) .AND. (j .NE. 3*Nelem)) THEN  ! Avoid (non-existing) HeIII (j=5) and electrons (j=3*Nelem)
!         
!            DESTROY_SMALL(i,j) = DESTROY(idvec2(i),idvec3(j))   ! Destruction of i by j. Dont need to put any DESTROY_SMALL(j,i), the loop goes through all
!            
!            CTH(i,j) = DE(idvec2(i),idvec3(j))

!            CTABIN(i,j) = CTABINBIG(idvec2(i),idvec3(j)) 

!            CTLEVEL(i,j) = CTLEVELBIG(idvec2(i),idvec3(j))

!            XTOEXC(i,j) = XTOEXCBIG(idvec2(i),idvec3(j))

!            DO m = 1, 3*Nelem+Nmol

!               IF ((m .NE. 5) .AND. (m .NE. 3*Nelem)) THEN    ! Avoid (non-existing) HeIII (j=5) and electrons (j=3*Nelem)

!                  
!                  CREATE_SMALL(i,j,m) = CREATE(idvec2(i),idvec3(j),idvec3(m))  ! Creation of i by j and m

!               END IF
!               
!            END DO
!       
!         END IF
!      
!      END DO

!   END DO

!END IF


!CONTAINS

!  SUBROUTINE GETABINS(i,j,itoion,itorec,iion,irec)

!    INTEGER :: i,j,itoion,itorec,iion,irec

!    itoion = abintoion(i)   ! get abin value of ionizing element

!    itorec = abintorec(j)   ! abin value of recombining element

!    IF (itoion .EQ. 1) THEN  ! If the element to be ionized is HI...

!       iion = 70  ! .. HII is at abin=70 (special location)

!    ELSE

!       iion = itoion + 1   ! ..otherwise the ion is at location itoion + 1

!    END IF

!    IF (itorec .EQ. 70) THEN   ! if the element to recombine is HII..

!       irec = 1           ! the recombined element is 1 (HI)

!    ELSE  ! ..otherwise its one step down the abin list

!       irec = itorec - 1

!    END IF

!  END SUBROUTINE GETABINS
print*, ctlevel(1,1)
CREATE = 0
DESTROY = 0
DE = 0
CTABINBIG = 0
CTLEVELBIG = 0

!!DESTROY!
!DO i = 1, 2*Nelem
!    DO j = 1, 3*Nelem
!        DESTROY(map2(i), map3(j)) = DESTROY_SMALL(i, j) 
!    END DO
!END DO

!call printMatrix('temp/res_chemrec/' // 'destroy_new.dat', DESTROY,Nabin+Nmol,Nabin+Nmol)


!!CREATE!
!DO i = 1, 2*Nelem
!    DO j = 1, 3*Nelem
!        DO m = 1, 3* Nelem
!            ct = CREATE_SMALL(i, j, m)
!            CREATE(map2(i), map3(j), map3(m)) = ct
!            

!!            call printMatrix('test_new.dat', CREATE(i, :, :),Nabin+Nmol,Nabin+Nmol)
!        END DO
!    END DO
!    write(i_str, '(i3)') i+100
!    fname = trim(i_str) // '_new.dat'
!    fname = trim(fname)
!    call printMatrix('temp/res_chemrec/' // fname, CREATE(i, :, :),Nabin+Nmol,Nabin+Nmol)
!END DO


!!CTH!
!DO i = 1, 2*Nelem
!    DO j = 1, 3*Nelem
!        DE(map2(i), map3(j)) = CTH(i, j) 
!    END DO
!END DO

!call printMatrix('temp/res_chemrec/cth_new.dat', DE,Nabin+Nmol,Nabin+Nmol)

!!CTABIN!
!DO i = 1, 2*Nelem
!    DO j = 1, 3*Nelem
!        CTABINBIG(map2(i), map3(j)) = CTABIN(i, j) 
!    END DO
!END DO

!call printMatrix('temp/res_chemrec/ctabin_new.dat', CTABINBIG,Nabin+Nmol,Nabin+Nmol)

!!CTLEVEL!
!DO i = 1, 2*Nelem
!    DO j = 1, 3*Nelem
!!        if (CTLEVEL(i, j ) .ne. 0) then
!!            print*, i, j, CTLEVEL(i, j)
!!        end if
!        CTLEVELBIG(map2(i), map3(j)) = CTLEVEL(i, j)
!!        print*, i, j, map2(i), map3(j), CTLEVEL(i, j)
!    END DO
!END DO

!call printMatrix('temp/res_chemrec/ctlevel_new.dat', CTLEVELBIG,Nabin,Nabin)

!!XTOEXCBIG
!DO i = 1, 2*Nelem
!    DO j = 1, 3*Nelem

!        XTOEXCBIG(map2(i), map3(j)) = XTOEXC(i, j) 
!    END DO
!END DO

!call printMatrix('temp/res_chemrec/xtoexc_new.dat', XTOEXCBIG,Nabin,Nabin)



CONTAINS

subroutine printMatrix(fname, array, n, m)
    implicit none
    character(len=35) :: fname
    double precision, intent(in) :: array(n,m)
    integer, intent(in) :: n,m
    integer :: i
    
    OPEN( 10, FILE=fname)

    do i = 1,n
        WRITE ( 10, *) array(i,:)
    end do

    close(10)

end subroutine printMatrix



END SUBROUTINE CHARGETRANSFER


