SUBROUTINE CHARGETRANSFERHEATING(nnuclei, x, DESTROY, CTH, ctheating)
USE M1, ONLY : Nelem, Nmol, myid, Nstages
IMPLICIT NONE

DOUBLE PRECISION, DIMENSION(Nelem), INTENT(IN) :: nnuclei
DOUBLE PRECISION, DIMENSION(Nstages*Nelem+Nmol), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: DESTROY, CTH
DOUBLE PRECISION, INTENT(OUT) :: ctheating

DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol) :: ndouble
DOUBLE PRECISION, DIMENSION(Nstages*Nelem+Nmol) :: ntriple
INTEGER :: i, j, m

! --------------------------------------------------
! TASK :    Compute the heating due to charge transfer reactions
! INPUT :   nnuclei        Number density of elements (all ions) [cm^-3]
!                          1 H,2 He,3 C,4 N, ...
!           x              Solutions to ionization balance
!                          1-2 HI and HII, 3-5 HeI-HeIII, 6-8 CI-CIII etc
!           DESTROY        Reaction rates [cm^3 s^-1]
!           CTH            Energy released per reaction [erg]
!                          NOTE should NOT be stored symmetrically (then double cunts)
! OUTPUT :  ctheating      [erg s^-1 cm^-3]
! Comments: This is a 'dumb' routine, it just multiplies the input
! numbers and adds them. Consideration of fractions of elements in
! the ground state etc should be dealt with as input.
! --------------------------------------------

ctheating = 0 ! Important
ndouble = 0
ntriple = 0

! ----------------------------------------------

ndouble(1) = nnuclei(1)*x(1)  ! n(HI)

ndouble(2) = nnuclei(1)*x(2)  ! n(HII)

DO m = 2, Nelem

   ndouble((Nstages-1)*(m-1)+1) = nnuclei(m)*x(Nstages*(m-1)+0)  ! m =2 --> 2(m-1)+1 = 3    3(m-1) + 0 = 3 :  m=3 --> 2(m-1)+1 = 5  3(m-1) + 0 = 6

   ndouble((Nstages-1)*(m-1)+2) = nnuclei(m)*x(Nstages*(m-1)+1)   
   
END DO
 ! FIXME molecule hardcoded
IF (Nmol > 0) THEN

   ndouble((Nstages-1)*Nelem + 1) = nnuclei(3)*x(Nstages*Nelem+1)     ! CO 

   ndouble((Nstages-1)*Nelem + 2) = nnuclei(3)*x(Nstages*Nelem+2)    ! CO+

   ndouble((Nstages-1)*Nelem + 3) = nnuclei(9)*x(Nstages*Nelem+3)   ! SiO

   ndouble((Nstages-1)*Nelem + 4) = nnuclei(9)*x(Nstages*Nelem+4)   ! SiO+

   ndouble((Nstages-1)*Nelem + 5) = nnuclei(1)*x(Nstages*Nelem+5)/2.  ! H2

   ndouble((Nstages-1)*Nelem + 6) = nnuclei(1)*x(Nstages*Nelem+6)/2.  ! H2+

   ndouble((Nstages-1)*Nelem + 7) = nnuclei(1)*x(Nstages*Nelem+7)  ! H-

   ndouble((Nstages-1)*Nelem + 8) = nnuclei(1)*x(Nstages*Nelem+8)/3.  ! H3+

END IF

ntriple(1) = nnuclei(1)*x(1)        ! HI  Hydrogen two states only (neutral and ionized)

ntriple(2) = nnuclei(1)*x(2)        ! HII

ntriple(3) = 0.

DO m = 2, Nelem             ! All other atoms use three states (neutral, singly ion. and doubly ion.)

   ntriple(Nstages*(m-1)+1) = nnuclei(m)*x(Nstages*(m-1)+1)

   ntriple(Nstages*(m-1)+2) = nnuclei(m)*x(Nstages*(m-1)+2)

   ntriple(Nstages*(m-1)+3) = nnuclei(m)*x(Nstages*(m-1)+3)
   
END DO
 ! FIXME molecule hardcoded
IF (Nmol > 0) THEN

   ntriple(3*Nelem + 1) = nnuclei(3)*x(Nstages*Nelem+1)     ! CO 

   ntriple(Nstages*Nelem + 2) = nnuclei(3)*x(Nstages*Nelem+2)    ! CO+

   ntriple(Nstages*Nelem + 3) = nnuclei(9)*x(Nstages*Nelem+3)   ! SiO

   ntriple(Nstages*Nelem + 4) = nnuclei(9)*x(Nstages*Nelem+4)   ! SiO+

   ntriple(Nstages*Nelem + 5) = nnuclei(1)*x(Nstages*Nelem+5)/2.  ! H2

   ntriple(Nstages*Nelem + 6) = nnuclei(1)*x(Nstages*Nelem+6)/2.  ! H2+

   ntriple(Nstages*Nelem + 7) = nnuclei(1)*x(Nstages*Nelem+7)  ! H-

   ntriple(Nstages*Nelem + 8) = nnuclei(1)*x(Nstages*Nelem+8)/3.  ! H3+

END IF


DO i = 1, (Nstages-1)*Nelem+Nmol  ! Destroy i...

   DO j = 1, Nstages*Nelem+Nmol !..with j..

       ctheating = ctheating + ndouble(i)*ntriple(j)*DESTROY(i,j)*CTH(i,j)   ! n1*n2*rate*energy --> energy per unit volume
       ! CARE this only avoids double counting if DESTROY or CTH are not stored symmetrically.
       if (ndouble(i)*ntriple(j)*DESTROY(i,j)*CTH(i,j) .gt. 0) then
            print*, i, j, ctheating, DESTROY(i, j), CTH(i, j)
       end if

   END DO

END DO

END SUBROUTINE CHARGETRANSFERHEATING
