SUBROUTINE createbeta_c(atom, N_levels, TAU_S, T, Nlambda, lambdavec, alfa_c_vec, BETA_C)
USE M1, ONLY : time, atomm
USE PARAMETERS, ONLY : k, mp, pi
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: N_levels, Nlambda
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(N_levels, N_levels), INTENT(IN) :: TAU_S
DOUBLE PRECISION, DIMENSION(Nlambda), INTENT(IN) :: lambdavec, alfa_c_vec
DOUBLE PRECISION, DIMENSION(N_levels, N_levels), INTENT(OUT) :: BETA_C
! ======================================================
! Task : Computes continuum absorption probabilities (per scattering ??) for all lines in input atom
! Input : atom          Atom to be computed
!         N_levels      Number of levels in atom
!         TAU_S         Matrix with Sobolev optical depths
!         T             Temperature
!         Nlambda       Number of wavelenghts in output vector
!         lambdavec     Vector with wavelengths to be computed
!         alfa_c_vec    Vector with continuum absorption coefficients [cm^-1]        
! Output: BETA_C        Matrix with beta_c values [dim.less]    
! =====================================================

INTEGER :: i,j, bin				! Loop indeces
DOUBLE PRECISION :: dnu, alfa_l, F, F_PR, F_use, lambda, psi, alfa_c, gamma, a

! ----------------------------------------

BETA_C = 0   ! Imp

DO i = 1, N_levels-1   ! For each lower level..

   DO j = (i+1), N_levels

      lambda = atom%LAMBDA(j,i)  

      IF (lambda < lambdavec(Nlambda-1) .AND. lambda > lambdavec(1) .AND. TAU_S(j,i) > 10) THEN

         CALL findbin2(Nlambda-1, lambdavec(1:Nlambda-1), lambda, bin)

         alfa_c = alfa_c_vec(bin+1)

         dnu = SQRT(2*k*T/(atom%MASS*mp))/(lambda*1E-8)  ! Thermal line width dnu = nu0*v_th/c = v_th/lambda = sqrt(2kT/m)/lambda

         alfa_l = TAU_S(j,i)/(lambda*1E-8*dnu*time)   ! Sobolev length = dv_D/(dv/ds) = c dnu/nu/(1/t) = dnu*lambda*t. Li & McCray 1995 eq 43 
         
         psi = 1/TAU_S(j,i)

         IF (alfa_c > 0) THEN

            IF (alfa_c < 0.1*alfa_l) THEN  ! This is normally the case

               ! Complete redistribution Hummer & Rybicki 1985
      
               F = 2*(-DLOG(DSQRT(pi)*(alfa_c/alfa_l + 2*psi*(-DLOG(DSQRT(pi)*(alfa_c/alfa_l + 2*psi*2)))**0.5)))**0.5

               ! Partial redistribution Chugai 1987

               gamma = SUM(atom%A(j,1:j-1))

               a = gamma/(4*pi*dnu)

               F_PR = 1.87*(a*alfa_l/alfa_c)**0.25   ! NOTE to be multiplied by alfa_c/alfa_l --> alfa_c/alfa_l^0.75

               F_use = MAXVAL([F, F_PR])  ! CARE does this make sense

               IF (atom%NAME .EQ. ' OI  ') F_use = F  ! CARE

               BETA_C(j,i) = alfa_c/alfa_l*F_use


            ELSE   ! continuum opacity is larger than 10 % of line opacity

               BETA_C(j,i) = alfa_c/(alfa_l + alfa_c)  ! CARE

            END IF

         ELSE

            BETA_C(j,i) = 0

         END IF       

      END IF

   END DO

END DO




END	
		
