SUBROUTINE createbeta_c_single(tau_s, R, a, beta_c)
USE PARAMETERS, ONLY : k, mp, pi
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: tau_s, R, a
DOUBLE PRECISION, INTENT(OUT) :: beta_c
! ======================================================
! Task : Computes continuum absorption probability for a single transition
! Input:
!         tau_s         Sobolev optical depth
!         R             alfa_c / alfa_l
!         a             sum A_down ( (4 pi Delta nu) Chugai 1987
! Output: beta_c        beta_c [dim.less]    
! =====================================================

DOUBLE PRECISION :: F_CR, F_PR, F_use, psi

! ----------------------------------------

if (tau_s > 0) then

  psi = 1/tau_s

else

  psi = 0

  print *, 'WARNING, tau_s not > 0'

end if

IF (R > 0) THEN

        IF (R < 0.1) THEN  ! This is normally the case

               ! Complete redistribution (CR) Hummer & Rybicki 1985
      
               F_CR = 2*(-DLOG(DSQRT(pi)*(R + 2*psi*(-DLOG(DSQRT(pi)*(R + 2*psi*2)))**0.5)))**0.5 ! DC

               ! Partial redistribution (PR) Chugai 1987

               F_PR = 1.87*(a*R)**0.25   ! NOTE to be multiplied by alfa_c/alfa_l --> alfa_c/alfa_l^0.75

               F_use = MAXVAL([F_CR, F_PR])  ! CARE does this make sense

               beta_c = R*F_use

         ELSE   ! continuum opacity is larger than 10 % of line opacity

               beta_c = R/(1. + R)  ! CARE. Make sure stays < 1

         END IF

ELSE

      beta_c = 0

END IF       


END
