SUBROUTINE createtau_s(atom, N_upper, N_levels, levels_pop, TAU_S)
USE M1
USE PARAMETERS
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: N_upper, N_levels
DOUBLE PRECISION, DIMENSION(N_levels), INTENT(IN) :: levels_pop
DOUBLE PRECISION, DIMENSION(N_upper, N_levels),  INTENT(OUT) :: TAU_S
! ======================================================
! Task : Compute the Sobolev optical depths given the level populations
! INPUT : atom     Type atomm (M1)
!         N_upper  Maximum upper level
!         N_levels Maximum lower level
!        levels_pop_pop   : Level populations [cm^-3]
! OUTPUT : TAU_S      Optical depths
! GLOBALS : time
! ===================================================
DOUBLE PRECISION :: n1, n2, g1, g2
INTEGER :: i, j

! -----------------------------------------------------------------

IF (minval(levels_pop) < 0) THEN

   print *, 'ERROR in createtau_s, negative population encountered for atom', atom%NAME

   STOP 1

END IF


TAU_S = 0                                       ! Initialize

DO i = 1, N_levels-1   ! For each lower level....optical depths to lower levels above N_levels is zero

   DO j = (i+1), N_upper     ! for each upper level...

      n1 = levels_pop(i)      ! number density of lower level......i is always less than N_levels

      g1 = atom%LEVELS_G(i)   ! statistical weight of lower level

      g2 = atom%LEVELS_G(j)   ! statistical weight of upper level

      IF (j <= N_levels) THEN    ! If upper level has non-zero population, subtract stimulated emission

         n2 = levels_pop(j)   ! number density of upper level

         TAU_S(j,i) = atom%A(j,i)*(atom%LAMBDA(j,i)*1e-8)**3/(8*pi)*g2/g1*time*(n1-g1/g2*n2) ! Castor 1970, Kozma 1998 part I eq 30, Li McCray 1996 eq 4..multiply n1 into paranthesis to avoid possible div. by zero.

      ELSE  ! j > N_levels --> population of upper level is zero : ignore stimulated emission

         TAU_S(j,i) = atom%A(j,i)*(atom%LAMBDA(j,i)*1e-8)**3/(8*pi)*g2/g1*time*n1        ! Div by zero checked

      END IF

   END DO

END DO

IF (atom%NAME .EQ. ' HI  ') TAU_S(2,1) = 0   ! HI twophoton

IF (atom%NAME .EQ. 'HeI  ') TAU_S(3,1) = 0  ! HeI twophotons


END
