SUBROUTINE DEEXCITE(z, levelj, Nup, P, beta_s, beta_c, level_out, photon, ionization)
USE M1, ONLY : myid, idum
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: z
INTEGER, INTENT(IN) :: levelj, Nup
DOUBLE PRECISION, DIMENSION(2*(levelj-1)+Nup), INTENT(IN) :: P
DOUBLE PRECISION, DIMENSION(levelj-1), INTENT(IN) :: beta_s, beta_c
INTEGER, INTENT(OUT) :: level_out
LOGICAL, INTENT(OUT) :: photon, ionization
! ==============================================
! Task : Make a random deexcitation of an excited atomic state
! INPUT :  z        (Already drawn) random number between 0 and 1
!         levelj    Level to be deexcited
!         Nup       Number of upwards transtions allowed
!         P         Array with probabilities for deeexcitation channels
!         beta_s   Array with Sobolev escape probabilities
!         beta_c   Array with continuum destruction probabilities
! OUTPUT :level_out : Level drawn as deexcitation level
!         photon     Boolean : was the deexcitation radiative?
!         ionization BOolean : was the deexcitation a continuum ionization?
! GLOBALS : --
! =============================================
DOUBLE PRECISION :: PC, Psum, p_rad, z2
INTEGER :: k
LOGICAL :: hit
REAL :: zreal
REAL, EXTERNAL :: ran1
! -----------------------------------------------

PC = 0
k = 0
hit = .FALSE.          ! Imp

! ----------------------------------------

Psum = SUM(P)

if (isNaN(Psum)) then
  print *, 'ERROR in deexcite', levelj, P
  stop 1
end if

! Check input arguments
IF (Psum .LT. 0.99 .OR. Psum .GT. 1.01) THEN
   WRITE(*,*) 'ERROR in deexcite. P vector does not add to one.'
   STOP 1
ELSE IF ( (z<0) .OR. (z>1) ) THEN
   WRITE(*,*) 'ERROR in deexcite : z not between 0 and 1.'
   STOP 1
ELSE IF ( levelj < 2) THEN 
   WRITE(*,*) 'ERROR in deexcite : level below 2.'
   STOP 1
END IF

DO WHILE (hit .EQV. .FALSE.)
	
   k = k + 1

   if (k > 2*(levelj-1)+Nup) then
	print *, 'ERROR in deexcite, k too large', k, levelj, z, Psum, PC
        stop 1
   end if 
	
   PC = PC + P(k)  ! Accumulated probability
	
   IF (PC .GE. z) THEN   ! Transition hit, accumulated prob larger than z
	
      IF (k < levelj) THEN                 ! Collision down                   
	
         level_out = k
              
         photon = .FALSE.

         ionization = .FALSE.

      ELSE IF (k > 2*(levelj-1)) THEN     ! Collision up

         level_out = k - (levelj-2)

         photon = .FALSE.

         ionization = .FALSE.

         IF (myid <= 1) PRINT *, 'Upcollision', levelj, level_out
		
      ELSE                                   ! RADIATIVE
		
         level_out = k - (levelj-1)
			
         p_rad = beta_s(level_out)/(beta_s(level_out) + beta_c(level_out))  ! Prob that photon escapes (is not continuum absorbed)

         !CALL RANDOM_NUMBER(z2)

         zreal = ran1(idum)

         z2 = zreal

         IF (z2 < p_rad) THEN
	
            photon = .TRUE.
            ionization = .FALSE.

         ELSE

            photon = .false.
            ionization = .TRUE.
           
         END IF
			
      END IF
				
      hit = .TRUE.

      EXIT                                    ! Important

   END IF
	
END DO


END
