SUBROUTINE depositionfunction(DIMS, x_MAT, Nblobs, deposition_tot_pervol, gammadep_pervol)
use M1, ONLY : Nr, Ncore, v_out, time, kappa_56Ni, kappa_56Co, kappa_57Co, kappa_44Ti, kappa_pos, globid, IDP, leakage, BLEND
use PARAMETERS, ONLY : mp, pi
use INTERFACE, ONLY : findbin2
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: DIMS, Nblobs
DOUBLE PRECISION, DIMENSION(:, :), INTENT(IN) :: x_MAT  ! Nr*DIMS
! LOGICAL, POINTER, INTENT(IN) :: BLEND
DOUBLE PRECISION, DIMENSION(:), INTENT(OUT) :: deposition_tot_pervol, gammadep_pervol  ! Nr
! ===========================================
! Task   : Computes the total energy deposition of gamma-rays, positrons, electrons and X-rays from 56Ni, 56Co, 57Co, 44Ti, in each zone, using gray transfer (effective opacities).
! INPUT  : DIMS               Size of array of properties (x_MAT) for each zone
!          x_MAT              2D Array with zone properties
!          af                 The area factor for each core zone, area of each zone divided by total area of all core blobs.
!          BLEND              Boolean switch to tell if core is blended (macroscopically mixed) or 'classical' (concentric shells)
!          Nblobs             Number of clumps that each core zone is divided into
! OUTPUT : deposition_tot_pervol   Array with total deposition (erg /s /cm3) in each zone
!          gammadep_pervol     ENergy deposition from gamma rays only (erg /s/cm3)
! ===========================================
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: r_in, r_out, rmiddle, nvec, Amean, Zmean, M_44Ti_vec, M_57Co_vec, M_56Ni_vec, M_56Co_vec, volume, L_56Ni, L_56Co, L_57Co, L_44Ti, L_leptons, L_xrays, sigma_56Ni, sigma_56Co, sigma_57Co, sigma_44Ti, sigma_leptons, sigma_xrays, Evec, Evec_wleptons, deposition_positrons_wholeshell, deposition_electrons_wholeshell, absorbed_positrons_wholeshell, absorbed_electrons_wholeshell, MZoverA, af_accum, thetavec, thetablobvec, impactvec, blob_radii, ff, af
INTEGER :: i, j, m, Ntheta, Nphi, shell, Nsplit, R_index_temp, R_index_next, nt, bin, bin2, np, Nthetablobs, counter
DOUBLE PRECISION :: Rcore, theta, dr, dV, r0, dtheta, r_sender, rho, tau_56Ni, tau_56Co, tau_57Co, tau_44Ti, Jfield_56Ni, Jfield_56Co, Jfield_57Co, Jfield_44Ti, thetastart,  gammadeposition_pp, gammadeposition_pervol, deposition_positrons_pervol, deposition_electrons_pervol, dJfield_56Ni, dJfield_56Co, dJfield_57, dJfield_44, rin, rout, r, fluxtot_56Ni, fluxtot_56Co, fluxtot_57, fluxtot_44, fluxtot_leptons, fluxtot_xrays, I_56Ni, I_56Co, I_57, I_44, I_leptons, I_xrays, dI_56Ni, dI_56Co, dI_57, dI_44, dI_leptons, dI_xrays, r_edgeinside, r_edgeoutside, r_smallest, r_edgehit, dr_toshelledge, dtau_56Ni, dtau_56Co, dtau_57, dtau_44, dtau_leptons, dtau_xrays, vol, cos_theta, sin_theta, voltot, volwhole, rnew, v_core, blobsize, blobsize_orig, dr_blobedge, cos_theta_int, sin_theta_int,r_int, z, rnext, cos_alpha, phi, dphi, positrons_tot, electrons_tot, afmax, tautot, dr_blobedge_motherblob=0
LOGICAL :: ALIVE, MIRROR, HOLEHIT, NEWBLOB=.FALSE.
DOUBLE PRECISION, DIMENSION(Nr) :: mass

! -------------------------------------------------------------------------------------------

ALLOCATE(r_in(Nr)); r_in=0
ALLOCATE(r_out(Nr)); r_out=0
ALLOCATE(rmiddle(Nr)); rmiddle=0
ALLOCATE(nvec(Nr)); nvec=0
ALLOCATE(Amean(Nr)); Amean=0
ALLOCATE(Zmean(Nr)); Zmean=0
ALLOCATE(M_44Ti_vec(Nr)); M_44Ti_vec = 0
ALLOCATE(M_57Co_vec(Nr)); M_57Co_vec = 0
ALLOCATE(M_56Ni_vec(Nr)); M_56Ni_vec = 0
ALLOCATE(M_56Co_vec(Nr)); M_56Co_vec = 0
ALLOCATE(volume(Nr)); volume=0
ALLOCATE(L_56Ni(Nr)); L_56Ni = 0
ALLOCATE(L_56Co(Nr)); L_56Co = 0
ALLOCATE(L_57Co(Nr)); L_57Co = 0
ALLOCATE(L_44Ti(Nr)); L_44Ti = 0
ALLOCATE(L_leptons(Nr)); L_leptons = 0
ALLOCATE(L_xrays(Nr)); L_xrays = 0
ALLOCATE(sigma_56Ni(Nr)); sigma_56Ni = 0
ALLOCATE(sigma_56Co(Nr)); sigma_56Co = 0
ALLOCATE(sigma_57Co(Nr)); sigma_57Co = 0
ALLOCATE(sigma_44Ti(Nr)); sigma_44Ti = 0
ALLOCATE(sigma_leptons(Nr)); sigma_leptons = 0
ALLOCATE(sigma_xrays(Nr)); sigma_xrays = 0
ALLOCATE(Evec(Nr)); Evec = 0
ALLOCATE(Evec_wleptons(Nr)); Evec_wleptons = 0
ALLOCATE(deposition_positrons_wholeshell(Nr)); deposition_positrons_wholeshell = 0
ALLOCATE(deposition_electrons_wholeshell(Nr)); deposition_electrons_wholeshell = 0
ALLOCATE(absorbed_positrons_wholeshell(Nr)); absorbed_positrons_wholeshell = 0
ALLOCATE(absorbed_electrons_wholeshell(Nr)); absorbed_electrons_wholeshell = 0
ALLOCATE(MZoverA(Nr)); MZoverA = 0
IF (Ncore > 0) THEN
   ALLOCATE(ff(Ncore)); ff=0
   ALLOCATE(af(Ncore)); af =0
   ALLOCATE(af_accum(Ncore+1)); af_accum=0
   ALLOCATE(blob_radii(Ncore)); blob_radii=0
END IF

Ntheta = 600  ! CARE number of different directions to send gamma rays off into

Nsplit = 300  ! CARE number of subzones each zone is split up in when emitting gamma rays

Nthetablobs = 30  ! 30 possible impact angles on new core blobs

Evec = 0  ! IMPORTANT

Evec_wleptons = 0  ! IMPORTANT

voltot = 0

bin = 0

tautot = 0

ALLOCATE(thetablobvec(Nthetablobs)); thetablobvec = 0

ALLOCATE(impactvec(Nthetablobs)); impactvec = 0

thetablobvec = [(pi/2.*i/(Nthetablobs-1),i=0,Nthetablobs-1)]  ! Impact angle on blob from From 0 to 90 degrees (relative surface normal)

impactvec = 0.5 - 0.5*COS(2*thetablobvec)    ! From 0 to 1..isotropic angle (half-sphere)

! -----------------------------------------------------------

IF (Ncore > 0) THEN

   ff = x_MAT(1:Ncore,DIMS-12)  ! filling factors for each core zone

   af = ff**(2./3.)

   af_accum(1) = 0 ! Make accumulated area factors, first is zero, Ncore + 1 contains sum of first Ncore

   DO i = 1, Ncore

      af_accum(i+1) = af_accum(i) + af(i)

   END DO

   afmax = af_accum(Ncore+1)

   af_accum = af_accum/afmax   ! Make sure its normalized to one

   v_core = v_out*x_MAT(Ncore,DIMS-10) ! core velocity in cm/s  x_MAT(:,DIMS-10) are normalized outer velocities

   Rcore = v_core*time   ! Core radius [cm]

   R0 = x_MAT(1,DIMS-11)*v_out*time  ! Ejecta minimum radius [cm]. x_MAT(:,DIMS-11) are the normalized inner radii

   blob_radii = ((Rcore**3-R0**3)*ff/DBLE(Nblobs))**(1./3.)   ! blob radii [cm]   4pi/3 blob_radii_i^3 = 4pi/3 (Rcore^3-R0**3)*ff_i / Nblobs

ELSE

   R0 = x_MAT(1,DIMS-11)*v_out*time  ! Ejecta minimum radius [cm]. x_MAT(:,DIMS-11) are the normalized inner radii

END IF

ALLOCATE(thetavec(Ntheta)); thetavec = 0

thetavec = [(i*pi/DBLE(Ntheta+1), i=1,Ntheta)]   ! From 0 (radially outwards) to pi (radially inwards), edges avoided, why?

r_in = x_MAT(:,DIMS-11)*v_out*time  ! Inner radii [cm]

r_out = x_MAT(:,DIMS-10)*v_out*time ! Outer radii [cm]

volume = x_MAT(:,DIMS-8)  ! Zone volumes [cm^3]

M_56Ni_vec = x_MAT(:,DIMS-13) ! Mass of 56Ni [Msun] (at current time)

M_56Co_vec = x_MAT(:,DIMS-7)  ! Mass of 56Co [Msun] (at current time)

M_57Co_vec = x_MAT(:, DIMS-6)  ! Mass of 57Co [Msun] (at current time)

M_44Ti_vec = x_MAT(:,DIMS-5)  ! Mass of 44Ti [Msun] (at current time)

Amean = x_MAT(:,DIMS-4)  ! Mean atomic weights [proton masses]

Zmean = x_MAT(:,DIMS-1)  ! Mean atomic charge []

nvec = x_MAT(:,DIMS-3)   ! Total number densities [cm^-3]

DO i = 1, Nr   ! For each zone...

   L_56Ni(i) = 5.41D42*M_56Ni_vec(i)/0.07    ! Gamma ray lum.   calc from Sur 1990, assume 1720 keV gamma rays

   L_56Co(i) = 8.83D41*M_56Co_vec(i)/0.07    ! Own calc. Gamma ray luminosities Kozma1998 part I , eq 20 has 8.82. Axelrod1980 gives correspondingly 8.64D41 but has tau=114 days which gives a lower value.  Woosley1989 give 8.90D40..hmm

   L_57Co(i) = 4.43D38*M_57Co_vec(i)/3.3e-3    ! own calc.K  F98-I eq 21 has 4.52 (2.0% higher)

   L_44Ti(i) = 3.78D36*M_44Ti_vec(i)/1.0e-4    ! av energy 0.596 MeV, life-time 87 years (Compare KF92 4.10E36, which assumes mean positron energy is half total decay energy)

   L_leptons(i) = 3.12D40*M_56Co_vec(i)/0.07 + 9.64D35*M_44Ti_vec(i)/1.0e-4 + 9.23D38*M_56Co_vec(i)/0.07 + 5.80D37*M_57Co_vec(i)/3.3E-3 +  1.75D34*M_44Ti_vec(i)/1E-4  ! First two positrons, last three electrons (Own calc based on numbers in Seitenzahl2011)

   L_xrays(i) = 1.17D37*M_57Co_vec(i)/3.3E-3   ! Seitenzahl 2009..neglegible for all others except 55Fe

   sigma_56Ni(i) = (Zmean(i)/Amean(i)/0.5)*kappa_56Ni*Amean(i)*mp

   sigma_56Co(i) = (Zmean(i)/Amean(i)/0.5)*kappa_56Co*Amean(i)*mp     ! Effective cross sections for gamma ray absorption [cm^2]

   sigma_57Co(i) = (Zmean(i)/Amean(i)/0.5)*kappa_57Co*Amean(i)*mp

   sigma_44Ti(i) = (Zmean(i)/Amean(i)/0.5)*kappa_44Ti*Amean(i)*mp

   sigma_leptons(i) = (Zmean(i)/Amean(i)/0.5)*kappa_pos*Amean(i)*mp

   sigma_xrays(i) = sigma_leptons(i) ! TEMP

END DO

IF (globid == IDP) PRINT *,'Total gamma-ray lum., lepton, xray lum:', SUM(L_56Ni) + SUM(L_56Co) + SUM(L_57Co) + sum(L_44Ti), SUM(L_leptons), SUM(L_xrays)

DO shell = 1, Nr   ! Emission from each shell...

   IF (shell <= Ncore) blobsize_orig = blob_radii(shell)   ! Radial size of the core blobs

   IF (M_44Ti_vec(shell) > 0 .OR. M_57Co_vec(shell) > 0 .OR. M_56Co_vec(shell) > 0 .OR. M_56Ni_vec(shell) > 0) THEN   !If there is any radioactive material present in this zone..

     DO i = 1, Nsplit        ! Split the shell into *Nsplit* sub-shells..

         rin = r_in(shell) + (i-1)*(r_out(shell) - r_in(shell))/DBLE(Nsplit)  ! all in cm..first one gets r_in, last r_in + dr*(Nplit-1)/(Nsplit)

         rout = r_in(shell) + i*(r_out(shell) - r_in(shell))/DBLE(Nsplit)

         r = 0.5*(rin + rout) ! take average radius as the emission point

         vol = 4*pi/3.*(rout**3 - rin**3)  ! volume of this segment [cm^3]

         volwhole = 4*pi/3.*(r_out(shell)**3 - r_in(shell)**3)  ! volume of whole zone [cm^3]

         fluxtot_56Ni = L_56Ni(shell)*vol/volwhole   ! Luminosity emitted by this subzone [erg/s] is total luminosity times the fraction of the volume

         fluxtot_56Co = L_56Co(shell)*vol/volwhole

         fluxtot_57 = L_57Co(shell)*vol/volwhole

         fluxtot_44 = L_44Ti(shell)*vol/volwhole

         fluxtot_leptons = L_leptons(shell)*vol/volwhole

         fluxtot_xrays = L_xrays(shell)*vol/volwhole

         voltot = voltot + vol

         IF (BLEND .AND. shell <= Ncore) THEN   ! If blended core zone..

            CALL RANDOM_NUMBER(z)

            r_int = blobsize_orig*z**(1./3.)   ! Random internal radius

            CALL RANDOM_NUMBER(z)

            cos_theta_int = -1 + 2*z   ! Random internal direction

            sin_theta_int = SQRT(1. - cos_theta_int**2)

            dr_blobedge_motherblob =  -r_int*cos_theta_int + DSQRT((r_int*cos_theta_int)**2 - (r_int**2 - blobsize_orig**2))  ! Distance to blob edge..law of cosines

            NEWBLOB = .FALSE.

         END IF

         DO nt = 1, Ntheta   ! For each direction..

            theta = thetavec(nt)

            dtheta = thetavec(2) - thetavec(1) ! CARE

            I_56Ni = fluxtot_56Ni*SIN(theta)*dtheta/2.  ! Total flux times part of the 4*pi that this direction takes up = dOmega/(4*pi) = sin(theta) dtheta*dphi/(4*pi) = sin(theta)*dtheta*2*pi/(4*pi) = sin(theta)*1/2 dtheta

            I_56Co = fluxtot_56Co*SIN(theta)*dtheta/2.

            I_57 = fluxtot_57*SIN(theta)*dtheta/2.

            I_44 = fluxtot_44*SIN(theta)*dtheta/2.  ! checked that this treatment agrees with using random directions cos_theta = -1 + 2*z for each photon

            I_leptons = fluxtot_leptons*SIN(theta)*dtheta/2.

            I_xrays = fluxtot_xrays*SIN(theta)*dtheta/2.

            ALIVE = .TRUE.

            sin_theta = SIN(theta)

            cos_theta = COS(theta)

            R_index_temp = shell  ! No BLEND, shell is shell. DO NOT MOVE

            HOLEHIT = .FALSE.

            MIRROR = .FALSE.

            NEWBLOB = .FALSE.

            dr_blobedge = dr_blobedge_motherblob

            counter = 0

            DO WHILE (ALIVE .EQV. .TRUE.)  ! Loop until beam escapes ejecta..

               counter = counter + 1

               r_edgeinside = r_in(R_index_temp)

               r_edgeoutside = r_out(R_index_temp)

               IF (cos_theta < 0) THEN  ! Inward moving photon..

                  r_smallest = r*sin_theta         ! angle is sin(pi-theta) = sin(theta)...Sin theta is positive always (theta between 0 and 180)

               ELSE

                  r_smallest = r        ! Photon is moving outward, smallest radius is the starting one..

               END IF

               IF (r_edgeinside > r_smallest) THEN      ! Photon is moving in and and inner shell surface is blocking photon path

                  r_edgehit = r_edgeinside

                  dr_toshelledge = -r*cos_theta - SQRT((r*cos_theta)**2 - (r**2 - r_edgehit**2))  ! Law of cosines..formula checked

                  IF (BLEND .AND. R_index_temp <= Ncore) THEN  ! Scrambled core..note can only go into this condition if there is an inner hole that has been hit, otherwise r_smallest is zero
                     MIRROR = .TRUE.    ! A photon inside the core(R_index_temp = Ncore) hitting an edge that is also inside the core -->

                  ELSE   ! there is no scrambled core...

                     R_index_next = R_index_temp - 1  ! Move one shell in

                     IF (R_index_next .EQ. 0) THEN   ! If the innermost shell has a non-zero velocity, a mirror bounce is needed also for a non-mixed core

                        MIRROR = .TRUE.

                        HOLEHIT = .TRUE.

                        R_index_next = 1

                     ELSE

                        MIRROR = .FALSE.

                     END IF

                  END IF

               ELSE  ! Photon hits outer edge (not inner)..unless another core blob is hit below

                  r_edgehit = r_edgeoutside

                  dr_toshelledge = -r*cos_theta + DSQRT((r*cos_theta)**2 - (r**2 - r_edgehit**2))  ! Law of cosines, positive solution since -r*cos(theta) is negative

                  IF (BLEND .AND. R_index_temp <= Ncore) THEN

                     R_index_next = Ncore + 1    ! This is overridden is ray hits blob edhe instead below

                  ELSE

                     R_index_next = R_index_temp + 1   ! Will be replaced below if a core blob is hit..(in that case R_index_next stays at R_index_temp = Ncore)

                  END IF

               END IF

               IF (BLEND .AND. (R_index_temp <= Ncore)) THEN  ! Photon is in scrambled core..

                  IF (NEWBLOB) THEN   ! Do we need to draw a new blob (i.e was last edge as blob edge)?

                     CALL RANDOM_NUMBER(z)

                     CALL findbin2(Ncore+1, af_accum, z, bin)  ! Make a random draw as to which blob type to enter, based on total area factors

                     blobsize = blob_radii(bin)

                     CALL RANDOM_NUMBER(z)

                     CALL findbin2(Nthetablobs, impactvec, z, bin2)  ! make a random draw for impact angle...CARE what is this based on? Let incoming photon have random angle --> same distribution as emitting photon cos z = -1 + 2z, i.e. let z choose a value in a uniformly spaced cos(theta) vec. Here we only have angles from 0 to 90 so use a mu vec between 0 and 1

                     theta = thetablobvec(bin2)

                     cos_theta_int = -1 + z   ! from -1 to 0, uniformly spaced

                     dr_blobedge = 2*blobsize*ABS(cos_theta_int)

                     NEWBLOB = .FALSE.

                     R_index_temp = bin

                  END IF

                  IF (dr_blobedge < dr_toshelledge) THEN  ! Next edge is the blob edge rather than the zone edge

                     dr = dr_blobedge

                     rnext = SQRT(r**2 + dr**2 - 2*r*dr*(-cos_theta))  ! Law of cosines..angle is cos(pi-theta) = -cos(theta) NOTE not cos_theta_int here

                     R_index_next = R_index_temp   ! Still in the same zone...

                     NEWBLOB = .TRUE.   ! Need to draw a new blob in next iteration

                  ELSE  ! Next edge is shell edge

                     dr = dr_toshelledge

                     IF (MIRROR .EQV. .FALSE.) THEN  ! Is the edge the inner hole? If no then we are leaving core..

                        rnext = Rcore*1.0001   ! To avoid numerical problems, put a bit inside inner radius

                        R_index_next = Ncore + 1

                        HOLEHIT = .FALSE.

                     ELSE ! The hit edge is the inner hole

                        HOLEHIT = .TRUE.

                        rnext = R0*1.0001

                        R_index_next = R_index_temp

                     END IF

                     dr_blobedge = dr_blobedge - dr  ! Reduce distance to blob edge (this can happen if the non-zero inner boundary is hit, we want to stay in the same blob)

                  END IF

               ELSE   ! Photon is not in a scrambled core..

                  dr = dr_toshelledge

                  rnext = r_edgehit

               END IF

               dtau_56Ni = sigma_56Ni(R_index_temp)*nvec(R_index_temp)*dr

               dtau_56Co = sigma_56Co(R_index_temp)*nvec(R_index_temp)*dr

               dtau_57 = sigma_57Co(R_index_temp)*nvec(R_index_temp)*dr

               dtau_44 = sigma_44Ti(R_index_temp)*nvec(R_index_temp)*dr

               dtau_leptons = sigma_leptons(R_index_temp)*nvec(R_index_temp)*dr

               dtau_xrays = sigma_xrays(R_index_temp)*nvec(R_index_temp)*dr

               dI_56Ni = I_56Ni*(1.-DEXP(-dtau_56Ni))

               dI_56Co = I_56Co*(1.-DEXP(-dtau_56Co))

               dI_57 = I_57*(1.-DEXP(-dtau_57))

               dI_44 = I_44*(1.-DEXP(-dtau_44))

               dI_leptons = I_leptons*(1.-DEXP(-dtau_leptons))

               dI_xrays = I_xrays*(1.-DEXP(-dtau_xrays))

               Evec(R_index_temp) = Evec(R_index_temp) + dI_56Ni + dI_56Co + dI_57 + dI_44

               Evec_wleptons(R_index_temp) = Evec_wleptons(R_index_temp) +  dI_56Ni + dI_56Co + dI_57 + dI_44 + dI_leptons + dI_xrays

               I_56Ni = I_56Ni - dI_56Ni

               I_56Co = I_56Co - dI_56Co

               I_57 = I_57 - dI_57

               I_44 = I_44 - dI_44

               I_leptons = I_leptons*DEXP(-dtau_leptons)

               I_xrays = I_xrays*DEXP(-dtau_xrays)

               IF (R_index_next <= Nr) THEN  ! Unless photon has escaped ejecta...

                  sin_theta = r/rnext*sin_theta; IF (sin_theta .GT. 1.) sin_theta = 1.

                  IF (cos_theta < 0 .AND. dr < r*ABS(cos_theta)) THEN  ! Inward doing photon, does not reach inflection point.. (sin(pi/2-x) = cos(x))

                     cos_theta = -SQRT(1.-sin_theta**2)

                  ELSE   ! Outward going photon or inward going photon that passes inflection point

                     cos_theta = +SQRT(1.-sin_theta**2)

                  END IF

                  IF (HOLEHIT) THEN  ! Reaches inner hole, flip sign

                     cos_theta = +SQRT(1.-sin_theta**2)

                     HOLEHIT = .FALSE.

                     MIRROR = .FALSE.

                  END IF

                  R_index_temp = R_index_next

                  r = rnext

               ELSE          ! Ray has reached outer edge

                  ALIVE = .FALSE.

                  r = 0.5*(rin + rout) ! CARE, get ready for new emission (new direction)

               END IF

            END DO

         END DO

      END DO

   END IF

END DO

OPEN(20,file=file_blobsize,STATUS='REPLACE')

DO i = 1, Ncore

   WRITE(20,*) v_core*(x_MAT(i,DIMS-12)/DBLE(Nblobs))**0.333/1E5    ! Write size of each core blob

END DO

CLOSE(20)

IF (MINVAL(Evec) < 0) THEN

   PRINT *, 'ERROR in depositionfunction: gamma deposition is negative'; STOP 1

END IF

! ------- POSITRONS ---------------------------------------------------------

!deposition_positrons_wholeshell = 3.12D40*M_56Co_vec/0.07 + 1.3D36*M_44Ti_vec/1e-4   ! Kozma1998 part I, eq 23 and 24. Axelrod1980 page 22 gives 3.02E40 for 56Co positrons which is 0.965 of the Kozma value, this is explained by him using tau=114 days instead of 111?. Woosley1989 give 3.12D40

DO i = 1, Nr

  MZoverA(i) = x_MAT(i,DIMS-8)*x_MAT(i,DIMS-4)*x_MAT(i,DIMS-3)*x_MAT(i,DIMS-1)/x_MAT(i,DIMS-4)   ! volume*Amean*n*Z/A   (volume*Amean*n = M)

END DO

IF (leakage == 0) THEN    ! If positron deposition should be local..

   WHERE (volume > 0)

      deposition_tot_pervol = (Evec + L_leptons + L_xrays)/volume   ! Total depositon, gamma rays plus positrons and electrons

      gammadep_pervol = Evec/volume

   ELSEWHERE

      deposition_tot_pervol = 0

      gammadep_pervol = 0

   END WHERE

ELSEIF (leakage == 1) THEN   ! else, used computed streamed depositions...

   WHERE (volume > 0)

      deposition_tot_pervol = Evec_wleptons/volume   ! Total depositon, gamma rays plus positrons

      gammadep_pervol = Evec/volume

   ELSEWHERE

      deposition_tot_pervol = 0

      gammadep_pervol = 0

   END WHERE

ELSEIF (leakage == 2) THEN  ! Split leptons between core zones

   WHERE (volume > 0)

      deposition_tot_pervol = Evec/volume ! gamma rays

      gammadep_pervol = Evec/volume

   ELSEWHERE

      deposition_tot_pervol = 0

      gammadep_pervol = 0

   END WHERE

   mass = x_MAT(:,DIMS-8)*x_MAT(:,DIMS-4)*x_MAT(:,DIMS-3)

   deposition_tot_pervol(1) = deposition_tot_pervol(1) + mass(1)/sum(mass(1:3))*sum(L_leptons+L_xrays)/volume(1)
   deposition_tot_pervol(2) = deposition_tot_pervol(2) + mass(2)/sum(mass(1:3))*sum(L_leptons+L_xrays)/volume(2)
   deposition_tot_pervol(3) = deposition_tot_pervol(3) + mass(3)/sum(mass(1:3))*sum(L_leptons+L_xrays)/volume(3)

END IF

DO i = 1, Nr

   IF (i <= Ncore) THEN

      tautot = tautot +  (Zmean(i)/Amean(i)/0.5)*kappa_56Co*Amean(i)*mp*nvec(i)*x_MAT(i,DIMS-9)*ff(i)

   ELSE

      tautot = tautot +  (Zmean(i)/Amean(i)/0.5)*kappa_56Co*Amean(i)*mp*nvec(i)*x_MAT(i,DIMS-9)

   END IF

END DO

IF (globid == IDP) then

 print *, 'tautot 56Co', tautot

 open(10,file='out/depositionfunction/taugamma.dat',STATUS='REPLACE')

 write(10,*) tautot

 close(10)

END IF

END SUBROUTINE depositionfunction
