ADDING ATOM OR MOLECULE TO DO LEVEL POPULATION CALCULATIONS FOR (AND INCORPORATE IN E.G. COOLING)
-------------------------------------

-------------------------------------
PREPARING THE DATA FILES
-------------------------------------

1. Creating data folder

Create a folder in `DATA/atoms`, for atoms, or `DATA/molecules` for molecules, with the name of the atom/molecule that you want to add. 
*NOTE* that fortran is not case sensitive, so make sure that no other molecule or atom shares the name (e.g. NiI would
become the same as NII, so one has to resolve this by using e.g. Ni_neutral). 

Naming conventions are the `name_of_atom/molecule` + `I` for neutral, `II` for singly ionized etc. 

2. Setting up the data file. 

Data files for the energy levels and transitions betweens these need to be created. For carbon monoxide, this is done with the python script /tools/energy_levels.py. The format of the data file is 
``` bash
C I level and line data from NIST. Rec. rates from Nahar et al. 1995. 
n       E[cm-1]    g       alfa (cm3/s)  T0[K]   Power-law   name/comment
1       0.000      1       9.01E-14      1000    -0.5         2s2.2p2 3P       
2       16.417     3       2.70E-13      1000    -0.4                 3P
Lines   Coll. strengths from Pradhan online.
i j WL[A]           A[s-1]     Upsilon(T0)  T0[K]   Power-law  
2 1 6091353.37      7.93E-08      0.24      5000    -0.5
3 1 2303433.26      1.72E-14      0.18      5000    -0.7
...
```
so that the first two lines of the file (not read) contains info about e.g. the sources for the line data. This is followed by level data for `N_leves` levels. 
If recombination rates are known, these come in columns 4 (alfa at some temperature T0), 5 (that T0) and 6 (power law exponent).

After the levels come another 2 dummy lines. Then come the transitions, with upper and lower level numbers, wavelength (in Angstrom) and Einstein coefficient A (s-1).
If collision strengths (Upsilon) are known, they come in columns 5 (value at some T0), 6 (that T0), and 7 (power law exponent).

The absence of presence of recombination rates and collision strength data is set in alloc.f90. If such are absent, leave the corresponding columns empty.

3. FOR A NEW ELEMENT ONLY (NOT A NEW ION STAGE, OR A NEW MOLECULE) : Add element to ejecta file

The number of elements in the explosion model must also be updated if a new element is added, in the DATA/explmodels/ directory. 
At the end of the explosion model file, add a line for each element, e.g.
```bash
26  0.1 0.0 0.0
```
where in the above example, we have put 10% of the new element (atomic number 26, so iron) in the first explosion zone, and none in the second and third zone.

4. MOLECULES ONLY: Update the moldata files

`DATA/mol_input/moldata_input.csv` contains information about all the included molecular species. To add a new molecule species this file. The file format is 
```bash
name, mbin, parent1, parent2, n_p1, n_p2, mass, charge
COI    ,1,4,14,1.0,1.0,28,0.0
```

-------------------------------------
CHANGES IN THE CODE
-------------------------------------

1. `ABINCODES`

Resides in DATA/mol_input. Append the list with the new atom/ion/molecule. In general we reserve 5 slots for each element (5 ionization stages), so even if you for now just add e.g. the neutral species, reserve all 5 slots for the other ions. Note this file is not actively used when running, but to create the chemical network files in pre-processing. Molecules are reserved for slots 126-135.

2. `m1.f90`

a. Increase Nabin. Add 5 for an atom/ion (make space for forst 5 ionization stages) and 2 for a molecule (make space for first 2 ionization stages).

b. Append the gval array in makeatoms.f90 to include the new species. These are the statistical weights of the ground multiplet.

3. `main.f90` and `main_radtrans.f90` (same changes needed in both)

a. Change N_atoms to include new species. E.g. if you add ZnI increase by 1, ZnI and ZnII increase by 2,..

b. Add the new species name to the atomm type list. (TYPE (atomm) :: atom, HI, HeI, HeII,...) 

c. Define the name of the new species in the line that starts with `HI%NAME=' HI  '; ...`
In general, first 2 characters are for the letter, then 3 slots for the ionization stage, so e.g. ' HI  ', 'HeII ', ' KIII'.

d. Add the new species to the atom(1:N_atoms) array. Ions of same element (e.g FeI, FeII) must be consecutive.

4. `alloc.f90`

a. Add info about the new species, e.g. 
```bash
  CASE ('CMI  ')  

     atom%SPECIES_TYPE = 'mol' (for atom either skip this (defaults to 'atm') or write 'atm')
     atom%Z = 1  ! Atomic number
     atom%MASS = 28.0
     atom%N = 98  ! Number of levels
     atom%N_levels = 50 ! Number of levels to do NLTE solutions for (may be smaller or equal to atom%N. 0 means use atom%N.)
     atom%NGROUND = 3 ! Number of levels counted into the "ground multiplet" (see more below)
     atom%N_lines = 3000  ! Number of transitions
     atom%FILE_ADDRESS = TRIM(dir_molecular_data) // '/CMI/cmI.dat'  ! use dir_atomic_data for atoms
     atom%ABIN = 106  ! ABIN code. Match with list in DATA/mol_input/ABINCODES and also in DATIE.SF
     atom%E_ION_GS = 14.014*eV   ! Ground state ionization energy in eV
     atom%RR_ALPHA_EXISTS = .FALSE. ! Do level-specific recombination rates exist in the data file?
     atom%RR_BETA_EXISTS = .FALSE. ! If yes, do also T-dependencies exist for these?
     atom%UPSILON0_EXISTS = .FALSE. ! Do collision strengths exist in the data file?
     atom%UPSILON0_BETA_EXISTS = .FALSE. ! If yes, do power law temperature dependencies exist for these?
     atom%MINSHELL = ..  ! Innermost electron shell that contributes to photoionization
     atom%MAXSHELL = ..  ! Outermost electron shell that contributes to photoionization
```

Note on NGROUND : The most important use of this quantity is that the code allows non-thermal ionizations and excitations
only from the levels included in it (Basically, a framework to handle these processes from excited states is not in place).
For most elements at temperatures <~ 1E4 K the majority of atoms are in the ground multiplet as there are typically significant
jumps in energy to the next higher states. As such its no big issues to ignore non-thermal processes from the (sparsely populated)
excited states. However, for some elements the excitation energies to first, second,.. excited multiplets can be quite small,
and especially if the temperature is high the population in the ground multiplet may start to fall significantly below unity.
One should be aware that in such cases the non-thermal rates may become less accurate (in particular will one start to
underestimate the non-thermal ionization rate).
There are also a few secondary uses of NGROUND: The "ground multiplet" values for photoionization are allocated to these states,
and default recombination rate fractions (30%) "to ground".

5. `modelreader.f90`

Manual adding needed for molecules. For atoms, don't need to do anything.

6. Spencer-Fano files (to get nonthermal ionization and excitation rates):

a. PARAM.SF : Increase MIE to equal the new Nabin value.

b. DATIE.SF : 
- Make new entries e.g. ICE1

c. sf.f : Update IEMAX to equal the new Nabin value.

d. fsigi.f (add ionization channels):
- Add new entry in the XI array, e.g. XI(ICE1)=5.538  (ionization energy in eV)
- Add new entry e.g. ELSEIF (IE.EQ.ICE1) THEN...fill with 1) Nmax values (number of shells contributing to ionization) 2) the four C coefficients (describe the ionization cross section) and 3) If Nmax=1, do XIE(1)=XI(IE). If Nmax>1, add further lines for each shell e.g. XIE(2)=24.2,XE(3)=37.5,...  Here the numbers have to be entered manually.

e. OPTIONAL: fsige.f (add excitation channels)
- Specify NL entry, e.g. NL(ICE1)=3   3 excitation channels
- Specify EX(ICE1,1), EX(ICE1,2), ... (excitation energies in eV)
- Add new entry ELSEIF(IE.EQ.ICE1)...  here cross sections (FSIGE) can be specified depending on energy Ekin.


-------------------------------------
IMPORTANT NOTES ON RUNNING
-------------------------------------

Currently, in a few places in the code (e.g. in alfa_photoionization) there remains hardcoding of what elements occupy which positions in the 2*Nelems listing (first 2 ionization stages for Nelems elements (the value of which is set in m1.f90).

This means currently new elements can be added, but none can be removed (e.g. if you want to run a model with only CeI, all the old Z<26 elements must still be included in the atoms array in main.f90/main_radtrans.f90). By setting their abundance to zero they dont do anything. Work to fix this is in progress.

-------------------------------------
PERMANENT CHANGES DONE
-------------------------------------

1. `files.f90` - Added new directory path:
```fortran
CHARACTER(LEN=*), PARAMETER :: dir_molecular_data = 'DATA/molecules'
```

2. `makeatoms.f90` - Added new condition, to ignore CO contribution:
```fortran
if (elname(3:5) == 'I  ' .and. elname(1:2) .ne. 'CM') then ! This atom is neutral
```

3. `modelreader.f90` - Made sure that the CO mass was read in properly, by changing the code to 
```fortran
                      + 2*DOT_PRODUCT(NFMAT(i,1:Nelem),ionization_fraction(i,1:Nelem,3)) 
! sl: this is only for when the code contains more than 3 ionization levels
!                      + 3*DOT_PRODUCT(NFMAT(i,1:Nelem),ionization_fraction(i,1:Nelem,4)) &
!                      + 4*DOT_PRODUCT(NFMAT(i,1:Nelem),ionization_fraction(i,1:Nelem,5)) 
```
This is due to the form of the ionfrac.dat*** file, where the fourth column represent the molecular mass, not the thricly ionized state. 

*NOTE* This will only work for the case of CO and maybe SiO if Si is used as a parent atom. This need to be fixed, if cooling from other molecules are to be implemented. 

A cycle statement was also added:
```fortran
IF (atom%NAME .eq. 'H2   ' .or. atom%NAME .eq. 'CMI  ') CYCLE
```
to catch exeptions involving CO. 














































