SUBROUTINE fe2HECSreader
USE M1, ONLY : globid
USE FILES, ONLY : file_HECS
USE MPI
IMPLICIT NONE

INTEGER, PARAMETER :: nlower=5, nlevR=262   ! CARE increase nlower (and add boltzmann factors for higher temperature. 1 is ok at 8 years). Number of levels in Ramsbottom FeII is 262
INTEGER, PARAMETER :: Ebinsmax=350  ! what is this?
INTEGER, PARAMETER :: ntrans=1000  ! Maximum number of transitions (the actual number (mtrans) is not known prior to computation because some levels dont exist). TEMP this needs to be synched with the value set in fsige.f in TWO places
! --------------------------------------------------------------
! TASK : Reads non-thermal cross sections for Fe II and stores in common block cfe2
!
!
! -------------------------------------------------------------
COMMON/cfe2/EFE2, CRFE2, C_IPOL, i_CFvec, j_CFvec, fitvec, hecvec, deltaEvec, mtrans, Ebinsvec, ies1   ! all data stored in this common block, also used by sf and fsige

real, DIMENSION(ntrans,Ebinsmax) :: EFE2, CRFE2, C_IPOL
real, DIMENSION(ntrans) :: deltaEvec, hecvec
INTEGER, DIMENSION(ntrans) :: i_CFvec, j_CFvec, fitvec, Ebinsvec, ies1
integer :: mtrans, ierr
INTEGER :: i,j, m,Ebins, j_CF, i_CF, ifit1, ifit2, itr
CHARACTER(len=54) :: infil, file
REAL :: deijx, aval, hec1, hec2
CHARACTER(len=3), DIMENSION(nlevR) :: lev

data lev/'001','002','003','004','005','006','007','008', &
      '009','010', &
      '011','012','013','014','015','016','017','018','019','020', &
      '021','022','023','024','025','026','027','028','029','030', &
      '031','032','033','034','035','036','037','038','039','040', &
      '041','042','043','044','045','046','047','048','049','050', &
      '051','052','053','054','055','056','057','058','059','060', &
      '061','062','063','064','065','066','067','068','069','070', &
      '071','072','073','074','075','076','077','078','079','080', &
      '081','082','083','084','085','086','087','088','089','090', &
      '091','092','093','094','095','096','097','098','099','100', &
      '101','102','103','104','105','106','107','108','109','110', &
      '111','112','113','114','115','116','117','118','119','120', &
      '121','122','123','124','125','126','127','128','129','130', &
      '131','132','133','134','135','136','137','138','139','140', &
      '141','142','143','144','145','146','147','148','149','150', &
      '151','152','153','154','155','156','157','158','159','160', &
      '161','162','163','164','165','166','167','168','169','170', &
      '171','172','173','174','175','176','177','178','179','180', &
      '181','182','183','184','185','186','187','188','189','190', &
      '191','192','193','194','195','196','197','198','199','200', &
      '201','202','203','204','205','206','207','208','209','210', &
      '211','212','213','214','215','216','217','218','219','220', &
      '221','222','223','224','225','226','227','228','229','230', &
      '231','232','233','234','235','236','237','238','239','240', &
      '241','242','243','244','245','246','247','248','249','250', &
      '251','252','253','254','255','256','257','258','259','260', &
      '261','262'/

itr = 0

IF (globid .EQ. 0) THEN   ! Only master reads files...

   DO i = 1, nlower     ! for each lower level..

      DO j = i+1, nlevR   ! for each upper level..

         itr = itr + 1    ! count up number of transitions

         infil=TRIM(file_HECS)//lev(i)//'to'//lev(j)

         OPEN(11,file=infil,status='OLD')

           READ(11,*) Ebins    ! Number of energy bins

           IF (Ebins .GT. Ebinsmax) PRINT*, 'Too many energy bins!', Ebins, Ebinsmax

           READ(11,'(1pe12.4)') deijx          ! excitation energy deltaE in eV

           READ(11,*) j_CF, i_CF     ! Corresponding j and i levels in my Fe II atom (checked 2015-03)

           IF ((j_CF == 0) .OR. (i_CF==0)) THEN      ! CARE These levels are not included CF:s FeII-atom, skip transition

              itr = itr - 1

              GOTO 20

           END IF

           READ(11,'(1pe12.4)') aval          ! A-value for the transition (not used for anything)

           READ(11,'(i4,1pe12.4,i4,1pe12.4)') ifit1, hec1, ifit2, hec2     !  Type and constant for high-energy fit (see fsige.f for how its used)

           IF (hec1 < 1e-17) THEN  ! CARE Skip ones w small cross sections (largest are of order 1e-15 cm2)

              itr = itr - 1

              GOTO 20

           END IF

           DO m = 1, Ebins

              READ(11,'(2e15.6)') efe2(itr,m), crfe2(itr,m)            ! electron energy in eV and cross section (cm^-2 ?)

           ENDDO

           DO m = 1, Ebins-1

              c_ipol(itr,m) = (crfe2(itr,m+1)-crfe2(itr,m))/(efe2(itr,m+1)-efe2(itr,m))  ! the slope

           ENDDO

           Ebinsvec(itr) = Ebins  ! Store number of energy bins in this transition

           i_CFvec(itr) = i_CF

           j_CFvec(itr) = j_CF

           fitvec(itr) = ifit1   ! CARE assume allowed transition fit (ln(E)/E) for all transitions..there is a very small difference between ln(E)/E and 1/E. Use ifit2 and hec2 otherwise
           hecvec(itr) = hec1

           deltaEvec(itr) = deijx

20         CLOSE(11)

        END DO

     END DO

     mtrans = itr

     ies1(1:ntrans) = 1

!print *, 'mtrans fe2HECS', mtrans

END IF



call MPI_BCAST(efe2, ntrans*Ebinsmax, MPI_REAL, 0, MPI_COMM_WORLD, ierr)  ! Communicate data to slaves..

call MPI_BCAST(crfe2, ntrans*Ebinsmax, MPI_REAL, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(c_ipol, ntrans*Ebinsmax, MPI_REAL, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(Ebinsvec, ntrans, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(i_CFvec, ntrans, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(j_CFvec, ntrans, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(fitvec, ntrans, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(hecvec, ntrans, MPI_REAL, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(deltaEvec, ntrans, MPI_REAL, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(mtrans, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(ies1, ntrans, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

!IF (globid .EQ. 1) THEN

!  OPEN(10,file='out/fe2HECSreader/hecs.dat',STATUS='REPLACE'); WRITE(10,*) 'mtrans', mtrans

!  WRITE(10,*) 'efe2', efe2(10,:)

!  WRITE(10,*) 'crfe2', crfe2(100,:)

!  WRITE(10,*) 'cipol', c_ipol(200,:)

!  WRITE(10,*) 'i_cf', i_cfvec

!  WRITE(10,*) 'j_cf', j_cfvec

!  WRITE(10,*) 'fitvec', fitvec

!  WRITE(10,*) 'hecvec', hecvec

!  WRITE(10,*) 'deltaEvec', deltaEvec

!  WRITE(10,*), 'Ebinsvec', Ebinsvec

!  WRITE(10,*) 'ies1', ies1

!  close(10)

!END IF



END SUBROUTINE fe2HECSreader
