SUBROUTINE feI_recomb(T, gf1, N_LEVELS, alphaans)
USE M1, ONLY : feI_levs, Ts                                                 !new000225 New Nahar-recombination data for Fe III (from CF)    
IMPLICIT NONE 
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(N_LEVELS) :: gf1
INTEGER, INTENT(IN) :: N_LEVELS
DOUBLE PRECISION, DIMENSION(N_LEVELS), INTENT(OUT) :: alphaans
! ------------------------------------------------------------------------------------------------
! Task : Compute specific recombination rates for Fe I
! Input : T        Temperature [K]
!         gf1      Statistical weights
!         N_LEVELS  Number of levels
! Output : alphaans  Recombination rates [cm^3 s^-1]
! Globals : - 
! ------------------------------------------------------
DOUBLE PRECISION, DIMENSION(500) :: alpha
INTEGER :: i, j, bin, levs
DOUBLE PRECISION :: intervalfrac, rec_multiplet, recsum, rec_multiplet_indirect, recadd, gsum, gtot, beta, C0
INTEGER, DIMENSION(FeI_levs) :: levsvec
INTEGER, DIMENSION(FeI_levs,7) :: IND
CHARACTER :: trash
DOUBLE PRECISION, DIMENSION(Ts), SAVE :: T_specific
DOUBLE PRECISION, DIMENSION(FeI_levs,Ts), SAVE :: RECMATRIX
LOGICAL, SAVE :: FIRST = .TRUE.

alpha = 0
alphaans = 0

! ----------------------------------------------------------------------------------------------

IF (T .LE. 0) THEN
        WRITE(*,*) 'ERROR in feI_recomb: T <= 0'
        STOP 1
ELSE IF ( N_LEVELS .LT. 1) THEN
        WRITE(*,*) 'ERROR in fe_recomb: N_LEVELS < 1'
        STOP 1
ELSE IF (N_LEVELS > 500) THEN
        WRITE(*,*) 'ERROR in t_feI_recomb. N_levels > 500'
        STOP 1
ELSE IF ( (gf1(1) .LE. 0) .OR. (gf1(N_LEVELS) .LE. 0) )THEN
        WRITE(*,*) 'ERROR in feI_recomb: gaunt factors not set.'
        STOP 1
END IF

IF (FIRST) THEN


     T_specific = [1.00E+01,  1.26E+01,    1.58E+01,    2.00E+01,   2.51E+01,   3.16E+01,   3.98E+01,   5.01E+01,   6.31E+01,   7.94E+01,   1.00E+02,   1.26E+02,   1.58E+02,   2.00E+02,   2.51E+02,   3.16E+02,   3.98E+02,   5.01E+02,   6.31E+02,   7.94E+02,   1.00E+03,   1.26E+03,   1.58E+03,   2.00E+03,  2.51E+03 , 3.16E+03,  3.98E+03,  5.01E+03,  6.31E+03,  7.94E+03,  1.00E+04,  1.26E+04,  1.58E+04,  2.00E+04,  2.51E+04,  3.16E+04,  3.98E+04,  5.01E+04,  6.31E+04,  7.94E+04,  1.00E+05,  1.26E+05,  1.58E+05,  2.00E+05,  2.51E+05,  3.16E+05,  3.98E+05,  5.01E+05,  6.31E+05,  7.94E+05]!  1.00E+06  1.26E+06  1.58E+06  2.00E+06  2.51E+06  3.16E+06  3.98E+06  5.01E+06  6.31E+06  7.94E+06  1.00E+07  1.26E+07  1.58E+07  2.00E+07  2.51E+07  3.16E+07  3.98E+07  5.01E+07  6.31E+07  7.94E+07  1.00E+08  1.26E+08  1.58E+08  2.00E+08  2.51E+08  3.16E+08  3.98E+08  5.01E+08  6.31E+08  7.94E+08  1.00E+09]


     RECMATRIX(1,:) = [1.81E-14,  2.22E-14,    2.55E-14,    2.76E-14,   2.87E-14,   2.87E-14,   2.79E-14,   2.64E-14,   2.45E-14,   2.24E-14,   2.02E-14,   1.83E-14,   1.70E-14,   1.70E-14,   1.92E-14,   2.38E-14,   3.02E-14,   3.69E-14,   4.25E-14,   4.59E-14,   4.67E-14,   4.51E-14,   4.17E-14,   3.73E-14,  3.26E-14,  2.81E-14,  2.45E-14,  2.19E-14,  2.04E-14,  1.95E-14,  1.88E-14,  1.79E-14,  1.67E-14,  1.51E-14,  1.32E-14,  1.13E-14,  9.35E-15,  7.57E-15,  6.02E-15,  4.71E-15,  3.62E-15,  2.76E-15,  2.08E-15,  1.56E-15,  1.16E-15,  8.60E-16,  6.37E-16,  4.69E-16,  3.46E-16,  2.55E-16]
    
     RECMATRIX(2,:) = [6.15E-13,  5.47E-13,    4.87E-13,    4.33E-13,   3.85E-13,   3.42E-13,   3.04E-13,   2.70E-13,   2.39E-13,   2.12E-13,   1.89E-13,   1.69E-13,   1.52E-13,   1.36E-13,   1.22E-13,   1.10E-13,   9.88E-14,   8.90E-14,   8.04E-14,   7.27E-14,   6.58E-14,   5.95E-14,   5.37E-14,   4.84E-14,  4.37E-14,  3.97E-14,  3.62E-14,  3.34E-14,  3.10E-14,  2.91E-14,  2.76E-14,  2.63E-14,  2.53E-14,  2.44E-14,  2.37E-14,  2.32E-14,  2.27E-14,  2.17E-14,  2.08E-14,  1.96E-14,  1.77E-14,  1.61E-14,  1.40E-14,  1.23E-14,  1.04E-14,  8.68E-15,  7.29E-15,  5.95E-15,  4.90E-15,  4.01E-15]

     RECMATRIX(3,:) = [1.47E-13,  1.31E-13,    1.17E-13,    1.04E-13,   9.25E-14,   8.23E-14,   7.31E-14,   6.49E-14,   5.76E-14,   5.10E-14,   4.53E-14,   4.03E-14,   3.63E-14,   3.35E-14,   3.21E-14,   3.21E-14,   3.31E-14,   3.48E-14,   3.64E-14,   3.75E-14,   3.76E-14,   3.65E-14,   3.45E-14,   3.17E-14,  2.86E-14,  2.55E-14,  2.26E-14,  2.01E-14,  1.80E-14,  1.63E-14,  1.50E-14,  1.39E-14,  1.30E-14,  1.22E-14,  1.15E-14,  1.10E-14,  1.05E-14,  9.78E-15,  9.20E-15,  8.56E-15,  7.64E-15,  6.87E-15,  5.94E-15,  5.17E-15,  4.34E-15,  3.60E-15,  3.01E-15,  2.44E-15,  2.00E-15,  1.63E-15]

     RECMATRIX(4,:) = [1.88E-12,  1.67E-12,    1.48E-12,    1.32E-12,   1.18E-12,   1.05E-12,   9.31E-13,   8.28E-13,   7.36E-13,   6.54E-13,   5.81E-13,   5.16E-13,   4.58E-13,   4.06E-13,   3.59E-13,   3.18E-13,   2.81E-13,   2.48E-13,   2.18E-13,   1.92E-13,   1.69E-13,   1.48E-13,   1.30E-13,   1.14E-13,  9.94E-14,  8.67E-14,  7.55E-14,  6.58E-14,  5.74E-14,  5.01E-14,  4.39E-14,  3.85E-14,  3.39E-14,  2.97E-14,  2.60E-14,  2.26E-14,  1.95E-14,  1.66E-14,  1.39E-14,  1.16E-14,  9.42E-15,  7.65E-15,  6.10E-15,  4.87E-15,  3.83E-15,  2.99E-15,  2.34E-15,  1.81E-15,  1.41E-15,  1.10E-15]

     RECMATRIX(5,:) = [1.93E-12,  1.73E-12,    1.54E-12,    1.37E-12,   1.22E-12,   1.09E-12,   9.72E-13,   8.66E-13,   7.72E-13,   6.88E-13,   6.13E-13,   5.46E-13,   4.86E-13,   4.33E-13,   3.85E-13,   3.42E-13,   3.05E-13,   2.71E-13,   2.40E-13,   2.13E-13,   1.89E-13,   1.67E-13,   1.48E-13,   1.31E-13,  1.15E-13,  1.01E-13,  8.91E-14,  7.87E-14,  6.98E-14,  6.21E-14,  5.55E-14,  4.96E-14,  4.43E-14,  3.95E-14,  3.51E-14,  3.09E-14,  2.69E-14,  2.29E-14,  1.93E-14,  1.60E-14,  1.30E-14,  1.05E-14,  8.30E-15,  6.55E-15,  5.08E-15,  3.92E-15,  3.03E-15,  2.31E-15,  1.78E-15,  1.36E-15]

     RECMATRIX(6,:) = [9.72E-13,  8.66E-13,    7.71E-13,    6.87E-13,   6.11E-13,   5.45E-13,   4.85E-13,   4.32E-13,   3.85E-13,   3.43E-13,   3.05E-13,   2.71E-13,   2.42E-13,   2.15E-13,   1.91E-13,   1.70E-13,   1.51E-13,   1.34E-13,   1.19E-13,   1.05E-13,   9.29E-14,   8.20E-14,   7.23E-14,   6.35E-14,  5.58E-14,  4.90E-14,  4.31E-14,  3.80E-14,  3.37E-14,  2.99E-14,  2.64E-14,  2.31E-14,  2.01E-14,  1.74E-14,  1.49E-14,  1.27E-14,  1.07E-14,  8.97E-15,  7.44E-15,  6.11E-15,  4.92E-15,  3.95E-15,  3.12E-15,  2.46E-15,  1.92E-15,  1.48E-15,  1.15E-15,  8.79E-16,  6.78E-16,  5.22E-16]

     RECMATRIX(7,:) = [1.39E-12,  1.71E-12,    1.95E-12,    2.10E-12,   2.16E-12,   2.14E-12,   2.05E-12,   1.93E-12,   1.78E-12,   1.62E-12,   1.46E-12,   1.31E-12,   1.16E-12,   1.02E-12,   8.96E-13,   7.82E-13,   6.79E-13,   5.88E-13,   5.06E-13,   4.33E-13,   3.66E-13,   3.08E-13,   2.56E-13,   2.12E-13,  1.75E-13,  1.44E-13,  1.18E-13,  9.62E-14,  7.86E-14,  6.42E-14,  5.24E-14,  4.26E-14,  3.45E-14,  2.78E-14,  2.23E-14,  1.78E-14,  1.42E-14,  1.12E-14,  8.83E-15,  6.96E-15,  5.44E-15,  4.27E-15,  3.31E-15,  2.59E-15,  2.00E-15,  1.54E-15,  1.19E-15,  9.16E-16,  7.09E-16,  5.48E-16]

     RECMATRIX(8,:) = [7.24E-13,  8.94E-13,    1.03E-12,    1.14E-12,   1.20E-12,   1.23E-12,   1.22E-12,   1.20E-12,   1.16E-12,   1.11E-12,   1.05E-12,   9.94E-13,   9.35E-13,   8.77E-13,   8.27E-13,   7.89E-13,   7.67E-13,   7.54E-13,   7.39E-13,   7.11E-13,   6.63E-13,   6.00E-13,   5.25E-13,   4.48E-13,  3.74E-13,  3.06E-13,  2.48E-13,  1.98E-13,  1.57E-13,  1.24E-13,  9.75E-14,  7.64E-14,  5.97E-14,  4.66E-14,  3.63E-14,  2.84E-14,  2.22E-14,  1.74E-14,  1.38E-14,  1.10E-14,  8.68E-15,  6.96E-15,  5.51E-15,  4.42E-15,  3.49E-15,  2.74E-15,  2.18E-15,  1.71E-15,  1.35E-15,  1.07E-15]

     RECMATRIX(9,:) = [3.41E-13,  4.21E-13,    4.86E-13,    5.33E-13,   5.61E-13,   5.71E-13,   5.66E-13,   5.50E-13,   5.25E-13,   4.96E-13,   4.63E-13,   4.29E-13,   3.95E-13,   3.64E-13,   3.40E-13,   3.29E-13,   3.35E-13,   3.53E-13,   3.73E-13,   3.84E-13,   3.77E-13,   3.54E-13,   3.18E-13,   2.76E-13,  2.31E-13,  1.90E-13,  1.52E-13,  1.20E-13,  9.41E-14,  7.30E-14,  5.64E-14,  4.34E-14,  3.33E-14,  2.55E-14,  1.95E-14,  1.50E-14,  1.16E-14,  8.96E-15,  7.01E-15,  5.54E-15,  4.35E-15,  3.47E-15,  2.74E-15,  2.19E-15,  1.73E-15,  1.36E-15,  1.08E-15,  8.44E-16,  6.68E-16,  5.28E-16]

     RECMATRIX(10,:) = [6.77E-15,  9.02E-15,    1.20E-14,    1.57E-14,   1.97E-14,   2.31E-14,   2.53E-14,   2.59E-14,   2.49E-14,   2.29E-14,   2.05E-14,   1.83E-14,   1.69E-14,   1.67E-14,   1.76E-14,   1.99E-14,   2.36E-14,   2.88E-14,   3.52E-14,   4.19E-14,   4.77E-14,   5.18E-14,   5.37E-14,   5.36E-14,  5.18E-14,  4.87E-14,  4.45E-14,  3.97E-14,  3.46E-14,  2.95E-14,  2.47E-14,  2.03E-14,  1.65E-14,  1.33E-14,  1.06E-14,  8.34E-15,  6.54E-15,  5.10E-15,  3.95E-15,  3.05E-15,  2.34E-15,  1.79E-15,  1.36E-15,  1.03E-15,  7.80E-16,  5.87E-16,  4.44E-16,  3.33E-16,  2.52E-16,  1.91E-16]

     RECMATRIX(11,:) = [1.28E-13,  1.58E-13,    1.81E-13,    1.95E-13,   2.00E-13,   1.96E-13,   1.83E-13,   1.66E-13,   1.45E-13,   1.24E-13,   1.05E-13,   8.77E-14,   7.43E-14,   6.45E-14,   5.91E-14,   5.94E-14,   6.66E-14,   8.02E-14,   9.70E-14,   1.12E-13,   1.22E-13,   1.24E-13,   1.20E-13,   1.10E-13,  9.70E-14,  8.30E-14,  6.94E-14,  5.69E-14,  4.59E-14,  3.67E-14,  2.91E-14,  2.30E-14,  1.81E-14,  1.42E-14,  1.11E-14,  8.69E-15,  6.77E-15,  5.25E-15,  4.07E-15,  3.15E-15,  2.41E-15,  1.85E-15,  1.41E-15,  1.08E-15,  8.21E-16,  6.21E-16,  4.73E-16,  3.57E-16,  2.72E-16,  2.07E-16]

     RECMATRIX(12,:) = [4.26E-13,  3.76E-13,    3.33E-13,    2.97E-13,   2.69E-13,   2.45E-13,   2.23E-13,   2.02E-13,   1.80E-13,   1.58E-13,   1.36E-13,   1.16E-13,   9.85E-14,   8.32E-14,   7.09E-14,   6.23E-14,   5.81E-14,   5.80E-14,   6.03E-14,   6.27E-14,   6.33E-14,   6.13E-14,   5.68E-14,   5.05E-14,  4.33E-14,  3.61E-14,  2.95E-14,  2.37E-14,  1.89E-14,  1.51E-14,  1.21E-14,  9.87E-15,  8.12E-15,  6.75E-15,  5.66E-15,  4.78E-15,  4.08E-15,  3.46E-15,  2.96E-15,  2.54E-15,  2.14E-15,  1.81E-15,  1.50E-15,  1.25E-15,  1.02E-15,  8.24E-16,  6.72E-16,  5.35E-16,  4.31E-16,  3.46E-16]

     RECMATRIX(13,:) = [8.64E-14,  1.06E-13,    1.21E-13,    1.32E-13,   1.37E-13,   1.37E-13,   1.33E-13,   1.26E-13,   1.17E-13,   1.06E-13,   9.50E-14,   8.33E-14,   7.17E-14,   6.08E-14,   5.09E-14,   4.27E-14,   3.68E-14,   3.35E-14,   3.30E-14,   3.47E-14,   3.77E-14,   4.11E-14,   4.39E-14,   4.57E-14,  4.63E-14,  4.54E-14,  4.33E-14,  4.01E-14,  3.62E-14,  3.19E-14,  2.75E-14,  2.32E-14,  1.93E-14,  1.58E-14,  1.27E-14,  1.01E-14,  7.94E-15,  6.16E-15,  4.73E-15,  3.60E-15,  2.72E-15,  2.04E-15,  1.52E-15,  1.13E-15,  8.35E-16,  6.15E-16,  4.54E-16,  3.33E-16,  2.45E-16,  1.81E-16]

     RECMATRIX(14,:) = [1.11E-13,  1.3550E-13,  1.5497E-13,  1.6797E-13, 1.7433E-13, 1.7455E-13, 1.6951E-13, 1.6031E-13, 1.4805E-13, 1.3379E-13, 1.1847E-13, 1.0291E-13, 8.7878E-14, 7.4145E-14, 6.2588E-14, 5.4055E-14, 4.9093E-14, 4.7718E-14, 4.9342E-14, 5.2863E-14, 5.6946E-14, 6.0396E-14, 6.2439E-14, 6.2751E-14, 6.1323E-14, 5.8325E-14, 5.4031E-14, 4.8800E-14, 4.3029E-14, 3.7104E-14, 3.1350E-14, 2.6006E-14, 2.1216E-14, 1.7046E-14, 1.3503E-14, 1.0558E-14, 8.1557E-15, 6.2287E-15, 4.7124E-15, 3.5356E-15, 2.6305E-15, 1.9474E-15, 1.4324E-15, 1.0510E-15, 7.6721E-16, 5.5862E-16, 4.0681E-16, 2.9517E-16, 2.1455E-16, 1.5596E-16]

     RECMATRIX(15,:) = [3.84E-15,  3.4144E-15,  3.0347E-15,  2.6952E-15, 2.3917E-15, 2.1200E-15, 1.8767E-15, 1.6587E-15, 1.4631E-15, 1.2876E-15, 1.1300E-15, 9.8854E-16, 8.6155E-16, 7.4774E-16, 6.4623E-16, 5.5716E-16, 4.8212E-16, 4.2332E-16, 3.8116E-16, 3.5265E-16, 3.3225E-16, 3.1469E-16, 2.9858E-16, 2.9720E-16, 3.7351E-16, 6.8714E-16, 1.4661E-15, 2.8501E-15, 4.7306E-15, 6.7601E-15, 8.5243E-15, 9.7218E-15, 1.0236E-14, 1.0109E-14, 9.4713E-15, 8.4906E-15, 7.3305E-15, 6.1253E-15, 4.9813E-15, 3.9585E-15, 3.0821E-15, 2.3639E-15, 1.7872E-15, 1.3386E-15, 9.9265E-16, 7.3109E-16, 5.3642E-16, 3.9128E-16, 2.8509E-16, 2.0733E-16]

     RECMATRIX(16,:) = [1.14E-12,  1.0134E-12,  8.9905E-13,  7.9638E-13, 7.0409E-13, 6.2097E-13, 5.4592E-13, 4.7794E-13, 4.1613E-13, 3.5973E-13, 3.0815E-13, 2.6103E-13, 2.1834E-13, 1.8045E-13, 1.4812E-13, 1.2237E-13, 1.0405E-13, 9.3359E-14, 8.9498E-14, 9.0637E-14, 9.4412E-14, 9.8639E-14, 1.0179E-13, 1.0303E-13, 1.0199E-13, 9.8552E-14, 9.2890E-14, 8.5404E-14, 7.6669E-14, 6.7301E-14, 5.7846E-14, 4.8737E-14, 4.0287E-14, 3.2702E-14, 2.6094E-14, 2.0494E-14, 1.5867E-14, 1.2125E-14, 9.1675E-15, 6.8678E-15, 5.1001E-15, 3.7668E-15, 2.7642E-15, 2.0227E-15, 1.4729E-15, 1.0697E-15, 7.7679E-16, 5.6218E-16, 4.0741E-16, 2.9525E-16]

     RECMATRIX(17,:) = [1.81E-15,  2.2290E-15,  2.5674E-15,  2.8038E-15, 2.9347E-15, 2.9696E-15, 2.9248E-15, 2.8193E-15, 2.6716E-15, 2.4992E-15, 2.3263E-15, 2.2305E-15, 2.4724E-15, 3.6011E-15, 6.2396E-15, 1.0535E-14, 1.5812E-14, 2.0859E-14, 2.4585E-14, 2.6471E-14, 2.6575E-14, 2.5277E-14, 2.3038E-14, 2.0293E-14, 1.7406E-14, 1.4659E-14, 1.2250E-14, 1.0293E-14, 8.7957E-15, 7.6696E-15, 6.7733E-15, 5.9784E-15, 5.2110E-15, 4.4548E-15, 3.7283E-15, 3.0603E-15, 2.4729E-15, 1.9673E-15, 1.5550E-15, 1.2223E-15, 9.4947E-16, 7.3997E-16, 5.7019E-16, 4.4209E-16, 3.3868E-16, 2.5894E-16, 1.9951E-16, 1.5191E-16, 1.1680E-16, 8.9866E-17]

     RECMATRIX(18,:) = [2.51E-14,  2.2372E-14,  1.9937E-14,  1.7765E-14, 1.5827E-14, 1.4099E-14, 1.2556E-14, 1.1180E-14, 9.9518E-15, 8.8555E-15, 7.8767E-15, 7.0023E-15, 6.2209E-15, 5.5220E-15, 4.8966E-15, 4.3363E-15, 3.8340E-15, 3.3830E-15, 2.9775E-15, 2.6124E-15, 2.2832E-15, 1.9860E-15, 1.7177E-15, 1.4782E-15, 1.2866E-15, 1.2454E-15, 1.6506E-15, 2.9885E-15, 5.6396E-15, 9.4602E-15, 1.3688E-14, 1.7303E-14, 1.9523E-14, 2.0075E-14, 1.9150E-14, 1.7200E-14, 1.4721E-14, 1.2123E-14, 9.6807E-15, 7.5434E-15, 5.7641E-15, 4.3367E-15, 3.2225E-15, 2.3712E-15, 1.7312E-15, 1.2560E-15, 9.0690E-16, 6.5220E-16, 4.6762E-16, 3.3447E-16]

     RECMATRIX(19,:) = [5.60E-14,  4.9478E-14,  4.3557E-14,  3.8168E-14, 3.3244E-14, 2.8753E-14, 2.4692E-14, 2.1060E-14, 1.7850E-14, 1.5047E-14, 1.2638E-14, 1.0668E-14, 9.3746E-15, 9.2923E-15, 1.1031E-14, 1.4719E-14, 1.9666E-14, 2.4655E-14, 2.8593E-14, 3.0931E-14, 3.1631E-14, 3.0930E-14, 2.9169E-14, 2.6731E-14, 2.3987E-14, 2.1226E-14, 1.8611E-14, 1.6205E-14, 1.4014E-14, 1.2024E-14, 1.0217E-14, 8.5815E-15, 7.1165E-15, 5.8262E-15, 4.7137E-15, 3.7755E-15, 3.0000E-15, 2.3618E-15, 1.8527E-15, 1.4474E-15, 1.1194E-15, 8.6713E-16, 6.6468E-16, 5.1149E-16, 3.8930E-16, 2.9555E-16, 2.2571E-16, 1.7057E-16, 1.2997E-16, 9.9098E-17]

     RECMATRIX(20,:) = [2.34E-15,  2.8782E-15,  3.3118E-15,  3.6134E-15, 3.7761E-15, 3.8087E-15, 3.7296E-15, 3.5630E-15, 3.3343E-15, 3.0684E-15, 2.7914E-15, 2.5527E-15, 2.4878E-15, 2.8719E-15, 4.0220E-15, 6.0344E-15, 8.6167E-15, 1.1227E-14, 1.3379E-14, 1.4823E-14, 1.5507E-14, 1.5477E-14, 1.4830E-14, 1.3730E-14, 1.2391E-14, 1.1024E-14, 9.7645E-15, 8.6633E-15, 7.7098E-15, 6.8696E-15, 6.1067E-15, 5.3939E-15, 4.7187E-15, 4.0842E-15, 3.5044E-15, 2.9936E-15, 2.5592E-15, 2.1660E-15, 1.8551E-15, 1.5936E-15, 1.3376E-15, 1.1399E-15, 9.4461E-16, 7.9324E-16, 6.4666E-16, 5.2358E-16, 4.2862E-16, 3.4185E-16, 2.7631E-16, 2.2255E-16]

     RECMATRIX(21,:) = [2.12E-15,  1.8991E-15,  1.7328E-15,  1.6259E-15, 1.5618E-15, 1.5123E-15, 1.4508E-15, 1.3621E-15, 1.2451E-15, 1.1092E-15, 9.6945E-16, 8.4101E-16, 7.3518E-16, 6.5753E-16, 6.1298E-16, 6.2485E-16, 7.7633E-16, 1.2380E-15, 2.1919E-15, 3.6568E-15, 5.3891E-15, 6.9826E-15, 8.0864E-15, 8.5709E-15, 8.5472E-15, 8.2624E-15, 7.9729E-15, 7.8677E-15, 8.0211E-15, 8.3605E-15, 8.6951E-15, 8.8139E-15, 8.5874E-15, 8.0068E-15, 7.1561E-15, 6.1587E-15, 5.1313E-15, 4.1583E-15, 3.2950E-15, 2.5625E-15, 1.9604E-15, 1.4821E-15, 1.1077E-15, 8.2197E-16, 6.0501E-16, 4.4292E-16, 3.2340E-16, 2.3496E-16, 1.7063E-16, 1.2375E-16]

     RECMATRIX(22,:) = [9.35E-16,  8.3696E-16,  7.5002E-16,  6.7287E-16, 6.0458E-16, 5.4430E-16, 4.9139E-16, 4.4558E-16, 4.0723E-16, 3.7734E-16, 3.5893E-16, 3.6948E-16, 4.7707E-16, 8.3038E-16, 1.5993E-15, 2.8218E-15, 4.3078E-15, 5.7185E-15, 6.7479E-15, 7.2498E-15, 7.2403E-15, 6.8302E-15, 6.1568E-15, 5.3463E-15, 4.4999E-15, 3.6909E-15, 2.9652E-15, 2.3449E-15, 1.8337E-15, 1.4238E-15, 1.1013E-15, 8.5131E-16, 6.6000E-16, 5.1600E-16, 4.1023E-16, 3.3512E-16, 2.8373E-16, 2.4293E-16, 2.1701E-16, 1.9709E-16, 1.7347E-16, 1.5650E-16, 1.3540E-16, 1.1906E-16, 1.0035E-16, 8.3680E-17, 7.0566E-17, 5.7515E-17, 4.7527E-17, 3.9020E-17]

     RECMATRIX(23,:) = [2.20E-15,  2.7106E-15,  3.1159E-15,  3.3943E-15, 3.5415E-15, 3.5691E-15, 3.4975E-15, 3.3529E-15, 3.1664E-15, 2.9722E-15, 2.7995E-15, 2.6689E-15, 2.5945E-15, 2.5845E-15, 2.6306E-15, 2.7017E-15, 2.7533E-15, 2.7468E-15, 2.6628E-15, 2.5031E-15, 2.2841E-15, 2.0291E-15, 1.7620E-15, 1.5038E-15, 1.2750E-15, 1.1132E-15, 1.1178E-15, 1.4809E-15, 2.4116E-15, 3.9511E-15, 5.8529E-15, 7.6699E-15, 8.9754E-15, 9.5388E-15, 9.3634E-15, 8.6160E-15, 7.5249E-15, 6.3014E-15, 5.1019E-15, 4.0208E-15, 3.1008E-15, 2.3506E-15, 1.7574E-15, 1.2997E-15, 9.5273E-16, 6.9357E-16, 5.0220E-16, 3.6200E-16, 2.6008E-16, 1.8636E-16]

     RECMATRIX(24,:) = [4.13E-16,  5.4090E-16,  6.8362E-16,  8.3278E-16, 9.6775E-16, 1.0630E-15, 1.1005E-15, 1.0771E-15, 1.0033E-15, 8.9705E-16, 7.7823E-16, 6.6454E-16, 5.6903E-16, 4.9981E-16, 4.6439E-16, 4.8088E-16, 5.9755E-16, 9.0287E-16, 1.4831E-15, 2.3328E-15, 3.3085E-15, 4.1911E-15, 4.8104E-15, 5.1316E-15, 5.2457E-15, 5.3016E-15, 5.4699E-15, 5.9333E-15, 6.8087E-15, 8.0205E-15, 9.2781E-15, 1.0217E-14, 1.0580E-14, 1.0309E-14, 9.5089E-15, 8.3719E-15, 7.0914E-15, 5.8160E-15, 4.6507E-15, 3.6426E-15, 2.8018E-15, 2.1288E-15, 1.5975E-15, 1.1904E-15, 8.7937E-16, 6.4609E-16, 4.7371E-16, 3.4545E-16, 2.5199E-16, 1.8363E-16]

     RECMATRIX(25,:) = [1.15E-14,  1.0152E-14,  8.9564E-15,  7.8760E-15, 6.8993E-15, 6.0153E-15, 5.2148E-15, 4.4901E-15, 3.8355E-15, 3.2477E-15, 2.7260E-15, 2.2732E-15, 1.8965E-15, 1.6153E-15, 1.4724E-15, 1.5337E-15, 1.8620E-15, 2.4982E-15, 3.4696E-15, 4.7727E-15, 6.3070E-15, 7.8418E-15, 9.0852E-15, 9.8086E-15, 9.9327E-15, 9.5325E-15, 8.7799E-15, 7.8644E-15, 6.9369E-15, 6.0979E-15, 5.4120E-15, 4.9100E-15, 4.5750E-15, 4.3413E-15, 4.1226E-15, 3.8508E-15, 3.4986E-15, 3.0752E-15, 2.6186E-15, 2.1651E-15, 1.7421E-15, 1.3731E-15, 1.0612E-15, 8.0935E-16, 6.0887E-16, 4.5370E-16, 3.3615E-16, 2.4710E-16, 1.8125E-16, 1.3256E-16]

     RECMATRIX(26,:) = [3.30E-15,  4.0581E-15,  4.6435E-15,  5.0101E-15, 5.1488E-15, 5.0854E-15, 4.8654E-15, 4.5388E-15, 4.1495E-15, 3.7313E-15, 3.3098E-15, 2.9151E-15, 2.6153E-15, 2.5479E-15, 2.8757E-15, 3.6614E-15, 4.7959E-15, 6.0784E-15, 7.3468E-15, 8.5049E-15, 9.4571E-15, 1.0078E-14, 1.0272E-14, 1.0054E-14, 9.5647E-15, 8.9943E-15, 8.4775E-15, 8.0517E-15, 7.6788E-15, 7.2897E-15, 6.8240E-15, 6.2527E-15, 5.5859E-15, 4.8626E-15, 4.1332E-15, 3.4428E-15, 2.8220E-15, 2.2755E-15, 1.8219E-15, 1.4490E-15, 1.1366E-15, 8.9353E-16, 6.9321E-16, 5.4061E-16, 4.1596E-16, 3.1911E-16, 2.4659E-16, 1.8813E-16, 1.4492E-16, 1.1166E-16]

     RECMATRIX(27,:) = [3.46E-14,  3.0799E-14,  2.7414E-14,  2.4387E-14, 2.1680E-14, 1.9259E-14, 1.7095E-14, 1.5175E-14, 1.3512E-14, 1.2140E-14, 1.1073E-14, 1.0266E-14, 9.6263E-15, 9.0558E-15, 8.4882E-15, 7.9005E-15, 7.2978E-15, 6.6920E-15, 6.0905E-15, 5.4962E-15, 4.9128E-15, 4.3478E-15, 3.8119E-15, 3.3157E-15, 2.8673E-15, 2.4714E-15, 2.1324E-15, 1.8601E-15, 1.6706E-15, 1.5745E-15, 1.5601E-15, 1.5901E-15, 1.6171E-15, 1.6039E-15, 1.5350E-15, 1.4151E-15, 1.2603E-15, 1.0849E-15, 9.1338E-16, 7.5383E-16, 6.0737E-16, 4.8609E-16, 3.8168E-16, 2.9953E-16, 2.3129E-16, 1.7759E-16, 1.3696E-16, 1.0425E-16, 7.9958E-17, 6.1305E-17]

     RECMATRIX(28,:) = [1.15E-16,  1.1562E-16,  1.3366E-16,  1.6915E-16, 2.1454E-16, 2.5739E-16, 2.8657E-16, 2.9672E-16, 2.8875E-16, 2.6787E-16, 2.4090E-16, 2.1427E-16, 1.9266E-16, 1.7872E-16, 1.7507E-16, 1.9556E-16, 2.9514E-16, 5.9426E-16, 1.2299E-15, 2.2308E-15, 3.4404E-15, 4.5810E-15, 5.4070E-15, 5.8190E-15, 5.8697E-15, 5.6911E-15, 5.4162E-15, 5.1399E-15, 4.9078E-15, 4.7143E-15, 4.5166E-15, 4.2650E-15, 3.9333E-15, 3.5282E-15, 3.0796E-15, 2.6233E-15, 2.1889E-15, 1.7901E-15, 1.4448E-15, 1.1520E-15, 9.0395E-16, 7.0604E-16, 5.4387E-16, 4.1869E-16, 3.1837E-16, 2.4100E-16, 1.8302E-16, 1.3759E-16, 1.0407E-16, 7.8727E-17]

     RECMATRIX(29,:) = [2.05E-13,  1.8295E-13,  1.6321E-13,  1.4557E-13, 1.2981E-13, 1.1574E-13, 1.0318E-13, 9.1972E-14, 8.1972E-14, 7.3050E-14, 6.5092E-14, 5.7993E-14, 5.1660E-14, 4.6010E-14, 4.0970E-14, 3.6472E-14, 3.2457E-14, 2.8872E-14, 2.5669E-14, 2.2807E-14, 2.0248E-14, 1.7957E-14, 1.5905E-14, 1.4065E-14, 1.2414E-14, 1.0935E-14, 9.6433E-15, 8.6219E-15, 8.0414E-15, 8.0575E-15, 8.6440E-15, 9.5290E-15, 1.0316E-14, 1.0676E-14, 1.0473E-14, 9.7529E-15, 8.6754E-15, 7.4119E-15, 6.1410E-15, 4.9602E-15, 3.9131E-15, 3.0456E-15, 2.3319E-15, 1.7738E-15, 1.3333E-15, 9.9597E-16, 7.4371E-16, 5.5088E-16, 4.0917E-16, 3.0374E-16]

     RECMATRIX(30,:) = [2.65E-13,  2.3604E-13,  2.1047E-13,  1.8764E-13, 1.6727E-13, 1.4909E-13, 1.3288E-13, 1.1842E-13, 1.0553E-13, 9.4025E-14, 8.3767E-14, 7.4618E-14, 6.6457E-14, 5.9176E-14, 5.2679E-14, 4.6881E-14, 4.1704E-14, 3.7081E-14, 3.2950E-14, 2.9257E-14, 2.5952E-14, 2.2993E-14, 2.0341E-14, 1.7961E-14, 1.5825E-14, 1.3917E-14, 1.2291E-14, 1.1205E-14, 1.1207E-14, 1.2872E-14, 1.6253E-14, 2.0584E-14, 2.4606E-14, 2.7200E-14, 2.7841E-14, 2.6629E-14, 2.4063E-14, 2.0743E-14, 1.7229E-14, 1.3887E-14, 1.0915E-14, 8.4269E-15, 6.3993E-15, 4.8094E-15, 3.5748E-15, 2.6377E-15, 1.9388E-15, 1.4161E-15, 1.0333E-15, 7.5266E-16]

     RECMATRIX(31,:) = [1.09E-13,  9.7192E-14,  8.6681E-14,  7.7292E-14, 6.8908E-14, 6.1425E-14, 5.4748E-14, 4.8789E-14, 4.3473E-14, 3.8730E-14, 3.4499E-14, 3.0724E-14, 2.7355E-14, 2.4348E-14, 2.1664E-14, 1.9268E-14, 1.7127E-14, 1.5213E-14, 1.3501E-14, 1.1968E-14, 1.0595E-14, 9.3646E-15, 8.2607E-15, 7.2696E-15, 6.3800E-15, 5.5850E-15, 4.8915E-15, 4.3351E-15, 3.9840E-15, 3.8998E-15, 4.0705E-15, 4.3826E-15, 4.6684E-15, 4.7863E-15, 4.6722E-15, 4.3418E-15, 3.8595E-15, 3.2977E-15, 2.7330E-15, 2.2079E-15, 1.7422E-15, 1.3557E-15, 1.0377E-15, 7.8885E-16, 5.9254E-16, 4.4227E-16, 3.2986E-16, 2.4407E-16, 1.8102E-16, 1.3417E-16]

     RECMATRIX(32,:) = [2.95E-14,  3.6309E-14,  4.1862E-14,  4.5770E-14, 4.7976E-14, 4.8630E-14, 4.7995E-14, 4.6372E-14, 4.4049E-14, 4.1279E-14, 3.8262E-14, 3.5156E-14, 3.2077E-14, 2.9112E-14, 2.6325E-14, 2.3754E-14, 2.1406E-14, 1.9268E-14, 1.7314E-14, 1.5516E-14, 1.3855E-14, 1.2316E-14, 1.0895E-14, 9.5875E-15, 8.3902E-15, 7.3019E-15, 6.3340E-15, 5.5315E-15, 4.9805E-15, 4.7596E-15, 4.8591E-15, 5.1481E-15, 5.4295E-15, 5.5346E-15, 5.3840E-15, 4.9910E-15, 4.4271E-15, 3.7754E-15, 3.1209E-15, 2.5138E-15, 1.9779E-15, 1.5333E-15, 1.1695E-15, 8.8499E-16, 6.6204E-16, 4.9201E-16, 3.6499E-16, 2.6879E-16, 1.9820E-16, 1.4600E-16]

     RECMATRIX(33,:) = [8.85E-14,  7.8766E-14,  7.0091E-14,  6.2383E-14, 5.5530E-14, 4.9433E-14, 4.4004E-14, 3.9170E-14, 3.4861E-14, 3.1021E-14, 2.7595E-14, 2.4539E-14, 2.1810E-14, 1.9372E-14, 1.7193E-14, 1.5243E-14, 1.3498E-14, 1.1933E-14, 1.0528E-14, 9.2663E-15, 8.1306E-15, 7.1072E-15, 6.1839E-15, 5.3500E-15, 4.5968E-15, 3.9217E-15, 3.3454E-15, 2.9495E-15, 2.8924E-15, 3.3324E-15, 4.2775E-15, 5.5100E-15, 6.6730E-15, 7.4468E-15, 7.6737E-15, 7.3733E-15, 6.6810E-15, 5.7679E-15, 4.7895E-15, 3.8540E-15, 3.0230E-15, 2.3246E-15, 1.7583E-15, 1.3137E-15, 9.7108E-16, 7.1208E-16, 5.1915E-16, 3.7638E-16, 2.7201E-16, 1.9602E-16]

     RECMATRIX(34,:) = [2.38E-13,  2.1178E-13,  1.8876E-13,  1.6821E-13, 1.4989E-13, 1.3355E-13, 1.1898E-13, 1.0597E-13, 9.4376E-14, 8.4029E-14, 7.4799E-14, 6.6566E-14, 5.9227E-14, 5.2700E-14, 4.6911E-14, 4.1786E-14, 3.7239E-14, 3.3177E-14, 2.9517E-14, 2.6194E-14, 2.3167E-14, 2.0410E-14, 1.7904E-14, 1.5633E-14, 1.3580E-14, 1.1736E-14, 1.0119E-14, 8.8189E-15, 7.9950E-15, 7.7734E-15, 8.1039E-15, 8.7238E-15, 9.2765E-15, 9.4776E-15, 9.2095E-15, 8.5142E-15, 7.5267E-15, 6.3970E-15, 5.2697E-15, 4.2299E-15, 3.3181E-15, 2.5638E-15, 1.9500E-15, 1.4709E-15, 1.0973E-15, 8.1323E-16, 6.0136E-16, 4.4165E-16, 3.2459E-16, 2.3829E-16]

     RECMATRIX(35,:) = [2.56E-13,  2.2814E-13,  2.0328E-13,  1.8111E-13, 1.6135E-13, 1.4373E-13, 1.2801E-13, 1.1399E-13, 1.0148E-13, 9.0323E-14, 8.0361E-14, 7.1466E-14, 6.3520E-14, 5.6418E-14, 5.0067E-14, 4.4382E-14, 3.9290E-14, 3.4724E-14, 3.0624E-14, 2.6939E-14, 2.3622E-14, 2.0632E-14, 1.7934E-14, 1.5497E-14, 1.3296E-14, 1.1328E-14, 9.6714E-15, 8.6193E-15, 8.7232E-15, 1.0506E-14, 1.3951E-14, 1.8271E-14, 2.2247E-14, 2.4830E-14, 2.5534E-14, 2.4466E-14, 2.2105E-14, 1.9033E-14, 1.5765E-14, 1.2658E-14, 9.9092E-15, 7.6059E-15, 5.7441E-15, 4.2847E-15, 3.1630E-15, 2.3164E-15, 1.6864E-15, 1.2211E-15, 8.8119E-16, 6.3405E-16]

     RECMATRIX(36,:) = [1.82E-13,  1.6295E-13,  1.4601E-13,  1.3079E-13, 1.1709E-13, 1.0478E-13, 9.3781E-14, 8.4041E-14, 7.5474E-14, 6.7964E-14, 6.1367E-14, 5.5518E-14, 5.0232E-14, 4.5327E-14, 4.0643E-14, 3.6079E-14, 3.1607E-14, 2.7285E-14, 2.3236E-14, 1.9607E-14, 1.6510E-14, 1.3989E-14, 1.2010E-14, 1.0489E-14, 9.3146E-15, 8.3798E-15, 7.6047E-15, 6.9775E-15, 6.5845E-15, 6.5639E-15, 6.9708E-15, 7.6717E-15, 8.3821E-15, 8.8132E-15, 8.7993E-15, 8.3334E-15, 7.5219E-15, 6.5050E-15, 5.4410E-15, 4.4262E-15, 3.5107E-15, 2.7416E-15, 2.1039E-15, 1.6015E-15, 1.2040E-15, 8.9888E-16, 6.7010E-16, 4.9553E-16, 3.6710E-16, 2.7169E-16]

     RECMATRIX(37,:) = [1.41E-13,  1.2637E-13,  1.1313E-13,  1.0108E-13, 8.9995E-14, 7.9806E-14, 7.0526E-14, 6.2194E-14, 5.4813E-14, 4.8338E-14, 4.2678E-14, 3.7722E-14, 3.3347E-14, 2.9433E-14, 2.5869E-14, 2.2575E-14, 1.9512E-14, 1.6694E-14, 1.4176E-14, 1.2023E-14, 1.0266E-14, 8.8843E-15, 7.8148E-15, 6.9757E-15, 6.2902E-15, 5.6979E-15, 5.1631E-15, 4.6884E-15, 4.3251E-15, 4.1484E-15, 4.1897E-15, 4.3871E-15, 4.6055E-15, 4.7072E-15, 4.6127E-15, 4.3168E-15, 3.8683E-15, 3.3310E-15, 2.7801E-15, 2.2597E-15, 1.7920E-15, 1.4004E-15, 1.0755E-15, 8.1986E-16, 6.1714E-16, 4.6141E-16, 3.4460E-16, 2.5523E-16, 1.8946E-16, 1.4052E-16]

     RECMATRIX(38,:) = [8.92E-14,  7.9588E-14,  7.0963E-14,  6.3249E-14, 5.6354E-14, 5.0191E-14, 4.4683E-14, 3.9761E-14, 3.5365E-14, 3.1446E-14, 2.7964E-14, 2.4880E-14, 2.2159E-14, 1.9769E-14, 1.7677E-14, 1.5841E-14, 1.4210E-14, 1.2729E-14, 1.1358E-14, 1.0072E-14, 8.8645E-15, 7.7378E-15, 6.6984E-15, 5.7503E-15, 4.8946E-15, 4.1314E-15, 3.4674E-15, 2.9323E-15, 2.5870E-15, 2.4954E-15, 2.6617E-15, 2.9947E-15, 3.3412E-15, 3.5601E-15, 3.5777E-15, 3.3954E-15, 3.0643E-15, 2.6464E-15, 2.2110E-15, 1.7974E-15, 1.4249E-15, 1.1136E-15, 8.5542E-16, 6.5265E-16, 4.9171E-16, 3.6810E-16, 2.7547E-16, 2.0441E-16, 1.5213E-16, 1.1315E-16]

     RECMATRIX(39,:) = [8.99E-15,  8.0474E-15,  7.2065E-15,  6.4578E-15, 5.7925E-15, 5.2028E-15, 4.6817E-15, 4.2232E-15, 3.8226E-15, 3.4791E-15, 3.2027E-15, 3.0333E-15, 3.0580E-15, 3.3889E-15, 4.0767E-15, 5.0246E-15, 5.9965E-15, 6.7264E-15, 7.0387E-15, 6.9019E-15, 6.4025E-15, 5.6805E-15, 4.8711E-15, 4.0733E-15, 3.3450E-15, 2.7142E-15, 2.2001E-15, 1.8395E-15, 1.6947E-15, 1.8148E-15, 2.1743E-15, 2.6540E-15, 3.0905E-15, 3.3513E-15, 3.3799E-15, 3.1942E-15, 2.8567E-15, 2.4408E-15, 2.0094E-15, 1.6052E-15, 1.2516E-15, 9.5730E-16, 7.2085E-16, 5.3622E-16, 3.9492E-16, 2.8859E-16, 2.0962E-16, 1.5148E-16, 1.0907E-16, 7.8298E-17]

     RECMATRIX(40,:) = [2.66E-14,  2.3683E-14,  2.1113E-14,  1.8818E-14, 1.6769E-14, 1.4941E-14, 1.3310E-14, 1.1854E-14, 1.0554E-14, 9.3942E-15, 8.3585E-15, 7.4334E-15, 6.6069E-15, 5.8680E-15, 5.2071E-15, 4.6155E-15, 4.0854E-15, 3.6101E-15, 3.1832E-15, 2.7995E-15, 2.4541E-15, 2.1428E-15, 1.8620E-15, 1.6084E-15, 1.3796E-15, 1.1749E-15, 1.0019E-15, 8.9072E-16, 9.0051E-16, 1.0910E-15, 1.4661E-15, 1.9445E-15, 2.3934E-15, 2.6942E-15, 2.7889E-15, 2.6858E-15, 2.4360E-15, 2.1036E-15, 1.7462E-15, 1.4042E-15, 1.1006E-15, 8.4534E-16, 6.3873E-16, 4.7653E-16, 3.5180E-16, 2.5760E-16, 1.8747E-16, 1.3569E-16, 9.7862E-17, 7.0362E-17]

     RECMATRIX(41,:) = [1.64E-14,  1.4566E-14,  1.2877E-14,  1.1360E-14, 9.9968E-15, 8.7715E-15, 7.6721E-15, 6.6905E-15, 5.8241E-15, 5.0752E-15, 4.4486E-15, 3.9547E-15, 3.6279E-15, 3.5411E-15, 3.7707E-15, 4.3143E-15, 5.0514E-15, 5.8176E-15, 6.5260E-15, 7.1928E-15, 7.8376E-15, 8.3922E-15, 8.7190E-15, 8.7028E-15, 8.3181E-15, 7.6317E-15, 6.7618E-15, 5.8360E-15, 4.9644E-15, 4.2243E-15, 3.6490E-15, 3.2251E-15, 2.9078E-15, 2.6455E-15, 2.4003E-15, 2.1542E-15, 1.9053E-15, 1.6467E-15, 1.4080E-15, 1.1879E-15, 9.7739E-16, 8.0374E-16, 6.4594E-16, 5.2107E-16, 4.1148E-16, 3.2290E-16, 2.5516E-16, 1.9803E-16, 1.5526E-16, 1.2156E-16]

     RECMATRIX(42,:) = [2.39E-14,  2.1268E-14,  1.8954E-14,  1.6883E-14, 1.5031E-14, 1.3373E-14, 1.1889E-14, 1.0564E-14, 9.3840E-15, 8.3446E-15, 7.4426E-15, 6.6827E-15, 6.0834E-15, 5.6660E-15, 5.4246E-15, 5.3059E-15, 5.2226E-15, 5.0891E-15, 4.8532E-15, 4.5051E-15, 4.0682E-15, 3.5811E-15, 3.0834E-15, 2.6063E-15, 2.1701E-15, 1.7849E-15, 1.4563E-15, 1.1953E-15, 1.0270E-15, 9.8181E-16, 1.0671E-15, 1.2448E-15, 1.4421E-15, 1.5860E-15, 1.6334E-15, 1.5788E-15, 1.4441E-15, 1.2594E-15, 1.0590E-15, 8.6441E-16, 6.8704E-16, 5.3724E-16, 4.1262E-16, 3.1427E-16, 2.3634E-16, 1.7649E-16, 1.3160E-16, 9.7332E-17, 7.2119E-17, 5.3385E-17]


     !DO i = 1, FeI_levs

     !   READ(22,'(A28)',ADVANCE='NO') trash
      
     !   READ(22,*) RECMATRIX(i,1:Ts)
             
     !END DO

     !CLOSE(22)

     FIRST = .FALSE.   ! Important

END IF

CALL findbin2(Ts, T_specific, T, bin)

IF (bin == Ts) bin = Ts - 1  ! avoid last bin since bin+1 is used below

i=1; levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/1,2,3,4,5/) ) ! a5D
i=2; levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/6,7,8,9,10/) ) ! a5F
i=3; levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/14,15,16/) ) ! a5P
i=4; levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/18,21,23,25,26/) ) ! z7D
i=5; levsvec(i) = 7; IND(i,1:levsvec(i)) = ( (/34,36,38,40,41,42,43/) ) ! z7F
i=6; levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/44,47,50/) ) ! z7P
i=7; levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/54,56,58,61,62/) ) ! z5D
i=8; levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/65,66,67,69,70/) ) ! z5F 
i=9; levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/73,78,79/) ) ! z5P
i=10;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/88,90,93,95,97/) ) ! y5D
i=11;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/91,96,98,100,103/) ) ! y5F
i=12;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/104,105,106,108,110/) ) ! z5G
i=13;levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/113,117,119/) ) ! y5P
i=14;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/124,125,128,130,132/) ) ! x5D
i=15;levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/126,127,131/) ) ! y7P
i=16;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/129,133,134,137,138/) ) ! x5F
i=17;levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/141,144,148/) ) ! x5P
i=18;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/143,151,155,159,160/) ) ! e7D
i=19;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/158,161,164,168,170/) ) ! w5D
i=20;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/162,163,173,175,176/) ) ! (3F)sp 5D
i=21;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/165,166,167,169,172/) ) ! (3F)sp 5D
i=22;levsvec(i) = 1; IND(i,1:levsvec(i)) = 171 ! y5S
i=23;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/174,177,181,183,186/) ) ! e5D
i=24;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/187,188,189,190,191/) ) ! x5G
i=25;levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/195,196,197/) ) ! w5P
i=26;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/200,202,212,213,214/) ) ! (4P)4p 5D
i=27;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/207,218,226,234,237/) ) ! e5F
i=28;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/217,220,223,225,228/) ) ! w5G
i=29;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/257,262,267,270,273/) ) ! 7D
i=30;levsvec(i) = 7; IND(i,1:levsvec(i)) = ( (/266,272,276,280,284,288,291/) ) ! 5p 7F
i=31;levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/274,289,299/) ) ! 7P
i=32;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/279,283,292,298,302/) ) ! f5D
i=33;levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/281,286,296/) ) ! e7P
i=34;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/282,293,301,315,324/) ) ! e5G
i=35;levsvec(i) = 7; IND(i,1:levsvec(i)) = ( (/290,300,316,320,328,330,331/) ) ! e7G
i=36;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/304,326,334,346,351/) ) ! u5F
i=37;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/308,322,335,347,349/) ) ! t5D
i=38;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/309,329,333,339,342/) ) ! f5F
i=39;levsvec(i) = 5; IND(i,1:levsvec(i)) = ( (/321,344,355,360,361/) ) ! g5D 
i=40;levsvec(i) = 1; IND(i,1:levsvec(i)) = 332 ! e7S
i=41;levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/338,350,357/) ) ! u5P
i=42;levsvec(i) = 3; IND(i,1:levsvec(i)) = ( (/348,353,356/) ) ! e5P


DO i = 1, FeI_levs

   levs = levsvec(i)

   beta = LOG10(RECMATRIX(i,bin)/RECMATRIX(i,bin+1))/LOG10(T_specific(bin)/T_specific(bin+1))

   C0 = RECMATRIX(i,bin)/((T_specific(bin))**beta)

   rec_multiplet = C0*T**beta   
   
   IF (MAXVAL(IND(i,1:levs)) .LE. N_LEVELS) THEN
   
      gtot = SUM(gf1(IND(i,1:levs)))

      DO j = 1, levs 

         alpha(IND(i,j)) = rec_multiplet*gf1(IND(i,j))/gtot

      END DO
      
   END IF

END DO

alphaans = alpha(1:N_LEVELS)


END
