RECURSIVE SUBROUTINE followphoton_ff(zone, r, E, lambda, sender, scattered, origzone, lastzone, origr, origlambda, dt_core, r_inblob, N_atoms, atoms, RT, ION_STRUCTURE, EXCCUBE_ABIN, NMOLMAT, radii, ne_vec, alfa_dust, Nlines, Nlinesvec, TAUCUBE, OLI, Nblobs, blobsize, af_accum, WLbins, xlog)
USE M1, ONLY : Nr, Ncore, Nabin, Npis, Nelem, v_out, atomm, myid, time, lambdacut, lambda_radfield, lambdaend_radfield, E_radfield, Nradfield, Nmol, N_CONTABS, colldeexc, OUTPHOTONS_SC, OUTPHOTONS_ABIN, OUTPHOTONS_LASTZONE, OUTPHOTONS_ORIGZONE, IDP, h2diss, DOPRINT, FIRFLUX, NPHOTOEXC, PHOTOEXC, RECURSIVES, Ephotoexc, idum, ngvec, energyin, energyout, BLEND, Nstages
USE PARAMETERS, ONLY : pi, c, h
USE INTERFACE, ONLY: findbin2, nextline2, nextline3, nextionedge, alfa_photoionization, createbetaandQvec, makeP, deexcite, twophoton_ay      ! Must not include interface for followphoton itself....
IMPLICIT NONE
DOUBLE PRECISION, INTENT(INOUT) :: r, E, lambda, dt_core, r_inblob
TYPE(atomm), DIMENSION(N_atoms) :: atoms
INTEGER, INTENT(IN) :: N_atoms, sender, Nlines, WLbins
INTEGER, INTENT(INOUT) :: zone, origzone, lastzone, Nblobs, scattered
DOUBLE PRECISION, INTENT(INOUT) :: origlambda, origr
DOUBLE PRECISION, DIMENSION(:,:), INTENT(IN) :: RT  !DIM: Nr*2
DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: radii, blobsize  ! DIM: Nr+1, Ncore
DOUBLE PRECISION, DIMENSION(:), INTENT(INOUT) :: ne_vec, alfa_dust  ! DIM:Nr
DOUBLE PRECISION, DIMENSION(:,:), INTENT(INOUT) :: ION_STRUCTURE  ! DIM:Nr*Nabin
REAL, DIMENSION(:,:,:), INTENT(IN) :: EXCCUBE_ABIN  ! DIM:Nr*Nabin*Npis
DOUBLE PRECISION, DIMENSION(:,:), INTENT(INOUT) :: NMOLMAT  !  DIM:Nr*Nmol
INTEGER, DIMENSION(:), INTENT(INOUT) :: Nlinesvec  ! DIM:Nr
REAL, DIMENSION(:,:,:), INTENT(INOUT) :: TAUCUBE  ! DIM:Nr*Nlines*6
LOGICAL, INTENT(INOUT) :: OLI
! LOGICAL, POINTER, INTENT(INOUT) :: BLEND
DOUBLE PRECISION, DIMENSION(:), INTENT(INOUT) :: af_accum  ! DIM:Ncore+1
DOUBLE PRECISION, DIMENSION(WLbins) :: xlog
DOUBLE PRECISION :: scr1,scr2
! ========================================================================
! Task : Follows a photon (packet) of comoving wavelength lambda end energy E, injected in zone, position r
! Input: Zone       Zone number
!        r          Dimensionless radius [r/r_out]
!        E          Energy of photon packet [ergs]
!        lambda     Wavelength [A]
!        sender     ID of emitting element
!        scattered  0=unscattered, 1=scattered
!        origzone   Which zone was the photon originally created in?
!        lastzone   Which zone was the photon emitted or last scattered?
!        origr      Which (dim.less) radius was the photon originally emitted from?
!        origlambda Which was the original wavelength [A] of the photon?
!        dt_core    Distance travelled through the core [dim.less]
!        r_inblob   The core consists of spherical blobs: current radius in a blob [normalized to blob radius]
!        N_atom     Number of atoms (length of atoms array)
!        atoms      Array with atoms (type atomm (M1)). These atoms need to have atomic data allocated.
!        RT         Array with zone radii and temperatures [dim less, K]
!        IONSTRUCTURE   Number densities for all ions (DIM:Nr*Nabin)   [cm^-3]
!        EXCCUBE_ABIN  Number density for first Npis states (DIM:Nr*Nabin*Npis) [cm^-3]. Need for continuum opacities.
!        NMOLMAT    Number density of molecules [cm^-3] (DIM:Nr*Nmol)
!        radii      Inner radii of all zones [dim.less]
!        ne_vec     Electron number density in each zone [cm^-3] DIM:Nr
!        alfa_dust  Dust absorption coefficient (gray) in each zone [cm^-1]. DIM:Nr
!        Nlines     Second dimension of TAUCUBE
!        Nlinesvec  Number of actual lines per zone (DIM:Nr)
!        TAUCUBE    Optical depths (DIM:Nr*Nlines*6)
!        OLI        Boolean, should only photoionization by locally emitted (same zone) photons be allowed?
!        BLEND      Boolean, should the zones in the core be macroscopically mixed?
!        Nblobs     Number of blobs per core zone
!        af_accum   Accumulated area factors for the core zones [dim.less]. af ~ ff^(2/3). af_accum(3) = af_1 + af_2 + af_3. Normalize so af_accum(Ncore) = 1.
!        wlbins     Length of wavelength array to compute emergent spectrum in  [A]
!        xlog       Wavelength array [A]. DIM:wlbins
! Output : None. Fills up global variables OUTPHOTONS_*, N_CONTABS, atoms%PHOTOEXCRATES, FIRFLUX
! ==================================================================
DOUBLE PRECISION :: lambda_line, lambda_next, z, cos_theta, sin_theta, ne, r_lastscatter, r_nextpos, r_nextline, r_edgeinside, r_edgeoutside, r_edgehit, r_smallest, dr_nextline, dr_toshelledge, dr_blobedge, r_fluor, lambda_fluor, E_deexcite, dr_tonext, tau_thisphoton, lambdaionedge, dr_nextionedge, r_nextionedge, tau_accum, tauline, T, Doppler_shift, lambda_escaped, alfa, alfa_Hn2, gamma, dtau, dtau_es, tau_es_this, Psum, nuse_FeI, alfa_star, Eheat, alfa_2s, alfa_2p, alfa_FeI, dt, cos_theta_int, cos_theta_es, sin_theta_int, r_inblob_new, dr_signchange, r_inblob_fluor, cf, nphotons, ionlimit, taumol, theta, beta_l, v, Eperphoton, relsum, r_temp, f_T, f_pass, left
INTEGER :: inner, POS_atomwithclosestline, i,linej, linei, level_out, from, fromsource, j, bin, bin2, bin3, Nup, sender_fluor, zone_fluor, lineind, zone_next, Ntheta
LOGICAL :: FOUNDNEXTLINE, NEXTLINEWITHIN, PHOTONWITHIN, LINEHIT, NEEDNEWLINE, NEEDNEWIONEDGE, IONEDGEEXISTS, ALIVE, PHOTON,BLOBEDGEHIT, IONIZATION, ERROR, MOLLINE, HOLEHIT, PRINTON=.FALSE., CROSSING
TYPE(atomm) :: atom
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: beta_s, beta_c, beta_e, Qvecdown, Qvecup, P, thetavec, impactvec
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: ALFA_MAT
INTEGER :: ndoppler=1  ! CARE
REAL, EXTERNAL :: ran1  ! The random number generator
! -----------------------------------------------------------------------------------------------

ALLOCATE(ALFA_MAT((Nstages-1)*Nelem+Nmol+3,maxval([Npis,ngvec]))); ALFA_MAT = 0

r_lastscatter = r                 ! Important
ALIVE =.TRUE.                     ! Important
NEEDNEWLINE = .TRUE.              ! Important
NEEDNEWIONEDGE = .TRUE.
BLOBEDGEHIT = .FALSE.             ! Important
tau_accum = 0                     ! Important
gamma = 1.
Nup = 1 ! CARE  Maximum number of upward transitions allowed following a line absorption.
ionlimit = lambdacut  ! CARE  over this wavelength photoionizations cannot occur
MOLLINE = .FALSE.
left = 1.   ! Fraction of photon energy remaining
alfa = 0.
HOLEHIT = .FALSE.

! ---------------------------------------------------------------------------------------------

Ntheta = 10    ! CARE Number of different blob impact angles to choose from..

ALLOCATE(thetavec(Ntheta)); thetavec = 0

ALLOCATE(impactvec(Ntheta)); impactvec = 0

thetavec = [(pi/2.*i/(Ntheta-1),i=0,Ntheta-1)]  ! Angle used for impact on blobs, impact angle is zero to pi/2. First angle is zero (impact perpendicular to surface), last is pi/2 (tangential to surface)

impactvec = -0.5*COS(2*thetavec) + 0.5     ! impact mu on blobs....theta=0 --> cos_impact = -0.5 + 0.5 = 0 --> thetavec = pi/2 --> cos_impact = 1  CARE is this done right?

z = DBLE(ran1(idum))

tau_thisphoton = -LOG(1-z)  ! Draw a random 'lifetime' tau for this photon. z can be 0 but not 1 -->avoid log 0 ..-log(1) = 0, -log(0.001) = 6.9

z = DBLE(ran1(idum))

cos_theta = 1. - 2*z      ! Draw a random (isotropic) angle... (Li & McCray 1994, Lucy1999)

v = r*v_out    ! cm/s

!cos_theta = (cos_theta + v/c)/(1. + v/c*cos_theta)    ! Aberration RL eq 4.8b CARE

sin_theta = DSQRT(1.-cos_theta**2)      ! Sin(theta) always positive since 0<theta<180  DSQRT is just as fast as SQRT (tested)

IF (BLEND .AND. zone <= Ncore) THEN  ! If photon is in (scrambled) core..

   r_inblob = MINVAL([r_inblob, blobsize(zone)])  ! Make sure r_inblob is not larger than the blob radius (numerical edge effects)

   z = DBLE(ran1(idum))

   cos_theta_int = -1. + 2*z  ! Random direction in blob  -1 to 1

   sin_theta_int = DSQRT(1. - cos_theta_int**2)

   dr_blobedge =  -r_inblob*cos_theta_int + DSQRT((r_inblob*cos_theta_int)**2 - (r_inblob**2 - blobsize(zone)**2))  ! Distance to blob edge..law of cosines

ELSE    ! Blobs not relevant, either unscrambled core or photon is outside the core

   dr_blobedge = 1000.

END IF


DO WHILE (ALIVE .EQV. .TRUE.) 	! Loop until photon is destroyed or escapes..

   BLOBEDGEHIT = .FALSE.   ! needed after adding Thomson scattering

   IF (PRINTON) PRINT *,'Position and WL and zone and mu:', r, lambda, zone, cos_theta

   IF (NEEDNEWLINE) THEN     ! Need to find out which is the closest line...(only done if wavelength is new)

      CALL nextline3(zone, lambda, Nlinesvec(zone), TAUCUBE, lineind, linej, linei, lambda_line, POS_atomwithclosestline, FOUNDNEXTLINE, NMOLMAT(zone,1), MOLLINE, taumol)   ! Get the next line..lines are listed in order in TAUCUBE...

      NEEDNEWLINE = .FALSE.      ! Do not remove

   END IF

   IF (FOUNDNEXTLINE) THEN  ! There exists a line in the zone, redward of current lambda..

      IF (MOLLINE .EQV. .FALSE.) THEN  ! Its an atomic line...
     
         atom = atoms(POS_atomwithclosestline); IF (PRINTON) PRINT *, 'Nextline:',atom%NAME,lambda_line,linei,linej
     
      ELSE  ! Its a molecular line...

         !IF (myid .LE. 1 .AND. DOPRINT) PRINT *, 'Next line is molecular.'

      END IF

      dr_nextline = c/v_out*(lambda_line - lambda)/lambda  ! The distance to the line in dimensionless units (radius of SN = 1)

      r_nextline = DSQRT(r**2 + dr_nextline**2 - 2*r*dr_nextline*(-cos_theta))    ! Dimless radius of the line resonance position, law of cosines

      IF (r_nextline <= 1.) THEN   ! Line is inside ejecta..

         NEXTLINEWITHIN = .TRUE.

      ELSE  ! Line is outside ejecta..

         IF (PRINTON) PRINT *, 'Closest line is outside.'

         NEXTLINEWITHIN = .FALSE.
         FOUNDNEXTLINE = .FALSE.
         dr_nextline = 100.
         r_nextline = 100.

      END IF

   ELSE  ! There is no line to interact with..

      NEXTLINEWITHIN = .FALSE.
      dr_nextline = 100.
      r_nextline = 100.

   END IF

   IF (NEEDNEWIONEDGE) THEN  ! TRUE if last interaction was passing an ionization edge..get a new one

      CALL nextionedge(lambda, lambdaionedge, IONEDGEEXISTS)  ! Get the wavelength of the next ionization edge..

      IF (IONEDGEEXISTS) THEN

         dr_nextionedge = c/v_out*(lambdaionedge - lambda)/lambda  ! Dimensionless distance to the next ion. edge

         r_nextionedge = DSQRT(r**2 + dr_nextionedge**2 - 2*r*dr_nextionedge*(-cos_theta))  ! Dimless radius of next ionizatione edge, law of cosines

      ELSE  ! There is no more ionization edges to consider..

         dr_nextionedge = 100.

      END IF

      IF (PRINTON) PRINT *, 'dr_nextionedge:', dr_nextionedge

      NEEDNEWIONEDGE = .FALSE.

   END IF

   r_edgeinside = radii(zone)   ! Radius of the inner edge of the current zone..

   IF (BLEND .AND. zone <= Ncore) THEN  ! Zone is a blob in a scrambled core..

      r_edgeoutside = radii(Ncore+1)  ! All radii until Ncore + 1 is zero in the scrambled case : use Ncore + 1..radii are the inner radii

   ELSE  ! scrambling is off or photon is outside core, outside edge is then radii(zone+1)..

      r_edgeoutside = radii(zone+1)

   END IF

   IF (cos_theta < 0) THEN   ! Photon going inwards, smallest impact parameter is sin r*sin_theta..

      r_smallest = r*sin_theta                   ! Sin theta should be positive always (theta between 0 and 180)

   ELSE  ! Photon going outwards..smallest radius is the starting point

      r_smallest = r

   END IF

   IF ( (cos_theta < 0.) .AND. (r_edgeinside > r_smallest) ) THEN     ! Inner shell surface is blocking photon path..

      r_edgehit = r_edgeinside   ! the photon hits the inner shell

      dr_toshelledge = -r*cos_theta - DSQRT((r*cos_theta)**2 - (r**2 - r_edgehit**2))

      if (dr_toshelledge < 0) then

         ALIVE=.FALSE.
         E=0.
         print *, 'WARNING neg dr_toshelledge 1st pos. Killing packet.', cos_theta, r, r_edgeinside, r_smallest
         EXIT

      end if

      IF ((zone >= Ncore + 1) .OR. (BLEND .EQV. .FALSE.)) THEN

         zone_next = zone - 1  ! Zone on the inside. Note this is only realized if surface hit is the next event

         if (zone_next == 0) zone_next = 1  ! Mirror

      ELSE

         zone_next = zone  ! Inward zone in a blended core --> it must be a mirror reflection (otherwise we would hit blob edge

      END IF

   ELSE

      r_edgehit = r_edgeoutside  ! Photon hits outside edge

      dr_toshelledge = -r*cos_theta + DSQRT((r*cos_theta)**2 - (r**2 - r_edgehit**2))


      if (dr_toshelledge < 0) then

         ALIVE=.FALSE.
         E=0
         print *, 'WARNING neg dr_toshelledge 2nd pos. Killing packet.', cos_theta, r, r_edgeinside, r_smallest, r_edgeoutside
         EXIT

      end if

      IF (BLEND .AND. zone <= Ncore) THEN

        zone_next = Ncore + 1    ! If next edge is a new blob, zone_next will be redrawn anyway..if the photon atctually exits the core, Ncore + 1 is the correct next zone

      ELSE  ! unscrambled core or photon is outside core..next zone is zone + 1

         zone_next = zone + 1  ! Note only realized if surface hit is the next event

      END IF

   END IF

   IF ( (NEXTLINEWITHIN) .AND. (dr_nextline < dr_toshelledge) .AND. (dr_nextline <= dr_nextionedge) .AND. (dr_nextline < dr_blobedge) ) THEN         ! next obstacle is line  CARE changed <= to < on first

      r_nextpos = r_nextline

      dr_tonext = dr_nextline

      if (dr_tonext < 0) then

         ALIVE=.FALSE.
         E=0.
         print *, 'WARNING, neg dr_nextline. Killing packet.'
         EXIT

      end if

      zone_next = zone   ! Same zone (did not hit any edge)

      LINEHIT = .TRUE.

      lambda_next = lambda_line

      IF (PRINTON) PRINT *, 'Moving to next line.'

      CROSSING = .FALSE.

   ELSE IF ( (IONEDGEEXISTS) .AND. (dr_nextionedge < dr_nextline) .AND. (dr_nextionedge < dr_toshelledge) .AND. (dr_nextionedge < dr_blobedge) ) THEN        ! next 'obstacle' is an ionization edge

      r_nextpos = r_nextionedge

      dr_tonext = dr_nextionedge

      if (dr_tonext < 0) then

         ALIVE=.FALSE.
         E=0
         print *, 'WARNING, neg dr_nextionedge. Killing packet.'
         EXIT

      end if

      zone_next = zone  ! Same zone (did not hit boundary)

      lambda_next = lambdaionedge

      !if (lambda_next < lambda) print *, "WARNING lambda > lambda_ionedge. Proceeding."

      LINEHIT = .FALSE.

      NEEDNEWIONEDGE = .TRUE.

      IF (PRINTON) PRINT *, 'Moving to next ionization edge.'

      CROSSING = .FALSE.

   ELSE IF (dr_toshelledge < dr_blobedge) THEN      ! Next obstacle is shell edge

      IF(PRINTON) PRINT *, 'Moving to next shell'

      r_nextpos = r_edgehit

      dr_tonext = dr_toshelledge

      ! zone_next has been set above

      LINEHIT = .FALSE.

      gamma = 1./SQRT(1.-(dr_tonext*v_out/c)**2)

      lambda_next = lambda*(1 + v_out/c*dr_tonext)*gamma    ! Rybicki Lightman eq 4.11 with cos_theta = -1 (homologous flow)
      IF (cos_theta < 0 .AND. r_nextpos < 1.01*radii(1)) THEN  ! Moving into central hole, flip sign (mirror treatment)

         HOLEHIT = .TRUE.

         gamma = 1./SQRT(1.-((dr_tonext+2*radii(1)*abs(cos_theta))*v_out/c)**2)  ! CARE use ABS?

         lambda_next = lambda*(1 + v_out/c*(dr_tonext + 2*radii(1)*abs(cos_theta)))*gamma

      END IF

      IF (zone .EQ. Ncore+1 .AND. zone_next .EQ. Ncore .AND. BLEND) THEN  ! If moving into the core..

         IF(PRINTON) PRINT *, 'Moving into the core'

         BLOBEDGEHIT = .TRUE.

         z = DBLE(ran1(idum))

         CALL findbin2(Ntheta, impactvec, z, bin)  ! Find impact angle

         theta = thetavec(bin)

         cos_theta_int = -COS(pi/2. - theta)

         sin_theta_int = DSQRT(1.-cos_theta_int**2)

         IF (PRINTON) PRINT *, 'Photon crossing INTO core'

      END IF

      IF (HOLEHIT .EQV. .FALSE.) dr_blobedge = 1000.

      NEEDNEWLINE = .TRUE.   ! New zone, need to get completely new line

      IF (HOLEHIT .EQV. .FALSE.) THEN

         CROSSING = .TRUE.

      ELSE

         CROSSING = .FALSE.

      END IF

   ELSE   ! Next obstacle is blob edge..

      IF (PRINTON) PRINT *, 'dr_nextshell', dr_toshelledge

      BLOBEDGEHIT = .TRUE.

      r_nextpos = DSQRT(r**2 + dr_blobedge**2 + 2*r*dr_blobedge*cos_theta)        ! Law of cosines

      dr_tonext = dr_blobedge

      if (dr_tonext < 0) then

          ALIVE=.FALSE.
          E=0
          print *, 'WARNING. neg dr_blobedge. Killing packet.'
          EXIT

      end if

      LINEHIT = .FALSE.

      gamma = 1./SQRT(1.-(dr_tonext*v_out/c)**2)

      lambda_next = lambda*(1 + v_out/c*dr_tonext)*gamma   ! Rybicki Lightman eq 4.11 with cos_theta = -1 (homologous flow)
      IF(PRINTON) PRINT *, 'Moving to blob edge at radius', r_nextpos

      NEEDNEWLINE = .TRUE.   ! New zone need to get completely new line

      CROSSING = .TRUE.

   END IF

! CONTINUUM OPACITIES ------------------------------------------------------------------

   T = RT(zone,2)   ! Temperature in this zone (needed for Bremsstrahlung continuum)

   ne = ne_vec(zone)  ! Electron number density (needed for Bremsstrahlung continuum)

   IF (lambda < ionlimit) THEN  ! Compute continuum opacities?

      CALL alfa_photoionization(lambda, N_atoms, atoms, EXCCUBE_ABIN(zone,:,:), NMOLMAT(zone,1:Nmol), alfa_dust(zone), T, ne, ION_STRUCTURE(zone,:), alfa, ALFA_MAT)

   ELSE

      alfa = 0; ALFA_MAT = 0

   END IF

   !alfa = 0 !

   !ALFA_MAT = 0 !

   !if (myid == 2) print *, 'apa cont', lambda, zone, alfa*v_out*time*0.15

   dtau = alfa*dr_tonext*v_out*time ; ! Continuum dtau (including electron scattering)

  ! IF (dtau < 0.) THEN

  !    PRINT *, 'Warning: dtau negative'

  ! END IF

  ! if (dtau < 0) then

  !    print *, 'WARNING, dtau < 0', dtau, alfa, dr_tonext, v_out, time

  ! end if

   dtau_es = ALFA_MAT((Nstages-1)*Nelem+Nmol+3,1)*dr_tonext*v_out*time  ! Just the electron scattering part

   tau_accum = tau_accum + dtau

   IF (alfa > 0.) THEN   ! If there is any (positive) continuum opacity...

      Eperphoton = h*c/(lambda*1d-8)  ! Energy per photon (ergs)

      scr1=E*(1.-exp(-dtau))/alfa
      scr2=scr1/Eperphoton
      DO j = 1, max(Npis,maxval(ngvec))   ! For each excited state that can absorb...

         DO i = 1, (Nstages-1)*Nelem + Nmol + 2  ! CARE should it not be 1, 2*Nelem?       Create photoionization rates



            N_CONTABS(i, j, 1, zone) =  N_CONTABS(i, j, 1, zone) +  ALFA_MAT(i,j)*scr2  ! Increase NUMBER of absorbed photons by the total number of absorbed photons E/Eperphoton(1-exp(-tau)) times the fraction of the opacity that this continuum constitutes

            N_CONTABS(i, j, 2, zone) =  N_CONTABS(i, j, 2, zone) +  ALFA_MAT(i,j)*scr1  ! Increase total ENERGY absorbed in this continuum

         END DO

      END DO

   END IF

   left = left*exp(-dtau)  ! Fraction NOT continuum-interacting

   IF (PRINTON) PRINT *, 'tau_accum', tau_accum

   IF (lambda < lambdaend_radfield) THEN  ! ! Add to radiation field vector?

      CALL findbin2(Nradfield, lambda_radfield, lambda, bin)  ! Find wavelength bin

      IF (bin .NE. 0) THEN

         IF (dtau > 1.) THEN   ! If significant continuum depth exists, damp contribution..note do NOT damp with exp(-tau), the typical distance travelled is dx/tau!

            E_radfield(zone, bin) = E_radfield(zone, bin) + 1./(4.*pi)*E*dr_tonext*v_out*time/dtau   ! Epr for J (Delta-t=1). Lucy 1999a eqv 13, note volume division occurs elsewhere

         ELSE

            E_radfield(zone, bin) = E_radfield(zone, bin) + 1./(4.*pi)*E*dr_tonext*v_out*time

         END IF

      END IF

   END IF

   IF (alfa > 0) THEN  ! If there is ANY opacity (to avoid division with zero)

      f_T = ALFA_MAT((Nstages-1)*Nelem+Nmol+3,1)/alfa*(1.-exp(-dtau))  ! Fraction of photons that Thomson scatter crossing dr_tonext

   ELSE

      f_T = 0

   END IF

   f_pass = exp(-dtau)   ! Fraction of photons that survive continuum absorption and Thomson scattering crossing dr_tonext

   IF (DOPRINT .and. myid==IDP) PRINT *, 'f_T f_pass', f_T, f_pass

   IF (left < 1E-4) THEN   ! If bug enough chunk is lost (>99.9%), skip following the rest CARE check sensitivity

      ALIVE = .FALSE.

      EXIT

   END IF

   E = E*(f_T + f_pass)  ! Energy that is NOT continuum absorbed (=Thomson scatters or passes). Formula checked.

   !if (f_T + f_pass > 1.) then

      !print *, 'WARNING, E boost as f_T + f_pass >1', f_T + f_pass, f_T, f_pass, alfa, dtau

   !end if

   tau_accum = tau_accum - dtau   ! Dont count up tau_tot for the surviving photons (they are not terminated)

   IF (f_T > 1E-5) THEN   ! TEMP Only bother to make random draw if Thomson fraction is non-neglegible (This is FRACTION of photons passing this particular path length that are Thomson scattered, not the actual optical depth for the path length)

      z = DBLE(ran1(idum))

      IF (z < f_T/(f_pass + f_T)) THEN   ! Select ALL photons to Thomson scatter every *tau_T/tau_tot* time (rather than a fraction $tau_T/tau_tot# of them every time

         z = DBLE(ran1(idum))

         dr_tonext = dr_tonext/maxval([2d0,dtau_es])  ! Distance to the Thomson scattering...never let be larger than to the next crit. point

         !dr_tonext = -log(1. + z*(exp(-dtau)-1.))/dtau*dr_tonext   ! Draw position of the scattering.

!         IF (myid .LE. 1 .AND. DOPRINT) print *, 'th dr_tonext  maxval', dr_tonext, maxval([1.01d0,dtau_es])

         gamma = 1./SQRT(1.-(dr_tonext*v_out/c)**2)

         lambda_next = lambda*(1 + v_out/c*dr_tonext)*gamma   ! Comoving lambda at that point

         E = E*(lambda/lambda_next)**ndoppler  ! CARE Energy is lowered in the local frame

         lambda = lambda_next;

         r_temp = DSQRT(r**2 + dr_tonext**2 - 2*r*dr_tonext*(-cos_theta))   ! Move photon to that point

         r = r_temp

         IF ((zone <= Ncore) .AND. (r > radii(Ncore+1))) zone = Ncore + 1   ! photon has moved outside core, radii is inner radius

         IF (PRINTON) print *, 'thomson at', r, cos_theta, sin_theta, 'New lambda:  E', lambda, E

         z = DBLE(ran1(idum))

         cos_theta = 1. - 2*z   ! New random angle (approximate Rayleigh phase function with uniform distribution, Rayleigh distr has no analytic solution, anyway its vvery similar to isotropic scattering)

         sin_theta = dsqrt(1.-cos_theta**2)

         NEEDNEWIONEDGE = .TRUE.  ! Dont need new line since we didnt reach point (verified)

         IF (BLEND .AND. zone <= Ncore) THEN   ! Photon is in scrambled core..

            r_inblob_new = DSQRT(r_inblob**2 + dr_tonext**2 + 2*r_inblob*dr_tonext*cos_theta_int)

!           IF (myid .LE. 1 .AND. DOPRINT) print *, 'r_inblob_new', r_inblob_new

            dr_signchange = r_inblob*ABS(cos_theta_int)  ! The distance where the sign of the direction cosine changes (for inward bound photon).

            sin_theta_int = sin_theta_int*r_inblob/r_inblob_new; IF(sin_theta_int > 1.) sin_theta_int = 1.

            IF (cos_theta_int < 0 .AND. dr_tonext <= dr_signchange) THEN   ! Photon is going inwards at the new position

               cos_theta_int = -DSQRT(1. - sin_theta_int**2)

            ELSE  ! Photon is going outwards at the new position

               cos_theta_int = +DSQRT(1. - sin_theta_int**2)

            END IF

            r_inblob = r_inblob_new  ! Do not move up

            dt_core = dt_core + dr_tonext/(radii(Ncore+1))  ! CARE

            dr_blobedge =  -r_inblob*cos_theta_int + DSQRT((r_inblob*cos_theta_int)**2 - (r_inblob**2 - blobsize(zone)**2))  ! Distance to blob edge..law of cosines


         END IF

         CYCLE    ! Follow the scattered photon from top of cycle

      END IF

   END IF

   !IF (tau_accum > tau_thisphoton) THEN  ! Continuum absorption happens..

   !   ALIVE = .FALSE.   ! Kill photon..

!      IF (myid .LE. 1 .AND. DOPRINT) PRINT *, 'Photon suffers continuum abs. tau_c:', tau_accum,'WL:', lambda

      !TAU_MAT = ALFA_MAT/SUM(ALFA_MAT)  ! Normalized opacities (NOT real taus)

      !DO i = 1, 2*Nelem + Nmol + 2  ! CARE

      !   DO j = 1, Npis

      !      Eperphoton = h*c/(lambda*1d-8)

      !      N_CONTABS(zone, i, j, origzone, 1) =  N_CONTABS(zone, i, j, origzone, 1) +  E*TAU_MAT(i,j)/Eperphoton

      !      N_CONTABS(zone, i, j, origzone, 2) =  N_CONTABS(zone, i, j, origzone, 2) +  E*TAU_MAT(i,j)

      !   END DO

      !END DO

!      CLOSE(100)

      ! Add contribution to internal radiation field:

      !IF (lambda < lambdaend_radfield) THEN   ! Only bother for wavelengths below set cutoff

      !   CALL findbin(Nradfield, lambda_radfield, lambda, bin)

      !   IF (bin .NE. 0) THEN

      !      E_radfield(zone, bin) = E_radfield(zone, bin) + 1./(4.*pi)*E*dr_tonext*v_out*time/tau_accum  ! Shorten pathlength by a factor tau_accum      Lucy 1999a eqv 13, note volume division occurs elsewhere

      !   END IF

      !END IF

   !   EXIT

   !ELSE   ! No continuum absorption, use dr_tonext for constructing radiation field

      !IF (lambda < lambdaend_radfield) THEN

      !   CALL findbin(Nradfield, lambda_radfield, lambda, bin)

       !  IF (bin .NE. 0) THEN

       !     E_radfield(zone, bin) = E_radfield(zone, bin) + 1./(4.*pi)*E*dr_tonext*v_out*time   ! Lucy 1999a eqv 13, note volume division occurs elsewhere


       !  END IF

      !END IF


  ! END IF

   sin_theta = r/r_nextpos*sin_theta; IF (sin_theta .GT. 1.) sin_theta = 1.       ! Law of sines..and then correct small numerical errors

   IF ( (cos_theta < 0) .AND. (dr_tonext < abs(r*cos_theta)) ) THEN  ! Only true if cos_theta negativ ie ingoing and minimum impact parameter does not get passed (then cos theta becomes positive)

      cos_theta = -DSQRT(1.-sin_theta**2)

   ELSE   ! Photon is going outwards, positive cos

      cos_theta = +DSQRT(1.-sin_theta**2)

   END IF

   IF (HOLEHIT) THEN

      cos_theta = +DSQRT(1.-sin_theta**2)

      HOLEHIT = .FALSE.

      NEEDNEWLINE = .TRUE.

   END IF




   !IF (((zone_next .NE. zone) .OR. (BLOBEDGEHIT .EQV. .TRUE.)) .AND. (OLI .EQV. .TRUE.)) THEN  ! IF OLI is on only

   !   ALIVE = .FALSE.

   !   !gamma = 1./SQRT(1.-(v_out/c)**2)

   !   lambda_escaped = lambda*(1. - v_out/c*cos_theta)*gamma  !  ! Rybicki Lightman eq 4.11...see also Kasen2006 eq 17  (1-v/c*mu1)*gamma*(1-v/c*mu2)*gamma = (1-v/c*mu1)/(1+v/c*mu2)

  !    E = E*(lambda/lambda_escaped)**ndoppler  ! CARE

  !    CALL findbin(Wlbins, xlog, lambda_escaped, bin)

   !   IF (bin .NE. 0 .AND. bin .NE. Wlbins) THEN

   !      OUTPHOTONS_SC(bin, scattered+1) = OUTPHOTONS_SC(bin, scattered+1) + E/(xlog(bin+1)-xlog(bin))

   !      OUTPHOTONS_ABIN(bin, sender) = OUTPHOTONS_ABIN(bin, sender) + E/(xlog(bin+1)-xlog(bin))

   !      OUTPHOTONS_LASTZONE(bin, lastzone) = OUTPHOTONS_LASTZONE(bin, lastzone) + E/(xlog(bin+1)-xlog(bin))

   !      OUTPHOTONS_ORIGZONE(bin, origzone) =  OUTPHOTONS_ORIGZONE(bin, origzone) + E/(xlog(bin+1)-xlog(bin))

   !   END IF

   !   CALL findbin(Wlbins_small, xlog_small, lambda_escaped, bin2)

   !   CALL findbin(Wlbins_small, xlog_small, origlambda, bin3)

   !   EXIT

   !END IF

   E = E*(lambda/lambda_next)**ndoppler  ! CARE Energy is lowered in the local frame

   lambda = lambda_next; IF (PRINTON) PRINT *, 'New lambda:', lambda

   r = r_nextpos  ! Do not move up

   dr_nextionedge = dr_nextionedge - dr_tonext


   IF (BLEND .AND. zone <= Ncore) THEN   ! Photon is in scrambled core..

      dr_blobedge = dr_blobedge - dr_tonext    ! Reduce distance to blob edge

      r_inblob_new = DSQRT(r_inblob**2 + dr_tonext**2 + 2*r_inblob*dr_tonext*cos_theta_int)

      dr_signchange = r_inblob*ABS(cos_theta_int)

      sin_theta_int = sin_theta_int*r_inblob/r_inblob_new; IF(sin_theta_int > 1.) sin_theta_int = 1.

      IF (cos_theta_int < 0 .AND. dr_tonext <= dr_signchange) THEN   ! Photon is going inwards

         cos_theta_int = -DSQRT(1. - sin_theta_int**2)

      ELSE  ! Photon is going outwards..

         cos_theta_int = +DSQRT(1. - sin_theta_int**2)

      END IF

      r_inblob = r_inblob_new  ! Do not move up

      dt_core = dt_core + dr_tonext/(radii(Ncore+1))  ! CARE

   END IF

   IF (CROSSING .AND. BLOBEDGEHIT .EQV. .FALSE.) THEN  ! This was a boundary crossing

      energyout(zone) = energyout(zone) + E

      IF (zone_next <= Nr) energyout(zone_next) = energyout(zone_next) - E

   ELSEIF (CROSSING .AND. BLOBEDGEHIT .EQV. .TRUE.) THEN

      energyout(zone) = energyout(zone) + E   ! Energy into blob added below

   END IF

   zone = zone_next  ! Update photon position. Do not move up

   IF (BLOBEDGEHIT) THEN   ! A new blob is encountered, make random draw

      z = DBLE(ran1(idum))

      CALL findbin2(Ncore+1, af_accum, z, zone)  ! selects ZONE type (1:Ncore)

      if (zone > 0) then

        energyout(zone) = energyout(zone) - E

      else

        print *, 'ERROR, zone=0', af_accum
        zone = 1
      end if

      IF (PRINTON) PRINT *, 'new blob zone selected', zone

      r_inblob = blobsize(zone)  ! Start at

      z = DBLE(ran1(idum))

      CALL findbin2(Ntheta, impactvec, z, bin)  ! impactvector

      theta = thetavec(bin)

      cos_theta_int = -COS(pi/2. - theta) ! Impact parameter on new blob

      sin_theta_int = DSQRT(1.-cos_theta_int**2)

      dr_blobedge = 2*blobsize(zone)*ABS(cos_theta_int)

      BLOBEDGEHIT = .FALSE.  ! Important

      IF (PRINTON) PRINT *, 'BLOBEDGEHIT New zone   blobsize  dr_blobedge', zone, blobsize, dr_blobedge

      NEEDNEWLINE = .TRUE.  ! New zone, get new line

   ELSE IF (LINEHIT) THEN     ! LINE INTERACTION ---------------------------------------------

      IF (MOLLINE .EQV. .FALSE.) THEN  ! An atomic line..

         tauline = TAUCUBE(zone,lineind,6)  ! Extract line optical depth

         IF (PRINTON) WRITE(*,'(A10,F5.2, A10, I4, I4, F10.4, ES10.2)'), 'LINEHIT!', r, atom%NAME, linei, linej, lambda_line, tauline

      ELSE  ! Molecular line..

         tauline = taumol

         IF (PRINTON) WRITE(*,'(A15, F5.2, I4, I4, F10.4, ES10.2)'), 'MOLECULEHIT!', r, linei, linej, lambda_line, tauline

      END IF

      IF (RECURSIVES > 300) THEN

           tauline = 0 ! CARE if too deep in recursion dont add more line absorptions

            print *, 'WARNING RECURSIVE LIMIT REACHED'

      END IF

      !tauline = 0 ! TEMP sets line opacities to 0

      tau_accum = tau_accum + ABS(tauline)  ! Add line optical depth to total..if line optical depth is negative interaction should still occur (in the form of stimulated emission)..CARE is this correct treatment

      IF (PRINTON) PRINT *, 'tau_accum', tau_accum

      IF ((PHOTOEXC >= 1) .AND. (atom%ABIN <= NABIN) .AND. (tauline > 0.)) THEN   ! Normal absorption, depopulate lower level

         atoms(atom%SEQ)%PHOTOEXCRATES(zone,linei,linej) = atoms(atom%SEQ)%PHOTOEXCRATES(zone,linei,linej) +  E/(h*c/(lambda*1E-8))*(1.-exp(-tauline))

      ELSE IF ((PHOTOEXC >= 1) .AND. (atom%ABIN <= NABIN) .AND. (tauline < 0.) .AND. (abs(tauline)<10.)) THEN  ! Stimulated emission, depopulate upper level

         atoms(atom%SEQ)%PHOTOEXCRATES(zone,linei,linej) = atoms(atom%SEQ)%PHOTOEXCRATES(zone,linei,linej) +  E/(h*c/(lambda*1E-8))*(1.-exp(-tauline))   !  (1-exp-tauline) is negative...CAREFUL dont dwitch this on without changing so that negative opt. depth are also counted in N_ABS   ! CARE

         !print *, 'Stim1', exp(-tauline)

      END IF

      IF ((PHOTOEXC == 2) .AND. (atom%ABIN <= Nabin) .AND. (linej <= atom%N_LEVELS) .AND. (tauline > -10.)) THEN   ! straight lambda iteration, decrease photon energy by absorption ONLY if upper level is included in NLTE solutions

         Ephotoexc(zone) = Ephotoexc(zone) + E*(1. - exp(-tauline))   ! increase total amount of photoexc. energy

         E = E*exp(-tauline)  ! Decrease packet energy by the absorption factor

         left = left*exp(-tauline)

         tau_accum = tau_accum - ABS(tauline)  ! Photons have been absorbed, dont add tauline to total for the survivors..important, gives too low fluxes without this

  	  IF (left < 1E-4) THEN  ! CARE

	    ALIVE = .FALSE.

            EXIT

         END IF

      END IF

      IF ((tau_accum > tau_thisphoton) .AND. ((PHOTOEXC <=1) .OR. (linej > atom%N_LEVELS))) THEN  ! Photon is absorbed, if photoexc = 0 or 1 follow deexc
         IF (tauline > 0) THEN   ! Absorption (rather than stimulated emission)

            IF (MOLLINE .EQV. .FALSE. ) THEN  ! By an atomic line..

               IF (PRINTON) THEN

                  PRINT *, 'Photon absorbed! By', atom%NAME, linei, linej, lambda_line

               END IF

               ALIVE = .FALSE.

               from = linej   ! Upper level

               ne = ne_vec(zone)   ! Electron number density (cm^-3)

               T = RT(zone, 2)    ! temperature

               lastzone = zone

               Eheat = 0  ! importantou

               nphotons = E/(h*c/(lambda*1E-8))  ! If lambda has changed, so has E so E*lambda = n = constant

               cf = 1.

               IF ((PHOTOEXC == 1) .AND. (atom%ABIN < Nabin)) THEN  

                  E = E*atom%LEVELS_E(linej)/(atom%LEVELS_E(linej)-atom%LEVELS_E(linei))  ! CARE The excitation energy of the lower state is now being converted to deexcitation energy

               END IF

               DO WHILE (from .GT. 1)             ! Follow output from the excited atom until its back in its ground state

                  ALLOCATE(beta_s(from-1)); beta_s = 0
                  ALLOCATE(beta_c(from-1)); beta_c = 0
                  ALLOCATE(beta_e(from-1)); beta_e = 0
                  ALLOCATE(Qvecdown(from-1)); Qvecdown = 0
                  ALLOCATE(P(2*(from-1)+Nup) ); P = 0
                  ALLOCATE(Qvecup(Nup)); Qvecup = 0

                  CALL createbetaandQvec(atom, from, Nup, EXCCUBE_ABIN(zone,1:Nabin,1:maxval([Npis,ngvec])), NMOLMAT(zone,1:Nmol), ION_STRUCTURE(zone,1:Nabin), alfa_dust(zone), N_atoms, atoms, zone, T, ne, beta_s, beta_c, Qvecdown, Qvecup)

                  beta_l = 0. ! CARE should treat H 3-1 better

                  beta_e = (beta_s + beta_c + beta_l)/(1. + beta_c + beta_l) ! The effective escape probability, taking continuum absorption and line absorption into account

                  CALL makeP(atom, from, Nup, ne, beta_e, Qvecdown, Qvecup, P)	 ! Compute probability array for various deexcitation channels

                  z = DBLE(ran1(idum))

                  CALL deexcite(z, from, Nup, P, beta_s, beta_c, level_out, PHOTON, IONIZATION)	 ! Randomly choose one deexcitation channel

                  DEALLOCATE(beta_s)
                  DEALLOCATE(beta_c)
                  DEALLOCATE(beta_e)
                  DEALLOCATE(Qvecdown)
                  DEALLOCATE(Qvecup)
                  DEALLOCATE(P)

                  E_deexcite = E*(atom%LEVELS_E(from)-atom%LEVELS_E(level_out))/atom%LEVELS_E(from) ! E*fraction of step to ground

                  E = E - E_deexcite  ! CARE can increase if upward collision

                  IF (PHOTON .EQV. .TRUE.) THEN       ! If a photon was sent out we need to follow it..

                     zone_fluor = zone

                     r_fluor = r

                     r_inblob_fluor = r_inblob

                     IF (atom%NAME .EQ. ' HI  ' .AND. from .EQ. 2) THEN

                        CALL twophoton_ay("HI ",lambda_fluor, ERROR); IF (PRINTON) PRINT *, 'Two-photon H fluorescence!', lambda_fluor

                     ELSE IF (atom%NAME .EQ. 'HeI  ' .AND. from .EQ. 3 .AND. level_out .EQ. 1) THEN

                        CALL twophoton_ay("HeI",lambda_fluor, ERROR)

                     ELSE

                        lambda_fluor = real(atom%LAMBDA(from, level_out))			        ! Get wavelength of the photon. CARE has to be real as long as TAUCUBE wavelengths are in real, otherwise can self-lock

                     END IF

                     IF (PRINTON) PRINT *, 'Radiative from ', from, '  to level: ', level_out, lambda_fluor

                     IF (lambda_fluor < lambdacut) THEN

                        sender_fluor = atom%ABIN

                        scattered = 1

                        RECURSIVES = RECURSIVES + 1

                        CALL followphoton_ff(zone_fluor, r_fluor, E_deexcite, lambda_fluor, sender_fluor, scattered, origzone, lastzone, origr, origlambda, dt_core, r_inblob_fluor, N_atoms, atoms, RT, ION_STRUCTURE, EXCCUBE_ABIN, NMOLMAT, radii, ne_vec, alfa_dust, Nlines, Nlinesvec, TAUCUBE, OLI, Nblobs, blobsize, af_accum, WLbins, xlog)                ! We have a new photon at a new position and wavelength - follow it!

                        RECURSIVES = RECURSIVES - 1

                     ELSE

                        IF (PRINTON) PRINT *, 'Photon WL > cutoff, not following.'

                        FIRFLUX = FIRFLUX + E_deexcite

                        IF (zone <= Nr) energyout(zone) = energyout(zone) + E_deexcite

                     END IF

                  ELSE IF (IONIZATION .EQV. .TRUE.) THEN  ! Photon deexcited radiatively into the continuum

                     zone_fluor = zone

                     lambda_fluor = atom%LAMBDA(from, level_out)

                     IF (PRINTON) PRINT *,'Ionizational deexcitation from', from,'to',level_out

                     T = RT(zone,2)   ! Temperature in this zone (needed for Bremsstrahlung continuum)

                     ne = ne_vec(zone)  ! Electron number density (needed for Bremsstrahlung continuum)

                     CALL alfa_photoionization(lambda_fluor, N_atoms, atoms, EXCCUBE_ABIN(zone_fluor,:,:), NMOLMAT(zone_fluor,1:Nmol), alfa_dust(zone_fluor), T, ne, ION_STRUCTURE(zone,:), alfa, ALFA_MAT)

                     IF (sum(ALFA_MAT) > 0.) THEN

                       
                        scr2= E_deexcite/SUM(ALFA_MAT) 
                        scr1= scr2/(h*c/(lambda_fluor*1e-8)) 
                        DO i = 1, (Nstages-1)*Nelem + Nmol + 2  ! CARE

                           DO j = 1, maxval([Npis,ngvec])

                              N_CONTABS(i, j, 1, zone) =  N_CONTABS(i, j, 1, zone) +  scr1*ALFA_MAT(i,j)

                              N_CONTABS(i, j, 2, zone) =  N_CONTABS(i, j, 2, zone) +  scr2*ALFA_MAT(i,j)

                           END DO

                        END DO

                     END IF

                  ELSE

                     Eheat = Eheat + E_deexcite*cf   ! collisional deexcitation --> heating

                     IF (PRINTON) PRINT *, 'Collisional from ', from,'  to level: ', level_out, 'Eheat', Eheat

                  END IF

                  from = level_out

               END DO

               colldeexc(zone) = colldeexc(zone) + Eheat   ! Add total heating produced in deexcitation route

            ELSE  ! Molecular line

               IF (PRINTON) PRINT *, 'CO photodissociation!'

            END IF

         ELSE   ! tauline is negative..

            E = 1*E   !CARE  Let packet proceed with twice the energy (twice as many photons) CARE should also follow fluorescence from linei not done now

            PRINT *, 'Stimulated emission!', lambda

         END IF

         NEEDNEWLINE = .TRUE.

      ELSE    ! PHOTOEXC=2 and j < N_levels or Photon goes through line for PHOTOEXC=0 and 1

         IF (PRINTON) PRINT *, 'Photon goes through line. Getting next', lineind, Nlinesvec(zone)

         lineind = lineind + 1  ! Wavelengths are sorted in ascending order..

         IF (lineind <= Nlinesvec(zone)) THEN  ! Make sure it was not the last line in the list..

            FOUNDNEXTLINE = .TRUE.

            IF (TAUCUBE(zone,lineind,1) > DBLE(Nabin)) THEN

               MOLLINE = .TRUE.

            ELSE

               MOLLINE = .FALSE.

               POS_atomwithclosestline = NINT(TAUCUBE(zone,lineind,2))

               linei = NINT(TAUCUBE(zone,lineind,3))

               linej = NINT(TAUCUBE(zone,lineind,4))

               lambda_line = TAUCUBE(zone,lineind,5)

               atom = atoms(POS_atomwithclosestline)   ! tauline is extracted when line is hit

            END IF

            IF (PRINTON) PRINT *, 'Next line sequental. Name,i,j,lambda,tau:', atom%NAME, linei, linej, lambda_line, TAUCUBE(zone,lineind,6)

         ELSE   ! If it was the last line in the last, no further line targets

            FOUNDNEXTLINE = .FALSE.

            NEXTLINEWITHIN = .FALSE.

            dr_nextline = 100.

            r_nextline = 100.

         END IF

         NEEDNEWLINE = .FALSE.  ! Either way, dont search for new line at top

      END IF

   END IF

   IF (r >= 1.) THEN              ! Photon leaves grid alive..storing

      IF (PRINTON) PRINT *, 'Photon escapes! WL:', lambda

      ALIVE = .FALSE.

      gamma = 1./SQRT(1.-(v_out/c)**2)

      lambda_escaped = lambda*(1. - v_out/c*cos_theta)*gamma  ! RybickiLightman eq 4.11

      E = E*(lambda/lambda_escaped)**ndoppler  ! CARE

      CALL findbin2(WLbins, xlog, lambda_escaped, bin)

      IF (bin .NE. 0 .AND. bin .NE. Wlbins) THEN

         if (isnan(E/(xlog(bin+1)-xlog(bin)))) then

           print *, 'WARNING ISNAN', E, bin, xlog(bin+1), xlog(bin), lambda

         else

           OUTPHOTONS_SC(bin, scattered+1) = OUTPHOTONS_SC(bin, scattered+1) + E/(xlog(bin+1)-xlog(bin))

           OUTPHOTONS_ABIN(bin, sender) = OUTPHOTONS_ABIN(bin, sender) + E/(xlog(bin+1)-xlog(bin))

           OUTPHOTONS_LASTZONE(bin, lastzone) = OUTPHOTONS_LASTZONE(bin, lastzone) + E/(xlog(bin+1)-xlog(bin))

           OUTPHOTONS_ORIGZONE(bin, origzone) =  OUTPHOTONS_ORIGZONE(bin, origzone) + E/(xlog(bin+1)-xlog(bin))

         end if

      ELSE   ! Wavelength is outside binning range

         FIRFLUX = FIRFLUX + E

      END IF

   END IF

END DO


END
