      FUNCTION FSIGI(IE,Ekin,Nstages)
C
C   TOTAL IONIZATION CROSS SECTIONS = DIRECT IONIZATION + 
C         EXCITATION-AUTOIONIZATION (WHERE DATA AVAILABLE)      AUTOIONIZATION CAN HAPPEN WHEN TWO ELECTRONS ARE ECITED SIMULTANEOUSLY
C   DATA FROM 'ARNAUD AND ROTHENFLUG, 1985, ASTRON. ASTROPHYS. 
C   SUPPL. SER. 60, 425-457' FOR ALL ELEMENTS EXCEPT O I AND FE.
C   O I : -------------------
C     DATA FOR O I IS TAKEN FROM LAHER AND GILMORE, 1990.
C     IONIZATION CROSS SECTION INCLUDE IONIZATION TO THE GROUNDSATE 4So, AND TO
C     THE EXCITED STATES 2Do AND 2Po, AND TO THE INNER IONIZATION STATE
C     2s2p4 4P. IONIZATION TO THE G.S. INCLUDE AUTOIONIZATION.      
C
C     OE = ENERGY BINS IN eV
C     OFRAC = % OF THE TOTAL ION. CROSS SECTION THAT GOES INTO A CERTAIN
C           STATE IN O II.
C           1 : 4So (ground state)
C           2 : 2Do  
C           3 : 2Po
C           4 : 4P
C     OSIG = TOTAL CROSS SECTION = SUM OF THE 4 O II STATES.  
C
C   FE : ----------------------
C   THE IONIZATION DATA IS TAKEN FROM ARNAUD AND RAYMOND, 1992,
C   AP.J., 398, 394.
C
C     THE ELEMENTS INCLUDED ARE GIVEN IN FILE 'DATIE'

      IMPLICIT NONE
      INCLUDE 'PARAM.SF'
      INCLUDE 'DATIE.SF'
      REAL    :: FSIGI
      INTEGER :: IE,Nstages           ! Element code (see DATIE.SF)
      REAL    :: Ekin            ! Kinetic energy of electron
      REAL    :: C(5,4),XIE(5),SIG(5),OE(25),OSIG(25),OFRAC(4,25)
      REAL    :: FRAC(4)
      REAL    :: XI,XIO,EX,EXMIN,AB,XEL,XIECR,SIGCR,SIGEA,U
      REAL    :: SSUM,C1,SIGT,XIEA,A1,A2,A3,A4,A5
      INTEGER :: NL,IEMAX,NOIF,NMAX,J,iebase
      DATA OE/13.6,14.,16.,18.,20.,25.,30.,35.,40.,50.,60.,70.,80.,90.,
     &        100.,150.,200.,300.,400.,500.,600.,700.,800.,900.,1000./
      DATA (OFRAC(1,I),I=1,25)/0.,100.,100.,86.8,76.7,68.0,65.2,64.5,
     &     63.0,59.4,56.4,53.0,50.4,47.7,46.1,42.0,39.0,36.8,36.1,
     &     35.6,35.6,35.5,35.5,35.5,35.5/               
      DATA (OFRAC(2,I),I=1,25)/0.,0.,0.,13.2,19.4,24.1,23.7,22.6,
     &     21.4,20.6,20.8,22.0,23.4,24.5,25.5,27.6,29.0,29.1,29.1,
     &     29.1,29.2,29.3,29.4,29.5,29.5/  
      DATA (OFRAC(3,I),I=1,25)/0.,0.,0.,0.,3.9,7.9,9.6,9.8,9.7,10.0,
     &     10.7,11.7,12.4,13.6,14.1,15.6,16.5,17.8,17.7,17.6,17.4,17.4,
     &     17.2,17.1,17.0/ 
      DATA (OFRAC(4,I),I=1,25)/0.,0.,0.,0.,0.,0.,1.5,3.1,5.9,10.,12.1,
     &     13.3,13.8,14.2,14.3,14.8,15.5,16.3,17.1,17.7,17.8,17.8,
     &     17.9,17.9,18.0/
      DATA (OSIG(I),I=1,25)/0.,2.6,16.3,26.5,36.0,53.0,68.8,82.3,96.5,
     &     115.3,124.3,128.2,131.0,132.0,133.0,131.0,124.0,104.2,89.3,
     &     79.2,69.4,62.2,56.5,51.9,48.1/     

      COMMON/CENERG/XI(MIE),XIO(4),EX(MIE,MIL),EXMIN
      COMMON/CABUND/AB(MIE),NL(MIE),XEL,IEMAX
      COMMON/COI/NOIF
C these parameters are used by the routine crion.f  to
C check the cross sections.
      COMMON/CCRION/XIECR(5),SIGCR(5),SIGEA,NMAX

      U=Ekin/XI(IE)    ! Electron energy divided by ionization potential (ground state)
      FSIGI=0.
      IF (U.LE.1.) GOTO 99   ! Electron energy below ionization threshold--> skip
      IF (AB(IE).LE.0.) GOTO 99  ! Abundance of element is zero --> skip
      
      C = 0.   ! Initialize..
     
      IF (IE.EQ.IH1) THEN   ! Photoionization cross section fits from Arnaud Rothenflug 1985
C  H I
        NMAX=1
        C(1,1)=22.8
        C(1,2)=-12.0
        C(1,3)=1.9
        C(1,4)=-22.6    ! Double checked
        XIE(1)=XI(IE)
      ELSEIF (IE.EQ.IHE1) THEN
C  HE I
        NMAX=1
        C(1,1)=17.8
        C(1,2)=-11.0
        C(1,3)=7.0
        C(1,4)=-23.2
        XIE(1)=XI(IE)   !D.c.
      ELSEIF (IE.EQ.IHE2) THEN
C  HE II
        NMAX=1
        C(1,1)=14.4
        C(1,2)=-5.6
        C(1,3)=1.9
        C(1,4)=-13.3   !D.c.
        XIE(1)=XI(IE)
      ELSEIF (IE.EQ.IC1) THEN
C  C I
        NMAX=2   ! Two shells of electrons
C  2p2 (Valence shell, 4 electrons, IP 11.3 eV)
        C(1,1)=6.0
        C(1,2)=-16.0
        C(1,3)=12.0
        C(1,4)=-15.1
C  2s2  (Inner shell, 2 electrons, IP 16.6 eV)
        C(2,1)=24.3
        C(2,2)=-7.8
        C(2,3)=2.5
        C(2,4)=-24.0
        XIE(1)=XI(IE)
        XIE(2)=16.6     !D.c.
      ELSEIF (IE.EQ.IC2) THEN
C  C II
        NMAX=2
C  2p Valence shell 3 electrons, IP 24.4 eV
        C(1,1)=16.0
        C(1,2)=-9.0
        C(1,3)=2.5
        C(1,4)=-10.5
C  2s2  Inner shell two electrons, IP 30.9 eV
        C(2,1)=23.7
        C(2,2)=-7.6
        C(2,3)=2.5
        C(2,4)=-21.7
        XIE(1)=XI(IE)
        XIE(2)=30.9    ! D.c.
      ELSEIF (IE.EQ.IC3) THEN
C  C III
        NMAX=2
C  2s2
        C(1,1)=23.2
        C(1,2)=-7.4
        C(1,3)=2.5
        C(1,4)=-19.4
C  1s2
        C(2,1)=20.0
        C(2,2)=-5.6
        C(2,3)=4.1
        C(2,4)=-18.0

        if (Nstages <= 3) C = 1E-5   ! CARE dont ionize more than to III

        XIE(1)=XI(IE)
        XIE(2)=325.
      ELSEIF (IE.EQ.IC4) THEN
C  C IV
        NMAX=2
C  2s
        C(1,1)=8.2
        C(1,2)=-2.7
        C(1,3)=1.4
        C(1,4)=-6.6
C  1s2
        C(2,1)=20.0
        C(2,2)=-5.6
        C(2,3)=4.1
        C(2,4)=-18.0

        if (Nstages <= 4) C = 1E-5
C 
        XIE(1)=XI(IE)
        XIE(2)=343.0
      ELSEIF (IE.EQ.IC5) THEN
C  C V
        NMAX=1
C  1s2
        C(1,1)=20.4
        C(1,2)=-6.1
        C(1,3)=4.5
        C(1,4)=-18.0
C 
        XIE(1)=XI(IE)
      ELSEIF (IE.EQ.IN1) THEN
C  N I
        NMAX=2
C  2p3  ! Valance shell, 5 electrons, IP 14.5 eV
        C(1,1)=19.5
        C(1,2)=-30.5
        C(1,3)=15.0
        C(1,4)=-29.0
C  2s2  ! Inner shell, 2 electrons, IP 20.3 eV
        C(2,1)=19.0
        C(2,2)=-4.5
        C(2,3)=2.8
        C(2,4)=-20.2
        XIE(1)=XI(IE)
        XIE(2)=20.3    ! D.c.
      ELSEIF (IE.EQ.IN2) THEN
C  N II
        NMAX=2
C  2p2 ! Valence shell, 4 electrons, IP 29.6 eV
        C(1,1)=21.0
        C(1,2)=-9.0
        C(1,3)=5.3
        C(1,4)=-22.5
C  2s2 ! Inner shell, 2 electrons, IP 36.7 eV
        C(2,1)=18.5
        C(2,2)=-4.3
        C(2,3)=2.8
        C(2,4)=-18.0
        XIE(1)=XI(IE)
        XIE(2)=36.7
C New 990824 data for N III, IV and V 
      ELSEIF (IE.EQ.IN3) THEN
C  N III
        NMAX=2
C  2p
        C(1,1)=16.0
        C(1,2)=-7.5
        C(1,3)=2.3
        C(1,4)=-10.0
C  2s2
        C(2,1)=18.1
        C(2,2)=-4.0
        C(2,3)=2.8
        C(2,4)=-15.8
 
        if (Nstages <= 3) C = 1D-5  ! CARE dont ionize more than III

        XIE(1)=XI(IE)
        XIE(2)=55.8
      ELSEIF (IE.EQ.IN4) THEN
C  N IV
        NMAX=2
C  2s2
        C(1,1)=17.6
        C(1,2)=-3.8
        C(1,3)=2.8
        C(1,4)=-13.6
C  1s2
        C(2,1)=20.5
        C(2,2)=-5.8
        C(2,3)=4.1
        C(2,4)=-18.0

        if (Nstages <= 4) C = 1E-5

        XIE(1)=XI(IE)
        XIE(2)=471.0
      ELSEIF (IE.EQ.IN5) THEN
C  N V
        NMAX=2
C  2s
        C(1,1)=10.5
        C(1,2)=-3.3
        C(1,3)=1.4
        C(1,4)=-7.7
C  1s2
        C(2,1)=20.5
        C(2,2)=-5.8
        C(2,3)=4.1
        C(2,4)=-18.0
        XIE(1)=XI(IE)
        XIE(2)=493.0
C      ELSEIF (IE.EQ.IO1) THEN  OI special treatment further down
C  O I
C        NMAX=2
C  2p4
CCJH        C(1,1:4)=<9.5,-17.5,12.5,-19.5>
C        C(1,1)=9.5
C        C(1,2)=-17.5
C        C(1,3)=12.5
C        C(1,4)=-19.5
C  2s2
CCJH        C(2,1:4)=<18.2,-4.0,2.8,-20.2>
C        C(2,1)=18.2
C        C(2,2)=-4.0
C        C(2,3)=2.8
C        C(2,4)=-20.2
CCJH        XIE(1:2)=<XI(IE),28.5>
C        XIE(1)=XI(IE)
C        XIE(2)=28.5
      ELSEIF (IE.EQ.IO2) THEN
C  O II
        NMAX=2
C  2p3 Valance shell, 5 electrons, IP 35.1 eV
        C(1,1)=25.0
        C(1,2)=-8.0
        C(1,3)=8.4
        C(1,4)=-29.5
C  2s2 Inner shell, 2 electrons, IP 42.6 eV
        C(2,1)=17.8
        C(2,2)=-3.8
        C(2,3)=2.9
        C(2,4)=-18.1
        XIE(1)=XI(IE)
        XIE(2)=42.6   ! D.c.
      ELSEIF (IE.EQ.IO3) THEN
C  O III
        NMAX=2
C  2p2 Valence shell, 4 electrons
        C(1,1)=25.0
        C(1,2)=-7.0
        C(1,3)=5.0
        C(1,4)=-18.0
C  2s2
        C(2,1)=17.3
        C(2,2)=-3.5
        C(2,3)=2.9
        C(2,4)=-16.1
C
        if (Nstages <= 3) C = 1D-5   ! CARE dont ionize more than III

        XIE(1)=XI(IE)
        XIE(2)=63.8
      ELSEIF (IE.EQ.IO4) THEN
C  O IV
        NMAX=2
C  2p
        C(1,1)=15.0
        C(1,2)=-5.0
        C(1,3)=2.2
        C(1,4)=-10.5
C  2s2
        C(2,1)=16.8
        C(2,2)=-3.3
        C(2,3)=2.8
        C(2,4)=-14.1

        if (Nstages <= 4) C = 1E-5
C
        XIE(1)=XI(IE)
        XIE(2)=87.6
      ELSEIF (IE.EQ.IO5) THEN
C  O V
        NMAX=2
C  2s2
        C(1,1)=16.4
        C(1,2)=-3.0
        C(1,3)=2.9
        C(1,4)=-12.0
C  1s2
        C(2,1)=20.8
        C(2,2)=-6.0
        C(2,3)=4.1
        C(2,4)=-18.0
C
        XIE(1)=XI(IE)
        XIE(2)=644.0
      ELSEIF (IE.EQ.INE1) THEN
C  NE I
        NMAX=2
C  2P6 Valance shell, 8 electrons, IP 21.6 eV
        C(1,1)=40.0
        C(1,2)=-42.0
        C(1,3)=18.0
        C(1,4)=-56.0
C  2S2 Inner shell, 2 electrons, IP 48.5 eV. Significantly small cross section per electron than for valence ones.
        C(2,1)=19.0
        C(2,2)=-4.9
        C(2,3)=2.8
        C(2,4)=-22.0   ! D.c
        XIE(1)=XI(IE)
        XIE(2)=48.5
C New 990824 data for Ne II, III, IV and V 
      ELSEIF (IE.EQ.INE2) THEN
C  NE II
        NMAX=2
C  2P5  ! Valence shell, 7 electrons, IP 41.1 eV
        C(1,1)=37.0
        C(1,2)=-33.0
        C(1,3)=15.5
        C(1,4)=-46.0
C  2S2 Inner shell, 2 electron IP 66.4 eV
        C(2,1)=18.6
        C(2,2)=-4.6
        C(2,3)=2.8
        C(2,4)=-20.2
        XIE(1)=XI(IE)
        XIE(2)=66.4
      ELSEIF (IE.EQ.INE3) THEN
C  NE III
        NMAX=2
C  2P4
        C(1,1)=33.0
        C(1,2)=-17.5
        C(1,3)=11.2
        C(1,4)=-33.0
C  2S2
        C(2,1)=18.2
        C(2,2)=-4.4
        C(2,3)=2.8
        C(2,4)=-18.4

        if (Nstages <= 3) C = 1D-5  ! CARE dont ionize more than III

        XIE(1)=XI(IE)
        XIE(2)=86.2
      ELSEIF (IE.EQ.INE4) THEN
C  NE IV
        NMAX=2
C  2P3
        C(1,1)=34.0
        C(1,2)=-10.0
        C(1,3)=7.5
        C(1,4)=-25.0
C  2S2
        C(2,1)=17.8
        C(2,2)=-4.0
        C(2,3)=2.8
        C(2,4)=-16.7

        if (Nstages <= 4) C = 1E-5

        XIE(1)=XI(IE)
        XIE(2)=108.0
      ELSEIF (IE.EQ.INE5) THEN
C  NE V
        NMAX=2
C  2P2
        C(1,1)=25.5
        C(1,2)=-8.5
        C(1,3)=4.5
        C(1,4)=-16.8
C  2S2
        C(2,1)=17.4
        C(2,2)=-3.8
        C(2,3)=2.8
        C(2,4)=-14.9
        XIE(1)=XI(IE)
        XIE(2)=139.0
      ELSEIF (IE.EQ.INA1) THEN
C  NA I
        NMAX=2
C  3S Valence shell, 1 electron, IP 5.14 eV
        C(1,1)=16.0
        C(1,2)=-1.0
        C(1,3)=0.2
        C(1,4)=-13.5
C  2P6 First inner shell, 6 electrons, IP 34 eV
        C(2,1)=63.9
        C(2,2)=-27.0
        C(2,3)=33.0
        C(2,4)=-80.0

        XIE(1)=XI(IE)
        XIE(2)=34.0     ! D.c   The cross section for the inner shell is about half the valence one. This means that the effective ionization potential is about 2/3*5.14 + 1/3*34 = 15 eV, 3 times higher than the valence one!
      ELSEIF (IE.EQ.INA2) THEN
C  NA II
        NMAX=2
C  2P6 !Valence shell, 6 electrons, IP 47.3 eV
        C(1,1)=40.0
        C(1,2)=-28.0
        C(1,3)=19.4
        C(1,4)=-60.0
C  2S2  ! First inner shell, 2 electrons IP 80.1 eV
        C(2,1)=19.2
        C(2,2)=-5.3
        C(2,3)=2.8
        C(2,4)=-21.2  ! D.c.
C  SPECIAL TREATMENT FOR NA II; THE CROSS SECTION GIVEN BY ARNAUD AND 
C  ROTHENFLUG IS LESS THAN ZERO CLOSE TO THE IONIZATION POTENTIAL
        XIE(1)=47.286
        XIE(2)=80.1
C New 990824 data for Na III, IV and V 
      ELSEIF (IE.EQ.INA3) THEN
C  NA III
        NMAX=2
C  2P5
        C(1,1)=50.1
        C(1,2)=-20.2
        C(1,3)=14.8
        C(1,4)=-41.7
C  2S2
        C(2,1)=18.8
        C(2,2)=-5.0
        C(2,3)=2.8
        C(2,4)=-19.6

        if (Nstages <= 3) C = 1D-5  ! CARE dont ionize more than III

        XIE(1)=XI(IE)
        XIE(2)=102.0
      ELSEIF (IE.EQ.INA4) THEN
C  NA IV
        NMAX=2
C  2P4
        C(1,1)=43.3
        C(1,2)=-16.3
        C(1,3)=10.7
        C(1,4)=-33.4
C  2S2
        C(2,1)=18.4
        C(2,2)=-4.7
        C(2,3)=2.8
        C(2,4)=-18.0

        if (Nstages <= 4) C = 1E-5

        XIE(1)=XI(IE)
        XIE(2)=126.0
      ELSEIF (IE.EQ.INA5) THEN
C  NA V
        NMAX=2
C  2P3
        C(1,1)=35.1
        C(1,2)=-12.4
        C(1,3)=7.2
        C(1,4)=-25.1
C  2S2
        C(2,1)=18.0
        C(2,2)=-4.3
        C(2,3)=2.8
        C(2,4)=-16.3
        XIE(1)=XI(IE)
        XIE(2)=151.0
      ELSEIF (IE.EQ.IMG1) THEN
C  MG I
        NMAX=3
C  3S2, Valence shell, 2 electrons, IP 7.6 eV
        C(1,1)=18.0
        C(1,2)=-1.0
        C(1,3)=0.6
        C(1,4)=-4.0
C  2P6, FIrst inner shell, 6 electrons IP 54.0 eV
        C(2,1)=37.7
        C(2,2)=-30.0
        C(2,3)=24.8
        C(2,4)=-62.0
C  2S2 Second inner shell, 2 electrons, IP 92.2 eV
        C(3,1)=17.6
        C(3,2)=-5.2
        C(3,3)=3.3
        C(3,4)=-19.0     ! D.c.  Significantly smaller cross sections for inner shell electrons, but huge IP can make difference anyway.
        XIE(1)=XI(IE)
        XIE(2)=54.0
        XIE(3)=92.2
      ELSEIF (IE.EQ.IMG2) THEN
C  MG II
        NMAX=3
C  3S  Valence shell, 1 electron, IP 15 eV
        C(1,1)=9.0
        C(1,2)=-3.6
        C(1,3)=0.3
        C(1,4)=-5.4
C  2P6  IP 65 eV
        C(2,1)=37.7
        C(2,2)=-30.0
        C(2,3)=24.8
        C(2,4)=-62.0
C  2S2   IP 104.5 eV
        C(3,1)=17.6
        C(3,2)=-5.2
        C(3,3)=3.3
        C(3,4)=-19.0   ! D.c.
        XIE(1)=XI(IE)
        XIE(2)=65.0
        XIE(3)=104.5
      ELSEIF (IE.EQ.IMG3) THEN
C  MG III
        NMAX=2
C  2P6
        C(1,1)=55.5
        C(1,2)=-24.1
        C(1,3)=18.7
        C(1,4)=-65.0
C  2S2
        C(2,1)=19.3
        C(2,2)=-5.6
        C(2,3)=2.8
        C(2,4)=-20.5
C
        if (Nstages <= 3) C = 1D-5   ! CARE dont ionize more than III

        XIE(1)=XI(IE)
        XIE(2)=119.0
      ELSEIF (IE.EQ.IMG4) THEN
C  MG IV
        NMAX=2
C  2P5
        C(1,1)=50.1
        C(1,2)=-20.2
        C(1,3)=14.2
        C(1,4)=-41.7
C  2S2
        C(2,1)=19.0
        C(2,2)=-5.3
        C(2,3)=2.8
        C(2,4)=-19.0

        if (Nstages <= 4) C = 1E-5
C
        XIE(1)=XI(IE)
        XIE(2)=144.0
      ELSEIF (IE.EQ.IMG5) THEN
C  MG V
        NMAX=2
C  2P4
        C(1,1)=43.3
        C(1,2)=-16.3
        C(1,3)=10.3
        C(1,4)=-33.4
C  2S2
        C(2,1)=18.6
        C(2,2)=-4.9
        C(2,3)=2.8
        C(2,4)=-17.5
C
        XIE(1)=XI(IE)
        XIE(2)=172.0
      ELSEIF (IE.EQ.ISI1) THEN
C  SI I
        NMAX=2
C  3P2 Valence shell, 2 electrons, IP 8.1 eV
        C(1,1)=74.5
        C(1,2)=-49.4
        C(1,3)=1.3
        C(1,4)=-54.6
C  3S2  First inner shell, 2 electrons, IP 13.5 eV
        C(2,1)=53.8
        C(2,2)=-35.8
        C(2,3)=1.4
        C(2,4)=-40.7
        XIE(1)=XI(IE)
        XIE(2)=13.5
      ELSEIF (IE.EQ.ISI2) THEN
C  SI II
        NMAX=2
C  3P, Valence shell, 1 electron, IP 16.3 eV
        C(1,1)=50.4
        C(1,2)=-33.4
        C(1,3)=0.6
        C(1,4)=-36.9
C  3S2
        C(2,1)=55.1
        C(2,2)=-37.2
        C(2,3)=1.4
        C(2,4)=-41.0
        XIE(1)=XI(IE)
        XIE(2)=22.9   ! D.c.
      ELSEIF (IE.EQ.ISI3) THEN
C  SI III
        NMAX=3
C  3S2
        C(1,1)=19.8
        C(1,2)=-5.7
        C(1,3)=1.3
        C(1,4)=-11.9
C  2P6
        C(2,1)=66.7
        C(2,2)=-24.8
        C(2,3)=18.7
        C(2,4)=-65.0
C  2S2
        C(3,1)=22.0
        C(3,2)=-7.2
        C(3,3)=3.3
        C(3,4)=-20.9
C
        if (Nstages <= 3) C = 1D-5 ! CARE Dont ionize more than III

        XIE(1)=XI(IE)
        XIE(2)=133.0
        XIE(3)=176.6
      ELSEIF (IE.EQ.ISI4) THEN
C  SI IV
        NMAX=3
C  3S
        C(1,1)=9.0
        C(1,2)=-3.0
        C(1,3)=0.6
        C(1,4)=-5.8
C  2P6
        C(2,1)=66.7
        C(2,2)=-24.8
        C(2,3)=18.7
        C(2,4)=-65.0
C  2S2
        C(3,1)=22.0
        C(3,2)=-7.2
        C(3,3)=3.3
        C(3,4)=-20.9

        if (Nstages <= 4) C = 1E-5
C
        XIE(1)=XI(IE)
        XIE(2)=148.0
        XIE(3)=193.5
      ELSEIF (IE.EQ.ISI5) THEN
C  SI V
        NMAX=2
C  2P6
        C(1,1)=72.0
        C(1,2)=-24.1
        C(1,3)=17.4
        C(1,4)=-50.0
C  2S2
        C(2,1)=19.6
        C(2,2)=-6.2
        C(2,3)=2.8
        C(2,4)=-19.0
C
        XIE(1)=XI(IE)
        XIE(2)=217.0
      ELSEIF (IE.EQ.IS1) THEN
C  S I
        NMAX=2
C  3P4 Valence shell, 4 electrons, IP 10.4 eV
        C(1,1)=6.0
        C(1,2)=-22.0
        C(1,3)=20.0
        C(1,4)=-20.0
C  3S2  First inner shell, 2 electrons, IP 20.2 eV
        C(2,1)=51.3
        C(2,2)=-33.2
        C(2,3)=1.4
        C(2,4)=-40.2
        XIE(1)=XI(IE)
        XIE(2)=20.2      ! D.c.
      ELSEIF (IE.EQ.IS2) THEN
C  S II
        NMAX=2
C  3P3  Valence shell, 3 electrons, IP 23.4 eV
        C(1,1)=98.7
        C(1,2)=-65.4
        C(1,3)=1.9
        C(1,4)=-72.3
C  3S2  First inner shell, IP 30.7 eV
        C(2,1)=52.5
        C(2,2)=-34.5
        C(2,3)=1.4
        C(2,4)=-40.5
        XIE(1)=XI(IE)
        XIE(2)=30.7         ! D.c.
      ELSEIF (IE.EQ.IS3) THEN
C  S III
        NMAX=2
C  3P2
        C(1,1)=74.5
        C(1,2)=-49.4
        C(1,3)=1.3
        C(1,4)=-54.6
C  3S2
        C(2,1)=53.8
        C(2,2)=-35.8
        C(2,3)=1.4
        C(2,4)=-40.7
C
        if (Nstages <= 3) C = 1D-5  ! CARE dont ionize more than III
        
        XIE(1)=XI(IE)
        XIE(2)=43.8
      ELSEIF (IE.EQ.IS4) THEN
C  S IV
        NMAX=2
C  3P
        C(1,1)=50.4
        C(1,2)=-33.4
        C(1,3)=0.6
        C(1,4)=-36.9
C  3S2
        C(2,1)=55.1
        C(2,2)=-37.2
        C(2,3)=1.4
        C(2,4)=-41.0

        if (Nstages <= 4) C = 1E-5
C
        XIE(1)=XI(IE)
        XIE(2)=57.6
      ELSEIF (IE.EQ.IS5) THEN
C  S V
        NMAX=3
C  3S2
        C(1,1)=19.8
        C(1,2)=-5.7
        C(1,3)=1.6
        C(1,4)=-11.9
C  2P6
        C(2,1)=73.2
        C(2,2)=-27.0
        C(2,3)=15.8
        C(2,4)=-61.1
C  2S2
        C(3,1)=23.1
        C(3,2)=-8.0
        C(3,3)=3.3
        C(3,4)=-19.5
C
        XIE(1)=XI(IE)
        XIE(2)=239.0
        XIE(3)=288.2
      ELSEIF (IE.EQ.IAR1) THEN
C  AR I
        NMAX=2
C  3P6 Valence shell, 6 electrons, IP 15.8 eV
        C(1,1)=171.1
        C(1,2)=-78.0
        C(1,3)=3.8
        C(1,4)=-169.0
C  3S2 First inner shell, 2 electrons IP 29.2 eV
        C(2,1)=48.7
        C(2,2)=-30.5
        C(2,3)=1.4
        C(2,4)=-39.7
        XIE(1)=XI(IE)
        XIE(2)=29.2      !D.c.
c
C New 990824 Argon data for Ar II, III, IV and V 
      ELSEIF (IE.EQ.IAR2) THEN
C  AR II
        NMAX=2
C  3P5 Valence shell, 5 electrons, IP 27.6 eV
        C(1,1)=147.0
        C(1,2)=-97.4
        C(1,3)=3.2
        C(1,4)=-107.7
C  3S2  First inner shell, 2 electrons, IP 41.7 eV
        C(2,1)=50.0
        C(2,2)=-31.8
        C(2,3)=1.4
        C(2,4)=-40.0    ! D.c.
c
        XIE(1)=XI(IE)
        XIE(2)=41.7
      ELSEIF (IE.EQ.IAR3) THEN
C  AR III
        NMAX=2
C  3P4
        C(1,1)=122.8
        C(1,2)=-81.4
        C(1,3)=2.6
        C(1,4)=-90.0
C  3S2
        C(2,1)=51.3
        C(2,2)=-33.2
        C(2,3)=1.4
        C(2,4)=-40.2

        if (Nstages <= 3) C = 1D-5  ! CARE dont ionize more than III

        XIE(1)=XI(IE)
        XIE(2)=55.5
      ELSEIF (IE.EQ.IAR4) THEN
C  AR IV
        NMAX=2
C  3P3
        C(1,1)=98.7
        C(1,2)=-65.4
        C(1,3)=1.9
        C(1,4)=-72.3
C  3S2
        C(2,1)=52.5
        C(2,2)=-34.5
        C(2,3)=1.4
        C(2,4)=-40.5

        if (Nstages <= 4) C = 1E-5

        XIE(1)=XI(IE)
        XIE(2)=70.4
      ELSEIF (IE.EQ.IAR5) THEN
C  AR V
        NMAX=2
C  3P2
        C(1,1)=74.5
        C(1,2)=-49.4
        C(1,3)=1.3
        C(1,4)=-54.6
C  3S2
        C(2,1)=53.8
        C(2,2)=-35.8
        C(2,3)=1.4
        C(2,4)=-40.7
        XIE(1)=XI(IE)
        XIE(2)=87.6
c
      ELSEIF (IE.EQ.ICA1) THEN
C  CA I
        NMAX=3
C  4S2 Valence shell, 2 electrons, IP 6.1 eV
        C(1,1)=2.5
        C(1,2)=-2.5
        C(1,3)=8.0
        C(1,4)=-5.5
C  3P6 First inner shell, 6 electrons, IP 28.0 eV
        C(2,1)=74.3
        C(2,2)=-24.2
        C(2,3)=7.0
        C(2,4)=-63.0
C  3S2  Second inner shell, 2 electrons, IP 40.3 eV
        C(3,1)=17.6
        C(3,2)=-3.8
        C(3,3)=1.9
        C(3,4)=-13.8    !D.c.
        XIE(1)=XI(IE)
        XIE(2)=28.0
        XIE(3)=40.3
      ELSEIF (IE.EQ.ICA2) THEN
C  CA II
        NMAX=3
C  4S Valence shell, 1 electron, IP 11.9 eV
        C(1,1)=7.9
        C(1,2)=-2.0
        C(1,3)=0.2
        C(1,4)=-6.0
C  3P6  IP 37 eV
        C(2,1)=74.3
        C(2,2)=-24.2
        C(2,3)=7.0
        C(2,4)=-68.0
C  3S2 IP 45.2 eV
        C(3,1)=17.6
        C(3,2)=-3.8
        C(3,3)=1.9
        C(3,4)=-13.8
        XIE(1)=XI(IE)
        XIE(2)=37.0
        XIE(3)=45.2
      ELSEIF (IE.EQ.ICA3) THEN
C  CA III
        NMAX=2
C  3P6
        C(1,1)=74.3
        C(1,2)=-24.2
        C(1,3)=7.0
        C(1,4)=-63.0
C  3S2
        C(2,1)=17.6
        C(2,2)=-3.8
        C(2,3)=1.9
        C(2,4)=-13.8
C
        if (Nstages <= 3) C = 1D-5   ! CARE dont ionize more than III

        XIE(1)=XI(IE)
        XIE(2)=70.1
      ELSEIF (IE.EQ.ICA4) THEN
C  CA IV
        NMAX=2
C  3P5
        C(1,1)=55.8
        C(1,2)=-15.8
        C(1,3)=6.4
        C(1,4)=-44.5
C  3S2
        C(2,1)=16.2
        C(2,2)=-3.2
        C(2,3)=1.8
        C(2,4)=-11.6

        if (Nstages <= 4) C = 1E-5
C
        XIE(1)=XI(IE)
        XIE(2)=86.4
      ELSEIF (IE.EQ.ICA5) THEN
C  CA V
        NMAX=2
C  3P4
        C(1,1)=47.1
        C(1,2)=-14.5
        C(1,3)=4.8
        C(1,4)=-35.5
C  3S2
        C(2,1)=18.9
        C(2,2)=-5.1
        C(2,3)=1.6
        C(2,4)=-13.2
C
        XIE(1)=XI(IE)
        XIE(2)=104.0
      ELSEIF (IE.EQ.IFE1) THEN
C  FE I  DATA FROM ARNAU & RAYMOND 1992
        NMAX=3
C  4S2 Valence shell, 2 electrons, IP 7.09 eV
        C(1,1)=31.9
        C(1,2)=-15.0
        C(1,3)=0.32
        C(1,4)=-28.1
C  3D6  IP 9.0 eV
        C(2,1)=15.0
        C(2,2)=-16.7
        C(2,3)=7.00
        C(2,4)=-12.7
C  3P6  IP 59.0 eV
        C(3,1)=115.0
        C(3,2)=-72.4
        C(3,3)=9.57
        C(3,4)=-107.0  !D.c.
C
        XIE(1)=XI(IE)
        XIE(2)=9.0
        XIE(3)=59.0
      ELSEIF (IE.EQ.IFE2) THEN
C  FE II  Arnaud & Raymond 1992
        NMAX=3
C  4S Valence shell, 1 electron, IP 16.18 eV
        C(1,1)=17.4
        C(1,2)=-3.27
        C(1,3)=0.16
        C(1,4)=-10.2
C  3D6  IP 24.83 eV
        C(2,1)=30.1
        C(2,2)=-38.8
        C(2,3)=18.6
        C(2,4)=-45.7
C  3P6  IP 83.37 eV
        C(3,1)=115.0
        C(3,2)=-72.4
        C(3,3)=9.57
        C(3,4)=-107.0  ! D.c.
C
        XIE(1)=XI(IE)
        XIE(2)=24.83
        XIE(3)=83.37
      ELSEIF (IE.EQ.IFE3) THEN
C  FE III
        NMAX=3
C  3D6
        C(1,1)=84.8
        C(1,2)=-67.6
        C(1,3)=21.0
        C(1,4)=-84.1
C  3P6
        C(2,1)=87.7
        C(2,2)=-49.6
        C(2,3)=16.4
        C(2,4)=-84.8
C  3S2
        C(3,1)=25.9
        C(3,2)=-11.7
        C(3,3)=2.32
        C(3,4)=-23.9

        if (Nstages <= 3) C = 1D-5  ! CARE dont ionize more than III
C
        XIE(1)=XI(IE)
        XIE(2)=93.28
        XIE(3)=131.9
      ELSEIF (IE.EQ.IFE4) THEN
C  FE IV
        NMAX=3
C  3D5
        C(1,1)=77.4
        C(1,2)=-43.9
        C(1,3)=19.6
        C(1,4)=-81.9
C  3P6
        C(2,1)=79.1
        C(2,2)=-30.0
        C(2,3)=8.38
        C(2,4)=-74.6
C  3S2
        C(3,1)=16.7
        C(3,2)=-3.44
        C(3,3)=2.32
        C(3,4)=-15.1

        if (Nstages <= 4) C = 1E-5
C
        XIE(1)=XI(IE)
        XIE(2)=111.9
        XIE(3)=151.0
      ELSEIF (IE.EQ.IFE5) THEN
C  FE V
        NMAX=3
C  3D4
        C(1,1)=48.1
        C(1,2)=-20.4
        C(1,3)=16.2
        C(1,4)=-48.4
C  3P6
        C(2,1)=66.8
        C(2,2)=-18.9
        C(2,3)=9.29
        C(2,4)=-60.5
C  3S2
        C(3,1)=13.4
        C(3,2)=-0.410
        C(3,3)=2.33
        C(3,4)=-10.9
C
        XIE(1)=XI(IE)
        XIE(2)=132.7
        XIE(3)=172.4
      ELSEIF (IE.EQ.IFE6) THEN
C  FE VI
        NMAX=3
C  3D3
        C(1,1)=36.9
        C(1,2)=-11.8
        C(1,3)=10.3
        C(1,4)=-31.5
C  3P6
        C(2,1)=67.0
        C(2,2)=-18.6
        C(2,3)=9.43
        C(2,4)=-56.5
C  3S2
        C(3,1)=12.7
        C(3,2)=-0.086
        C(3,3)=2.42
        C(3,4)=-8.97
C
        XIE(1)=XI(IE)
        XIE(2)=155.5
        XIE(3)=195.8
c      ELSEIF (IE.EQ.IFE7) THEN
cC  FE VII
c        NMAX=3
cC  3D2
c        C(1,1)=14.6
c        C(1,2)=-4.36
c        C(1,3)=5.98
c        C(1,4)=-10.5
cC  3P6
c        C(2,1)=67.9
c        C(2,2)=-20.6
c        C(2,3)=9.82
c        C(2,4)=-53.7
cC  3S2
c        C(3,1)=15.6
c        C(3,2)=-2.29
c        C(3,3)=2.3
c        C(3,4)=-10.6
C
c        XIE(1)=XI(IE)
c        XIE(2)=180.0
c        XIE(3)=220.9
C
      ELSEIF (IE.EQ.ICO1) THEN
C  CO I  WHERE IS THIS DATA FROM? NOT IN AR85
        NMAX=3
C  4S2 Valence shell, 2 electrons IP 
        C(1,1)=3.9
        C(1,2)=-1.3
        C(1,3)=0.4
        C(1,4)=-1.9
C  3D7
        C(2,1)=11.1
        C(2,2)=-3.5
        C(2,3)=0.35
        C(2,4)=-5.35
C  3P6
        C(3,1)=69.9
        C(3,2)=-23.7
        C(3,3)=9.5
        C(3,4)=-51.7
C
        XIE(1)=XI(IE)
        XIE(2)=9.0
        XIE(3)=59.0
      ELSEIF (IE.EQ.ICO2) THEN
C  CO II
        NMAX=3
C  3D8
        C(1,1)=28.4
        C(1,2)=-8.8
        C(1,3)=0.9
        C(1,4)=-13.6
C  3P6
        C(2,1)=69.9
        C(2,2)=-23.7
        C(2,3)=9.5
        C(2,4)=-51.7
C  3S2
        C(3,1)=19.2
        C(3,2)=-5.7
        C(3,3)=2.3
        C(3,4)=-12.7
C
        XIE(1)=XI(IE)
        XIE(2)=81.0
        XIE(3)=142.0
      ELSEIF (IE.EQ.ICO3) THEN
C  CO III
        NMAX=3
C  3D7
        C(1,1)=38.6
        C(1,2)=-12.3
        C(1,3)=1.2
        C(1,4)=-18.7
C  3P6
        C(2,1)=69.9
        C(2,2)=-23.7
        C(2,3)=9.5
        C(2,4)=-51.7
C  3S2
        C(3,1)=19.2
        C(3,2)=-5.7
        C(3,3)=2.3
        C(3,4)=-12.7
C
        if (Nstages <= 3) C = 1D-5  ! CARE dont ionize more than III

        XIE(1)=XI(IE)
        XIE(2)=103.0
        XIE(3)=141.0
      ELSEIF (IE.EQ.ICO4) THEN
C  CO IV
        NMAX=3
C  3D6
        C(1,1)=45.1
        C(1,2)=-14.3
        C(1,3)=1.4
        C(1,4)=-21.8
C  3P6
        C(2,1)=69.9
        C(2,2)=-23.7
        C(2,3)=9.5
        C(2,4)=-51.7
C  3S2
        C(3,1)=19.2
        C(3,2)=-5.7
        C(3,3)=2.3
        C(3,4)=-12.7

        if (Nstages <= 4) C = 1E-5
C
        XIE(1)=XI(IE)
        XIE(2)=125.0
        XIE(3)=162.0
      ELSEIF (IE.EQ.ICO5) THEN
C  CO V
        NMAX=3
C  3D5
        C(1,1)=43.2
        C(1,2)=-13.8
        C(1,3)=1.3
        C(1,4)=-20.8
C  3P6
        C(2,1)=69.9
        C(2,2)=-23.7
        C(2,3)=9.5
        C(2,4)=-51.7
C  3S2
        C(3,1)=19.2
        C(3,2)=-5.7
        C(3,3)=2.3
        C(3,4)=-12.7
C
        XIE(1)=XI(IE)
        XIE(2)=147.0
        XIE(3)=184.0
C
C
      ELSEIF (IE.EQ.INI1) THEN
C  NI I
        NMAX=3
C  4S2 Valence shell, 2 electrons, IP 8.7 eV
        C(1,1)=2.5
        C(1,2)=-0.8
        C(1,3)=0.2
        C(1,4)=-1.2
C  3D8  IP 10 eV
        C(2,1)=12.6
        C(2,2)=-4.0
        C(2,3)=0.4
        C(2,4)=-6.1
C  3P6   IP 73 eV
        C(3,1)=69.9
        C(3,2)=-23.7
        C(3,3)=9.5
        C(3,4)=-51.7
        XIE(1)=XI(IE)
        XIE(2)=10.0
        XIE(3)=73.0
      ELSEIF (IE.EQ.INI2) THEN
C  NI II
        NMAX=3
C  3D9 Valence, 9 electrons, IP 18.2 eV
        C(1,1)=32.0
        C(1,2)=-10.0
        C(1,3)=1.0
        C(1,4)=-15.4
C  3P6
        C(2,1)=69.9
        C(2,2)=-23.7
        C(2,3)=9.5
        C(2,4)=-51.7
C  3S2
        C(3,1)=19.2
        C(3,2)=-5.7
        C(3,3)=2.3
        C(3,4)=-12.7     ! D.c.
C
        XIE(1)=XI(IE)
        XIE(2)=97.0
        XIE(3)=142.0
      ELSEIF (IE.EQ.INI3) THEN
C  NI III
        NMAX=3
C  3D8
        C(1,1)=44.4
        C(1,2)=-14.1
        C(1,3)=1.4
        C(1,4)=-21.5
C  3P6
        C(2,1)=69.9
        C(2,2)=-23.7
        C(2,3)=9.5
        C(2,4)=-51.7
C  3S2
        C(3,1)=19.2
        C(3,2)=-5.7
        C(3,3)=2.3
        C(3,4)=-12.7

        if (Nstages <= 3) C = 1E-5  ! CARE dont ionize more than III
C
        XIE(1)=XI(IE)
        XIE(2)=122.0
        XIE(3)=166.0
      ELSEIF (IE.EQ.INI4) THEN
C  NI IV
        NMAX=3
C  3D7
        C(1,1)=50.3
        C(1,2)=-16.0
        C(1,3)=1.6
        C(1,4)=-24.3
C  3P6
        C(2,1)=69.9
        C(2,2)=-23.7
        C(2,3)=9.5
        C(2,4)=-51.7
C  3S2
        C(3,1)=19.2
        C(3,2)=-5.7
        C(3,3)=2.3
        C(3,4)=-12.7

        if (Nstages <= 4) C = 1E-5
C
        XIE(1)=XI(IE)
        XIE(2)=146.0
        XIE(3)=190.0
      ELSEIF (IE.EQ.INI5) THEN
C  NI V
        NMAX=3
C  3D6
        C(1,1)=49.9
        C(1,2)=-15.9
        C(1,3)=1.6
        C(1,4)=-24.1
C  3P6
        C(2,1)=69.9
        C(2,2)=-23.7
        C(2,3)=9.5
        C(2,4)=-51.7
C  3S2
        C(3,1)=19.2
        C(3,2)=-5.7
        C(3,3)=2.3
        C(3,4)=-12.7
C
        XIE(1)=XI(IE)
        XIE(2)=171.0
        XIE(3)=215.0


      ELSEIF(IE.EQ.ICM1) then

		NMAX=1          
		C(1,1)=-59.2       ! Sofie L. fits to Itikawa 2015
		C(1,2)=-9.28E-3
		C(1,3)=36.4
		C(1,4)=24.0
		XIE(1)=XI(IE)

      ELSEIF(IE.EQ.ISM1) then

		NMAX=1          
		C(1,1)=-93.9       ! Sofie L. fits to Itikawa 2015
		C(1,2)=2.43E-2
		C(1,3)=74.5
		C(1,4)=25.2
		XIE(1)=XI(IE)

      ELSEIF(IE.EQ.ISS1) then

		NMAX=1          
		C(1,1)=-112.0       ! Sofie L. fits to Itikawa 2015
		C(1,2)=1.1
		C(1,3)=86.1
		C(1,4)=49.9
		XIE(1)=XI(IE)

      ELSEIF(IE.EQ.IUM1) then

		NMAX=1          
		C(1,1)=-93.9       ! Sofie L. fits to Itikawa 2015
		C(1,2)=2.43E-2
		C(1,3)=74.5
		C(1,4)=25.2
		XIE(1)=XI(IE)

      ELSE  ! use Lotz formula for atoms, CO I for molecules  

		 if (IE <= 125) then 
			iebase = 106
		 elseif (IE >= 146) then
			iebase = 146
		 else ! Molecule (126-145 slots) !TEMP
			iebase = 0
         end if

         IF ((iebase > 0) .and. MOD(IE-iebase,5) .GE. (Nstages-1)) THEN !don't ionise above Nstages

            NMAX=1
            C=1E-5
            XIE(1)=XI(IE)

         ELSEIF (iebase > 0) THEN ! iebase > 0 makes sure its not a molecule

			NMAX=1
			C=0
			C(1,3)=4.  ! Lotz 1967, with psi=1
			XIE(1)=XI(IE)

         ELSE  ! Molecule with unknown cross section - use cross section of CO I

            NMAX=1
	    	C(1,1)=-59.2       
	    	C(1,2)=-9.28E-3
	    	C(1,3)=36.4
	    	C(1,4)=24.0
	    	XIE(1)=XI(IE)

         END IF

      ENDIF 

      SSUM=0.

      IF (IE.EQ.IO1) THEN
C  SPECIAL TREATMENT FOR O I ; INCLUDING AUTO-IONIZATION
        IF (Ekin.LE.1000.) THEN  ! If electron energy is less than 1000 eV
          DO J=1,24
            IF (Ekin.LE.OE(J)) GOTO 5  ! OE(J) = energy in eV in bin J : This loop finds the bin of Ekin
          ENDDO
 5        CONTINUE
          C1=(Ekin-OE(J-1))/(OE(J)-OE(J-1))       ! the fraction of the energy bin that Ekin is into 
          SIGT=OSIG(J-1)+C1*(OSIG(J)-OSIG(J-1))    ! interpolate cross section. OSIG is of order 50 --> SIGT is of order 50
          DO I=1,4
            FRAC(I)=OFRAC(I,J-1)+C1*(OFRAC(I,J)-OFRAC(I,J-1))  ! Interpolate fractions [%]
            IF (Ekin.LT.XIO(I)) FRAC(I)=0.  ! If electron energy is smaller than ionization threshold, fraction=0
          ENDDO  
        ELSEIF (Ekin.GT.1000.) THEN  ! If electron energy is over 1000 eV
          SIGT=(-43100+13200*LOG(Ekin))/Ekin     ! Bethe approximation? SIGT is of order 50
CJH          FRAC(1:4)=<35.5,29.5,17.0,18.0>
          FRAC(1)=35.5   ! Fractions in high energy limit
          FRAC(2)=29.5
          FRAC(3)=17.0
          FRAC(4)=18.0  
        ENDIF  
 9      CONTINUE
        IF (NOIF.GT.0) SIGT=SIGT*FRAC(NOIF)*0.01  ! 0.01 converts % to fraction. 
        SSUM=SIGT*1.E-18  ! Convert from Mega-barn to cm^2
       
      ELSE  
C  THE REST OF THE ELEMENTS
        DO J=1,NMAX       ! For each shell..
          U=Ekin/XIE(J)   ! Normalized energy...    
          IF (U.GT.1) THEN   ! Electron energy must be larger than ion. potential of this shell..
            SIG(J)=(C(J,1)*(1.-1./U)+C(J,2)*(1.-1./U)**2.+C(J,3)*LOG(U)+   ! Cross section value for this shell (AR85 formula)
     &             C(J,4)*LOG(U)/U)/(U*XIE(J)*XIE(J)) 
            SIG(J)=SIG(J)*1.E-14
          ELSE  !..otherwise cross section is zero.
            SIG(J)=0.
          ENDIF
          SSUM = SSUM + SIG(J)    ! Total cross section value (sum of all shells)
        ENDDO
      ENDIF

C   EXCITATION-AUTOIONIZATION..where is data from?

      SIGEA=0.
      IF (IE.EQ.IC4) THEN
C  C IV
      ELSEIF (IE.EQ.IMG2) THEN
C  MG II
        U=Ekin/52.
        IF (U.GT.1.) SIGEA=2.8E-17*(1.-1./U)/U  
      ELSEIF (IE.EQ.ISI2) THEN
C  SI II
        U=Ekin/77.6  
        IF (U.GT.1.) SIGEA=2.63E-17*(1.-1./U**3)/U
      ELSEIF (IE.EQ.ISI4) THEN
C  SI IV
        U=Ekin/104.
        IF (U.GT.1.) SIGEA=1.3E-17*(1.-1./U)/U  
      ELSEIF (IE.EQ.IS2) THEN
C  S II
        U=Ekin/97.8  
        IF (U.GT.1.) SIGEA=1.60E-17*(1.-1./U**3)/U
      ELSEIF (IE.EQ.ICA1) THEN
C  CA I
        U=Ekin/25.  
        IF (U.GT.1.) SIGEA=6.0E-17*(1.+1.12*LOG(U))/U
      ELSEIF (IE.EQ.ICA2) THEN
C  CA II      
        U=Ekin/25.  
        IF (U.GT.1.) SIGEA=9.8E-17*(1.+1.12*LOG(U))/U
      ELSEIF (IE.EQ.ICA5) THEN
C  CA V
        U=Ekin/287.81
        IF (U.GT.1.) SIGEA=(3.475E-18/U)*(1.-1./(U*U*U))
      ELSEIF (IE.GE.IFE1.AND.IE.LE.IFE6) THEN
        IF (IE.EQ.IFE1) THEN
C  FE I
          XIEA=0.
          A1=0.
          A2=0.
          A3=0.
          A4=0.
          A5=0.
        ELSEIF (IE.EQ.IFE2) THEN
C  FE II
          XIEA=0.
          A1=0.
          A2=0.
          A3=0.
          A4=0.
          A5=0.
        ELSEIF (IE.EQ.IFE3) THEN
C  FE III
          XIEA=57.3
          A1=0.878
          A2=0.
          A3=-115.0
          A4=71.2
          A5=47.9

          A1 = 0  ! TEMP no auto-ionization for Fe III
          A2 = 0
          A3 = 0
          A4 = 0
          A5 = 0
        ELSEIF (IE.EQ.IFE4) THEN
C  FE IV
          XIEA=60.0
          A1=3.02
          A2=67.5
          A3=-59.8
          A4=46.0
          A5=0.
        ELSEIF (IE.EQ.IFE5) THEN
C  FE V
          XIEA=81.0
          A1=0.
          A2=0.280
          A3=-16.2
          A4=35.5
          A5=0.
        ELSEIF (IE.EQ.IFE6) THEN
C  FE VI
          XIEA=95.0
          A1=0.
          A2=0.
          A3=-5.83
          A4=12.3
          A5=8.85          
        ENDIF
        SIGEA=0.
        IF (XIEA.GT.0.) THEN
          U=Ekin/XIEA
          IF (U.GT.1) THEN
            SIGEA=(A1+A2*(1.-1./U)+A3*(1.-1./(U*U))+A4*(1.-1./(U*U*U))+
     &            A5*LOG(U))/(U*XIEA) 
            SIGEA=SIGEA*1.E-16
          ENDIF
        ENDIF
      ENDIF

      FSIGI = SSUM + SIGEA    ! Total cross section is collisional ionization + autoionization

 99   CONTINUE
      if (FSIGI.lt.0. .and. (ie .ne. 25)) write(*,701) 'WARNING, PROBLEM IN FSIGI, FSIGI=',ie,Ekin,FSIGI   ! ERROR, FSIGI less than zero, Note Na II (IE=25) has a known problem close to threshold.

      if (FSIGI.lt.0) FSIGI=1D-30

 701  FORMAT(A,I4,1pe12.4,6E12.4)

      do i=1,5
        xiecr(i)=0.
        sigcr(i)=0.
      enddo
      do i=1,nmax
        xiecr(i)=xie(i)
        sigcr(i)=sig(i)
      enddo

      RETURN
      END        

      SUBROUTINE EION           ! Sets all ionization potentials
      IMPLICIT NONE
      INCLUDE 'PARAM.SF'
      INCLUDE 'DATIE.SF'
      REAL :: XI,XIO,EX,EXMIN
      COMMON/CENERG/XI(MIE),XIO(4),EX(MIE,MIL),EXMIN

      XI(IH1)=13.595
      XI(IHE1)=24.587
      XI(IHE2)=54.418
      XI(IC1)=11.87
      XI(IC1)=11.260
      XI(IC2)=24.383
      XI(IC3)=47.9
      XI(IC4)=64.5
      XI(IC5)=392.0
      XI(IN1)=14.534
      XI(IN2)=29.601
      XI(IN3)=47.4
      XI(IN4)=77.5
      XI(IN5)=97.9
      XI(IO1)=13.618
c ionization potentials to excited states in O II
      XIO(1)=13.618
      XIO(2)=16.9
      XIO(3)=18.6
      XIO(4)=28.5
c
      XI(IO2)=35.121
      XI(IO3)=54.935
      XI(IO4)=77.4
      XI(IO5)=114.0
c      XI(INE1)=21.6
      XI(INE1)=21.565
c      XI(INE2)=41.1
      XI(INE2)=40.963
      XI(INE3)=63.5
      XI(INE4)=97.1
      XI(INE5)=126.0
      XI(INA1)=5.139
c      XI(INA2)=47.286
c  kolla upp ion. pot. tvarsnittet blir < 0  for 47.3
      XI(INA2)=47.286
      XI(INA3)=71.7
      XI(INA4)=99.0
      XI(INA5)=138.0
      XI(IMG1)=7.646
      XI(IMG2)=15.035
      XI(IMG3)=80.1
      XI(IMG4)=109.0
      XI(IMG5)=141.0
      XI(ISI1)=8.152
      XI(ISI2)=16.349
      XI(ISI3)=33.493
      XI(ISI4)=45.1
      XI(ISI5)=167.0
      XI(IS1)=10.360
      XI(IS2)=23.338
      XI(IS3)=35.0
      XI(IS4)=47.3
      XI(IS5)=72.7
      XI(IAR1)=15.759
      XI(IAR2)=27.630
      XI(IAR3)=40.735
      XI(IAR4)=59.7
      XI(IAR5)=75.2
      XI(ICA1)=6.113
      XI(ICA2)=11.872
      XI(ICA3)=51.2
      XI(ICA4)=67.3
      XI(ICA5)=84.5
c ionization potentials from Arnaud and Raymond,1992
C                       FE I
c      XI(IFE1)=7.90
c      XI(IFE1)=7.870
      XI(IFE1) = 7.902
C                       FE II
c      XI(IFE2)=16.18
      XI(IFE2) = 16.188
C                       FE III
c      XI(IFE3)=34.75
      XI(IFE3) = 30.651
C                       FE IV
      XI(IFE4)=53.74
C                       FE V
      XI(IFE5)=75.15
C                       FE VI
      XI(IFE6)=98.69
C                       FE VII
c      XI(IFE7)=124.2
C
c the following ionization potentials are from Arnaud and Rothenflug,
c 1985.
c      XI(IFE1)=7.9
c      XI(IFE2)=16.2
c      XI(IFE3)=30.7
c      XI(IFE4)=54.8
c      XI(IFE5)=75.0
c      XI(IFE6)=99.0
c      XI(IFE7)=125.0
C                       CO I
c      XI(ICO1)=7.86
      XI(ICO1) = 7.881
C                       CO II
c     XI(ICO2)=17.06
      XI(ICO2) = 17.084
C                       CO III
      XI(ICO3)=33.50
C                       CO IV
      XI(ICO4)=51.3
C                       CO V
      XI(ICO5)=79.5
C                       NI I
c     XI(INI1)=8.7
c     XI(INI1)=7.635
      XI(INI1) = 7.640
C                       NI II
c     XI(INI2)=18.2
      XI(INI2)=18.169
C                       NI III
      XI(INI3)=35.187
C                       NI IV
      XI(INI4)=54.9
C                       NI V
      XI(INI5)=75.5
C                       Ti I
      XI(ITI1)=6.828
C                       Ti II
      XI(ITI2)=13.576
C                       Ti III
      XI(ITI3)=27.491
C                       Ti IV
      XI(ITI4)=43.267
C                       Al I
      XI(IAL1)=5.986
C                       Al II
      XI(IAL2)=18.829
C                       Al III
      XI(IAL3)=28.447
C                       Al IV
      XI(IAL4)=119.99
C                       Cr I
      XI(ICR1)=6.767
C                       Cr II
      XI(ICR2)=16.486
C                       Cr III
      XI(ICR3)= 30.959  
C                       Cr IV
      XI(ICR4)=49.16
C                       Mn I      
      XI(IMN1)=7.434
C                       Mn II      
      XI(IMN2)=15.640
C                       Mn III
      XI(IMN3)=33.668
C                       Mn IV
      XI(IMN4)=51.21
C                       Sc I
      XI(ISC1)=6.561
C                       Sc II     
      XI(ISC2)=12.800
C                       Sc III
      XI(ISC3)=24.756
C                       Sc IV
      XI(ISC4)=73.489
C                       V I
      XI(IV1)=6.746
C                       V II      
      XI(IV2)=14.634
C                       V III
      XI(IV3)=29.311
C                       V IV
      XI(IV4)=46.709
C                       K I
      XI(IK1)=4.341
C                       K II      
      XI(IK2)=31.625
C                       K III
      XI(IK3)=45.80
C                       K IV
      XI(IK4)=60.917

C MOLECULES ===================

C                       CO I
      XI(ICM1) = 14.01 
      XI(ICM2) = 41.8
C                       SiO
      XI(ISM1) = 11.6
      XI(ISM2) = 20.1
C                       SiS
      XI(ISS1) = 10.53
      XI(ISS2) = 17.5
C                       SO
      XI(IUM1) = 10.29
      XI(IUM2) = 19.4

      RETURN
      END






