SUBROUTINE GETF(neq, x, nnuclei, gamma, gammacollion, alpha, T, CREATE, DESTROY, MOLON, xmat_prev, nmol_prev, f, frac_dissociate, mols)
USE M1, ONLY : time, time_prev, TIMEDEPENDENT_ION, Nelem, Nmol, map_abin2elem3, map2, map3, map_abin2elem2, Nabin, mmol, Nstages, globid
IMPLICIT NONE
INTEGER, INTENT(IN) :: neq
DOUBLE PRECISION, DIMENSION(neq), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION(Nelem+1), INTENT(IN) :: nnuclei
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol), INTENT(IN) :: gamma, gammacollion, alpha
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: CREATE
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: DESTROY
DOUBLE PRECISION, DIMENSION(Nelem,Nstages+1), INTENT(IN) :: xmat_prev
DOUBLE PRECISION, DIMENSION(Nmol), INTENT(IN) :: nmol_prev
LOGICAL, INTENT(IN) :: MOLON
DOUBLE PRECISION, INTENT(IN) :: frac_dissociate
DOUBLE PRECISION, DIMENSION(neq), INTENT(OUT) :: f
   
TYPE(mmol), DIMENSION(Nmol), INTENT(IN) :: mols
type(mmol) :: mol
   
DOUBLE PRECISION, DIMENSION(neq) :: f2
! =================================================
! Task :  Computes the net ionization rates for ionization solution x
! Input : neq        Number of elements in equation system
!         n           Number density vector
!         gamma       Ionization rates for first and singly ionized species
!         alpha       Recombination rates for first and singly ionized species
!         CREATE
!         DESTROY         
! Output: f           Array with all net ionization rates [s^-1 cm^-3]
! ===============================================
DOUBLE PRECISION, DIMENSION(Nstages-1) :: chemicalcreation=0, chemicaldestruction=0, radiativecreation=0, radiativedestruction=0, recombinativecreation=0,recombinativedestruction=0
DOUBLE PRECISION :: alfa_1, alfa_2, gamma_1, gamma_2, ntot, radiativecreation_n, radiativecreation_i, chemicalcreation_n, chemicalcreation_i, chemicalcreation_d, recombinativecreation_n, recombinativecreation_i, recombinativecreation_d, radiativedestruction_n, radiativedestruction_i, radiativedestruction_d, chemicaldestruction_n, chemicaldestruction_i, chemicaldestruction_d, recombinativedestruction_n, recombinativedestruction_i, recombinativedestruction_d, recombinativecreation_m, recombinativedestruction_m, radiativecreation_m, radiativedestruction_m, chemicalcreation_m, chemicaldestruction_m, creation_n, oneoverdt, expterm
INTEGER :: i, j, k, m, pos2_n, posdouble, elpos, pos3_n, sel, n, indi, abin_parent1, abin_parent2, abin_mol
DOUBLE PRECISION :: ne, ne2, frac_recombdissociate, temp, temp4, temp5, temp6, divi1, divi2, charge_mol
DOUBLE PRECISION, DIMENSION(Nstages*Nelem+Nmol) :: nvec                   ! Densities for each ion and molecule
LOGICAL :: SIOPRINT, COPRINT
! -------------------------------------------------------------
INTEGER, DIMENSION(20) :: species = 0
DOUBLE precision, DIMENSION(20) ::  multi = 0
   
nvec = 0
   
SIOPRINT = .false.
   
COPRINT = .false.

frac_recombdissociate = 0.999    ! describes the ratio between XY+ + e- that produces into X + Y vs XY. 
                       ! This is here dominated by the first reaction. 
elpos = 3           ! position of electron density in x
   
f = 0.

ne2 = 0.   ! Important
   
ntot = nnuclei(Nelem+1)          ! Total number density
   
ne = ntot*x(3)                   ! Electron number density [cm^{-3}]
   
nvec(1) = nnuclei(1)*x(1)        ! HI
nvec(2) = nnuclei(1)*x(2)        ! Protons
 
if (Nstages >= 4) nvec(4) = nnuclei(1)*x(4)
 
if (Nstages >= 5) nvec(5) = nnuclei(1)*x(5)
 
DO m = 2, Nelem             ! All other atoms use Nstages ionization states 

	DO i = 1, Nstages

		nvec(Nstages*(m-1)+i) = nnuclei(m)*x(Nstages*(m-1)+i)

    END DO
   
END DO
   
!-------------------------------------------------------
! 1. Setting nvec for the molecules
! 2. Calculating the photodissocation and recombination
! for molecules and their parent atoms.  
!-------------------------------------------------------
   
if (Nmol > 0) then

	do i=1,Nmol

		mol = mols(i)

		! sets the nvec = nuclei number density * x of the molecules / (number of n nuclei in the molecule):

		nvec(map_abin2elem3(mol%abin)) = nnuclei(map_abin2elem2(mol%abin_parent1) / (Nstages-1) + 1)*x(map_abin2elem3(mol%abin)) / mol%n_p1
        
        ! if ionic molecules, the contribution is added to the electron density:

		ne2 = ne2 + mol%z * nvec(map_abin2elem3(mol%abin))

        ! sets the rates for photodiss (for neutral molecules) 
        ! and recombination (for single ionized molecules)
        ! contribution for the parent atoms: 

		! neutral molecules:

		if (mod((mol%mbin), 2) /= 0) then

			! the rate of change of parent1 depends onthe photodiss rate of the neutral molecule * how many parent atoms it contains

			f(map_abin2elem3(mol%abin_parent1)) = f(map_abin2elem3(mol%abin_parent1)) + nvec(map_abin2elem3(mol%abin))*gamma(map_abin2elem2(mol%abin))*frac_dissociate * mol%n_p1
            
            ! checks if the molecule has more than one parent
            if (mol%abin_parent2 /= 0) then

				f(map_abin2elem3(mol%abin_parent2)) = f(map_abin2elem3(mol%abin_parent2)) + nvec(map_abin2elem3(mol%abin))*gamma(map_abin2elem2(mol%abin))*frac_dissociate * mol%n_p2

			endif
            
       	! singly ionized molecules
    	elseif (mod((mol%mbin), 2) == 0) then

			! for singly ionized molecule, the rate o f change of parent 1 (and parent 2)
            ! depend on the recombination rates of that molecule, i.e. XY+ + e- -> X + Y

			f(map_abin2elem3(mol%abin_parent1)) = f(map_abin2elem3(mol%abin_parent1)) + nvec(map_abin2elem3(mol%abin))*ne*alpha(map_abin2elem2(mol%abin)-1)*frac_recombdissociate * mol%n_p1
            
            ! checks if the molecule has more than one parent

			if (mol%abin_parent2 /= 0) then

				f(map_abin2elem3(mol%abin_parent2)) = f(map_abin2elem3(mol%abin_parent2)) + nvec(map_abin2elem3(mol%abin))*ne*alpha(map_abin2elem2(mol%abin)-1)*frac_recombdissociate * mol%n_p2
               
            endif      

		endif
   
      end do

end if

!-------------------------------------------------------
! --- HYDROGEN -----------------------------------------
!-------------------------------------------------------
   
alfa_1 = alpha(1)
   
creation_n = 0.  ! Important
   
! iterate over all charge transfer + moleule creation
DO i = 1, neq
	DO j = 1, neq
    	creation_n = creation_n + CREATE(1,i,j)*nvec(i)*nvec(j)
    END DO
END DO
      
IF (TIMEDEPENDENT_ION .EQV. .FALSE.) THEN
   
   f(1) = f(1) + creation_n + nvec(2)*ne*alfa_1 - nvec(1)*(gamma(1) + gammacollion(1)*ne + DOT_PRODUCT(DESTROY(1,:),nvec))
            ! chemical creation + Radiative rec - losses by nonthermal and radiative processes, and chemical 
   
ELSE
   
   f(1) = f(1) + creation_n + nvec(2)*ne*alfa_1 - nvec(1)*(gamma(1) + gammacollion(1)*ne + 1./(time-time_prev) + DOT_PRODUCT(DESTROY(1,:),nvec)) + nnuclei(1)*xmat_prev(1,1)/(time-time_prev)
   
END IF
   
ne2 = ne2 + 1.*nvec(2)    ! Protons contribute one electron each (H2, H2+ and H- are added later)

if (Nstages >= 4) then ! CARE Add fake equations for slots where ions dont exist

  f(4) = 10.*nvec(4)

  if (Nstages >= 5) then

	f(5) = 10.*nvec(5)

  end if

end if

! ALL OTHER ELEMENTS:
   
DO m = 2, Nelem
   
   	pos3_n = Nstages*(m-1) + 1         ! Position of the neutral element in the Nstages*Nelem array
   
   	pos2_n = (Nstages-1)*(m-1) + 1          ! Position of the neutral in the (Nstages-1)*Nelem array

 	chemicalcreation = 0. ! Important
	chemicaldestruction = 0.
	radiativecreation = 0.
	radiativedestruction = 0.
	recombinativecreation = 0.
	recombinativedestruction = 0.

    !-------------------------------------------------------
    ! CREATION
    !-------------------------------------------------------
    ! chemical creation

    ! Add all creations by chemical reactions A + B = neutral/ionized/doubly ion.]
    ! by looping over all inputs in the CREATE vector

	DO i = 1, neq          
      
    	DO j = 1, neq

        	DO k = 1, Nstages-1

				chemicalcreation(k) = chemicalcreation(k) + CREATE(pos2_n+k-1,i,j)*nvec(i)*nvec(j)

                !if (globid == 5 .and. i == 79 .and. CREATE(pos2_n+k-1,i,j)*nvec(i)*nvec(j) > 0.) print *, 'uuu',j,k,CREATE(pos2_n+k-1,i,j)*nvec(i)*nvec(j)

            END DO
   
      	END DO
   
  	END DO

  	!-------------------------------------------------------
    ! recombination creation (XII + e- -> XI) calculated using alpha_tot, for neutral and ionized atoms

	DO k = 1, Nstages-1

		recombinativecreation(k) = nvec(pos3_n+k)*ne*alpha(pos2_n + k - 1)

   	END DO
      
    !-------------------------------------------------------
    ! radiative creation (XI + hv -> XII + e) photo-ionization

   	radiativecreation(1) = 0.                        ! Neutral cannot be created by photoionization
     
	DO k = 2, Nstages-1

		radiativecreation(k) = nvec(pos3_n+k-2)*(gamma(pos2_n+k-2) + gammacollion(pos2_n+k-2)*ne)

    END DO
      
    !-------------------------------------------------------
    ! DESTRUCTION
    !-------------------------------------------------------
    ! recombinative destruction, (XII + e- -> XI)

    recombinativedestruction(1) = 0.                       ! Neutral cannot recombine
    DO k = 2, Nstages-1

		recombinativedestruction(k) = recombinativecreation(k-1)

    END DO
      
    !-------------------------------------------------------
    ! radiative destruction (XI + hv -> XII + e) 
    DO k = 1, Nstages-1

		radiativedestruction(k) = nvec(pos3_n+k-1)*(gamma(pos2_n+k-1) + gammacollion(pos2_n+k-1)*ne)

    END DO
      
    !-------------------------------------------------------
    ! chemical destruction, with rates from DESTROY
    DO k = 1, Nstages-1

		chemicaldestruction(k) = nvec(pos3_n+k-1)*DOT_PRODUCT(DESTROY(pos2_n+k-1,:),nvec)

    END DO
   
    IF (TIMEDEPENDENT_ION .EQV. .FALSE.) THEN

		DO k = 1, Nstages-1

			f(pos3_n+k-1) = f(pos3_n+k-1) + (radiativecreation(k) + recombinativecreation(k) + chemicalcreation(k)) - (recombinativedestruction(k) + radiativedestruction(k) + chemicaldestruction(k))

        END DO

    ELSE
	
		DO k = 1, Nstages-1

			f(pos3_n+k-1) = f(pos3_n+k-1) + (radiativecreation(k) + recombinativecreation(k) + chemicalcreation(k)) - (recombinativedestruction(k) + radiativedestruction(k) + chemicaldestruction(k) + 1./(time-time_prev)*nvec(pos3_n+k-1)) + nnuclei(m)*xmat_prev(m,k)/(time-time_prev)

      	END DO
   
  	END IF
      
    IF (Nstages >= 4 .and. m == 2) THEN ! CARE fake equation for "He IV" and "He V" which do not exist

    	f(Nstages+3) = 10.*nvec(Nstages+4) ! The Nstages equation will be replace by number conservation, the "He IV" should enter in earlier equation(s)

        if (Nstages >= 5) f(Nstages+4) = 10.*nvec(Nstages+5)

    END IF

   	! ------------------------------------------------------------------------------------------------------
   	! ------------------------------------------------------------------------------------------------------ 
      
    ! conservation of free electrons, all the electrons from ions are added to the total electron density count
	DO k = 2, Nstages

		ne2 = ne2 + (k-1)*nvec(pos3_n+k-1)

    END DO

END DO
  
f(elpos) = ne - ne2    ! Add this since we are using ne as an 'additional' paramete
   
! ------------------------------------------------------------------------------------------------------ 
! MOLECULES ::::::equations for creation = destruction of molecules :::::::::::::::::::::
! ------------------------------------------------------------------------------------------------------ 
    
IF (Nmol > 0) THEN  ! Do only if molecules included...

	! loops through all the molecules
  	DO sel = 1, Nmol
   
   		radiativecreation_m = 0.; recombinativecreation_m = 0.; chemicalcreation_m = 0;
        radiativedestruction_m = 0; recombinativedestruction_m = 0.; chemicaldestruction_m = 0;
         
        !-------------------------------------------------------
        ! CREATION
        !-------------------------------------------------------
        ! chemical creation and destruction

        DO i = 1, neq          ! Add all creations by chemical reactions A + B = [N,I,D]

			DO j = 1, neq

				chemicalcreation_m = chemicalcreation_m + CREATE((Nstages-1)*Nelem+sel,i,j)*nvec(i)*nvec(j)

                !if (globid == 5 .and. sel == 1 .and. CREATE((Nstages-1)*Nelem+sel,i,j)*nvec(i)*nvec(j) > 0.) print *, 'uuu',i,j,CREATE((Nstages-1)*Nelem+sel,i,j)*nvec(i)*nvec(j)

              
			END DO

            !if (globid == 5 .and. sel == 1 .and. DESTROY((Nstages-1)*Nelem+sel,i)*nvec(i) > 0.) print *, 'ccc',i,DESTROY((Nstages-1)*Nelem+sel,i)*nvec(i)
		END DO
         
        chemicaldestruction_m = nvec(Nstages*Nelem+sel)*DOT_PRODUCT(DESTROY((Nstages-1)*Nelem+sel,:),nvec)
         
        !-------------------------------------------------------
        ! recombination rates and photo-ionization/dissociation
        ! rates for if the molecues is netural (sel = 1,3,5 ...)
        IF (mod(sel, 2) /= 0) THEN
   
        	recombinativecreation_m = nvec(Nstages*Nelem+sel+1)*ne*alpha((Nstages-1)*Nelem+sel)*(1-frac_recombdissociate) ! XY+ + e- --> XY  Gearhart 1999

			recombinativedestruction_m = 0.
            
           	radiativecreation_m = 0

			radiativedestruction_m = gamma((Nstages-1)*Nelem+sel)*nvec(Nstages*Nelem+sel)  ! Nonthermal and radiative destruction
         
         !-------------------------------------------------------
         ! recombination rates and photo-ionization/dissociation
         ! rates for if the molecues is singly ionized (sel = 2, 4, 6, ... )
      	ELSEIF (mod(sel,2) == 0) THEN
         
        	recombinativecreation_m = 0

			recombinativedestruction_m = nvec(Nstages*Nelem+sel)*ne*alpha((Nstages-1)*Nelem+sel - 1)
      
            radiativecreation_m = nvec(Nstages*Nelem+sel-1)*gamma((Nstages-1)*Nelem+sel-1)*(1-frac_dissociate) !CO + photon --> CO+ + e-

			radiativedestruction_m = 0.
         
       	END IF

  		f(Nstages*Nelem+sel) = (recombinativecreation_m + chemicalcreation_m + radiativecreation_m) - (chemicaldestruction_m + radiativedestruction_m + recombinativedestruction_m)
    
        !if (globid == 5 .and. sel == 1) print *, 'vvv', recombinativecreation_m, chemicalcreation_m, radiativecreation_m

        !if (globid == 5 .and. sel == 1) print *, 'www', chemicaldestruction_m, radiativedestruction_m, recombinativedestruction_m


		IF (TIMEDEPENDENT_ION .EQV. .TRUE.) THEN

			oneoverdt = 1./(time - time_prev)

			expterm = 3./(time + time_prev)

            if (globid == 5 .and. sel == 1) print *, 'xxx', nvec(Nstages*Nelem+sel)*(oneoverdt + expterm), nmol_prev(sel)*(oneoverdt - expterm)

			f(Nstages*Nelem+sel) = f(Nstages*Nelem+sel) - nvec(Nstages*Nelem+sel)*(oneoverdt + expterm) + nmol_prev(sel)*(oneoverdt - expterm) ! dn_i/dt + 3/t*n_i = ...  Then use the average of the 3/t*ni at current time and previous time (much more accurate than using the current time).

			!print *, 'hej', sel, f(Nstages*Nelem+sel), nvec(Nstages*Nelem+sel)*(oneoverdt + expterm) - nmol_prev(sel)*(oneoverdt - expterm), nvec(Nstages*Nelem+sel), nmol_prev(sel)

         END IF

	END DO
   
END IF
   
!-------------------------------------------------------
! CONSERVATION EQUATIONS 
!-------------------------------------------------------
! calculates the rate for the doubly ionized for all 
! elements, taking molecular contribution into account
   
!-------------------------------------------------------
! Special case for Hydrogen, as there is no double ionized H. 
f(2) = nvec(1) + nvec(2) - nnuclei(1) 
   
!-------------------------------------------------------
! Atomic contribution
DO m = 2, Nelem

	! the position of the netural element in 3*Nelem arrays
   	pos3_n = Nstages * (m-1) + 1

	! f for the conserved species, doubly ionized of an element, 
    ! is sum(f(n)) - nnuclei(i), sum over f for all other 
    ! species with same nuclei

	f(pos3_n+(Nstages-1)) = sum(nvec(pos3_n:pos3_n+Nstages-1)) - nnuclei(m)

END DO

!-------------------------------------------------------
! Molecular contribution
! To complete the conservation equation the atom 
! in molecules also need to be counted. 
DO m = 1, Nmol

	mol = mols(m)

	! special case for hydrogen, where protons are the 
   	! conserved species, at index 2.

	IF (mol%abin_parent1 .eq. 1) THEN

		f(2) = f(2) + nvec(map_abin2elem3(mol%abin)) * mol%n_p1

	ELSE

		! other molecules. Contribution to parent 1 is nvec of the molecule 
        ! (how much of that molecule) times the number of 
        ! atoms of p1 contained in the molecule (n_p1)

		f(map_abin2elem3(mol%abin_parent1) + Nstages -1) = f(map_abin2elem3(mol%abin_parent1) + Nstages - 1) + nvec(map_abin2elem3(mol%abin)) * mol%n_p1

	END IF

	! checks if there is a second parent atom in the molecule
    IF (mol%abin_parent2 .ne. 0) THEN

		! adds contribution to parent2 if this is the case
    	f(map_abin2elem3(mol%abin_parent2) + Nstages - 1) = f(map_abin2elem3(mol%abin_parent2) + Nstages - 1) + nvec(map_abin2elem3(mol%abin)) * mol%n_p2

	END IF

END DO

END
   
