SUBROUTINE GETJACNUM(DIMS, x, n, gamma, gammacollion, alpha, T, CREATE, DESTROY, MOLON, xmat_prev, nmol_prev, JAC, frac_dissociate, mols)
use M1, ONLY : Nelem, Nmol, myid, mmol, Nstages
use INTERFACE, ONLY : getf
IMPLICIT NONE

INTEGER, INTENT(IN) :: DIMS
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION(Nelem+1), INTENT(IN) :: n
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol), INTENT(IN) :: gamma, gammacollion, alpha
DOUBLE PRECISION, INTENT(IN) :: T, frac_dissociate
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: CREATE
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: DESTROY
LOGICAL, INTENT(IN) :: MOLON
DOUBLE PRECISION, DIMENSION(Nelem,Nstages+1) :: xmat_prev ! TEMPFIX
DOUBLE PRECISION, DIMENSION(DIMS,DIMS), INTENT(OUT) :: JAC
DOUBLE PRECISION, DIMENSION(Nmol) :: nmol_prev

TYPE(mmol), DIMENSION(Nmol), INTENT(IN) :: mols

! -------------------------------------------------------------
! Task :     Compute Jacobian (array with all partial derivates) of array function f
! Input :    DIMS       Dimension of f (and x and JAC)
!            x          Point in parameter space where the Jacobain should be computed
!            n          Total number density [cm^-3]
!            gamma     Ionization rates
!            alpha     Recombination rates
!            T         Temperature
!            CREATE    3D Array with reaction rates R (CREATED(2) + CREATE(3) --> CREATE(1)
!            DESTROY   2D ARRAY with rates to destroy DESTROY(1) by DESTROY(2) 
!            MOLON     Boolean, are molecules turned on?
! Output:    JAC       Jacobian (df_i / dx_j)
! Globals :  --
! ---------------------------------------------------------------

INTEGER :: i, m, j, ind, pos30_1, pos30_2
DOUBLE PRECISION, DIMENSION(DIMS) :: xtry, f, ftry
DOUBLE PRECISION :: T_dummy

! -------------------------------------------------------------

JAC = 0               ! Important

T_dummy = T

! -------------------------------------------------------------

CALL getf(DIMS, x, n, gamma, gammacollion, alpha, T_dummy, CREATE, DESTROY, MOLON, xmat_prev, nmol_prev, f, frac_dissociate, mols)  ! f = net creation rate for each ion

DO i = 1, DIMS

   xtry = x

   IF (x(i) .NE. 0) THEN

      xtry(i) = xtry(i)*1.01     ! Use 1% step sized to compute derivatives

   ELSE

      xtry(i) = 1E-2  ! CARE cannot be too small

   END IF

   CALL getf(DIMS, xtry, n, gamma, gammacollion, alpha, T_dummy, CREATE, DESTROY, MOLON, xmat_prev, nmol_prev, ftry, frac_dissociate, mols)

  ! IF (minval(abs(ftry - f)) .NE. 0) THEN

  JAC(:,i) = (ftry - f)/(xtry(i) - x(i))    ! Numerical estimate for the partial derivatives

  ! ELSE

  !    JAC(:,i) = 1D-30

  ! END IF

END DO




END
