SUBROUTINE hI_recomb_nl(ntot, T, alpha)
IMPLICIT NONE
INTEGER :: ntot
DOUBLE PRECISION :: T
DOUBLE PRECISION, DIMENSION(ntot) :: alpha
! ================================
! Task :             Computes specific recombination rates alpha(i) [cm^3 s^-1]for HI given the temperature
! INPUT : ntot       Number of nl-states to compute
!         T          Temperature [K]
! OUTPUT: alpha      Vector with the *ntot* rec values [cm^3 s^-1 ]
! =================================
DOUBLE PRECISION :: t4, c1, xxx, x1, b, eps, err, xl, xkm, fxk, fim, fail, fip, xkmax, Imin, Iplus, e00, y, const, a1
INTEGER :: n, l, lp, i, k, nlev, n2, l2, n_n
INTEGER, DIMENSION(:,:), ALLOCATABLE :: i_mat
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: en                   ! Should be in eV

DATA eps/0.000001/
DATA err/0.01/

INTERFACE

   SUBROUTINE romb2(aa, bb, err, fi, fail, k, y, n, l, lp, nm, nlm)
     IMPLICIT NONE
     DOUBLE PRECISION :: aa, bb, err, fi, fail,y
     INTEGER :: k, n, l, lp, nm
     INTEGER :: nlm(nm,0:nm)
   END SUBROUTINE romb2

   SUBROUTINE findup(n, l, lp, y, eps, xkm, fxk, xkmax, n_n, nlm)
     IMPLICIT NONE
     INTEGER :: n, l, lp, n_n
     DOUBLE PRECISION :: y, eps, xkmax
     DOUBLE PRECISION :: xkm, fxk
     INTEGER :: nlm(n_n,0:n_n)
   END SUBROUTINE findup

END INTERFACE

n_n = NINT(-0.5 + SQRT(0.25 + 2*ntot))  ! This formula computes the number of principal quantum states states given *ntot* number of nl states. FOr example ntot = 1 --> n=2, ntot = 3 --> n=2 etc  (note dont count spin states) 

ALLOCATE(en(n_n)); en = 0;  ! Vector with principal state energies

ALLOCATE(i_mat(n_n,0:n_n)); i_mat = 0  
      
e00 = 13.5972   ! Ground state ion. energy

DO i = 1, n_n
        
   en(i) = e00*(1.-1./DBLE(i)**2)   ! Bohr formula binding energy Eb_i = Eb0/n2 --> E_i = Eb0 - Ebi 

END DO


!  The recombination coefficients are calculated according to  Brocklehurst, M., 1971, MNRAS, 153, 471.

i_mat = 0

nlev = 0
      
DO n = 1, n_n  ! For each principal n..

   DO l = 0, (n-1)  ! For each angular momentum substate..
          
      i_mat(n,l) = nlev + l + 1  ! The sequential number for state n,l
      
   END DO

   nlev = nlev + n  ! Add n substates for each n
    
ENDDO


t4 = T/1.e4
      
y = 15.778/t4

const = 2.8128d-15
      
a1 = 0
      
DO n = 1, n_n   ! For each principal n..
                  
   c1 = const*2.*SQRT(y)/(DBLE(n)**2)
         
   DO l = 0, n-1  ! For each substate..

      IF (n .EQ. 18 .AND. l .EQ. 0) THEN  ! What is this?
      
         xxx = 1
            
      ENDIF

      xl = DBLE(l)
            
      IF (l .EQ. 0) THEN
               
         Imin = 0.
            
      ELSE

         lp = l-1

         !  find upper integration limit

         xkmax = 5.d0*(e00-en(n))/e00
               
         CALL findup(n,l,lp,y,eps,xkm,fxk,xkmax,n_n, i_mat)
               
         b = xkm**2

         n2 = n

         l2 = l

         CALL romb2(a1, b, err, fim, fail, k, y, n2, l2, lp, n_n, i_mat)
               
         Imin = xl*y*fim
            
      ENDIF

      lp = l + 1

      !  find upper integration limit

      xkmax = 5.d0*(e00-en(n))/e00
            
      CALL findup(n,l,lp,y,eps,xkm,fxk,xkmax, n_n, i_mat)
            
      b = xkm**2

      n2 = n

      l2 = l

      CALL romb2(a1, b, err, fip, fail, k, y, n2, l2, lp, n_n, i_mat)
            
      Iplus = DBLE(l+1)*y*fip
            
      i = i_mat(n,l)
            
      alpha(i) = c1*(Imin+Iplus)
         
   ENDDO

ENDDO


      
END



SUBROUTINE findup(n, l, lp, y, eps, xkm, fxk, xkmax, n_n, nlm)
IMPLICIT NONE

INTERFACE 

   DOUBLE PRECISION FUNCTION gf(n_n, n, l, xk, lp, nlm)
     IMPLICIT NONE
     INTEGER :: n_n, n, l, lp
     DOUBLE PRECISION :: xk
     INTEGER :: nlm(n_n,0:n_n)
   END FUNCTION gf

END INTERFACE

INTEGER :: n, l, lp, n_n
DOUBLE PRECISION :: y, eps, xkmax
DOUBLE PRECISION :: xkm, fxk
INTEGER :: nlm(n_n,0:n_n)

DOUBLE PRECISION :: gf0, theta0, fx0, xn, xk, sn2k2, gfk, theta, dk, xk2, fxko


gf0 = gf(n_n, n, l, 0d0, lp, nlm)

theta0 = gf0**2

fx0 = theta0

xn = DBLE(n)

xk2 = xkmax**2

xk = xkmax

sn2k2 = 1. + xn**2*xk2

gfk = gf(n_n, n, l, xk, lp, nlm)

theta = sn2k2*gfk**2     

fxk = sn2k2**2*theta*EXP(-xk2*y)/theta0

dk = xk*0.11
     
IF (fxk .LT. eps*0.1) THEN
         
   xk = SQRT(-LOG(eps)/y)
         
   xk2 = xk**2
         
   sn2k2 = 1.+xn**2*xk2
         
   gfk = gf(n_n, n, l, xk, lp, nlm)
         
   theta = sn2k2*gfk**2     
         
   fxk = sn2k2**2*theta*EXP(-xk2*y)/theta0
         
   dk = xk*0.11

   IF (fxk .GE. eps) THEN
            
      DO WHILE (fxk .GE. eps)
               
         xk = xk + dk
               
         xk2 = xk**2
               
         sn2k2 = 1.+xn**2*xk2
               
         gfk = gf(n_n, n, l, xk, lp, nlm)
               
         theta = sn2k2*gfk**2     
               
         fxk = sn2k2**2*theta*EXP(-xk2*y)/theta0
            
      ENDDO
         
   ELSEIF (fxk .LT. eps) THEN
            
      DO WHILE (fxk .LT. eps)
               
         fxko = fxk
               
         IF (xk .LT. dk) dk = xk*0.11
               
         xk = xk - dk
               
         xk2 = xk**2
               
         sn2k2 = 1.+xn**2*xk2
               
         gfk = gf(n_n, n, l, xk, lp, nlm)
               
         theta = sn2k2*gfk**2     
               
         fxk = sn2k2**2*theta*EXP(-xk2*y)/theta0
      ENDDO
            
      fxk = fxko
            
      xk = xk + dk
   
   ENDIF
      
ELSEIF (fxk .GE. eps) THEN
         
   DO WHILE (fxk .GE. eps)
           
      xk = xk + dk
            
      xk2 = xk**2
            
      sn2k2 = 1.+xn**2*xk2
            
      gfk = gf(n_n, n, l, xk, lp, nlm)
            
      theta = sn2k2*gfk**2     
            
      fxk = sn2k2**2*theta*EXP(-xk2*y)/theta0
         
   ENDDO

ENDIF

xkm = xk


END



SUBROUTINE romb2(aa, bb, err, fi, fail, k, y, npass, l, lp, nm, nlm)
IMPLICIT NONE
DOUBLE PRECISION :: aa, bb, err, fi, fail,y
INTEGER :: k, npass, l, lp, nm
INTEGER :: nlm(nm,0:nm)


INTERFACE

   DOUBLE PRECISION FUNCTION fx(xk2, y, n, l, lp, nm, nlm)
     IMPLICIT NONE
     DOUBLE PRECISION :: xk2, y
     INTEGER :: n, l, lp, nm
     INTEGER :: nlm(nm,0:nm)
   END FUNCTION fx

END INTERFACE

DOUBLE PRECISION, DIMENSION(100) :: tr, w
DOUBLE PRECISION :: a, b, dx, fi1, fi2, tdx, x
INTEGER :: i, kk, j, m, n


a = aa
b = bb
dx = (b-a)/2.
fi1 = 0.5*(fx(a, y, npass, l, lp, nm, nlm) + fx(b, y, npass, l, lp, nm, nlm))
fi2 = fx(a+dx, y, npass, l, lp, nm, nlm)
tr(1) = dx*(fi1+fi2)
n = 1
k = 1
w(1) = 4.
1     fi = tr(1)
w(k+1) = 4.*w(k)
n = 2*n
k = k+1
tdx = dx
dx = dx/2.
x = a+dx
DO 2 i = 1,n
   
   fi2 = fi2 + fx(x, y, npass, l, lp, nm, nlm)
2  x = x + tdx
   tr(k) = dx*(fi1+fi2)
   kk = k-1
DO 3 j = 1, kk
      m = k-j
3     tr(m) = (w(j)*tr(m+1)-tr(m))/(w(j)-1.)
      fail = ABS(tr(1)-fi)/ABS(tr(1))
      IF(fail-err)6,6,4
4     IF(k-100)1,6,6
6     fi = tr(1)

END



DOUBLE PRECISION function fx(xk2, y, n, l, lp, nm, nlm)
IMPLICIT NONE
DOUBLE PRECISION :: xk2, y
INTEGER :: n, l, lp, nm
INTEGER :: nlm(nm,0:nm)

INTERFACE

   DOUBLE PRECISION FUNCTION gf(n_n, n, l, xk, lp, nlm)
     IMPLICIT NONE
     INTEGER :: n_n, n, l, lp
     DOUBLE PRECISION :: xk
     INTEGER :: nlm(n_n,0:n_n)
   END FUNCTION gf

END INTERFACE

DOUBLE PRECISION :: xk, xn, xl, xlp, sn2k2, gfk, theta


xk = SQRT(xk2)

xn = DBLE(n)

xl = DBLE(l)

xlp = DBLE(lp)

sn2k2 = 1. + xn**2*xk2

gfk = gf(nm, n, l, xk, lp, nlm)
      
theta = sn2k2*gfk**2

fx = sn2k2**2*theta*EXP(-xk2*y)


END


DOUBLE PRECISION FUNCTION gf(n_n, n, l, xk, lp, nlm)
IMPLICIT NONE
INTEGER :: n_n, n, l, lp
DOUBLE PRECISION :: xk
INTEGER :: nlm(n_n,0:n_n)

DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, SAVE :: G0p, G0m
INTEGER :: i, is, nl, n_nm1, lr, il, nmax
DOUBLE PRECISION :: pi, xn, xl, xlp, xnf1, xnf2, xprod, s, cgf, sn2k2, c1, c2, c3, xlr, Gm, GKnp(0:n_n),GKnm(0:n_n)
LOGICAL :: FIRST = .TRUE.

DATA pi/3.141593/

INTERFACE

   SUBROUTINE fG0(n_n, nmax, G0p, G0m, nlm)
     IMPLICIT NONE
     INTEGER :: n_n, nmax
     DOUBLE PRECISION, DIMENSION(nmax) :: G0p, G0m
     INTEGER :: nlm(n_n,0:n_n)
   END SUBROUTINE fG0

END INTERFACE

nmax = 0.5*n_n*(n_n+1)

IF (FIRST .EQV. .TRUE.) THEN

   ALLOCATE(G0p(nmax)); G0p = 0
   ALLOCATE(G0m(nmax)); G0m = 0

   FIRST  = .FALSE.  

   CALL fG0(n_n, nmax, G0p, G0m, nlm)

ENDIF
      
xn = DBLE(n)

xl = DBLE(l)

xlp = DBLE(lp)

xnf1 = 1.d0
      
DO i = 2, n+l

   xnf1 = xnf1*DBLE(i)

ENDDO

xnf2 = 1.d0
      
DO i = 2,n-l-1

   xnf2 = xnf2*DBLE(i)

ENDDO

xprod = 1.d0
      
DO is = 0, lp

   s = DBLE(is)

   xprod = xprod*(1.+s**2*xk**2)

ENDDO
      
cgf = SQRT(xnf1*xprod/xnf2)*((2.*xn)**(l-n))

IF (xk .EQ. 0.) THEN

   nl = nlm(n,l)
         
   IF (lp .EQ. l+1) THEN
   
      Gm = G0p(nl)
         
   ELSEIF (lp .EQ. l-1) THEN   
   
      Gm = G0m(nl)

   ELSE

      Gm = 0.d0

   END IF
      
ELSEIF (xk .GT. 0.) THEN

!  K > 0
   sn2k2 = 1. + xn**2*xk**2


   DO i = 0, n_n
   
      GKnp = 0.d0

      GKnm = 0.d0
   
   ENDDO

   c1 = 1./SQRT(1.-EXP(-2.*pi/xk))
   
   c2 = EXP(2.*xn-(2./xk)*ATAN(xn*xk))
   
   c3 = sn2k2**(n+2)
   
   n_nm1 = nlm(n,n-1)
        
   GKnp(n-1) = (c1*c2/c3)*G0p(n_nm1)

   IF (n .EQ. 2) THEN
   
      GKnm(n-1) = (sn2k2/(2.*xn))*GKnp(n-1)
      
      GKnp(0) = (2.*xn-1)*sn2k2*xn*GKnp(n-1)
        
   ELSEIF (n.GT.2) THEN
   
      GKnm(n-1) = (sn2k2/(2.*xn))*GKnp(n-1)
      
      GKnp(n-2) = (2.*xn-1)*sn2k2*xn*GKnp(n-1)
      
      GKnm(n-2) = (2.*xn-1)*(4.+(xn-1.)*sn2k2)*GKnm(n-1)

      DO il = n-3,1,-1
        
         lr = il+2
              
         xlr = DBLE(lr)
               
         GKnp(il) = (4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr-1.)*sn2k2)*GKnp(lr-1)-4.*xn*xn*(xn*xn-xlr*xlr)*(1.+(xlr+1.)*(xlr+1.)*xk*xk)*GKnp(lr)

         lr = il+1
              
         xlr = DBLE(lr)
               
         GKnm(il) = (4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr+1.)*sn2k2)*GKnm(lr)-4.*xn*xn*(xn*xn-(xlr+1.)*(xlr+1.))*(1.+xlr*xlr*xk*xk)*GKnm(lr+1)
               
         IF (il .LE. l) GOTO 9
            
      ENDDO

      ! l = 0
            
      lr = 2
            
      xlr = DBLE(lr)
            
      GKnp(0) = (4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr-1.)*sn2k2)*GKnp(lr-1)-4.*xn*xn*(xn*xn-xlr*xlr)*(1.+(xlr+1.)*(xlr+1.)*xk*xk)*GKnp(lr)

         
   ENDIF
 9       CONTINUE

   IF (lp .EQ. l+1) THEN
            
      Gm = GKnp(l)
         
   ELSEIF (lp .EQ. l-1) THEN   
            
      Gm = GKnm(l)
         
   ELSE
            
      Gm = 0.d0
         
   ENDIF
      
ENDIF

gf = cgf*Gm



      
END FUNCTION gf

! ----------------------------------------------------------------------------------

SUBROUTINE fG0(n_n, nmax, G0p, G0m, nlm)
IMPLICIT NONE
INTEGER :: n_n, nmax
DOUBLE PRECISION, DIMENSION(nmax) :: G0p, G0m
INTEGER :: nlm(n_n,0:n_n)

INTEGER :: i, n, l, n_nm1, n_nm2, n_0, nl, lr, n_lrm1, n_lr, n_nmn1, n_lrp1
DOUBLE PRECISION :: pi, xn, xnf3, xlr

data pi/3.141593/

!  G0(n,l,lp) = G(n,l,0,lp) <=> K=0.
!  G0p(nl) = G(n,l,0,l+1)
!  G0m(nl) = G(n,l,0,l-1)

G0p = 0.
G0m = 0.

!  n = 1
      
G0p(nlm(1,0)) = sqrt(pi/2.)*8.*4.*exp(-2.)

!  n = 2
      
n = 2
      
xn = dble(n)
      
xnf3 = 6.
      
n_nm1 = nlm(n,n-1)
      
n_0 = nlm(n,0)
      
G0p(n_nm1) = sqrt(pi/2.)*(8.*xn/xnf3)*((4.*xn)**xn)*exp(-2.*xn)
      
G0m(n_nm1) = (1./(2.*xn))*G0p(n_nm1)
      
G0p(n_0) = (2.*xn-1.)*xn*G0p(n_nm1)      

DO n = 3, n_n
         
   xn = DBLE(n)
         
   xnf3 = 1.d0
         
   DO i = 1, 2*n-1
            
      xnf3 = xnf3*DBLE(i)
         
   ENDDO
         
   n_nm1 = nlm(n,n-1)
         
   n_nm2 = nlm(n,n-2)
         
   G0p(n_nm1) = SQRT(pi/2.)*(8.*xn/xnf3)*((4.*xn)**xn)*EXP(-2.*xn)
         
   G0m(n_nm1) = (1./(2.*xn))*G0p(n_nm1)

   G0p(n_nm2) = (2.*xn-1.)*xn*G0p(n_nm1)
         
   G0m(n_nm2) = (2.*xn-1.)*(4.+(xn-1.))*G0m(n_nm1)

   DO l = n-3,1,-1
            
      nl = nlm(n,l)

      lr = l+2
            
      n_lrm1 = nlm(n,lr-1)
            
      n_lr = nlm(n,lr)
            
      xlr = DBLE(lr)

      G0p(nl) = (4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr-1.))*G0p(n_lrm1)-4.*xn*xn*(xn*xn-xlr*xlr)*G0p(n_lr)

      lr = l+1
            
      n_lrp1 = nlm(n,lr+1)
            
      n_lr = nlm(n,lr)
            
      xlr = DBLE(lr)

      G0m(nl)=(4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr+1.))*G0m(n_lr)-4.*xn*xn*(xn*xn-(xlr+1.)*(xlr+1.))*G0m(n_lrp1)
         
   ENDDO
! l = 0
         
   lr = 2
         
   n_lrm1 = nlm(n,lr-1)
         
   n_lr = nlm(n,lr)
         
   xlr = DBLE(lr)
         
   G0p(nlm(n,0))=(4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr-1.))*G0p(n_lrm1)-4.*xn*xn*(xn*xn-xlr*xlr)*G0p(n_lr)

ENDDO


END SUBROUTINE fG0
