SUBROUTINE HI_RECOMBSPECTRUM_RECRATES(TOLEVEL, CHI_EV, LEVELG, T, alfa)
USE PARAMETERS, ONLY : k, me, eV, c, pi
IMPLICIT NONE
INTEGER, INTENT(IN) :: TOLEVEL                                       ! Level in an atom with only n=2 split 
DOUBLE PRECISION, INTENT(IN) :: CHI_EV, T, LEVELG         
DOUBLE PRECISION, INTENT(OUT) :: alfa
! ======================================================
!     RECOMBINATION (USING THE MILNE RELATION)                  ! NOTE E AND DE N EV !!
!     AVERAGE THE EMISSION OVER THE INTERVAL SO THAT THE TOTAL EMISSION
!     IS CORRECT
! Task :        Compute rec emission [erg cm^-33 s^-1 eV^-1] for level TOLEVEL
! Input         TOLEVEL     Which level? 1=1s, 2=2s, 3=3p etc
!               CHI_EV      Ionization potential of this level [eV] 
!               LEVELG      Statistical weight of this level  
!               T           Temperature
! Output       alfa         ergcm3perseV
! Globals :    
! ==============================================
INTEGER :: nlow, nllevs, i
DOUBLE PRECISION :: F1, F2, F3, fac, gauntfactor, sigma_ionization, wll, sig2s, sig2p, gauntfactor_2s, gauntfactor_2p, gbf, f, E_EV, dv, v, hnu, sigma_fb
INTEGER, PARAMETER :: Nv=1000
DOUBLE PRECISION, DIMENSION(Nv) :: vvec

alfa = 0  ! important

vvec = [(i*10*sqrt(2*k*T/me)/DBLE(Nv),i=1,Nv)]    ! Array with velocities relevant for temperature T (0.01 sqrt(2kT/m) to 10 sqrt(2kT/m)

DO i = 1, Nv    ! For each electron velocity..

   v = vvec(i)   ! Velocity [cm s^-1]

   E_EV = CHI_EV + 0.5*me*v**2/eV   ! energy of the recombination photon [eV]

   IF (TOLEVEL <= 6) THEN   ! 1s, 2s, 3p, 3s, 3p, 3d

      IF (TOLEVEL .EQ. 1) THEN  ! 1s

         nlow = 1

         !gauntfactor = gbf(nlow,E_EV)  ! Gaunt factor needed for the ionization sigma

         !sigma_ionization = 7.9e-18*nlow*gauntfactor*(CHI_EV/E_EV)**3   ! Ground states has sigma ~ E^-3

         CALL phfit2(1, 1, 1, E_EV, sigma_ionization)  ! Verner Ferland 1996 for ground state...atom 1, electrons 1, shell 1

      ELSE IF (TOLEVEL .EQ. 2) THEN   ! 2s

         nlow = 2    

         wll = 12398.41896/E_EV   ! Wavelength in Ångströms (E_EV is in eV)

         CALL hy_2s2p(wll, gauntfactor_2s, gauntfactor_2p, sig2s, sig2p)   ! Complex cross section

         sigma_ionization = sig2s

      ELSE IF (TOLEVEL .EQ. 3) THEN   ! 2p

         nlow = 2      

         wll = 12398.41896/E_EV

         CALL hy_2s2p(wll, gauntfactor_2s, gauntfactor_2p, sig2s, sig2p)  ! complex cross section
         
         sigma_ionization = sig2p
      
      ELSE 

         nlow = 3  ! 3s, 3p, 3d

         gauntfactor = gbf(nlow,E_EV)

         sigma_ionization = 7.9e-18*nlow*gauntfactor*(CHI_EV/E_EV)**3  !  source? Use sigma ~ E^-3 for these

      END IF

   ELSE  ! 4s and above

      nlow = 0; nllevs = 0

      DO WHILE (nllevs < TOLEVEL)

         nlow = nlow + 1

         nllevs = nllevs + nlow   ! principal state nlow has nlow substates 1 : 1s, 2: 2s 2p : etc)

      END DO
      
      gauntfactor = 1.  !    gbf only covers 1s to 3p

      sigma_ionization = 7.9E-18*nlow*gauntfactor*(CHI_EV/E_EV)**3

   ENDIF

   hnu = E_EV*eV   ! Photon energy in ergs

   sigma_fb = sigma_ionization*hnu**2/(me**2*c**2*v**2)*LEVELG  ! RybickiLightman eq 10.62

   f = 4*pi*(me/(2*pi*k*T))**1.5*v**2*exp(-0.5*me*v**2/(k*T))  !RL Eq 10.60 [1 / cm/s]

   dv = vvec(2) - vvec(1)  ! CARE

   alfa = alfa + v*f*sigma_fb*dv    ! Add contribution by this speed interval to the total free-bound rate [cm^3 s^-1]

END DO


!  AVERAGE OVER THE ENERGY INTERVAL

!ergcm3perseV = LEVELG*1.31159E-4*E_EV**3*(EXP(F2+F3) - EXP(F2-F3))*sigma_ionization/(2*F3*T**1.5)

!ergcm3perseV = LEVELG*T**(-3./2.)*1.31159E-4*E_EV**3*sigma_ionization*k*T/(dE_eV_foraverage*eV)*(-EXP(-F2) + EXP(-F1))

! avoid these integrated formulas as they involve the differences between potentially small numbers

!ergcm3perseV = LEVELG*T**(-3./2.)*1.31159E-4*E_EV**3*sigma_ionization*EXP(-F1)
   
275 CONTINUE


END
      
              
SUBROUTINE HY_2S2P(wl, gauntfactor_2s, gauntfactor_2p, sig2s, sig2p)    ! photo cross sections for 2s and 2p of hydrogen. From Bethe & Salpeter
USE PARAMETERS
IMPLICIT NONE
! ::::::::::::::::::::::::::::::::::::::::::::
! TASK : compute the ionization cross section from 2s and 2p states in HI
! Input :    wl               Wavelength in ..?
! Output :   gauntfactor_2s   gauntfactor
!            gauntfactor_2p   gauntfactor
!            sig2s
!            sig2p   
! ::::::::::::::::::::::::::::::::::::::::::

DOUBLE PRECISION eu, con,wl,gauntfactor_2s,gauntfactor_2p,sig2s,sig2p,ac,z, nu,nu2,np2, nu0,nucgs,fac

eu = 4.803e-10
      
nu0 = c/911.9837D-8
      
con = 2.D0**11*pi*eu**2 / (3*me*c)
      
nu = 911.9837D0/wl  ! frequency in Ry

z = 1.d0

nu2 = z/4.  ! = 0.25

IF (nu > nu2) THEN

   np2 = 2*dsqrt(nu2/(nu-nu2))

   ac = datan(2./np2)

   fac = dexp(-4.0*np2*ac)/(1. - dexp(-2.0*pi*np2) )

   sig2s = (1.0 + 3.0*nu2/nu)*fac

   sig2p = (3.0 + 8.0*nu2/nu)*fac

   nucgs = nu*nu0

   sig2s = con*sig2s/(nucgs*(nu/nu2)**3)

   ! BS give the total 2s-shell cross section. divide by 2

   sig2s = sig2s/2.

   sig2p = con*sig2p/(nucgs*(nu/nu2)**4)

   ! 2p divide by 6

   sig2p = sig2p/6.

   ! Gaunt factors

   gauntfactor_2s = sig2s*(nu/nu2)**3/(7.9e-18*2.)

   gauntfactor_2p = sig2p*(nu/nu2)**3/(7.9e-18*2.)
   
ELSE

   sig2s = 0

   sig2p = 0

   gauntfactor_2s = 0

   gauntfactor_2p = 0

END IF

  
         


END
      
                  

