SUBROUTINE indexnuclei(N_atoms, atoms, N_nuclei)
USE M1, ONLY : atomm
IMPLICIT NONE
INTEGER, INTENT(IN) :: N_atoms
TYPE(atomm), DIMENSION(N_atoms), INTENT(INOUT) :: atoms
INTEGER, INTENT(OUT) :: N_nuclei
! =======================================
! Task :  Index nuclei 
! Input : N_atoms         Number of atoms in the ATOMS array
!         atoms           Array of atoms (type atomm, module m1)
! Output : atoms         atom%SEQ and atom%POS are set
!          N_nuclei       Number of elements found in the atoms array (Fe I, Fe II, Fe III are all one element, etc)
! Globals : none
! =======================================
INTEGER :: i, pos
CHARACTER(len=5) :: name
CHARACTER(len=2) :: nucleus, oldnucleus
! --------------------------------------------------------

N_nuclei = 0              ! Important
pos = -1                  ! Important
oldnucleus = 'xx'

! -----------------------------------------------------

DO i = 1, N_atoms

   name = atoms(i)%NAME
   
   nucleus = name(1:2)

   IF (nucleus .NE. oldnucleus) THEN  ! IF Fe II has position 5, Fe III should be at 6, and the next new element starts at 7 (because the Fe III abundance needs to be stored to compute free-bound processes to Fe II)

      pos = pos+2

      N_nuclei = N_nuclei +1

   ELSE
      
      pos = pos + 1    ! If FeII was at 5 and we now come to FeIII, take one step up
      
   END IF

   atoms(i)%SEQ = i     ! SEQ is the sequential position

   atoms(i)%POS  = pos  ! POS is the position in the x array

   oldnucleus = nucleus

END DO


END SUBROUTINE indexnuclei
        
