MODULE INTERFACE
USE memalloc_shared  ! This collects all memory allocation code into one call
IMPLICIT NONE

INTERFACE

SUBROUTINE MOLECULES(T, DESTROY, CREATE, DE)
USE M1, ONLY : Nabin, Nmol
USE PARAMETERS, ONLY : eV, k
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(Nabin+Nmol,Nabin+Nmol), INTENT(INOUT) :: DESTROY, DE
DOUBLE PRECISION, DIMENSION(Nabin+Nmol,Nabin+Nmol,Nabin+Nmol), INTENT(INOUT) :: CREATE

END SUBROUTINE MOLECULES

SUBROUTINE MGI_COLLRATES(T, UPSILON)
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(11,11), INTENT(OUT) :: UPSILON
END SUBROUTINE MGI_COLLRATES

DOUBLE PRECISION FUNCTION BETA_S_FUNC(tau)
DOUBLE PRECISION, INTENT(IN) :: tau
END

SUBROUTINE BREMSSTRAHLUNG(Z, nplus, ne, T, lambdastart, lambdaend, stepatstart, N, lambdavec, emissvec_integrated)
!  USE PARAMETERS
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: Z
  DOUBLE PRECISION, INTENT(IN) :: nplus, ne, lambdastart, lambdaend, stepatstart, T
  INTEGER, INTENT(OUT) :: N
  DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE,  INTENT(OUT) :: lambdavec, emissvec_integrated
END SUBROUTINE BREMSSTRAHLUNG

SUBROUTINE PAGUESSES(itoion, itorec, dE, ct)
IMPLICIT NONE

INTEGER, INTENT(IN) :: itoion, itorec
DOUBLE PRECISION, INTENT(IN) :: dE
DOUBLE PRECISION, INTENT(out) :: ct

END SUBROUTINE PAGUESSES

SUBROUTINE PESC_STATIC(tau, pesc)
IMPLICIT NONE

DOUBLE PRECISION, INTENT(IN) :: tau
DOUBLE PRECISION, INTENT(OUT) :: pesc

END

SUBROUTINE NLTEAPA(Lstar, Tstar, ne, filename, Ha, Hb, Hg, Hd, He, Pa, Pb, Pg, Pd, Pe, tau_lyman, tau_balmer, tau_paschen, R, tau_ha, tau_hb, tau_thomson, lsob_rel, inflow, outflow, nk, n2, taulyb)
USE M1
USE PARAMETERS
IMPLICIT NONE
! ------------------------------
! Task : Compute NLTE solution for any atom, given the density, temperature
! -----------------------------
CHARACTER(len=100) :: filename
DOUBLE PRECISION, INTENT(IN) :: Lstar, Tstar, ne
DOUBLE PRECISION, INTENT(OUT) :: Ha, Hb, Hg, Hd, He, Pa, Pb, Pd, Pe, Pg, tau_lyman, tau_balmer, tau_paschen, R, tau_ha, tau_thomson, tau_hb, lsob_rel, taulyb
DOUBLE PRECISION, DIMENSION(5,10) :: inflow, outflow
DOUBLE PRECISION, INTENT(OUT) :: nk, n2

END

SUBROUTINE FINDRESONANCE(iion, irec, ED_ground, Nlev_ion, Nlev_rec, energies_ion, energies_rec, abin_best, ctlevel_best, ED_best)
IMPLICIT NONE
!IN
INTEGER, INTENT(IN) :: iion, irec  ! abin values of target atoms
DOUBLE PRECISION, INTENT(IN) :: ED_ground ! Energy defect (eV) of ground state channel
INTEGER, INTENT(IN) :: Nlev_ion, Nlev_rec ! Number of energy levels
DOUBLE PRECISION, DIMENSION(1:Nlev_ion) :: energies_ion ! Excited states (eV)
DOUBLE PRECISION, DIMENSION(1:Nlev_rec) :: energies_rec
!OUT
INTEGER, INTENT(OUT) :: abin_best, ctlevel_best
DOUBLE PRECISION, INTENT(OUT) :: ED_best

END SUBROUTINE FINDRESONANCE

!SUBROUTINE COLLION(Nlev, T, ionrate, recrate)
!USE PARAMETERS, ONLY : k
!IMPLICIT NONE
!INTEGER, INTENT(IN) :: Nlev
!DOUBLE PRECISION, INTENT(IN) :: T
!DOUBLE PRECISION, DIMENSION(Nlev), INTENT(OUT) :: ionrate, recrate
!END SUBROUTINE COLLION

SUBROUTINE DUSTATTENUATION(x, vin, vout, tau, attenuation)
IMPLICIT NONE

DOUBLE PRECISION, INTENT(IN) :: x, tau, vin, vout
DOUBLE PRECISION, INTENT(OUT) :: attenuation

END SUBROUTINE DUSTATTENUATION

SUBROUTINE TOTRECRATES_GOULD(name, T, alpha)
IMPLICIT NONE

CHARACTER(len=5), INTENT(IN) :: name
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, INTENT(OUT) :: alpha

END SUBROUTINE TOTRECRATES_GOULD

SUBROUTINE CASCADEMATRIX2(ngrounduse, N, AMAT, CASCMAT, PMAT)
IMPLICIT NONE
! IN AND OUT
INTEGER, INTENT(IN) :: ngrounduse, N    ! Number of levels
DOUBLE PRECISION, DIMENSION(N,N), INTENT(IN) :: AMAT  ! A-values
DOUBLE PRECISION, DIMENSION(N,N), INTENT(OUT) :: CASCMAT, PMAT
END SUBROUTINE CASCADEMATRIX2

SUBROUTINE READRUNPARAMETERS(time_days, file_explmodel, tau_dust, vterm, Nblobs, tau_tresh, sf_tresh, REALRECCONT, FORCECAII, OLI, Dustzonemax, N_atoms, atoms)
USE M1
IMPLICIT NONE

DOUBLE PRECISION, INTENT(OUT) :: time_days
INTEGER, INTENT(IN) :: N_atoms
TYPE(atomm), DIMENSION(N_atoms), INTENT(INOUT) :: atoms
CHARACTER(len=100), INTENT(OUT) :: file_explmodel
DOUBLE PRECISION, INTENT(OUT) :: tau_dust, tau_tresh, sf_tresh
DOUBLE PRECISION, POINTER, INTENT(OUT) :: vterm
INTEGER, INTENT(OUT) :: Nblobs, Dustzonemax
LOGICAL, INTENT(OUT) :: REALRECCONT, FORCECAII,OLI
! LOGICAL, POINTER, INTENT(OUT) :: BLEND

END SUBROUTINE READRUNPARAMETERS

SUBROUTINE MAKEATOMS(N_atoms, atoms, T_typical, maxN)
USE M1, ONLY : atomm
IMPLICIT NONE

INTEGER, INTENT(IN) :: N_atoms
TYPE(atomm), DIMENSION(N_atoms), INTENT(INOUT) :: atoms
DOUBLE PRECISION, INTENT(INOUT) :: T_typical
INTEGER, INTENT(OUT) :: maxN

END SUBROUTINE MAKEATOMS

SUBROUTINE WRITELOGS(N_atoms)
INTEGER, INTENT(IN) :: N_atoms
END SUBROUTINE WRITELOGS

subroutine phfit2(nz,ne,is,e,s)
      implicit real*8(a-h,o-z)
      integer nz,ne,is
      real*8 e,s
END subroutine phfit2

!SUBROUTINE EFFRECRATES(N, atom, alpha, BETA, alphaeff)
!USE M1
!IMPLICIT NONE

!INTEGER, INTENT(IN) :: N
!TYPE(atomm), INTENT(IN) :: atom
!DOUBLE PRECISION, DIMENSION(N), INTENT(IN) :: alpha
!DOUBLE PRECISION, DIMENSION(N,N) :: BETA
!DOUBLE PRECISION, DIMENSION(N), INTENT(OUT) :: alphaeff

!END SUBROUTINE EFFRECRATES

!SUBROUTINE RECCONTINUUM(atom, T, ne, nplus, Nlambda, lambdarec, Erec)
!!USE PARAMETERS
!USE M1
!IMPLICIT NONE

!TYPE(atomm), INTENT(IN) :: atom
!DOUBLE PRECISION, INTENT(IN) :: T, ne, nplus
!INTEGER, INTENT(IN) :: Nlambda
!DOUBLE PRECISION, DIMENSION(Nlambda), INTENT(IN) :: lambdarec
!DOUBLE PRECISION, DIMENSION(Nlambda), INTENT(OUT) :: Erec

!END SUBROUTINE RECCONTINUUM

SUBROUTINE MGI_RECOMB(T, N, alpha)
IMPLICIT NONE

DOUBLE PRECISION, INTENT(IN) :: T
INTEGER, INTENT(IN) :: N
DOUBLE PRECISION, DIMENSION(N), INTENT(OUT) :: alpha

END

SUBROUTINE NAI_RECOMB(T, N, alpha)
IMPLICIT NONE

DOUBLE PRECISION, INTENT(IN) :: T
INTEGER, INTENT(IN) :: N
DOUBLE PRECISION, DIMENSION(N), INTENT(OUT) :: alpha

END

SUBROUTINE molcoolCF(isel,te,denel,denmol,time,cooling,coolv1,coolv2)
      implicit real*8(a-h,o-z)
      INTEGER :: isel

    END SUBROUTINE molcoolCF

SUBROUTINE oI_recomb(T, gf1, N_LEVELS, alphaans)
  USE M1                                                 !new000225 New Nahar-recombination data for Fe III (from CF)
  IMPLICIT NONE
  DOUBLE PRECISION, INTENT(IN) :: T
  DOUBLE PRECISION, DIMENSION(N_LEVELS) :: gf1
  INTEGER, INTENT(IN) :: N_LEVELS
  DOUBLE PRECISION, DIMENSION(N_LEVELS), INTENT(OUT) :: alphaans
END SUBROUTINE oI_recomb

SUBROUTINE RECLINES_SMOOTH(atom, T, ne, nplus, eexc, N, Nsplit, lambdarec, Erec)
!USE PARAMETERS
USE M1
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: Nsplit
DOUBLE PRECISION, INTENT(IN) :: T, ne, nplus, eexc
INTEGER, INTENT(IN) :: N
DOUBLE PRECISION, DIMENSION(N, Nsplit), INTENT(OUT) :: lambdarec, Erec

END SUBROUTINE RECLINES_SMOOTH

SUBROUTINE STATEQ(R_index, N_atoms, atoms, EXCMAX, DIMS, x, deposition_tot_pervol, IONRATES, fracmat_exc, fground, Nlambda, lambdavec, alfa_c, T, CTINMAT, Tuse, EXC_MATRIX, convprobs)
USE M1
!USE PARAMETERS
USE FILES
IMPLICIT NONE

INTEGER, INTENT(IN) :: R_index, N_atoms, EXCMAX, DIMS, Nlambda
TYPE(atomm), DIMENSION(N_atoms), INTENT(IN) :: atoms
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, INTENT(INOUT) :: deposition_tot_pervol, T, Tuse
DOUBLE PRECISION, DIMENSION(Nabin,EXCMAX) :: CTINMAT
REAL, DIMENSION(:,:), INTENT(IN) :: IONRATES
DOUBLE PRECISION, DIMENSION(Nabin, SFMAX), INTENT(IN) :: fracmat_exc
DOUBLE PRECISION, DIMENSION(Nabin), INTENT(IN) :: fground
DOUBLE PRECISION, DIMENSION(Nlambda), INTENT(IN) :: lambdavec, alfa_c
DOUBLE PRECISION, DIMENSION(N_atoms,EXCMAX), INTENT(OUT) :: EXC_MATRIX
LOGICAL, DIMENSION(N_atoms), INTENT(OUT), OPTIONAL :: convprobs
END SUBROUTINE STATEQ

SUBROUTINE createbeta_l(atom, N_levels, TAU_S, T, BETA_L)
USE M1
IMPLICIT NONE
! ----------------------------------------
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: N_levels
DOUBLE PRECISION, DIMENSION(N_LEVELS, N_LEVELS), INTENT(IN) :: TAU_S
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(N_LEVELS, N_LEVELS), INTENT(OUT) :: BETA_L
END SUBROUTINE createbeta_l

SUBROUTINE TWOPHOTON_AY(name, lambda, ERROR)
IMPLICIT NONE
CHARACTER(len=3), INTENT(IN) :: name
DOUBLE PRECISION, INTENT(OUT) :: lambda
LOGICAL, INTENT(OUT) :: ERROR
END SUBROUTINE TWOPHOTON_AY

SUBROUTINE WRITEIONEDGES(N_atoms, atoms)
USE M1
!USE PARAMETERS
USE FILES
IMPLICIT NONE

INTEGER, INTENT(IN) :: N_atoms
TYPE(atomm), DIMENSION(N_atoms), INTENT(IN) :: atoms
END SUBROUTINE WRITEIONEDGES

SUBROUTINE statisticalandthermalequilibrium(N_atoms, atoms, R_index, DIMS, x, abinvec, fground, nmolvec, EXCmax, dep_tot_pervol, radheating, IONRATES, alfa_dust, tau_tresh, abintresh, T_prev, xe_prev, T_eq, EXC_MATRIX, cool_co, cool_sio, fv1_co, fv1_sio, ntheatrate, convprobs)           ! A SUBROUTINE THAT SOLVES FOR T, ION. AND EXC STRUCTURE
use M1
!USE PARAMETERS
USE FILES
IMPLICIT NONE
INTEGER :: N_atoms, DIMS, EXCmax
INTEGER, INTENT(IN) :: R_index
TYPE(atomm), DIMENSION(N_atoms) :: atoms
DOUBLE PRECISION, DIMENSION(DIMS) :: x
DOUBLE PRECISION, DIMENSION(Nabin), INTENT(INOUT) :: abinvec, fground
DOUBLE PRECISION, DIMENSION(maxval([1,Nmol])), INTENT(IN) :: nmolvec
REAL, DIMENSION(:,:) :: IONRATES
DOUBLE PRECISION :: dep_tot_pervol
DOUBLE PRECISION, INTENT(IN) :: radheating, alfa_dust, tau_tresh, abintresh, T_prev, xe_prev
DOUBLE PRECISION, INTENT(OUT) :: T_eq, cool_co, cool_sio, fv1_co, fv1_sio
DOUBLE PRECISION, DIMENSION(N_atoms, EXCmax), INTENT(OUT) :: EXC_MATRIX
DOUBLE PRECISION, INTENT(OUT) :: ntheatrate
LOGICAL, DIMENSION(N_atoms+1), INTENT(OUT) :: convprobs
END SUBROUTINE statisticalandthermalequilibrium

SUBROUTINE netheating(R_index, heatrate, N_atoms, atoms, EXCmax, EXC_MATRIX, ne, DIMS, x, T, IONRATES_COLLION, nh, cool_co, cool_sio, fv1_co, fv1_sio, PRINTT)
USE M1
!USE PARAMETERS
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: heatrate
INTEGER, INTENT(IN) :: R_index, N_atoms, EXCmax, DIMS
TYPE(atomm), DIMENSION(N_atoms), INTENT(IN) :: atoms
DOUBLE PRECISION, DIMENSION(N_atoms, EXCmax), INTENT(IN) :: EXC_MATRIX
DOUBLE PRECISION, INTENT(IN) :: ne
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, INTENT(IN) :: T
REAL, DIMENSION(Nabin,Npis), INTENT(IN) :: IONRATES_COLLION
DOUBLE PRECISION, INTENT(OUT) :: nh, cool_co, cool_sio, fv1_co, fv1_sio
LOGICAL, INTENT(IN) :: PRINTT
END SUBROUTINE netheating

subroutine co_photodiss(ireadph,denco,tday,wl,tau)
      implicit real*8(a-h,o-z)

      real*8 atot(50),fvlvu(50),nvlvu(50),eta(50)
      character*8 char
      integer iul(50)
      integer vp
      real*8 tau(40),wl(40)
end subroutine co_photodiss

! Sf cannot be handled, the include files are in F77 line break format which cannot be read here

SUBROUTINE depositionfunction(DIMS, x_MAT, Nblobs, deposition_tot_pervol, gammadep_pervol)
use M1
!USE PARAMETERS
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: DIMS, Nblobs
DOUBLE PRECISION, DIMENSION(:, :), INTENT(IN) :: x_MAT
! LOGICAL, POINTER, INTENT(IN) :: BLEND
DOUBLE PRECISION, DIMENSION(:), INTENT(OUT) :: deposition_tot_pervol, gammadep_pervol
END SUBROUTINE depositionfunction

SUBROUTINE depositionfunction_compton(DIMS, x_MAT, af, Nblobs, deposition_tot_pervol, gammadep_pervol)
use M1
!USE PARAMETERS
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: DIMS, Nblobs
DOUBLE PRECISION, DIMENSION(:, :), INTENT(IN) :: x_MAT
DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: af
! LOGICAL, POINTER, INTENT(IN) :: BLEND
DOUBLE PRECISION, DIMENSION(:), INTENT(OUT) :: deposition_tot_pervol, gammadep_pervol
END SUBROUTINE depositionfunction_compton

SUBROUTINE COMPTON(E, sigma)
USE PARAMETERS, ONLY : me, c
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: E  ! Photon energy in in erg
DOUBLE PRECISION, INTENT(OUT) :: sigma ! Cross section in cm^2
END SUBROUTINE COMPTON

SUBROUTINE COMPTONDIFF(Ein, Eout, angle)
USE PARAMETERS, ONLY : me, c, pi
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: Ein  ! Photon energy in in erg
DOUBLE PRECISION, INTENT(OUT) :: Eout, angle
END SUBROUTINE COMPTONDIFF

!SUBROUTINE RECLINES(atom, T, ne, nplus, N, lambdarec, Erec)
!!USE PARAMETERS
!USE M1
!IMPLICIT NONE
!TYPE(atomm), INTENT(IN) :: atom
!DOUBLE PRECISION, INTENT(IN) :: T, ne, nplus
!INTEGER, INTENT(OUT) :: N
!DOUBLE PRECISION, DIMENSION(N), INTENT(OUT) :: lambdarec, Erec
!END SUBROUTINE RECLINES

SUBROUTINE feI_recomb(T, gf1, N_LEVELS, alphaans)
USE M1                                                 !new000225 New Nahar-recombination data for Fe III (from CF)
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(N_LEVELS) :: gf1
INTEGER, INTENT(IN) :: N_LEVELS
DOUBLE PRECISION, DIMENSION(N_LEVELS), INTENT(OUT) :: alphaans
END SUBROUTINE feI_recomb

SUBROUTINE feII_recomb(T, N_LEVELS, alphaans)
use M1
IMPLICIT NONE                                                 !new000225 New Nahar-recombination data for Fe III (from CF
DOUBLE PRECISION, INTENT(IN) :: T
INTEGER, INTENT(IN) :: N_LEVELS
DOUBLE PRECISION, DIMENSION(N_LEVELS), INTENT(OUT) :: alphaans
END SUBROUTINE feII_recomb

SUBROUTINE NEXTIONEDGE(lambda, lambdaedge, IONEDGEEXISTS)
use M1
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: lambda
DOUBLE PRECISION, INTENT(OUT) :: lambdaedge
LOGICAL, INTENT(OUT) :: IONEDGEEXISTS
END SUBROUTINE NEXTIONEDGE

SUBROUTINE FOSC(Nfrom, fromvec, Nto, tovec, atom, fosctot)
USE M1
IMPLICIT NONE
INTEGER, INTENT(IN) :: Nfrom, Nto
INTEGER, DIMENSION(Nfrom), INTENT(IN) :: fromvec
INTEGER, DIMENSION(Nto), INTENT(IN) :: tovec
TYPE(atomm), INTENT(IN) :: atom
DOUBLE PRECISION, INTENT(OUT) :: fosctot
END SUBROUTINE FOSC

SUBROUTINE FINDBIN2(N, array, x, index)
IMPLICIT NONE
INTEGER, INTENT(IN) :: N
DOUBLE PRECISION, DIMENSION(N), INTENT(IN) :: array
DOUBLE PRECISION, INTENT(IN) :: x
INTEGER, INTENT(OUT) :: index
END SUBROUTINE FINDBIN2

SUBROUTINE FINDBIN3(inx, N, array, x, index)
IMPLICIT NONE
INTEGER, INTENT(IN) :: inx, N
REAL, DIMENSION(:,:), INTENT(IN) :: array

DOUBLE PRECISION, INTENT(IN) :: x
INTEGER, INTENT(OUT) :: index
END SUBROUTINE FINDBIN3

SUBROUTINE FEI_COLLISIONRATES(T, UPSILON)
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(10,10), INTENT(OUT) :: UPSILON
END SUBROUTINE FEI_COLLISIONRATES

SUBROUTINE FEII_COLLISIONRATES(T, UPSILON)
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(16,16), INTENT(OUT) :: UPSILON
END SUBROUTINE FEII_COLLISIONRATES

SUBROUTINE hI_recomb_nl(ntot, T, alpha)
IMPLICIT NONE
INTEGER :: ntot
DOUBLE PRECISION :: T
DOUBLE PRECISION, DIMENSION(ntot) :: alpha
END SUBROUTINE hI_recomb_nl

SUBROUTINE HI_RECOMBSPECTRUM(TOLEVEL, CHI, G, T, E, ergcm3perseV)
IMPLICIT NONE
INTEGER, INTENT(IN) :: TOLEVEL                                       ! Level in an atom with only n=2 split
DOUBLE PRECISION, INTENT(IN) :: CHI, T, E, G          ! Ionization energy of TOLEVEL. Temperature. The energy (in eV) and interval(eV) we want emission at. Statistical weight of TOLEVEL
DOUBLE PRECISION, INTENT(OUT) :: ergcm3perseV
END SUBROUTINE HI_RECOMBSPECTRUM

SUBROUTINE CEXH(NLOWER, NUPPER, T, nmax, Evec, CE)
    !USE PARAMETERS, ONLY : k, pi
    IMPLICIT NONE
    INTEGER, INTENT(IN) :: NLOWER, NUPPER, nmax
    DOUBLE PRECISION, INTENT(IN) :: T
    DOUBLE PRECISION, DIMENSION(nmax), INTENT(IN) :: Evec
    DOUBLE PRECISION, INTENT(OUT) :: CE
END SUBROUTINE CEXH

SUBROUTINE hI_coll_differentn(T, N_levels, Evec, gvec, A, Q)
!USE PARAMETERS
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
INTEGER, INTENT(IN) :: N_levels
DOUBLE PRECISION, DIMENSION(N_levels), INTENT(IN) :: Evec, gvec
DOUBLE PRECISION, DIMENSION(N_levels, N_levels), INTENT(IN) :: A
DOUBLE PRECISION, DIMENSION(N_levels, N_levels), INTENT(INOUT) :: Q
END SUBROUTINE hI_coll_differentn

SUBROUTINE twophoton(atomname, lambdastart_Angstrom, lambdaend_Angstrom, stepatstart_Angstrom, N, lambdavec_out, emissvec_integrated)
!USE PARAMETERS
USE FILES
IMPLICIT NONE
CHARACTER(len=3) :: atomname
DOUBLE PRECISION, INTENT(IN) :: lambdastart_Angstrom, lambdaend_Angstrom, stepatstart_Angstrom
integer, INTENT(OUT) :: N
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE,  INTENT(OUT) :: lambdavec_out, emissvec_integrated
END SUBROUTINE twophoton

SUBROUTINE MOLCOOLING(te, ne, n_co, n_sio, cool_co, cool_sio)
!USE PARAMETERS
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: te, ne, n_co, n_sio
DOUBLE PRECISION, INTENT(OUT) :: cool_co, cool_sio
END SUBROUTINE MOLCOOLING

SUBROUTINE regemorter(N, T_typical, atom)
USE M1
!USE PARAMETERS
IMPLICIT NONE
INTEGER, INTENT(IN) :: N
DOUBLE PRECISION, INTENT(INOUT) :: T_typical
TYPE(atomm), INTENT(INOUT) :: atom
END SUBROUTINE regemorter

SUBROUTINE SORTTAUMAT(Nlines, TAUMAT)
USE M1
IMPLICIT NONE
INTEGER, INTENT(IN) :: Nlines
DOUBLE PRECISION, DIMENSION(:,:), INTENT(INOUT) :: TAUMAT
END SUBROUTINE SORTTAUMAT

SUBROUTINE thinspectrum(R_index, lambdastart_A, lambdaend_A, emitend_A, stepatstart_A, DIMS, x, EMISS_MAT, N, wlvec, tau_dust)
USE M1
!USE PARAMETERS
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index
DOUBLE PRECISION, INTENT(IN) :: lambdastart_A, lambdaend_A, stepatstart_A, emitend_A
DOUBLE PRECISION, INTENT(INOUT) :: tau_dust
INTEGER, INTENT(IN) :: DIMS
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION(:,:), INTENT(IN) :: EMISS_MAT
INTEGER, INTENT(OUT) :: N
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: wlvec
END SUBROUTINE thinspectrum

SUBROUTINE fileEMISS_CUBE(R_index, EMISS_MAT, DIMS, x)
USE M1
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index, DIMS
DOUBLE PRECISION, DIMENSION(:,:), INTENT(IN) :: EMISS_MAT
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
END SUBROUTINE fileEMISS_CUBE

SUBROUTINE readtaucube(N_atoms, atoms, Nlines, TAUCUBE)
use M1
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: N_atoms, Nlines
TYPE(atomm), DIMENSION(N_atoms), INTENT(INOUT) :: atoms
REAL, DIMENSION(:,:,:), INTENT(OUT) :: TAUCUBE
END SUBROUTINE readtaucube

SUBROUTINE createbetaandQvec(atom, from, Nup, EXC_MAT, nmolvec, ndensvec_abin, alfa_dust, N_atoms, atoms, R_index, T, ne, beta_s, beta_c, Qvecdown, Qvecup)
!USE PARAMETERS
USE M1
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: from, R_index, Nup, N_atoms
REAL, DIMENSION(:,:), INTENT(IN) :: EXC_MAT
DOUBLE PRECISION, DIMENSION(Nmol), INTENT(IN) :: nmolvec
DOUBLE PRECISION, DIMENSION(Nabin), INTENT(INOUT) :: ndensvec_abin
TYPE(atomm), DIMENSION(N_atoms), INTENT(IN) :: atoms
DOUBLE PRECISION, INTENT(IN) :: T, ne, alfa_dust
DOUBLE PRECISION, DIMENSION(from-1), INTENT(OUT) :: beta_s, beta_c, Qvecdown
DOUBLE PRECISION, DIMENSION(Nup), INTENT(OUT) :: Qvecup
END SUBROUTINE createbetaandQvec

SUBROUTINE CHARGEREADER(atom, errsig)
USE M1
IMPLICIT NONE
TYPE(atomm), INTENT(INOUT) :: atom
INTEGER, INTENT(OUT) :: errsig
END SUBROUTINE CHARGEREADER

SUBROUTINE CHARGETRANSFERHEATING(nnuclei, x, DESTROY, CTH, ctheating)
USE M1
IMPLICIT NONE
DOUBLE PRECISION, DIMENSION(Nelem), INTENT(IN) :: nnuclei
DOUBLE PRECISION, DIMENSION(Nstages*Nelem+Nmol), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: DESTROY, CTH
DOUBLE PRECISION, INTENT(OUT) :: ctheating
END SUBROUTINE CHARGETRANSFERHEATING

SUBROUTINE alfa_photoionization(lambda, Natoms, atoms, EXC_MAT, nmolvec, alfa_dust, T, ne, ndensvec_abin, alfa, ALFA_MAT)      ! For photons above 500 \C5.
USE M1
!USE PARAMETERS
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: lambda, alfa_dust, T, ne
INTEGER, INTENT(IN) :: Natoms
TYPE(atomm), DIMENSION(Natoms), INTENT(IN) :: atoms
REAL, DIMENSION(:,:), INTENT(IN) :: EXC_MAT
DOUBLE PRECISION, DIMENSION(Nmol), INTENT(IN) :: nmolvec
DOUBLE PRECISION, DIMENSION(Nabin), INTENT(IN) :: ndensvec_abin
DOUBLE PRECISION, INTENT(OUT) :: alfa
DOUBLE PRECISION, DIMENSION(:, :), INTENT(OUT) :: ALFA_MAT
END SUBROUTINE alfa_photoionization



RECURSIVE SUBROUTINE followphoton_ff(zone, r, E, lambda, sender, scattered, origzone, lastzone, origr, origlambda, dt_core, r_inblob, N_atoms, atoms, RT, ION_STRUCTURE, EXCCUBE_ABIN, NMOLMAT, radii, ne_vec, alfa_dust, Nlines, Nlinesvec, TAUCUBE, OLI, Nblobs, blobsize, af_accum, WLbins, xlog)
USE M1
!USE PARAMETERS
IMPLICIT NONE
DOUBLE PRECISION, INTENT(INOUT) :: r, E, lambda, dt_core, r_inblob
TYPE(atomm), DIMENSION(N_atoms) :: atoms
INTEGER, INTENT(IN) :: N_atoms, sender, Nlines, WLbins
INTEGER, INTENT(INOUT) :: zone, origzone, lastzone, Nblobs, scattered
DOUBLE PRECISION, INTENT(INOUT) :: origlambda, origr
DOUBLE PRECISION, DIMENSION(:,:), INTENT(IN) :: RT
DOUBLE PRECISION, DIMENSION(:,:), INTENT(INOUT) :: ION_STRUCTURE
DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: radii, blobsize
DOUBLE PRECISION, DIMENSION(:), INTENT(INOUT) :: ne_vec, alfa_dust
REAL, DIMENSION(:,:,:), INTENT(IN) :: EXCCUBE_ABIN
DOUBLE PRECISION, DIMENSION(:,:), INTENT(INOUT) :: NMOLMAT
INTEGER, DIMENSION(:), INTENT(INOUT) :: Nlinesvec
REAL, DIMENSION(:,:,:), INTENT(INOUT) :: TAUCUBE
LOGICAL, INTENT(INOUT) :: OLI
! LOGICAL, POINTER, INTENT(INOUT) :: BLEND
DOUBLE PRECISION, DIMENSION(:), INTENT(INOUT) :: af_accum
DOUBLE PRECISION, DIMENSION(WLbins) :: xlog, xlog_small
END SUBROUTINE followphoton_ff

SUBROUTINE modelreader(vterm, N_atoms, atoms, file_explmodel, FORCECAII, DIMS, x_MAT, ION_STRUCTURE, ABIN_MAT, Dustzonemax)
USE M1
!USE PARAMETERS
USE FILES
IMPLICIT NONE
DOUBLE PRECISION, POINTER, INTENT(OUT) :: vterm
INTEGER, INTENT(IN) :: N_atoms, DIMS, Dustzonemax
TYPE(atomm), DIMENSION(N_atoms), INTENT(INOUT) :: atoms
DOUBLE PRECISION, DIMENSION(:, :), INTENT(OUT) :: x_MAT
DOUBLE PRECISION, DIMENSION(:, :), INTENT(OUT) :: ION_STRUCTURE, ABIN_MAT
LOGICAL, INTENT(IN) :: FORCECAII
CHARACTER(len=100), INTENT(IN) :: file_explmodel
! LOGICAL, POINTER :: BLEND
END SUBROUTINE modelreader


SUBROUTINE hI_freebound(atom, T, N, lambdavec, emissvec_integrated)
!USE PARAMETERS
USE M1
IMPLICIT NONE

TYPE(atomm), INTENT(IN) :: atom
DOUBLE PRECISION, INTENT(IN) :: T
INTEGER, INTENT(OUT) :: N
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: lambdavec, emissvec_integrated

END SUBROUTINE hI_freebound

!SUBROUTINE initialize_linemasks(lambda0, atoms)
!USE M1
!IMPLICIT NONE
!! -----------------------------------------------------------------------------------------
!DOUBLE PRECISION, INTENT(IN) :: lambda0
!TYPE(atomm), DIMENSION(:), INTENT(INOUT), TARGET :: atoms
!END SUBROUTINE initialize_linemasks

SUBROUTINE calculatecubes(R_index, N_atoms, atoms, EXCMAX, nmolvec, EXC_MAT_ABIN, tau_tresh, DIMS, x, ndensvec_abin, alfa_dust, LINESTOT, COMPUTESHELL, REALRECCONT, n_co, Nlines, TAUMAT, N_abs, EMISS_MAT, molcool, lum)
USE M1
!USE PARAMETERS
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index, N_atoms, EXCMAX, DIMS
TYPE(atomm), DIMENSION(N_atoms) :: atoms
DOUBLE PRECISION, DIMENSION(Nmol), INTENT(IN) :: nmolvec
DOUBLE PRECISION, DIMENSION(Nabin, EXCMAX),  INTENT(IN) :: EXC_MAT_ABIN
DOUBLE PRECISION, INTENT(IN) :: tau_tresh
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION(Nabin), INTENT(INOUT) :: ndensvec_abin
DOUBLE PRECISION, INTENT(IN) :: alfa_dust, molcool
INTEGER, INTENT(IN) :: LINESTOT, Nlines
LOGICAL, INTENT(IN) :: COMPUTESHELL, REALRECCONT
DOUBLE PRECISION, INTENT(IN):: n_co
DOUBLE PRECISION, DIMENSION(Nlines,6), INTENT(OUT) :: TAUMAT
INTEGER, INTENT(OUT) :: N_abs
DOUBLE PRECISION, DIMENSION(LINESTOT,6), INTENT(OUT) :: EMISS_MAT
DOUBLE PRECISION, INTENT(OUT) :: lum
END SUBROUTINE calculatecubes

SUBROUTINE hI_coll_internal(partner, T, ne, N_levels, A, Q)
!USE PARAMETERS
IMPLICIT NONE
INTEGER, INTENT(IN) :: partner, N_levels
DOUBLE PRECISION, INTENT(IN) :: T, ne
DOUBLE PRECISION, DIMENSION(N_levels,N_levels), INTENT(IN) :: A
DOUBLE PRECISION, DIMENSION(N_levels,N_levels), INTENT(INOUT) :: Q
END SUBROUTINE hI_coll_internal

SUBROUTINE boltzmann(T, atom, N_levels, boltzmannsol, Zout)                   ! Calculates the normalized Boltzmann distribution for given T
USE M1
!USE PARAMETERS
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: N_levels
DOUBLE PRECISION, DIMENSION(N_levels), INTENT(OUT)  :: boltzmannsol
DOUBLE PRECISION, OPTIONAL :: Zout
END SUBROUTINE boltzmann

SUBROUTINE CREATEQ(T, atom, Nsize, Q, ne)	                ! Calculates collisional rates q from Omega values and temperature (Red Osterbrock)
USE M1
!USE PARAMETERS
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: Nsize
DOUBLE PRECISION, DIMENSION(Nsize,Nsize), INTENT(OUT) :: Q
DOUBLE PRECISION :: ne
END SUBROUTINE CREATEQ

SUBROUTINE createtau_s(atom, N_upper, N_levels, levels_pop, TAU_S)
USE M1
!USE PARAMETERS
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: N_upper, N_levels
DOUBLE PRECISION, DIMENSION(N_levels), INTENT(IN) :: levels_pop
DOUBLE PRECISION, DIMENSION(N_upper, N_levels),  INTENT(OUT) :: TAU_S
END SUBROUTINE createtau_s

SUBROUTINE createbeta_s(name, N_up, N_down, TAU_S, BETA)
IMPLICIT NONE
CHARACTER(len=5), INTENT(IN) :: name
INTEGER, INTENT(IN) :: N_up, N_down
DOUBLE PRECISION, DIMENSION(N_up, N_down), INTENT(IN) :: TAU_S
DOUBLE PRECISION, DIMENSION(N_up, N_down), INTENT(OUT) :: BETA
END SUBROUTINE createbeta_s

SUBROUTINE FLUOR(atom, DIMS, x, EXCMAX, EXC_MAT_ABIN, Nfill, idvec, fracvec, power)
USE M1
!USE PARAMETERS
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: DIMS, Nfill, EXCMAX
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
INTEGER, DIMENSION(Nfill), INTENT(IN) :: idvec
DOUBLE PRECISION, DIMENSION(Nfill), INTENT(IN) :: fracvec
DOUBLE PRECISION, DIMENSION(Nabin,EXCMAX), INTENT(IN) :: EXC_MAT_ABIN
DOUBLE PRECISION, INTENT(IN) :: power
END SUBROUTINE FLUOR

SUBROUTINE lyalphalybeta_alloc(name, Ly_alpha_flow, Ly_beta_flow, N_levels, levels_pop, LYMAT)
IMPLICIT NONE
CHARACTER(len=5), INTENT(IN) :: name
DOUBLE PRECISION, INTENT(IN) :: Ly_alpha_flow, Ly_beta_flow
INTEGER, INTENT(IN) :: N_levels
DOUBLE PRECISION, DIMENSION(N_levels), INTENT(IN) :: levels_pop
DOUBLE PRECISION, DIMENSION(N_levels, N_levels), INTENT(OUT) :: LYMAT
END SUBROUTINE lyalphalybeta_alloc

SUBROUTINE createbeta_c(atom, N_levels, TAU_S, T, Nlambda, lambdavec, alfa_c_vec, BETA_C)
USE M1
!USE PARAMETERS
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: N_levels, Nlambda
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION(N_levels, N_levels), INTENT(IN) :: TAU_S
DOUBLE PRECISION, DIMENSION(Nlambda), INTENT(IN) :: lambdavec, alfa_c_vec
DOUBLE PRECISION, DIMENSION(N_levels, N_levels), INTENT(OUT) :: BETA_C
END SUBROUTINE createbeta_c

SUBROUTINE excitation_solver(name, N_levels, N, AMAT, QMAT, BETAMAT, NTEXCMAT, LYMAT, gammaion, GAMMAPHOTOEXC, alphain, threebodyrates, alphaout, ctanddirection, nk, nplus, ne, levels_pop, FAIL, NEG)
!USE PARAMETERS
USE M1
IMPLICIT NONE
CHARACTER(len=5) :: name
INTEGER, INTENT(IN) :: N_levels, N
DOUBLE PRECISION, DIMENSION(N_levels), INTENT(IN) :: alphain, threebodyrates, gammaion, ctanddirection
DOUBLE PRECISION, DIMENSION(N_levels, N_levels), INTENT(IN)  :: AMAT, QMAT, BETAMAT, NTEXCMAT, LYMAT
DOUBLE PRECISION, DIMENSION(N_levels, N), INTENT(IN) :: GAMMAPHOTOEXC
DOUBLE PRECISION, INTENT(IN) :: nk, nplus, ne, alphaout
DOUBLE PRECISION, DIMENSION(N_levels), INTENT(OUT) :: levels_pop
INTEGER, INTENT(OUT) :: FAIL, NEG
END SUBROUTINE excitation_solver

SUBROUTINE MAKEALPHA(T, alphatot , avexcenergy_diel)
use M1, ONLY : Nelem, Nstages
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem), INTENT(OUT) :: alphatot
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem), INTENT(OUT), OPTIONAL :: avexcenergy_diel
END SUBROUTINE MAKEALPHA

SUBROUTINE readlevelsandlines(atom, T_typical)
USE M1
!USE PARAMETERS
IMPLICIT NONE
TYPE(atomm), INTENT(INOUT) :: atom
DOUBLE PRECISION, INTENT(INOUT) :: T_typical
END SUBROUTINE readlevelsandlines

SUBROUTINE MAKEPHOTOEXCRATES(atom, Tstar, T, N_upper, GAMMAPHOTOEXC, gammaion, sigmaweighted, bfactorweighted, stimrec, nion)
USE M1
USE PARAMETERS
IMPLICIT NONE

TYPE(atomm), INTENT(IN) :: atom
DOUBLE PRECISION, INTENT(IN) :: Tstar, T
INTEGER, INTENT(IN) :: N_upper
DOUBLE PRECISION, DIMENSION(N_upper, N_upper), INTENT(OUT) :: GAMMAPHOTOEXC
DOUBLE PRECISIOn, DIMENSION(N_upper), INTENT(OUT) :: gammaion, sigmaweighted, bfactorweighted, nion, stimrec

END

SUBROUTINE makealphavec(atom, T, N, alpha_spec)
USE M1
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
DOUBLE PRECISION, INTENT(IN) :: T
INTEGER, INTENT(IN) :: N
DOUBLE PRECISION, DIMENSION(N), INTENT(OUT) :: alpha_spec
END SUBROUTINE makealphavec

SUBROUTINE makeP(atom, from, Nup, ne, beta_e, Qvecdown, Qvecup, Pvec)
USE M1
IMPLICIT NONE
TYPE(atomm), INTENT(INOUT) :: atom
INTEGER, INTENT(IN) :: from, Nup                                            ! The level to calculate de-excitation probabilities from
DOUBLE PRECISION, INTENT(IN) :: ne
DOUBLE PRECISION, DIMENSION(from-1), INTENT(IN) :: beta_e, Qvecdown
DOUBLE PRECISION, DIMENSION(Nup), INTENT(IN) :: Qvecup
DOUBLE PRECISION, DIMENSION(2*(from-1)+Nup), INTENT(OUT) :: Pvec
END SUBROUTINE makeP

SUBROUTINE SORTTAUCUBE(Nlinesvec,Nlines,TAUCUBE)
USE M1
IMPLICIT NONE
INTEGER, INTENT(IN) :: Nlines
INTEGER, DIMENSION(:), INTENT(IN) :: Nlinesvec
REAL, DIMENSION(:,:,:), INTENT(INOUT) :: TAUCUBE

END SUBROUTINE SORTTAUCUBE

SUBROUTINE DEEXCITE(z, levelj, Nup, P, beta_s, beta_c, level_out, photon, ionization)
USE M1
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: z
INTEGER, INTENT(IN) :: levelj, Nup
DOUBLE PRECISION, DIMENSION(2*(levelj-1)+Nup), INTENT(IN) :: P
DOUBLE PRECISION, DIMENSION(levelj-1), INTENT(IN) :: beta_s, beta_c
INTEGER, INTENT(OUT) :: level_out
LOGICAL, INTENT(OUT) :: photon, ionization
END SUBROUTINE DEEXCITE

SUBROUTINE IONIZATION(TESTRUN, nlev,sizeuse, Elevels, fracground2, fracground3, CTON, MOLON, totalnumberdensity, numberfractions, volume, Edep, gammarad, useshell, mols)
USE M1
!USE PARAMETERS
USE FILES
IMPLICIT NONE

LOGICAL, INTENT(IN) :: TESTRUN
INTEGER, DIMENSION(Nabin) :: nlev
INTEGER, INTENT(IN) :: sizeuse
DOUBLE PRECISION, DIMENSION(Nabin,sizeuse) :: Elevels
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem), INTENT(IN) :: fracground2
DOUBLE PRECISION, DIMENSION(Nstages*Nelem), INTENT(IN) :: fracground3
LOGICAL, INTENT(IN) :: CTON, MOLON
DOUBLE PRECISION, INTENT(IN) :: totalnumberdensity, volume, Edep
DOUBLE PRECISION, DIMENSION(Nelem), INTENT(IN) :: numberfractions
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol), INTENT(IN) :: gammarad
INTEGER, OPTIONAL :: useshell

TYPE(mmol), DIMENSION(Nmol), INTENT(IN) :: mols

END SUBROUTINE IONIZATION


!SUBROUTINE nextline(lambda,N_atoms, atoms, linej, linei, lambda_line, seq, FOUNDNEXTLINE, n_CO, MOLLINE, tau_mol)
!USE M1
!IMPLICIT NONE
!DOUBLE PRECISION, INTENT(IN) :: lambda, n_CO
!INTEGER, INTENT(IN) :: N_atoms
!TYPE(atomm), DIMENSION(N_atoms), INTENT(IN) :: atoms
!INTEGER, INTENT(OUT) :: linej, linei
!DOUBLE PRECISION, INTENT(OUT) :: lambda_line, tau_mol
!INTEGER, INTENT(OUT) :: seq
!LOGICAL, INTENT(OUT) :: FOUNDNEXTLINE, MOLLINE
!END SUBROUTINE nextline

SUBROUTINE nextline2(R_index, lambda, Nlineszone, TAUMAT, lineind, linej, linei, lambda_line, seq, FOUNDNEXTLINE, n_CO, MOLLINE, tau_mol)
USE M1
IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index, Nlineszone
DOUBLE PRECISION, INTENT(IN) :: lambda, n_CO
REAL, DIMENSION(Nlineszone, 6), INTENT(IN) :: TAUMAT
DOUBLE PRECISION, INTENT(OUT) :: lambda_line, tau_mol
INTEGER, INTENT(OUT) :: seq, lineind,linei,linej
LOGICAL, INTENT(OUT) :: FOUNDNEXTLINE
LOGICAL, INTENT(OUT) :: MOLLINE

END SUBROUTINE nextline2

SUBROUTINE nextline3(R_index, lambda, Nlineszone, TAUMAT, lineind, linej, linei, lambda_line, seq, FOUNDNEXTLINE, n_CO, MOLLINE, tau_mol)
USE M1
IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index, Nlineszone
DOUBLE PRECISION, INTENT(IN) :: lambda, n_CO
REAL, DIMENSION(:,:,:), INTENT(IN) :: TAUMAT
DOUBLE PRECISION, INTENT(OUT) :: lambda_line, tau_mol
INTEGER, INTENT(OUT) :: seq, lineind,linei,linej
LOGICAL, INTENT(OUT) :: FOUNDNEXTLINE
LOGICAL, INTENT(OUT) :: MOLLINE

END SUBROUTINE nextline3

SUBROUTINE ALLOC(atom)
USE M1
!USE PARAMETERS                  ! Uses eV
IMPLICIT NONE
TYPE (atomm), INTENT(INOUT) :: atom
END SUBROUTINE ALLOC

SUBROUTINE ALLOC_MOL(mols)
USE M1, ONLY : mmol, Nmol
IMPLICIT NONE
TYPE (mmol), DIMENSION(Nmol), INTENT(INOUT) :: mols
END SUBROUTINE ALLOC_MOL

SUBROUTINE matrixer(N,A)
IMPLICIT NONE
INTEGER, INTENT(IN) :: N
DOUBLE PRECISION, DIMENSION(N,N) :: A
END SUBROUTINE matrixer

SUBROUTINE GETJAC(DIMS, x, n, gamma, gammacollion, alpha, CTION, CTREC, JAC)
IMPLICIT NONE
INTEGER, INTENT(IN) :: DIMS
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION(16), INTENT(IN) :: n
DOUBLE PRECISION, DIMENSION(30), INTENT(IN) :: gamma, gammacollion, alpha
DOUBLE PRECISION, DIMENSION(30,30), INTENT(IN) :: CTION, CTREC
DOUBLE PRECISION, DIMENSION(DIMS,DIMS), INTENT(OUT) :: JAC
END SUBROUTINE GETJAC

SUBROUTINE CHARGETRANSFER(T, nlev, Elevels, CREATE_SMALL, DESTROY_SMALL, CTH, CTABIN, CTLEVEL, XTOEXC)
use M1
!USE PARAMETERS
IMPLICIT NONE

DOUBLE PRECISION, INTENT(IN) :: T
INTEGER, DIMENSION(Nabin) :: nlev
DOUBLE PRECISION, DIMENSION(Nabin, 1000) :: Elevels
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(OUT) :: CREATE_SMALL
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(OUT) :: DESTROY_SMALL, CTH
INTEGER, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(OUT) :: CTABIN, CTLEVEL
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(OUT) :: XTOEXC
END SUBROUTINE CHARGETRANSFER

SUBROUTINE GETJACNUM(DIMS, x, n, gamma, gammacollion, alpha, T, CREATE, DESTROY, MOLON, xmat_prev, nmol_prev, JAC, f_photodiss, mols)
use M1
IMPLICIT NONE

INTEGER, INTENT(IN) :: DIMS
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION(Nelem+1), INTENT(IN) :: n
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol), INTENT(IN) :: gamma, gammacollion, alpha
DOUBLE PRECISION, INTENT(IN) :: T, f_photodiss
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: CREATE
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: DESTROY
LOGICAL, INTENT(IN) :: MOLON
DOUBLE PRECISION, DIMENSION(Nelem,Nstages+1) :: xmat_prev
DOUBLE PRECISION, DIMENSION(Nmol) :: nmol_prev
DOUBLE PRECISION, DIMENSION(DIMS,DIMS), INTENT(OUT) :: JAC

TYPE(mmol), DIMENSION(Nmol), INTENT(IN) :: mols

END SUBROUTINE GETJACNUM

SUBROUTINE GETF(DIMS, x, n, gamma, gammacollion, alpha, T, CREATE, DESTROY, MOLON, xmat_prev, nmol_prev, f, f_photodiss, mols)
USE M1
IMPLICIT NONE
INTEGER, INTENT(IN) :: DIMS
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION(Nelem+1), INTENT(IN) :: n
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol), INTENT(IN) :: gamma, gammacollion, alpha
DOUBLE PRECISION, INTENT(IN) :: T, f_photodiss
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: CREATE
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol,Nstages*Nelem+Nmol), INTENT(IN) :: DESTROY
LOGICAL, INTENT(IN) :: MOLON
DOUBLE PRECISION, DIMENSION(Nelem,Nstages+1), INTENT(IN) :: xmat_prev
DOUBLE PRECISION, DIMENSION(Nmol) :: nmol_prev
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(OUT) :: f
TYPE(mmol), DIMENSION(Nmol), INTENT(IN) :: mols
END SUBROUTINE GETF

SUBROUTINE indexnuclei(N_atoms, atoms, N_nuclei)
USE M1
IMPLICIT NONE
INTEGER, INTENT(IN) :: N_atoms
TYPE(atomm), DIMENSION(N_atoms), INTENT(INOUT) :: atoms
INTEGER, INTENT(OUT) :: N_nuclei
END SUBROUTINE indexnuclei

!SUBROUTINE setcoremask(N_atoms, atoms, N_LOWER_MAX, N_UPPER_MAX, TAU_HYPERCUBE, tau_tresh)
!USE M1
!IMPLICIT NONE
!INTEGER, INTENT(IN) :: N_atoms, N_LOWER_MAX, N_UPPER_MAX
!TYPE(atomm), dimension(N_atoms), INTENT(INOUT) :: atoms
!DOUBLE PRECISION, DIMENSION(Nr,N_atoms,N_LOWER_MAX,N_UPPER_MAX), INTENT(IN) :: TAU_HYPERCUBE
!DOUBLE PRECISION, INTENT(INN) :: tau_tresh
!END SUBROUTINE setcoremask

SUBROUTINE DGESV( N, NRHS, A, LDA, IPIV, B, LDB, INFO )
  INTEGER            INFO, LDA, LDB, N, NRHS
  INTEGER            IPIV( * )
  DOUBLE PRECISION   A( LDA, * ), B( LDB, * )
END SUBROUTINE DGESV

SUBROUTINE checkerrors(N, sol, ans, tol, ERROR)
USE M1
INTEGER, INTENT(IN) :: N
DOUBLE PRECISION, DIMENSION(N), INTENT(IN) :: sol, ans
DOUBLE PRECISION, INTENT(IN) :: tol
LOGICAL, INTENT(OUT) :: ERROR
END SUBROUTINE checkerrors

SUBROUTINE allocGAMMAEXC(atom, deposition_tot_pervol, nk, Tuse, fracvec_channels, GAMMAMAT)
USE M1
!USE PARAMETERS
IMPLICIT NONE
TYPE(atomm), INTENT(IN) :: atom
DOUBLE PRECISION, INTENT(IN) :: deposition_tot_pervol, nk, Tuse
DOUBLE PRECISION, DIMENSION(SFMAX), INTENT(IN) :: fracvec_channels
DOUBLE PRECISION, DIMENSION(atom%N_levels, atom%N_levels), INTENT(OUT) :: GAMMAMAT
END SUBROUTINE allocGAMMAEXC

END INTERFACE

END MODULE INTERFACE
