SUBROUTINE lyalphalybeta_alloc(name, Ly_alpha_flow, Ly_beta_flow, N_levels, levels_pop, LYMAT)
IMPLICIT NONE
CHARACTER(len=5), INTENT(IN) :: name
DOUBLE PRECISION, INTENT(IN) :: Ly_alpha_flow, Ly_beta_flow
INTEGER, INTENT(IN) :: N_levels
DOUBLE PRECISION, DIMENSION(N_levels), INTENT(IN) :: levels_pop
DOUBLE PRECISION, DIMENSION(N_levels, N_levels), INTENT(OUT) :: LYMAT
DOUBLE PRECISION :: frac
! ================================================
! TASK : Create excitation rates corresponding to the Ly-alpha/Ly-beta radiative
!        pumping of certain lines.
! INPUT : name            Atom name
!         Ly_alpha_flow   Energy deposited by Ly-alpha [s^{-1} cm^{-3}]
!         Ly_beta_flow    Energy deposited by Ly-beta [s^{-1} cm^{-3}]
!         N_levels        Size of atom
!         levels_pop      Excitation structure of atom [cm^-3]
! OUTPUT : LYMAT          Matrix containing the excitation rates [s^{-1}]
! ================================================
LYMAT = 0

!frac = 1/9.

frac = 1./22 ! 22 transitions identified in Table B1 in Jerkstrand+2012. 

IF (name .EQ. 'FeII ') THEN


  LYMAT(10,323) = Ly_alpha_flow*frac/levels_pop(10)

  LYMAT(12,334) = Ly_alpha_flow*frac/levels_pop(12)

  LYMAT(11,330) = Ly_alpha_flow*frac/levels_pop(11)

  LYMAT(11,331) = Ly_alpha_flow*frac/levels_pop(11)

  LYMAT(12,336) = Ly_alpha_flow*frac/levels_pop(12)

  LYMAT(12,335) = Ly_alpha_flow*frac/levels_pop(12)

  LYMAT(13,337) = Ly_alpha_flow*frac/levels_pop(13)

   LYMAT(32,526) = Ly_alpha_flow*frac/levels_pop(32)  

   LYMAT(33,532) = Ly_alpha_flow*frac/levels_pop(33)  

   LYMAT(11,328) = Ly_alpha_flow*frac/levels_pop(11)

   LYMAT(34,533) = Ly_alpha_flow*frac/levels_pop(34) 

   LYMAT(13,336) = Ly_alpha_flow*frac/levels_pop(13)
    
   LYMAT(17,400) = Ly_alpha_flow*frac/levels_pop(17)

   LYMAT(25,465) = Ly_alpha_flow*frac/levels_pop(25) 

  

ELSE IF (name .EQ. 'CrII ') THEN

   LYMAT(4,358) = Ly_alpha_flow*frac/levels_pop(4)

   LYMAT(5,361) = Ly_alpha_flow*frac/levels_pop(5)
   
   LYMAT(5,358) = Ly_alpha_flow*frac/levels_pop(5)

   LYMAT(6,364) = Ly_alpha_flow*frac/levels_pop(6)

   LYMAT(3,353) = Ly_alpha_flow*frac/levels_pop(3)
  
   LYMAT(2,351) = Ly_alpha_flow*frac/levels_pop(2)

   LYMAT(4,353) = Ly_alpha_flow*frac/levels_pop(4)

   LYMAT(3,351) = Ly_alpha_flow*frac/levels_pop(3)


ELSE IF (name .EQ. ' OI  ') THEN

   LYMAT(1,23) = Ly_beta_flow*1./levels_pop(1)

END IF

END
   
