SUBROUTINE makeP(atom, from, Nup, ne, beta_e, Qvecdown, Qvecup, Pvec)
USE M1, ONLY : atomm
IMPLICIT NONE
TYPE(atomm), INTENT(INOUT) :: atom
INTEGER, INTENT(IN) :: from, Nup                                            ! The level to calculate de-excitation probabilities from
DOUBLE PRECISION, INTENT(IN) :: ne
DOUBLE PRECISION, DIMENSION(from-1), INTENT(IN) :: beta_e, Qvecdown 
DOUBLE PRECISION, DIMENSION(Nup), INTENT(IN) :: Qvecup
DOUBLE PRECISION, DIMENSION(2*(from-1)+Nup), INTENT(OUT) :: Pvec                 ! Vector with all transition probabilities, twice as long as from
! ======================================================
! Task : Compute probabilities for all deexcitation channels
! ======================================================
DOUBLE PRECISION, DIMENSION(2*(from-1)+Nup) :: R
DOUBLE PRECISION, DIMENSION(from-1) :: beta_e_dummy
DOUBLE PRECISION :: Rtot
INTEGER :: to, i

! ------------------------------------

beta_e_dummy = beta_e   ! VERY IMPORTANT
R = 0
Pvec = 0		! VERY IMPORTANT

! -------------------------------------

IF (from < 2) THEN
        WRITE(*,*) 'ERROR in makeP: from < 2', from
        STOP 1
ELSE IF (ne .LT. 0) THEN
        WRITE(*,*) 'ERROR in makeP: ne <= 0', ne
        STOP 1
ELSE IF (MINVAL(beta_e) <0 ) THEN
        WRITE(*,*) 'ERROR in makeP: min beta < 0'
        STOP 1
ELSE IF (MINVAL(Qvecdown) <0)  THEN
        WRITE(*,*) 'ERROR in makeP: min Q < 0'
        print *, Qvecdown
        stop 1  
END IF

IF (atom%NAME .EQ. ' HI  ' .AND. from .EQ. 2) THEN

   atom%A(2,1) = 8.29

   beta_e_dummy(1) = 1

ELSEIF (atom%NAME .EQ. 'HeI  ' .AND. from .EQ. 3) THEN

   atom%A(3,1) = 51.3

   beta_e_dummy(1) = 1

END IF

DO to = 1, (from-1)                                ! Bottom level can take any value from 1 to (from-1)
 
   R(to) = ne*Qvecdown(to)			        ! Collisions in the first (from-1) positions
     
   R((from-1) + to) = abs(beta_e_dummy(to))*atom%A(from,to)	! Radiative deexcitation in the latter
      
END DO

DO i = 1, Nup       ! Up collisions in the last part
   
   R(2*(from-1) + i) = ne*Qvecup(i)

END DO
   

Rtot = SUM(R)                                     ! The sum of all rates

IF (Rtot > 0) THEN

   Pvec = R/Rtot

ELSE

   WRITE(*,*) 'ERROR in makeP : no deecitation options'

   print *, atom%NAME, from, Qvecdown, ne

   stop 1

END IF

IF (atom%NAME .EQ. ' HI  ') THEN

   atom%A(2,1) = 0

ELSEIF (atom%NAME .EQ. 'HeI  ') THEN

   atom%A(3,1) = 0

END IF
   
END

