SUBROUTINE MAKEALPHA(T, alpha_tot, avrecpot_diel)
USE M1, ONLY : Nelem, myid, IDP, DOPRINT, Zvec, Nstages
USE INTERFACE, ONLY : totrecrates_gould
USE PARAMETERS, ONLY : eV
IMPLICIT NONE

DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem), INTENT(OUT) :: alpha_tot
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem), INTENT(OUT), OPTIONAL :: avrecpot_diel
! ----------------------------------------------
! TASK : Computes the btotal recombination rate for all elements for a given temperature
! Input :  T      Temperature
! Output : alpha  Array with total rec. rates for all atoms
! Globals :
! ----------------------------------------------
CHARACTER :: trash
DOUBLE PRECISION :: lambda, beta, C0, a, b, T0, T1, T2, Y, Z, si_default, Fi, psi_n, psi_n_1, psi_1, psi_2, psi_3, fv, x, psi, t4, alpha_first, alpha_ground, Adi, Bdi, alpha_diel, CC, Buse, alpha_rr, alpha_dr, alpha, alpha_heavy
INTEGER :: i, j, bin, N
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: logT, alphavec
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: DATAMAT
LOGICAL, SAVE :: FIRST=.TRUE.
LOGICAL :: GOULD
DOUBLE PRECISION, DIMENSION(100,Nstages-1) :: ALPHAMAT=0. ! TEMPFIX can it be variable? First index Z, second ion stage
!--------------------------------------------

alpha_heavy = 1E-11! CARE use as default value if not known. TEMP not well understood, energy balance problems if too small

IF (present(avrecpot_diel)) avrecpot_diel = 0

!si_default = 3E-12*(T/5000.)**(-0.5)

! ------ H I ---------------------------------------------------

lambda = 1.579E5/T

alpha = 5.197E-14*lambda**0.5*(0.4288 + 0.5*LOG(REAL(lambda))+0.469*lambda**(-0.333))  ! Rybicki Lightman 10.65a

ALPHAMAT(1,1) = alpha

! -------protons ------------------------------------------------------

ALPHAMAT(1,2) = 0

! ------- HE I ------------------------------------------------

N = 16

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT=RESHAPE([1.0,	3.432E-11,&    ! HeI Total rec rate, data from Hummer, Storey 297, 1998
&1.2,	2.616E-11,&
&1.4,	1.990E-11,&
&1.6,	1.509E-11,&
&1.8,	1.142E-11,&
&2.0,	8.613E-12,&
&2.2,	6.480E-12,&
&2.4,	4.862E-12,&
&2.6,	3.636E-12,&
&2.8,	2.711E-12,&
&3.0,	2.014E-12,&
&3.2,	1.491E-12,&
&3.4,	1.101E-12,&
&3.6,	8.096E-13,&
&3.8,	5.940E-13,&
&4.0,	4.350E-13],[2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin > 0 .AND. bin < N) THEN

   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))

   C0 = alphavec(bin)/((10**logT(bin))**beta)

   alpha = C0*T**beta

ELSE IF (T >= 10**(logT(N))) THEN

   alpha = alphavec(N)*(T/10**logT(N))**(-0.5)  ! CARE

ELSE

   alpha = alphavec(1)*(T/10**logT(1))**(-0.5)

END IF

ALPHAMAT(2,1) = alpha

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

! ----------------- HE II ---------------------------------

a = 1.891E-10
b = 0.7524
T0 = 9.37
T1 = 2.774E6

alpha = a/(SQRT(T/T0)*(1+SQRT(T/T0))**(1-b)*(1+SQRT(T/T1))**(1+b))             ! Verner Ferland 1995

ALPHAMAT(2,2) = alpha

! ------------------C I ----------------------------------

N = 31         ! NAHAR

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT = RESHAPE([1.0,	3.61E-11,&	! Total rec rate for CI...Nahar 101, 1995   31 Temperatures
&1.1,	3.15E-11,&	
&1.2,	2.75E-11,&	
&1.3,	2.40E-11,&
&1.4,	2.10E-11,&
&1.5,	1.83E-11,&
&1.6,	1.59E-11,&
&1.7,	1.39E-11,&
&1.8,	1.21E-11,&
&1.9,	1.05E-11,&
&2.0,	9.13E-12,&
&2.1,	7.94E-12,&
&2.2,	6.89E-12,&
&2.3,	5.99E-12,&
&2.4,	5.20E-12,&
&2.5,	4.51E-12,&
&2.6,	3.91E-12,&
&2.7,	3.39E-12,&
&2.8,	2.94E-12,&
&2.9,	2.55E-12,&
&3.0,	2.21E-12,&
&3.1,	1.91E-12,&
&3.2,	1.66E-12,&
&3.3,	1.44E-12,&
&3.4,	1.26E-12,&
&3.5,	1.10E-12,&
&3.6,	9.72E-13,&
&3.7,	8.67E-13,&
&3.8,	7.85E-13,&
&3.9,	7.25E-13,&
&4.0,	6.90E-13], [2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN

   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))
   C0 = alphavec(bin)/((10**logT(bin))**beta)
   alpha = C0*T**beta

END IF

ALPHAMAT(6,1) = alpha

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

! Dielectronic recombination irrelevant below ~ 20,000 K (Shull+ 1982)

! ------------------ C II -------------------------------

N = 36         ! NAHAR

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT = RESHAPE([1.0,   1.6253E-10,&  ! Nahar (see CI)
&  1.1,      1.4249E-10,&
&  1.2,      1.2475E-10,&
&  1.3,      1.0922E-10,&
&  1.4,      9.5559E-11,&
&  1.5,      8.3540E-11,&
&  1.6,      7.3017E-11,&
&  1.7,      6.3805E-11,&
&  1.8,      5.5644E-11,&
&  1.9,      4.8648E-11,&
&  2.0,      4.2401E-11,&
&  2.1,      3.6973E-11,&
&  2.2,      3.2230E-11,&
&  2.3,      2.8078E-11,&
&  2.4,      2.4454E-11,&
&  2.5,      2.1283E-11,&
&  2.6,      1.8522E-11,&
&  2.7,      1.6131E-11,&
&  2.8,      1.4102E-11,&
&  2.9,      1.2569E-11,&
&  3.0,      1.1584E-11,&
&  3.1,      1.1193E-11,&
&  3.2,      1.1209E-11,&
&  3.3,      1.1324E-11,&
&  3.4,      1.1248E-11,&
&  3.5,      1.0824E-11,&
&  3.6,      1.0050E-11,&
&  3.7,      9.0292E-12,&
&  3.8,      7.9100E-12,&
&  3.9,      6.8602E-12,&
&  4.0,      6.0166E-12,&
&  4.1,      5.4944E-12,&
&  4.2,      5.4044E-12,&
&  4.3,      6.0206E-12,&
&  4.4,      7.8648E-12,&
&  4.5,      1.1375E-11], [2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN

   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))

   C0 = alphavec(bin)/((10**logT(bin))**beta)

   alpha = C0*T**beta

END IF

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

ALPHAMAT(6,2) = alpha

! Dielectronic recombination irrelevant below ~15,000 K (Shull+ 1982)

! ----------------- N I ------------------------------------

alpha = 4.10E-13*(T/1E4)**(-0.608)                 ! Shull, van Steenberg

ALPHAMAT(7,1) = alpha

! Dielectronic rec irrelevant below ~25,000 K (Shull+ 1982)

! ----------------- N II ----------------------------------

alpha = 2.20E-12*(T/1E4)**(-0.639)                 ! Shull, van Steenberg

ALPHAMAT(7,2) = alpha

! Dielectronic rec irrelevant below ~25,000 K (Shull+ 1982)

! ------------------- O I ----------------------------------

N = 41

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT = RESHAPE([  1.0,      2.9130E-11,&   ! Nahar online
&  1.1,      2.5327E-11,&
&  1.2,      2.1962E-11,&
&  1.3,      1.9101E-11,&
&  1.4,      1.6543E-11,&
&  1.5,      1.4335E-11,&
&  1.6,      1.2413E-11,&
&  1.7,      1.0737E-11,&
&  1.8,      9.2813E-12,&
&  1.9,      8.0124E-12,&
&  2.0,      6.9126E-12,&
&  2.1,      5.9587E-12,&
&  2.2,      5.1216E-12,&
&  2.3,      4.4123E-12,&
&  2.4,      3.7851E-12,&
&  2.5,      3.2466E-12,&
&  2.6,      2.7813E-12,&
&  2.7,      2.3792E-12,&
&  2.8,      2.0325E-12,&
&  2.9,      1.7333E-12,&
&  3.0,      1.4763E-12,&
&  3.1,      1.2558E-12,&
&  3.2,      1.0655E-12,&
&  3.3,      9.0571E-13,&
&  3.4,      7.6840E-13,&
&  3.5,      6.5365E-13,&
&  3.6,      5.5829E-13,&
&  3.7,      4.7949E-13,&
&  3.8,      4.1443E-13,&
&  3.9,      3.6006E-13,&
&  4.0,      3.1404E-13,&
&  4.1,      2.7498E-13,&
&  4.2,      2.4610E-13,&
&  4.3,      2.4927E-13,&
&  4.4,      3.4472E-13,&
&  4.5,      6.2613E-13,&
&  4.6,      1.1578E-12,&
&  4.7,      1.9027E-12,&
&  4.8,      2.7154E-12,&
&  4.9,      3.4096E-12,&
&  5.0,      3.8434E-12],[2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN

   !beta = LOG10(alphavec(bin)/alphavec(bin+1))/(logT(bin)-logT(bin+1))

   !C0 = alphavec(bin)/((10**logT(bin))**beta)

   !alpha(9) = C0*T**beta

   alpha = alphavec(bin) + (alphavec(bin+1)-alphavec(bin))*(T - 10**logT(bin))/(10**logT(bin+1)-10**logT(bin))

END IF

ALPHAMAT(8,1) = alpha

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

! Dielectronic rec irrelevant below ~20,000 K (Shull+ 1982)

!- -------------------- O II --------------------------------

N = 41

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT = RESHAPE([1.0,     1.4548E-10,&  ! Nahar online
&  1.1,      1.2729E-10,&
&  1.2,      1.1120E-10,&
&  1.3,      9.7145E-11,&
&  1.4,      8.4796E-11,&
&  1.5,      7.3943E-11,&
&  1.6,      6.4460E-11,&
&  1.7,      5.6173E-11,&
&  1.8,      4.8837E-11,&
&  1.9,      4.2574E-11,&
&  2.0,      3.6980E-11,&
&  2.1,      3.2133E-11,&
&  2.2,      2.7906E-11,&
&  2.3,      2.4212E-11,&
&  2.4,      2.0994E-11,&
&  2.5,      1.8184E-11,&
&  2.6,      1.5744E-11,&
&  2.7,      1.3639E-11,&
&  2.8,      1.1823E-11,&
&  2.9,      1.0330E-11,&
&  3.0,      9.0600E-12,&
&  3.1,      8.0006E-12,&
&  3.2,      7.0893E-12,&
&  3.3,      6.2806E-12,&
&  3.4,      5.5502E-12,&
&  3.5,      4.8855E-12,&
&  3.6,      4.2864E-12,&
&  3.7,      3.7524E-12,&
&  3.8,      3.2792E-12,&
&  3.9,      2.8715E-12,&
&  4.0,      2.5156E-12,&
&  4.1,      2.2141E-12,&
&  4.2,      1.9713E-12,&
&  4.3,      1.8260E-12,&
&  4.4,      1.8999E-12,&
&  4.5,      2.4186E-12,&
&  4.6,      3.6280E-12,&
&  4.7,      5.6155E-12,&
&  4.8,      8.1722E-12,&
&  4.9,      1.0818E-11,&
&  5.0,      1.2988E-11], [2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN

   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))

   C0 = alphavec(bin)/((10**logT(bin))**beta)

   alpha = C0*T**beta

END IF

ALPHAMAT(8,2) = alpha

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

! Dielectronic rec irrelevant below ~25,000 K (Shull+ 1982)

! --------------- NE I ------------------------

alpha = 2.20E-13*(T/1E4)**(-0.759)            ! Shull, Steenberg

ALPHAMAT(10,1) = alpha

! Dielectronic rec irrelevant below ~50,000 K (Shull+ 1982)

! --------------NE II ----------------------------

alpha = 1.50E-12*(T/1E4)**(-0.693)            ! Shull, Steenberg

ALPHAMAT(10,2) = alpha

! Dielectronic rec irrelevant below ~50,000 K (Shull+ 1982)

! ----------------- NA I --------------------------------------

a = 5.641E-12                              ! SOURCE Verner Ferland 1996
b = 0.1749
T0 = 3.077E2
T1 = 2.617E6  ! DC

Y = SQRT(T/T0)
Z = SQRT(T/T1)!  DC

alpha = a*(Y*(1+Y)**(1-b)*(1+Z)**(1+b))**(-1)  ! DC

!alpha_tot(13) = 5.82E-12*(T/1E2)**(-0.682)            ! PA 1986

ALPHAMAT(11,1) = alpha

!---------- NA II ----------

alpha = 3E-12*(T/5000.)**(-0.5)    ! CARE made up value

ALPHAMAT(11,2) = alpha

! ---------------- MG I -----------------------------------

!IF (T > 1000) THEN

!   alpha(15) = 1.40E-13*(T/1E4)**(-0.855)                   ! Shull % Steenberg 1982

!ELSE

!   alpha(15) = 5.87E-12*(T/1E2)**(-0.681)                    ! Pequignot, Alvodrani 1986...161

!END IF

!GOULD = .FALSE.

!IF (GOULD) THEN

!call totrecrates_gould("MgI  ", T, alpha_tot(15))  

t4 = T/1e4

!if (myid == IDP) print *, 'alpha mgi rad, diel', alpha_tot(15),  1E-12*(1.20/t4 - 4.68 + 7.66*t4 - 0.59*t4**2)*t4**(-1.5)*exp(-1.62/t4)

!alpha_dr = T**(-1.5)*(3.87e-8*exp(-8415./T) + 4.73e-7*exp(-1.68e4/T) + 1.59e-3*exp(-5.00e4/T) + 2.63e-5*exp(-2.75e5/T))  ! Dielectronic from Altun+ 2006

!   alpha_tot(15) = alpha_tot(15) + 1E-12*(1.20/t4 - 4.68 + 7.66*t4 - 0.59*t4**2)*t4**(-1.5)*exp(-1.62/t4)  ! Dielectronic, Nussbaumer Storey 1986

!   alpha_tot(15) = alpha_tot(15) + alpha_dr 

alpha_ground =   1E-12*(0.10/t4 - 0.33 + 1.37*t4 - 0.074*t4**2)*t4**(-1.5)*exp(-1.27/t4)

alpha_first =    1E-12*(0.51/t4 - 2.89 + 7.44*t4 - 0.72*t4**2)*t4**(-1.5)*exp(-2.41/t4)

IF (present(avrecpot_diel)) avrecpot_diel(15) = (7.646*eV*alpha_ground + (7.646+2.71)*eV*alpha_first)/(alpha_ground + alpha_first)

!ELSE  ! Badnell 2006 radiative rec. and Altun 2006 dielectronic

A = 5.452E-11
B = 0.6845
T0 = 5.637
T1 = 1.551e6
CC = 0.3945
T2 = 8.360e5
Buse = B + CC*exp(-T2/T)

alpha_rr = A*( sqrt(T/T0)*(1 + sqrt(T/T0)**(1-Buse)*(1 + sqrt(T/T1)**(1+Buse))))**(-1)   ! Badnell 2006

alpha_dr = 1E-12*(1.20/t4 - 4.68 + 7.66*t4 - 0.59*t4**2)*t4**(-1.5)*exp(-1.62/t4)  ! Dielectronic, Nussbaumer Storey 1986

!alpha_dr = T**(-1.5)*(3.87e-8*exp(-8415./T) + 4.73e-7*exp(-1.68e4/T) + 1.59e-3*exp(-5.00e4/T) + 2.63e-5*exp(-2.75e5/T))  ! Dielectronic Altun 2006

alpha = alpha_rr + alpha_dr

ALPHAMAT(12,1) = alpha

!END IF

! ----------------MG II ------------------------------------

a = 1.920E-11                                 ! Verner Ferland 1996
b = 0.3028
T0 = 4.849E2
T1 = 5.890E6

Y = SQRT(T/T0)
Z = SQRT(T/T1)

alpha = a*(Y*(1+Y)**(1-b)*(1+Z)**(1+b))**(-1)

ALPHAMAT(12,2) = alpha

! Dielectronic rec irrelevant below ~50,000 K (Shull+ 1982)

! ----------------Si I --------------------------

N = 31                   ! NAHAR

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT = RESHAPE([1.0,	5.02E-11,&     ! Nahar 101, 1995
&1.1,	4.41E-11,&
&1.2,	3.87E-11,&
&1.3,	3.40E-11,&
&1.4,	2.99E-11,&
&1.5,	2.62E-11,&
&1.6,	2.30E-11,&
&1.7,	2.02E-11,&
&1.8,	1.77E-11,&
&1.9,	1.55E-11,&
&2.0,	1.36E-11,&
&2.1,	1.19E-11,&
&2.2,	1.05E-11,&
&2.3,	9.19E-12,&
&2.4,	8.06E-12,&
&2.5,	7.07E-12,&
&2.6,	6.20E-12,&
&2.7,	5.45E-12,&
&2.8,	4.79E-12,&
&2.9,	4.21E-12,&
&3.0,	3.72E-12,&
&3.1,	3.30E-12,&
&3.2,	2.94E-12,&
&3.3,	2.65E-12,&
&3.4,	2.40E-12,&
&3.5,	2.19E-12,&
&3.6,	2.01E-12,&
&3.7,	1.87E-12,&
&3.8,	1.76E-12,&
&3.9,	1.67E-12,&
&4.0,	1.60E-12], [2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN

   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))

   C0 = alphavec(bin)/((10**logT(bin))**beta)

   alpha = C0*T**beta

ELSEIF (T > 10**logT(N)) THEN

   alpha = alphavec(N)*(T/10**logT(N))**(-0.5)

END IF

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

Adi = 1.10E-3
Bdi = 0
T0 = 7.7E4
T1 = 0   ! DC

alpha = alpha + Adi*T**(-1.5)*exp(-T0/T)*(1 + Bdi*exp(-T1/T))  ! Dielectronic, Shull+ 1982,important for T >~ 8000 K.  DC

ALPHAMAT(14,1) = alpha

! --------------Si II --------------------

N = 31                       ! NAHAR

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT = RESHAPE([1.0,	1.32E-10,&   ! Nahar 101, 1995
&1.1,	1.15E-10,&
&1.2,	1.01E-10,&
&1.3,	8.77E-11,&
&1.4,	7.64E-11,&
&1.5,	6.65E-11,&
&1.6,	5.78E-11,&
&1.7,	5.02E-11,&
&1.8,	4.35E-11,&
&1.9,	3.79E-11,&
&2.0,	3.28E-11,&
&2.1,	2.84E-11,&
&2.2,	2.46E-11,&
&2.3,	2.12E-11,&
&2.4,	1.83E-11,&
&2.5,	1.58E-11,&
&2.6,	1.36E-11,&
&2.7,	1.17E-11,&
&2.8,	1.01E-11,&
&2.9,	8.68E-12,&
&3.0,	7.46E-12,&
&3.1,	6.42E-12,&
&3.2,	5.54E-12,&
&3.3,	4.79E-12,&
&3.4,	4.16E-12,&
&3.5,	3.63E-12,&
&3.6,	3.22E-12,&
&3.7,	2.94E-12,&
&3.8,	2.80E-12,&
&3.9,	2.82E-12,&
&4.0,	2.97E-12],[2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN

   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))

   C0 = alphavec(bin)/((10**logT(bin))**beta)

   alpha = C0*T**beta

END IF

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

Adi = 5.87E-3
Bdi = 7.53e-1
T0 = 9.63e4
T1 = 6.46e4  ! DC

alpha = alpha + Adi*T**(-1.5)*exp(-T0/T)*(1 + Bdi*exp(-T1/T))  ! Dielectronic, Shull+ 1982,important for T >~ 8000 K.  DC

ALPHAMAT(14,2) = alpha

! ------------ SI --------------------------

IF (T > 1000) THEN

   alpha = 4.10E-13*(T/1E4)**(-0.63)              ! Shull and Steenberg 1982

ELSE

   alpha = 1.05E-11*(T/1e2)**(-0.593)              ! Pequignot, Aldrovandi 1986

END IF

Adi = 1.62e-3
Bdi = 0
T0 = 1.25e5
T1 = 0  ! DC

alpha = alpha + Adi*T**(-1.5)*exp(-T0/T)*(1 + Bdi*exp(-T1/T))  ! Dielectronic, Shull+ 1982,important for T >~ 13000 K.  !DC

ALPHAMAT(16,1) = alpha

! ------------S II -------------------------------

N = 36             ! NAHAR ERRATUM

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT =RESHAPE([1.0,       1.4928E-10,& !  SII Nahar erratum 1996
&  1.1,       1.3062E-10,&
&  1.2,       1.1411E-10,&
&  1.3,       9.9667E-11,&
&  1.4,       8.6968E-11,&
&  1.5,       7.5800E-11,&
&  1.6,       6.6036E-11,&
&  1.7,       5.7511E-11,&
&  1.8,       4.9988E-11,&
&  1.9,       4.3595E-11,&
&  2.0,       3.7926E-11,&
&  2.1,       3.3047E-11,&
&  2.2,       2.8833E-11,&
&  2.3,       2.5217E-11,&
&  2.4,       2.2168E-11,&
&  2.5,       1.9626E-11,&
&  2.6,       1.7516E-11,&
&  2.7,       1.5717E-11,&
&  2.8,       1.4098E-11,&
&  2.9,       1.2635E-11,&
&  3.0,       1.1243E-11,&
&  3.1,       9.9649E-12,&
&  3.2,       8.8093E-12,&
&  3.3,       7.7800E-12,&
&  3.4,       6.8723E-12,&
&  3.5,       6.0730E-12,&
&  3.6,       5.3726E-12,&
&  3.7,       4.7600E-12,&
&  3.8,       4.2229E-12,&
&  3.9,       3.7630E-12,&
&  4.0,       3.3694E-12,&
&  4.1,       3.0583E-12,&
&  4.2,       2.8770E-12,&
&  4.3,       2.9894E-12,&
&  4.4,       3.8400E-12,&
&  4.5,       6.2893E-12], [2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN
   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))
   C0 = alphavec(bin)/((10**logT(bin))**beta)
   alpha = C0*T**beta
END IF

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

ALPHAMAT(16,2) = alpha

! Dielectronic only important for T > 25000 K (Shull+ 1982)

! ---------- AR I ----------------------------------

alpha = 3.77E-13*(T/1E4)**(-0.651)                ! Shull, Stenberg 1982

ALPHAMAT(18,1) = alpha

! Dielectronic rec. uniportant below ~50000 K (Shull+ 1982)

! --------- AR II --------------------------------

alpha = 1.95E-12*(T/1E4)**(-0.752)                ! Shull, Stenberg 1982

ALPHAMAT(18,2) = alpha

! Dielectronic rec. uniportant below ~50000 K (Shull+ 1982)

! ---------- K I ---------------------------

N = 11             ! Wane & Aymar 1987, radiative rec.

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT =RESHAPE([2.7,       1.938E-12,& 
&  3.0,       1.15E-12,&
&  3.3,       6.9E-13,&
&  3.48,      5.1E-13,&
&  3.60,      4.1E-13,&
&  3.70,      3.5E-13,&
&  3.78,      3.1E-13,&
&  3.85,      2.7E-13,&
&  3.90,      2.5E-13,&
&  3.95,      2.3E-13,&
&  4.0,       2.1E-13],[2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN
   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))
   C0 = alphavec(bin)/((10**logT(bin))**beta)
   alpha = C0*T**beta
END IF

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

ALPHAMAT(19,1) = alpha

! ---------- CA I --------------------------------

!alpha(23) = 1.12E-13*(T/1E4)**-0.9                 ! CARE these are very different Shull, Stenberg 1982
alpha = 5.58E-12*(T/1E2)**(-0.683)                ! Pequignot, Aldrovandi 1986...161  valid up to 1000 K

Adi = 3.28e-4
Bdi = 9.07e-2
T0 = 3.46e4
T1 = 1.64e4  ! DC

alpha_diel =  Adi*T**(-1.5)*exp(-T0/T)*(1 + Bdi*exp(-T1/T))   ! DC

alpha = alpha + alpha_diel  ! Dielectronic, Shull+ 1982,important for T >~ 3000 K.

ALPHAMAT(20,1) = alpha

! ---------- CA II -----------------------------

!alpha(24) = 6.78E-13*(T/1E4)**-0.8                  ! Shull, Stenberg 1982
alpha = 2.79E-11*(T/1E2)**(-0.647)                 ! Pequignot, Aldrovandi 1986....161 valid up to 1000 K

ALPHAMAT(20,2) = alpha

! Diel. recombination unimportant below ~50,000 K (Shull+1982)

! ------------------FE I-----------------------------------------

N = 61             ! NAHAR

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT = RESHAPE([1.0, 3.48E-11,& ! From Nahar, 122, 1997
&1.1, 3.14E-11,&
&1.2, 2.84E-11,&
&1.3, 2.58E-11,&
&1.4, 2.33E-11,&
&1.5, 2.09E-11,&
&1.6, 1.87E-11,&
&1.7, 1.67E-11,&
&1.8, 1.48E-11,&
&1.9, 1.31E-11,&
&2.0, 1.16E-11,&
&2.1, 1.02E-11,&
&2.2, 9.02E-12,&
&2.3, 7.97E-12,&
&2.4, 7.04E-12,&
&2.5, 6.27E-12,&
&2.6, 5.65E-12,&
&2.7, 5.15E-12,&
&2.8, 4.73E-12,&
&2.9, 4.34E-12,&
&3.0, 3.95E-12,&
&3.1, 3.55E-12,&
&3.2, 3.14E-12,&
&3.3, 2.75E-12,&
&3.4, 2.37E-12,&
&3.5, 2.02E-12,&
&3.6, 1.72E-12,&
&3.7, 1.47E-12,&
&3.8, 1.35E-12,&
&3.9, 1.52E-12,&
&4.0, 2.30E-12,&
&4.1, 3.89E-12,&
&4.2, 6.19E-12,&
&4.3, 8.72E-12,&
&4.4, 1.09E-11,&
&4.5, 1.21E-11,&
&4.6, 1.24E-11,&
&4.7, 1.17E-11,&
&4.8, 1.05E-11,&
&4.9, 8.95E-12,&
&5.0, 7.35E-12,&
&5.1, 5.86E-12,&
&5.2, 4.56E-12,&
&5.3, 3.48E-12,&
&5.4, 2.62E-12,&
&5.5, 1.95E-12,&
&5.6, 1.43E-12,&
&5.7, 1.05E-12,&
&5.8, 7.61E-13,&
&5.9, 5.51E-13,&
&6.0, 3.95E-13,&
&6.1, 2.83E-13,&
&6.2, 2.03E-13,&
&6.3, 1.45E-13,&
&6.4, 1.04E-13,&
&6.5, 7.38E-14,&
&6.6, 5.26E-14,&
&6.7, 3.74E-14,&
&6.8, 2.66E-14,&
&6.9, 1.90E-14,&
&7.0, 1.35E-14],[2,N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN

   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))

   C0 = alphavec(bin)/((10**logT(bin))**beta)

   alpha = C0*T**beta

END IF

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

Adi = 1.58e-3
Bdi = 4.56e-1
T0 = 6e4
T1 = 8.97e4

alpha = alpha + Adi*T**(-1.5)*exp(-T0/T)*(1 + Bdi*exp(-T1/T))  ! Dielectronic, Shull+ 1982,important for T >~ 5000 K.

ALPHAMAT(26,1) = alpha

! ---------------Fe II ------------------------------------

N = 41                 ! NAHAR

ALLOCATE(logT(N))
ALLOCATE(alphavec(N))
ALLOCATE(DATAMAT(2,N))

DATAMAT = RESHAPE([  1.0,      1.2320E-10,&  ! Nahar online
&  1.1,      1.0740E-10,&
&  1.2,      9.3445E-11,&
&  1.3,      8.1296E-11,&
&  1.4,      7.0646E-11,&
&  1.5,      6.1306E-11,&
&  1.6,      5.3169E-11,&
&  1.7,      4.6080E-11,&
&  1.8,      3.9809E-11,&
&  1.9,      3.4494E-11,&
&  2.0,      2.9744E-11,&
&  2.1,      2.5648E-11,&
&  2.2,      2.2091E-11,&
&  2.3,      1.8999E-11,&
&  2.4,      1.6327E-11,&
&  2.5,      1.4024E-11,&
&  2.6,      1.2066E-11,&
&  2.7,      1.0421E-11,&
&  2.8,      9.0320E-12,&
&  2.9,      7.9137E-12,&
&  3.0,      6.9674E-12,&
&  3.1,      6.1976E-12,&
&  3.2,      5.5704E-12,&
&  3.3,      5.0672E-12,&
&  3.4,      4.6825E-12,&
&  3.5,      4.4121E-12,&
&  3.6,      4.2517E-12,&
&  3.7,      4.1778E-12,&
&  3.8,      4.1447E-12,&
&  3.9,      4.1083E-12,&
&  4.0,      4.0159E-12,&
&  4.1,      3.8529E-12,&
&  4.2,      3.6347E-12,&
&  4.3,      3.4231E-12,&
&  4.4,      3.3123E-12,&
&  4.5,      3.3773E-12,&
&  4.6,      3.6147E-12,&
&  4.7,      3.9297E-12,&
&  4.8,      4.1834E-12,&
&  4.9,      4.2625E-12,&
&  5.0,      4.1260E-12], [2, N])

logT = DATAMAT(1,:)
alphavec = DATAMAT(2,:)

CALL findbin2(N, logT, DBLE(LOG10(T)), bin)

IF (bin .NE. 0) THEN
   beta = (LOG10(REAL(alphavec(bin)))-LOG10(REAL(alphavec(bin+1))))/(logT(bin)-logT(bin+1))
   C0 = alphavec(bin)/((10**logT(bin))**beta)
   alpha = C0*T**beta
ELSEIF (T < 10.) THEN
   alpha = alphavec(1)
ELSEIF (T > 1E5) THEN
   alpha = alphavec(N)
END IF

DEALLOCATE(logT)
DEALLOCATE(alphavec)
DEALLOCATE(DATAMAT)

ALPHAMAT(26,2) = alpha

! Dielectronic unimportant under 20,000 K (Shull+ 1982)

!  Fe III  TEMPfix

if (Nstages == 4) ALPHAMAT(26,Nstages-1) = 1E-11 

if (Nstages == 5) then
        
	ALPHAMAT(26,Nstages-2) = 1E-11
  	ALPHAMAT(26,Nstages-1) = 1E-11 

end if

! -------------- CO I ----------------------

ALPHAMAT(27,1) = ALPHAMAT(26,1)            ! CARE put cobalt I equal to FeI

! -------------- CO II ----------------------

ALPHAMAT(27,2) = ALPHAMAT(26,2)             ! CARE put cobalt II equal to FeII

! -------------NI I --------------------

alpha = 3.60E-13*(T/1E4)**(-0.7)             ! Shull, Steenberg 1982
!alpha(29) = 5.56E-12*(T/1E2)**(-0.681)            ! Pequignot, Aldrovandi 1986

Adi = 1.41E-3
Bdi = 4.69e-1
T0 = 9.82e4
T1 = 1.01e5  ! DC

alpha = alpha + Adi*T**(-1.5)*exp(-T0/T)*(1 + Bdi*exp(-T1/T))  ! Dielectronic, Shull+ 1982,important for T >~ 11000 K.

ALPHAMAT(28,1) = alpha

! ----------- NI II -----------------------

alpha = 1.00E-12*(T/1E4)**(-0.7)              ! Shull, Steenberg 1982

ALPHAMAT(28,2) = alpha

! Diel. unimportant under ~25,000 K (Shull +1982)

! ----------- TI I --------------------------

ALPHAMAT(22,1) = 5.50E-12*(T/1E2)**(-0.683)            ! PA 1986  valid up to 1000 K

! --------- TI II --------------------------

ALPHAMAT(22,2) = ALPHAMAT(26,2)                        ! CARE put equal to FeII

!------AL I ---------------------------------

ALPHAMAT(13,1) = 1.54E-11*(T/1E2)**(-0.567)            ! PA 1986 valid up to 1000 K

! ---- AL II --------------------------------

ALPHAMAT(13,2) = 0

! ----------- CR I -------------------------

ALPHAMAT(24,1) = ALPHAMAT(26,1)              ! CARE put equal to FeI

! ----------- CR II ------------------------

ALPHAMAT(24,2) = ALPHAMAT(26,2)              ! CARE put equal to FeII

! --------------- MN I ---------------------

ALPHAMAT(25,1) = 5.45E-12*(T/1E2)**(-0.686)            ! PA 1986 valid up to 1000 K

!--------------- MN II ---------------------

ALPHAMAT(25,2) = ALPHAMAT(26,2)             ! CARE put equal to FeII

! ----------- SC I --------------------------

ALPHAMAT(21,1) = ALPHAMAT(26,1) 

!  SC II ------------------------------------

ALPHAMAT(21,2) = ALPHAMAT(26,2)

! ---- V I ----------------------------------

ALPHAMAT(23,1) = ALPHAMAT(26,1)                  ! CARE put equal to FeI rate

! ---- V II ------------------------------

ALPHAMAT(23,2) = ALPHAMAT(26,2)                 ! CARE put equal to FeII

! Fill array ===============================================

DO i = 1, Nelem

   DO j = 1, Nstages-1

   	alpha = ALPHAMAT(Zvec(i),j) ! Get

   	IF (alpha == 0.) THEN

      	if (Zvec(i) <= 30) THEN

        	alpha = ALPHAMAT(26,j) ! Put equal to Fe value if no entry

        	if (DOPRINT) print *, 'WARNING, no rec. rate for element with atomic number',Zvec(i),'ion',j,'Putting equal to the corresponding iron ion rate.'

      	else

        	alpha = alpha_heavy

      	end if

   END IF

   alpha_tot((Nstages-1)*(i-1)+j) = alpha ! Allocate

   END DO

END DO

END
