SUBROUTINE MAKEATOMS(N_atoms, atoms, T_typical, maxN)
USE M1, ONLY : atomm, myid, globid,  Nelem,Nmol, map2, map3, massvec, Zvec, ievec, ngvec, minshellvec, maxshellvec, map_abin2elem2, map_abin2elem3, Nabin, lambdaion, ind_iron, ind_cobalt, ind_nickel, ind_titanium, ind_calcium, gval, Nstages, DOPRINT
USE INTERFACE, ONLY : alloc
USE PARAMETERS, ONLY : eV, h, c
USE MPI
IMPLICIT NONE
!INCLUDE 'mpif.h'

INTEGER, INTENT(IN) :: N_atoms
TYPE(atomm), DIMENSION(N_atoms), INTENT(INOUT) :: atoms
DOUBLE PRECISION, INTENT(INOUT) :: T_typical
INTEGER, INTENT(OUT) :: maxN

INTEGER :: i, j, ierr, ind1, ind2, ind3, ionstage, ionstage_last, Nuse
CHARACTER(len=5) :: elname

ind1 = 0 ! 1*Nelem list
ind2 = 0 ! 2*Nelem list (or, generically, (Nstages-1)*Nelem list)
ind3 = 0 ! 3*Nelem list (or, generically, Nstages*Nelem list)
ionstage_last = Nstages-1 ! Important, must be set to 2, or generically, Nstages-1

DO i = 1, N_atoms   ! Allocate atoms and atomic data

    if (atoms(i)%species_type .ne. 'mol') then

    	atoms(i)%species_type = 'atm'

    	atoms(i)%mbin = 0

   end if

   	CALL alloc(atoms(i))   ! Allocates basic parameters to the atoms..need to change here when adding atoms

   	elname = atoms(i)%NAME

   	IF (myid .EQ. 0) CALL readlevelsandlines(atoms(i), T_typical)  ! Master read levels and lines

    !Leave marked out, useful sometimes
    !IF (myid == 0 .AND. atoms(i)%NAME == 'FeIII') THEN

    	!call fosc(5,[1,2,3,4,5],595,tovec,atoms(i), fosctot)

    	!stop

   	!END IF

   CALL MPI_BCAST(atoms(i)%LAMBDA, atoms(i)%N*atoms(i)%N, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)   ! send to slaves

   CALL MPI_BCAST(atoms(i)%A, atoms(i)%N*atoms(i)%N, MPI_REAL, 0, MPI_COMM_WORLD, ierr)

   CALL MPI_BCAST(atoms(i)%LEVELS_E, atoms(i)%N, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

   call MPI_BCAST(atoms(i)%LEVELS_G, atoms(i)%N, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

   call MPI_BCAST(atoms(i)%RR_ALPHA0, atoms(i)%N_LEVELS, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

   call MPI_BCAST(atoms(i)%RR_T0, atoms(i)%N_LEVELS, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

   call MPI_BCAST(atoms(i)%RR_BETA, atoms(i)%N_LEVELS, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

   call MPI_BCAST(atoms(i)%UPSILON0, atoms(i)%N*atoms(i)%N, MPI_REAL, 0, MPI_COMM_WORLD, ierr)

   call MPI_BCAST(atoms(i)%UPSILON0_T0, atoms(i)%N*atoms(i)%N, MPI_REAL, 0, MPI_COMM_WORLD, ierr)

   call MPI_BCAST(atoms(i)%UPSILON0_BETA, atoms(i)%N*atoms(i)%N, MPI_REAL, 0, MPI_COMM_WORLD, ierr)

   IF (atoms(i)%N > maxN) maxN = atoms(i)%N

   if (elname(3:5) == 'I  ') then ! This atom/NLTE molecule is neutral

      ionstage = 0

      ind1 = ind1 + 1

      atoms(i)%DENSITY_ID = ind1

      if (ionstage_last == Nstages-1) then ! Last atom was highest ion, jump 1 step in 2*Nelem list

      	ind2 = ind2 + 1

		ind3 = ind3 + 1

      else ! Last atom was also neutral : need to jump 2 steps in 2*Nelem list

         ind2 = ind2 + Nstages - 1 - ionstage_last

         ind3 = ind3 + Nstages - ionstage_last

      end if

   elseif ((elname(3:5) == 'II ') .or. (elname(3:3) == '2')) then ! This atom is singly ionized...

      atoms(i)%DENSITY_ID = ind1 ! same as neutral

      ionstage = 1

      ind2 = ind2 + 1   ! Assume any lower ionizaton states are always present

      ind3 = ind3 + 1   ! Assume any lower ionizaton states are always present

   elseif (((elname(3:5)) == 'III' .or. (elname(3:3) == '3'))) then ! This atom is doubly ionized...

      atoms(i)%DENSITY_ID = ind1 ! same as neutral

      ionstage = 2

      if (Nstages > 3) then ! Dont increase this index if its the last ion (true if Nstages = 3)

	   ind2 = ind2 + 1

      end if

      ind3 = ind3 + 1  ! Assume any lower ionizaton states are always present

   elseif (((elname(3:5) == 'IV ') .or. (elname(3:3) == '4'))) then ! This atom is triply ionized

     atoms(i)%DENSITY_ID = ind1

     ionstage = 3

     if (Nstages > 4) then ! Dont increase this index if its the last ion (true if Nstages = 4)

       ind2 = ind2 + 1

     end if

     ind3 = ind3 + 1

   elseif (((elname(3:5) == 'V  ') .or. (elname(3:3) == '5'))) then ! This atom is quadrupply ionized

     atoms(i)%DENSITY_ID = ind1

     ionstage = 4

     ! ind2 not increased as we have ceiling at quadruply ionized

     ind3 = ind3 + 1

   end if

   ionstage_last = ionstage

   if (globid == 0 .AND. DOPRINT) print *, 'atom i ind1 ind2 ind3', atoms(i)%NAME, i, ind1, ind2, ind3, atoms(i)%DENSITY_ID

   if (ionstage == 0) then ! Only need to fill this once per element (take neutral)

      massvec(ind1) = atoms(i)%MASS  ! MASS is DP

      Zvec(ind1) = atoms(i)%Z  ! Z is INT

      if (Zvec(ind1) == 26) then

         ind_iron = ind1

      elseif (Zvec(ind1) == 27) then

         ind_cobalt = ind1

      elseif (Zvec(ind1) == 28) then

         ind_nickel = ind1

      elseif (Zvec(ind1) == 20) then

         ind_calcium = ind1

      elseif (Zvec(ind1) == 22) then

         ind_titanium = ind1

      end if

      map2(ind2) = atoms(i)%ABIN

      map3(ind3) = atoms(i)%ABIN

      if (elname == ' HI  ') then ! Special treatment for hydrogen: 1) Its ion (HII) is not at ABIN+1 but at position 70  2) THere is no HIII ion --> put 0 in map3

         map2(ind2+1) = 70

         map3(ind3+1) = 70

         DO j = 2, Nstages-1

			map3(ind3+j) = 0

         END DO

         !map3(ind3+2) = 0

      else

		Nuse = Nstages-2

        DO j = 1, Nuse

        	if (j < atoms(i)%Z .or. atoms(i)%species_type == 'mol') then  ! e.g. He has Z=2, can be ionized at most j=1 times

         		map2(ind2+j) = atoms(i)%ABIN + j

           	end if

       	END DO

		DO j = 1, Nuse + 1

        	if (j < atoms(i)%Z .or. atoms(i)%species_type == 'mol') then

            	map3(ind3+j) = atoms(i)%ABIN + j

            end if

    	END DO

        if (elname == 'HeI  ') then ! Special treatment for helium 1) HeIII is missing in ABIN list --> put zero in map3

			DO j = 2, Nstages-1

            	map3(ind3+j) = 0

            END DO

        end if

      end if

   end if

   Nuse = Nstages-2

   if (ionstage <= Nuse) then ! Dont fill 2*Nelem arrays if double ionized..

      ngvec(ind2) = atoms(i)%NGROUND  ! NGROUND is INT. Used in alfa_photoionization. Ok to let 0 for missing singly

      minshellvec(ind2) = atoms(i)%MINSHELL ! MINSHELL is int. Used in alfa_photoionation. Ok to let 0 (checks made in alfa_photoionization)

      maxshellvec(ind2) = atoms(i)%MAXSHELL ! MAXSHELL is int. = " =

      ievec(ind2) = atoms(i)%E_ION_GS/eV ! Used in chargetransfer.f90. CARE Needs to be set also for all singly ionized.

      lambdaion(ind2) = h*c/(atoms(i)%E_ION_GS)*1e8

      !ievec(ind2+1) = atoms(i)%NEXT_E_ION_GS  !TEMPFIX need or not?

      map_abin2elem2(atoms(i)%ABIN) = ind2  ! Prefill higher ions if they dont exist
   end if

   map_abin2elem3(atoms(i)%ABIN) = ind3

   if (elname == ' HI  ') then

        map_abin2elem2(70) = 2

        map_abin2elem3(70) = 2

   elseif (elname == 'HeI  ') then

		map_abin2elem2(atoms(i)%ABIN+1) = ind3 + 1

   elseif (ionstage == 0) then ! From C and up 5 slots available Prefill higher ion slots in case they are missing

     DO j = 1, Nuse

        map_abin2elem2(atoms(i)%ABIN + j) = ind2 + j ! Prefill higher ions if they dont exist

     END DO

     DO j = 1, Nuse + 1

     	map_abin2elem3(atoms(i)%ABIN+j) = ind3 + j

     END DO

   end if

END DO

! Add the molecule mapping
do i = 1, Nmol

    map2((Nstages-1)*Nelem + i) = Nabin + i

    map3(Nstages*Nelem + i) = Nabin + i

    map_abin2elem2(Nabin + i) = (Nstages-1)*Nelem + i

    map_abin2elem3(Nabin + i) = Nstages*Nelem + i

end do

if (globid == 0) then

   open(10,file='out/makeatoms/map2.txt',STATUS='REPLACE')
   open(11,file='out/makeatoms/map3.txt',STATUS='REPLACE')
   open(12,file='out/makeatoms/abinmaps.txt',STATUS='REPLACE')
   open(13,file='out/makeatoms/Zvec.txt',STATUS='REPLACE')
   !open(13,file='out/makeatoms/ngvec.txt',STATUS='REPLACE')

   do i = 1, (Nstages-1)*Nelem + Nmol

      if (i <= (Nstages-1)*Nelem) then

      	write(10,*) i, map2(i), ngvec(i), ievec(i)

      else

      	write(10,*) i, map2(i), ngvec(i), 0.

      end if

   end do

   do i = 1, Nstages*Nelem + Nmol

      write(11,*) i, map3(i)

   end do

   do i = 1, Nabin + Nmol

      write(12,*), i, map_abin2elem2(i), map_abin2elem3(i)

   end do

   do i = 1, Nelem

      write(13,*) i, Zvec(i)

   end do

   close(10)
   close(11)
   close(12)
   close(13)

end if

! Set ground state multiplet statistical weights used in chargetransfer.f90

gval = &
[2, &  ! H 1
1, 2, & ! He 2-3
9,6,1,0,0, & ! C 4-8
4,9,6,0,0, & ! N 9-13
9,4,9,0,0, & ! O 14-18
1,4,9,0,0, & ! Ne 19-23
2,1,4,0,0, & ! Na 24-28
1,2,1,0,0, & ! Mg 29-33
9,6,1,0,0, & ! Si 34-38
9,4,9,0,0, & ! S 39-43
1,4,5,0,0, & ! Ar 44-48
1,2,3,0,0, & ! Ca 49-53
25,30,25,0,0,0, & ! Fe 54-59
28,21,28,0,0, & ! Co 60-64
21,10,21,0,0, & ! Ni 65-69
1, &               ! HII
21,28,21,0,0, & ! Ti 71-75
6,1,2,0,0, & ! Al 76-80
7,6,25,0,0, & ! Cr 81-85
6,1,6,0,0, & ! Mn 86-90
10,15,10,0,0, & ! Sc 91-95
28,25,28,0,9, & ! V 96-100
2,1,6,0,0, &  ! K 101-105
9,8,9,6,0, &  ! Ce 106-110
5,4,1,0,0, &  ! Te 111-115
1,6,9,10,0, & ! Pt 116-120
9,8,9,0,0,  & ! & ! Th 121-125
1, 1,0,0,0, & ! Mol1
1, 1,0,0,0, & ! Mol2
1, 1,0,0,0, & ! Mol3
1, 1,0,0,0]   ! Mol4

END SUBROUTINE MAKEATOMS
