#! /bin/bash
np=$2
# Description ============================================
# Used to process a set of models as specified by a file provided as command-line argument no 2 (no 3 is how many CPU cores to use)
# e.g. ./master.sh myrunfile 256      Process myrunfile, use 256 cores.
# where myrunfile has entries like
# 001_300 n 0         Model 001_300, do not use previous runs as start point.
# 001_600 y 001_300   Model 001_600, use model 001_300 as start point.
# 002_300 y 0         Model 002_300, use whatever model is in working directory as start point.
# #002_600 y 0        Skip this model (by # sign)
#
# Each model here needs to have a runfile, i.e. ip001_300, ip001_600, ip002_300
# ==========================================================
# --------------------
let wlstart1="400"  # Initial iterations, low MC statistics ("first/second iterate")
let wlend1="25000"
wlres1="1.0"  # Resolution (number of A at 1000 A). R = 1000/wlres
let Ntot1="10000000" # Base number to calcule number of MC packets per bin from
let Nmin1="1" # Minimum number to send per shell and wavelength bin
let NminUV1="1" # Minimum number to send per shell and wavelength bin, UV
let Nmax1="10" # Maximum number to send per shell and wavelength bin
let nitermax1="30" # Max number of iterations allowed at this stage
# --------------------
let wlstart2="400"   # Higher MC statistics ("third iterate")
let wlend2="25000"
wlres2="1.0"
let Ntot2="100000000"
let Nmin2="3"
let NminUV2="3"
let Nmax2="30"
let nitermax2="5" 
# --------------------
let wlstart3="400"  # Highest-MC statistics ("fourth iterate", only for convergence problems)
let wlend3="25000"
wlres3="1.0"
let Ntot3="100000000"
let Nmin3="10"
let NminUV3="10"
let Nmax3="100"
let nitermax3="3" 
# -------------------------
let j="0"
echo "Logfile" > logfile
echo "Summary" > summary
let nsuccess="0"
declare -a array
declare -a iterarray
declare -a arrayget
while read -r idrun iter idget || [[ -n "$idrun" ]]; do
	if [ "${idrun:0:1}" = "#" ]; then
		echo "Marked out" $idrun >> logfile
	else
		let j+=1
        	array[$j]=$idrun
		iterarray[$j]=$iter
                arrayget[$j]=$idget
		echo "Reading model name", ${array[$j]} >> logfile
	fi
done < "$1"

echo $j "models to run" >> logfile

for ((i=1; i<=$j; i++)); do
        idrun=${array[$i]}
        idrunget=${arrayget[$i]}
	echo "==========================" >> logfile
        echo "Model $i, $idrun" >> logfile
	STARTTIME=$(date +%s)
	RUNFILE="ip$idrun"
	if [ ! -f $RUNFILE ]; then
	    echo "ERROR Runfile not found. Breaking." >> logfile
	    echo "$idrun 1 Runfile not found" >> summary	
	    continue
	else
	    echo "Found file $RUNFILE" >> logfile
	fi
	nrplus=$((`grep -h Nr $RUNFILE | awk '{print $2}'`+1))
	if [ $? = 0 ]; then
	    echo "Read number of shells $(($nrplus-1))" >> logfile
	else
	    echo "ERROR in reading number of shells. Aborting." >> logfile
	    echo "$idrun 1 Nr read error" >> summary
	    continue
	fi
    	if [ $idrunget = "0" ]; then
           echo "Not reading from saved.." >> logfile
        else
           echo "Doing getfiles, using run" $idrunget >> logfile
           ./getfiles.sh $idrunget $(($nrplus-1))
        fi
	if [ $np -lt $nrplus ]; then
	    echo "ERROR, too few cores. Breaking" >> logfile
	    echo "$idrun 1 Too few cores" >> summary
	    continue
	fi
	echo "Doing first iterate.sh ..." >> logfile
	./iterate.sh 1 $nrplus $np $idrun $RUNFILE y ${iterarray[$i]} $wlstart1 $wlend1 $wlres1 $Ntot1 $Nmin1 $NminUV1 $Nmax1 #  Single iteration with gamma deposition 
	if [ $? = 0 ]; then
	   echo "First iterate.sh OK." >> logfile
	else
	   echo "ERROR first suse. Breaking." >> logfile
	   echo "$idrun 1 First suse" >> summary
	   continue
	fi
	echo "Doing second iterate.sh..." >> logfile
	./iterate.sh $nitermax1 $nrplus $np $idrun $RUNFILE n y $wlstart1 $wlend1 $wlres1 $Ntot1 $Nmin1 $NminUV1 $Nmax1 # Low res iterations
        outsignal=$?
        echo "Outsignal second iterate.sh $outsignal" >> logfile
	if [ $outsignal = 0 ]; then
	    echo "Second iterate.sh converged." >> logfile
	elif [ $outsignal = 2 ]; then
	    echo "Second iterate.sh, exceeding niter.." >> logfile
	elif [ $outsignal = 3 ]; then
	    echo "Solution looks divergent..restarting.." >> logfile
	    ./iterate.sh 1 $nrplus $np $idrun $RUNFILE n n $wlstart1 $wlend1 $wlres1 $Ntot1 $Nmin1 $NminUV1 $Nmax1
	    ./iterate.sh $nitermax1 $nrplus $np $idrun $RUNFILE n y $wlstart1 $wlend1 $wlres1 $Ntot1 $Nmin1 $NminUV1 $Nmax1 
	    outsignalnew=$?
	    if [ $outsignalnew = 0 ]; then
		echo "Second iterate.sh converged." >> logfile
	    elif [ $outsignalnew = 2 ]; then
		echo "Second iterate.sh, exceeding niter.." >> logfile
	    else
		echo "ERROR second iterate.sh. Breaking." >> logfile	    
		ENDTIME=$(date +%s)
		minutes=$(bc -l <<< "scale=2; $[$ENDTIME-$STARTTIME]/60")
		echo "$idrun 1 second iterate.sh $minutes min" >> summary
		continue
	    fi
	else
	    echo "ERROR second iterate.sh. Breaking." >> logfile	    
            ENDTIME=$(date +%s)
            minutes=$(bc -l <<< "scale=2; $[$ENDTIME-$STARTTIME]/60")
	    echo "$idrun 1 second iterate.sh $minutes min" >> summary
	    continue
	fi
	echo "Doing third iterate.sh ..." >> logfile
	./iterate.sh $nitermax2 $nrplus $np $idrun $RUNFILE n y $wlstart2 $wlend2 $wlres2 $Ntot2 $Nmin2 $NminUV2 $Nmax2 # High res. iterations
        outsignal=$?
        if [ $outsignal = 2 ]; then
	    echo "Apa.." >> logfile
	    read -r convstat eb < out/process/eb.dat
	    tol=0.15 # Energy discrepancy tolerance
	    c1=$(bc -l <<< $eb'>'1.-$tol)
	    c2=$(bc -l <<< $eb'<'1.+$tol)
	    if [ $c1 = 1 ] && [ $c2 = 1 ]; then
		echo "Within 15%, trying a few more with very large MC count.." >> logfile
		./iterate.sh $nitermax3 $nrplus $np $idrun $RUNFILE n y $wlstart3 $wlend3 $wlres3 $Ntot3 $Nmin3 $NminUV3 $Nmax3
		outsignal=$?
	    else
		echo "Outside 15% error, giving up to improve...$eb $c1 $c2" >> logfile
	    fi
	fi
        echo "Out signal third $outsignal" >> logfile
	if [ $outsignal = 0 ]; then
	    echo "Final iterate.sh successful." >> logfile
	    ./process_special $idrun $RUNFILE       # Write line fluxes to file
	    echo "Doing putfiles..." >> logfile
	    ./putfiles.sh $idrun $((nrplus-1)) $RUNFILE
	    ./saveT $((nrplus-1))
            ENDTIME=$(date +%s)
	    minutes=$(bc -l <<< "scale=2; $[$ENDTIME-$STARTTIME]/60")
	    echo "$idrun 0 $minutes min" >> summary
	    let nsuccess+=1
        elif [ $outsignal = 2 ]; then
	    echo "Third iterate.sh, exceeed niter.." >> logfile
	    read -r convstat eb < out/process/eb.dat
	    ENDTIME=$(date +%s)
	    minutes=$(bc -l <<< "scale=2; $[$ENDTIME-$STARTTIME]/60")
	    echo "$idrun 2 Exceed niter, $eb $minutes" >> summary
	    echo "Doing putfiles..." >> logfile
	    ./putfiles.sh $idrun $((nrplus-1)) $RUNFILE
            ./saveT $((nrplus-1))
	else
	    echo "ERROR third iterate.sh" >> logfile
	    echo "$idrun 1 third iterate.sh" >> summary
	fi
done 
echo "Nmodels",$j, "computed, of which ", $nsuccess, "successful." >> logfile
echo "All done, TERMINATE." >> logfile
#echo "SUMO master.sh done" | mail -s "SUMO master.sh done" anders@mpa-garching.mpg.de
