SUBROUTINE modelreader(vterm, N_atoms, atoms, file_explmodel, FORCECAII, DIMS, x_MAT, ION_STRUCTURE, ABIN_MAT, Dustzonemax)
USE M1, ONLY : Nr, Nabin, Nmol, myid, map2, map_abin2elem2, R_index_str, atomm, taulife_44Ti, taulife_56Ni, taulife_56Co, taulife_57Co, taulife_57Ni, Ncore, ITERATIVERUN, M_56Co, M_44Ti, M_57Co, M_56Ni, v_out, Nelem, time, gridtype, dustedge, massvec, Zvec, ind_iron, ind_nickel, ind_cobalt, ind_titanium, ind_calcium, Nstages, Nmol_NLTE, BLEND, globid
!USE INTERFACE
USE PARAMETERS, ONLY : Msun, mp, pi
USE FILES
IMPLICIT NONE
DOUBLE PRECISION, POINTER, INTENT(OUT) :: vterm
INTEGER, INTENT(IN) :: N_atoms, DIMS, Dustzonemax
TYPE(atomm), DIMENSION(N_atoms), INTENT(INOUT) :: atoms
DOUBLE PRECISION, DIMENSION(:, :), INTENT(OUT) :: x_MAT  ! Nr*DIMS
DOUBLE PRECISION, DIMENSION(:, :), INTENT(OUT) :: ION_STRUCTURE, ABIN_MAT  ! Nr*Nbin
LOGICAL, INTENT(IN) :: FORCECAII
CHARACTER(len=100), INTENT(IN) :: file_explmodel
LOGICAL :: EXISTS

! =====================================
! Task:   Read explosion model. If this is an iterative run, also read ionization structure and temperature for each zone.
! Input:  N_atoms          Number of atoms used
!         atoms            Array with atoms (of type ATOMM(M1))
!         file_explmodel   Name of explosion model to use FOR FORMAT SEE THE READ LINES
!         DIMS             Length of vector with data for each zone (density etc)
!         FORCECAII        Boolean, should CaIII ions in the H zones be forced to be CaII?
!         BLEND            Logical specifying if core should be mixed in velocity
!         ITERATIVERUN     Logical specifying if this is an iteration
! Output: x_MAT            Matrix with data for each zone (Nr, DIMS)
!         ION_STRUCTURE    Matrix with ionization fractions for each zone (Nr, Nabin)
!         ABIN_MAT         Matrix with number fractions in each zone (Nr, Nabin)
! GLOBALS USED: time       Time [s] (should be set)
!               Nr         Number of zones (sbs)
!               Nabin      Number of ions (sbs)
!              v_out       set here
!              dustedge    set here
!              M_56Ni      set here
!              M_56Co      set here
!              M_57Co      set here
!              M_44Ti      set here
! ====================================

INTEGER, PARAMETER :: Nionlevs=5  ! TEMPFIX           
INTEGER :: i, j, ind, pos, ii, bin, shell, Zval, indi1        
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: radius_in, radius_out, velocity_in, velocity_out, shell_mass, shell_totalnumberdensity, Amean, Zmean, volume, thickness_cm, T, xe, radius_norm_in, radius_norm_out, ff, atomdensityvec, mass56Ni0_shells, mass57Ni0_shells, mass44Ti0_shells, totmass, mass56Ni_frac,  mass57Ni_frac, mass44Ti_frac   ! radius in cm, velocity in km/s
INTEGER, DIMENSION(:), ALLOCATABLE :: shelltype
TYPE(atomm) :: atom
CHARACTER(len=1) :: trash
DOUBLE PRECISION :: vcore, time_days, Ntot, n, sumsmall, Ekin, norm, vin, thomsonaccum, temp1
DOUBLE PRECISION, DIMENSION(:, :), ALLOCATABLE :: NMAT, NFMAT, MASSFRACS, NMOLMAT 
DOUBLE PRECISION, DIMENSION(:, :, :), ALLOCATABLE :: ionization_fraction, mol_numdens    
DOUBLE PRECISION, DIMENSION(Nelem) :: mass, masssynt, massprim
CHARACTER(Len=50) :: infile, outfile

! -------------------------------------------------------------------------------------------------

allocate(mol_numdens(Nr,Nmol, 5)); mol_numdens = 0 ! CARE the 5

ALLOCATE(radius_in(Nr)); radius_in = 0
ALLOCATE(radius_out(Nr)); radius_out = 0
ALLOCATE(velocity_in(Nr)); velocity_in = 0
ALLOCATE(velocity_out(Nr)); velocity_out = 0
ALLOCATE(shell_mass(Nr)); shell_mass = 0
ALLOCATE(shell_totalnumberdensity(Nr)); shell_totalnumberdensity = 0
ALLOCATE(Amean(Nr)); Amean = 0
ALLOCATE(Zmean(Nr)); Zmean = 0
ALLOCATE(volume(Nr)); volume = 0
ALLOCATE(thickness_cm(Nr)); thickness_cm = 0
ALLOCATE(shelltype(Nr)); shelltype = 0
ALLOCATE(T(Nr)); T =0
ALLOCATE(xe(Nr)); xe=0
ALLOCATE(radius_norm_in(Nr)); radius_norm_in = 0
ALLOCATE(radius_norm_out(Nr)); radius_norm_out = 0
ALLOCATE(ff(Nr)); ff =1.
ALLOCATE(atomdensityvec(Nr)); atomdensityvec=0
ALLOCATE(mass56Ni0_shells(Nr)); mass56Ni0_shells = 0
ALLOCATE(mass57Ni0_shells(Nr)); mass57Ni0_shells = 0
ALLOCATE(mass44Ti0_shells(Nr)); mass44Ti0_shells = 0
ALLOCATE(NMAT(Nr, Nelem)); NMAT=0
ALLOCATE(NFMAT(Nr, Nelem)); NFMAT=0
ALLOCATE(ionization_fraction(Nr,Nelem,Nionlevs)); ionization_fraction=0
ALLOCATE(MASSFRACS(110,Nr)); MASSFRACS=0 ! First index is atomic number
ALLOCATE(totmass(Nr)); totmass=0
ALLOCATE(mass56Ni_frac(Nr)); mass56Ni_frac=0
ALLOCATE(mass57Ni_frac(Nr)); mass57Ni_frac=0
ALLOCATE(mass44Ti_frac(Nr)); mass44Ti_frac=0
ALLOCATE(NMOLMAT(Nr,maxval([1,Nmol]))); NMOLMAT=0

ABIN_MAT = 0

ION_STRUCTURE = 0

ionization_fraction = 0 ! Imp (initialised above already??)

mass = 0

masssynt = 0

massprim = 0

x_MAT = 0

NMOLMAT = 0

time_days = time/(3600.*24.)

OPEN(10,file=TRIM(file_timedays),STATUS='REPLACE'); WRITE(10,*) time_days; close(10) !removed NINT type for time_days to allow decimal values

! READ EXPLOSION MODEL  ---------------------------------------------

OPEN(10, file = file_explmodel, STATUS='OLD', ACTION='READ')

     READ(10,*) trash

     READ(10,*) trash, totmass(1:Nr)    ! Total zone mass in Msun

     READ(10,*) trash, velocity_in(1:Nr)

     read(10,*) trash, velocity_out(1:Nr)

     read(10,*) trash, ff(1:Nr)

     read(10,*) trash

     READ(10,*) trash

     READ(10,*) trash, mass56Ni_frac(1:Nr)   ! These are the mass fractions of 56Ni + 56Co at time = 0

     READ(10,*) trash, mass57Ni_frac(1:Nr)   ! 57Ni + 57Co

     READ(10,*) trash, mass44Ti_frac(1:Nr)   ! 44Ti

     READ(10,*) trash

     DO

        READ(10,*,END=99) Zval, MASSFRACS(Zval,1:Nr)

     END DO

99 CLOSE(10)

if (ff(1) == 1d0) THEN

   BLEND = .FALSE.

ELSE

   BLEND = .TRUE.

end if

vin = velocity_in(1)

IF (BLEND) THEN

   vcore = velocity_out(Ncore)

   velocity_in(1:Ncore) = vin

   velocity_out(1:Ncore) = vcore

END IF

vterm = velocity_out(Nr)

! Find total masses for each element

IF (globid == 0) THEN
    print *, "Sum of mass fractions in each zone of explosion model:"

    DO i = 1, Nr
       PRINT '(A10, I3, F10.2)', 'Sumfracs: ', i, SUM(MASSFRACS(:,i)) + mass56Ni_frac(i) + mass57Ni_frac(i) + mass44Ti_frac(i)
       !norm =  SUM(MASSFRACS(:,i)) + mass56Ni_frac(i) + mass57Ni_frac(i) + mass44Ti_frac(i)
       !MASSFRACS(:,i) = MASSFRACS(:,i)/norm   ! Normalize mass fractions to unity
       !mass56Ni_frac(i) = mass56Ni_frac(i)/norm
       !mass57Ni_frac(i) = mass57Ni_frac(i)/norm
       !mass44Ti_frac(i) = mass44Ti_frac(i)/norm
    END DO
ENDIF

shell_mass = totmass  ! In Msun

radius_in = velocity_in*time*1e5 ! Assume homology

radius_out = velocity_out*time*1e5

thickness_cm = radius_out - radius_in

IF (BLEND) THEN

   volume = 4*pi/3.*(radius_out**3 - radius_in**3)*ff

ELSE

   volume = 4*pi/3.*(radius_out**3 - radius_in**3)

END IF

mass56Ni0_shells = mass56Ni_frac*totmass

mass57Ni0_shells = mass57Ni_frac*totmass

mass44Ti0_shells = mass44Ti_frac*totmass

v_out = velocity_out(Nr)*1e5      ! v_out in cm/s

IF (Dustzonemax > 0) THEN

   dustedge = velocity_out(Dustzonemax)/velocity_out(Nr)*v_out*time  ! CARE

ELSE

   dustedge = 0.

END IF

DO i = 1, Nr

   DO j = 1, Nelem-Nmol_NLTE

      NMAT(i,j) = shell_mass(i)*MASSFRACS(Zvec(j),i)*Msun/(massvec(j)*mp)  ! Total number of particles of this type

   END DO

   ! Decays from Co56 and Co57 adds to iron:

   NMAT(i,ind_iron) = NMAT(i,ind_iron) + &
                mass56Ni0_shells(i)*(1.-EXP(-time/taulife_56Ni)-(EXP(-time/taulife_56Co)-EXP(-time/taulife_56Ni))/(1-taulife_56Ni/taulife_56Co))*Msun/(massvec(ind_iron)*mp) + & 
                mass57Ni0_shells(i)*(1.-EXP(-time/taulife_57Ni)-(EXP(-time/taulife_57Co)-EXP(-time/taulife_57Ni))/(1-taulife_57Ni/taulife_57Co))*Msun/(massvec(ind_iron)*mp)

   ! Add non-decayed Co56 and Ni56 to the stable Co and Ni:

   NMAT(i,ind_cobalt) = NMAT(i,ind_cobalt) + & 
	        mass56Ni0_shells(i)*(EXP(-time/taulife_56Co) - EXP(-time/taulife_56Ni))/(1.-taulife_56Ni/taulife_56Co)*Msun/(massvec(ind_cobalt)*mp) + & 
                mass57Ni0_shells(i)*(EXP(-time/taulife_57Co) - EXP(-time/taulife_57Ni))/(1.-taulife_57Ni/taulife_57Co)*Msun/(massvec(ind_cobalt)*mp) ! Axelrod1980 eq 3.1b

   NMAT(i,ind_nickel) = NMAT(i,ind_nickel) + &
		mass56Ni0_shells(i)*exp(-time/taulife_56Ni)*Msun/(massvec(ind_nickel)*mp) + & 
                mass57Ni0_shells(i)*exp(-time/taulife_57Ni)*Msun/(massvec(ind_nickel)*mp)

   ! Add decayed Ti44 to stable calcium: (the intermediate Scandium step is very short)

   NMAT(i,ind_calcium) = NMAT(i,ind_calcium) + mass44Ti0_shells(i)*(1.-EXP(-time/taulife_44Ti))*Msun/(44*mp)

   ! Add non-decayed Titanium to the total Ti:

   NMAT(i,ind_titanium) = NMAT(i,ind_titanium) + mass44Ti0_shells(i)*EXP(-time/taulife_44Ti)*Msun/(44*mp)

   Ntot = SUM(NMAT(i,:))   ! Total number density in this shell

   IF (Ntot > 0) THEN

      NFMAT(i,:) = NMAT(i,:)/Ntot             ! Create number fractions

   ELSE

      NFMAT(i,:) = 0

   END IF

END DO

! READ IONIZATION STRUCTURE -----------------------------------------------

IF (ITERATIVERUN .EQV. .FALSE.) THEN   ! If this is NOT an iterative run

   T = 3000.

   ionization_fraction = 0 ! Initialize ionization_fraction matrix

   ionization_fraction(:,:,1) = 0.90 ! Neutrals

   ionization_fraction(:,:,2) = 0.09 ! Single ionised

   ionization_fraction(:,:,3) = 0.01 ! Doubly ionised

   !OPEN(40, file=file_ionandTstart, STATUS='OLD', ACTION='READ')

   !DO i = 1, Ncore+2   ! CARE

   !   READ(40,*) trash, Tzones(i)

   !   DO j = 1, 15     ! There is currently a starting guess only for first 15 elements

   !      READ(40,*) trash, ionization_fraction(i,j,1:3) ! CARE 3 or 4

   !   END DO

   !   DO j = 16, Nelem

    !     ionization_fraction(i,j,1:3) = ionization_fraction(i,13,1:3)  ! Put ionization of all elements above 15 equal to iron's

   !   END DO

   !END DO

   !CLOSE(40)

   !DO i = 1, Nr

   !   T(i) = Tzones(shelltype(i))

   !END DO

   !DO i = Ncore+3, Nr  ! CARE

   !   DO j = 1, Nelem

   !      ionization_fraction(i,j,1:3) = ionization_fraction(Ncore+2,j,1:3)  !CARE

   !   END DO

   !END DO

   !print *, 'Temperatures', T


ELSE           ! If ITERATIVERUN is TRUE

   DO i = 1, Nr
!-------------------------------------------------------------
! sl: adding special case for the molecules, imports mass
! from the molres file_explmodel
        OPEN(10,file=TRIM(file_molres)//R_index_str(i), STATUS='OLD', ACTION='READ')

        READ(10,*) trash

        do j=1, Nelem*Nstages 

            READ(10,*) trash

        end do

        do j=1, Nmol

            READ(10,*) mol_numdens(i,j, :)

        end do

        close(10)
!-------------------------------------------------------------
      inquire(file=TRIM(file_ionfracs)//R_index_str(i),exist=EXISTS)

      IF (EXISTS) THEN

      	OPEN(10,file=TRIM(file_ionfracs)//R_index_str(i), STATUS='OLD', ACTION='READ')

      	READ(10,*) trash

      	DO j = 1, Nelem

         	READ(10,*) trash, ionization_fraction(i,j,1:Nstages) 

      	END DO

        close(10)

      ELSE ! I.e. iterative run but no previous solution (when would this happen?)

		DO j = 1, Nelem

			ionization_fraction(i,j,1:2) = [0.99,0.01] 

        END DO

      END IF

      IF (FORCECAII .and. (i .EQ. 7 .OR. i >= 6)) THEN  ! TEMP CARE Force CaIII to be CaII in the H zones

         ionization_fraction(i,12,2) =  SUM(ionization_fraction(i,12,2:3))

         ionization_fraction(i,12,3) = 0 ! Remove CaIII

         IF (myid .eq. 1) PRINT *, 'Changed CaIII to CaII', ionization_fraction(i,12,2)

      END IF

      OPEN(75, file=TRIM(file_temps)//R_index_str(i), STATUS='OLD', ACTION='READ')

        READ(75,*) T(i)

      close(75)

   END DO

   WHERE (T .EQ. 0)  ! If I want to compute a new shell that has no previous solution

      T = 5000.   ! CARE

   END WHERE

END IF

DO i = 1, Nr  ! Load ABIN mat which is used for sf.(Spencer-Fano routine)

   ABIN_MAT(i,1) = NFMAT(i,1)*ionization_fraction(i,1,1)  ! HI

   ABIN_MAT(i,70) = NFMAT(i,1)*ionization_fraction(i,1,2) ! protons

   ABIN_MAT(i,2) = NFMAT(i,2)*ionization_fraction(i,2,1)  ! HeI

   ABIN_MAT(i,3) = NFMAT(i,2)*ionization_fraction(i,2,2)  ! HeII

   DO j = 3, Nelem  ! All others have 5 ionization stages

      pos = 5*j - 11 ! ABIN number for ion states I - III TEMP a hardcoding here that the Nelem array comes in same order as in ABIN...and no elements can be omitted.

      IF (j .GE. 14) pos = pos + 1   ! Fe has 6 levels, pos should be 1 higher above 14

      IF (j .GE. 16) pos = pos + 1   ! Skip over H II which is assigned before

	  if (j >= Nelem - Nmol_NLTE + 1) then  ! An NLTE molecule

	  	pos = 126 + 5*(j - (Nelem - Nmol_NLTE + 1))

		ABIN_MAT(i,126) = mol_numdens(i,1,2)/sum(NMAT(i,:))*volume(i) ! TEMP hardcodings

		ABIN_MAT(i,127) = mol_numdens(i,2,2)/sum(NMAT(i,:))*volume(i)

		ABIN_MAT(i,131) = mol_numdens(i,3,2)/sum(NMAT(i,:))*volume(i)

		ABIN_MAT(i,132) = mol_numdens(i,4,2)/sum(NMAT(i,:))*volume(i)

		ABIN_MAT(i,136) = mol_numdens(i,17,2)/sum(NMAT(i,:))*volume(i)

		ABIN_MAT(i,137) = mol_numdens(i,18,2)/sum(NMAT(i,:))*volume(i)

		ABIN_MAT(i,141) = mol_numdens(i,19,2)/sum(NMAT(i,:))*volume(i) 

		ABIN_MAT(i,142) = mol_numdens(i,20,2)/sum(NMAT(i,:))*volume(i)

      else

      	ABIN_MAT(i,pos:(pos+(Nstages-1))) = NFMAT(i,j)*ionization_fraction(i,j,1:Nstages)   

      end if


   END DO

END DO

radius_norm_in = radius_in(1:Nr)/radius_out(Nr)

radius_norm_out = radius_out(1:Nr)/radius_out(Nr)

DO i = 1, Nr

   Amean(i) = DOT_PRODUCT(NFMAT(i,1:Nelem), massvec)   ! Mean atomic mass [mp]

   Zmean(i) = DOT_PRODUCT(NFMAT(i,1:Nelem), Zvec)

   IF (volume(i) > 0 .AND. Amean(i) > 0) THEN

      shell_totalnumberdensity(i) = shell_mass(i)*Msun/(Amean(i)*mp*volume(i))

   ELSE

      shell_totalnumberdensity(i) = 0

   END IF

   xe(i) = 0.

   DO j = 2, Nstages

	  xe(i) = xe(i) + (j-1)*DOT_PRODUCT(NFMAT(i,1:Nelem),ionization_fraction(i,1:Nelem,j))

   END DO

   !xe(i) = DOT_PRODUCT(NFMAT(i,1:Nelem),ionization_fraction(i,1:Nelem,2)) &                 ! Electron fraction
   !                   + 2*DOT_PRODUCT(NFMAT(i,1:Nelem),ionization_fraction(i,1:Nelem,3)) & 
! sl: this is only for when the code contains more than 3 ionization levels
   !                   + 3*DOT_PRODUCT(NFMAT(i,1:Nelem),ionization_fraction(i,1:Nelem,4)) &
   !                   + 4*DOT_PRODUCT(NFMAT(i,1:Nelem),ionization_fraction(i,1:Nelem,5)) 

   ION_STRUCTURE(i,:) = ABIN_MAT(i,:)*shell_totalnumberdensity(i)   ! Number density for each ion
    
END DO

OPEN(8,file=file_numberfractionsused,STATUS='REPLACE')

do i = 1, N_atoms   
    atom = atoms(i)
    if ((atom%species_type .eq. 'mol') .and. (atom%name(3:5) == 'I  ')) THEN

		NFMAT(:,(map_abin2elem2(atom%abin)+1)/2) = SUM(mol_numdens(:,atom%MBIN:atom%MBIN+1, 2)) / shell_totalnumberdensity

	end if
end do

DO i = 1, Nr

	WRITE(8,'(I6,ES10.3, 100ES10.2)') i, shell_totalnumberdensity(i), NFMAT(i,1:Nelem) 

END DO

close(8)

DO i = 1, N_atoms

   atom = atoms(i)

!-------------------------------------------------------------
! sl: temp stuff for the molecule mass to be added
! adding special case for the currently included molecules
   IF (atom%species_type .eq. 'mol') THEN

        x_MAT(:,atom%POS) = mol_numdens(:,atom%MBIN, 2)

    ELSE ! An atom

        atomdensityvec = shell_totalnumberdensity*NFMAT(:,atom%DENSITY_ID)

        DO j = 0, 1  ! Always add the next higher ionization state, which is overwritten if that atom is actually computed (eg Fe II adds also Fe III, but if Fe III is also computed that position is overwritten with Fe III, and Fe IV is then also included. New ions jump 2 positions in indexnuclei.

            x_MAT (:,atom%POS+j) = atomdensityvec*ionization_fraction(:, atom%DENSITY_ID, atom%CHARGE + j + 1)

        END DO
    END IF

END DO


!-------------------------------------------------------------
! sl: temp stuff for the molecule mass to be added
! CO: 1
! SiO: 3
! H2: 5
!-------------------------------------------------------------
!print*, '----'
!print*, mol_numdens(3, 1:2)
!print*, mol_numdens(17, 1:2)
!print*, mol_numdens(3, 2) + mol_numdens(17,2)
!print*, '----'

!atom = atoms(49)
!print*, 'is this right?', atom%name, x_MAT(:,atom%POS), mol_numdens(1, 2)


!atom = atoms(50)
!print*, 'is this right?', atom%name, x_MAT(:,atom%POS), mol_numdens(3, 2)


!x_MAT(:,DIMS-15) = mol_numdens(5, 2)  ! H2
!x_MAT(:,DIMS-14) = mol_numdens(3, 2)  ! SiO
!x_MAT(:,DIMS-13) = mol_numdens(1, 2)  ! CO
!-------------------------------------------------------

x_MAT(:,DIMS-13) = mass56Ni0_shells*EXP(-time/taulife_56Ni)   ! Mass of 56Ni at current time  
x_MAT(:,DIMS-12) = ff ! Filling factors
x_MAT(:,DIMS-11) = radius_norm_in(1:Nr)
x_MAT(:,DIMS-10) = radius_norm_out(1:Nr)
x_MAT(:,DIMS-9) = thickness_cm ! Shell thickness
x_MAT(:,DIMS-8) = volume ! Shell volume
x_MAT(:,DIMS-7) = mass56Ni0_shells*(EXP(-time/taulife_56Co) - EXP(-time/taulife_56Ni))/(1.-taulife_56Ni/taulife_56Co) ! Mass of 56Co at this time. First 8 day decay 56Ni --> 56Co  (mass56Ni is actuall mass of 56Ni). Source : Axelrod1980 Eq 3.1b
x_MAT(:,DIMS-6) = mass57Ni0_shells*(EXP(-time/taulife_57Co) - EXP(-time/taulife_57Ni))/(1.-taulife_57Ni/taulife_57Co) ! Mass of 57Co at this time
x_MAT(:,DIMS-5) = mass44Ti0_shells*EXP(-time/taulife_44Ti) ! Mass of 44Ti at this time
x_MAT(:,DIMS-4) = Amean
x_MAT(:,DIMS-3) = shell_totalnumberdensity
x_MAT(:,DIMS-2) = xe
x_MAT(:,DIMS-1) = Zmean
x_MAT(:,DIMS) = T

M_56Ni = SUM(x_MAT(:,DIMS-13)) ! Nr array with all current 56Ni masses
M_56Co = SUM(x_MAT(:,DIMS-7))
M_57Co = SUM(x_MAT(:,DIMS-6))
M_44Ti = SUM(x_MAT(:,DIMS-5))

IF (myid == 0) THEN  ! CARE changed from 1 to 0 does it matter?
   IF (globid == 0) THEN  ! ONLY global master write to the logfile
       OPEN (UNIT=20, FILE=file_log, STATUS='OLD', POSITION='APPEND')

       WRITE(20,*) 'Mass 56Ni (initial - current) :', sum(mass56Ni0_shells), M_56Ni
       WRITE(20,*) 'Mass 56Co (initial - current) :', 0., M_56Co
       WRITE(20,*) 'Mass 57Co (initial - current) :', sum(mass57Ni0_shells), M_57Co
       WRITE(20,*) 'Mass 44Ti (initial - current) :', sum(mass44Ti0_shells), M_44Ti

       print *, 'Mass 56Ni (initial - current) :', sum(mass56Ni0_shells), M_56Ni
       print *, 'Mass 56Co (initial - current) :', 0., M_56Co
       print *, 'Mass 57Co (initial - current) :', sum(mass57Ni0_shells), M_57Co
       print *, 'Mass 44Ti (initial - current) :', sum(mass44Ti0_shells), M_44Ti

       WRITE(20,*) 'Filling factors:', ff

       WRITE(20,*) 'Number densities:', shell_totalnumberdensity

       WRITE(20,*) 'Temperatures:', T

       CLOSE(20)
   ENDIF

   OPEN( 7, file=file_tempsused, STATUS='REPLACE')
   OPEN(22, file=file_modelsummary, STATUS='REPLACE')
   OPEN(40, file=file_masses, STATUS='REPLACE')
   OPEN(25, file=file_volume, STATUS='REPLACE')
   OPEN(50, file=file_rads,STATUS='REPLACE')
   OPEN(300,file=file_ionstructure,STATUS='REPLACE')
   OPEN (UNIT=41, FILE=file_log, STATUS='OLD', POSITION='APPEND')
   OPEN(55, file=file_thomson,STATUS='REPLACE')
   OPEN(56, file='out/modelreader/ekin.dat',STATUS='REPLACE')

   WRITE(41,*) 'MASSES:'

   WRITE(50,*) Nr

   write(22,'(2A4, 4A12, 100A10)') 'i','Type','radius_in','volume','ff','vel_in','vel_out','Temp', 'mass','n.dens','density','Amean','Zmean','Zb/Ab'

   thomsonaccum = 0.

   DO i = 1, Nr

      thomsonaccum = thomsonaccum + x_MAT(i,DIMS-3)*x_MAT(i,DIMS-2)*thickness_cm(i)*ff(i)*6.65e-25

      write(55,*) i, x_MAT(i,DIMS-3)*x_MAT(i,DIMS-2)*thickness_cm(i)*ff(i)*6.65e-25, thomsonaccum

      WRITE(7,*) i, T(i)

      WRITE(50,*) radius_out(i)/radius_out(Nr)

      DO j = 1, (Nstages-1)*Nelem

         if (map2(j) > 0) WRITE(300,*) i, j, ION_STRUCTURE(i,map2(j))
         
      END DO

      DO j = 1, Nmol

         WRITE(300,*) i, (Nstages-1)*Nelem+j, 0.0 !NMOLMAT(i,j)
!       FIXME weird bug when writing the molecule masses, unclear why, might be because radmaker 
!       gives a non-zero value to their gamma?

      END DO

      WRITE(22,'(2I4, 5ES12.4, 12ES10.2)') i, shelltype(i), radius_in(i), volume(i), ff(i), velocity_in(i), velocity_out(i), T(i), shell_mass(i),shell_totalnumberdensity(i), shell_totalnumberdensity(i)*Amean(i)*mp, Amean(i), Zmean(i), Zmean(i)/Amean(i)

      WRITE(25,'(I6, 100ES10.3)') i, x_MAT(i,DIMS-8)

      DO j = 1, Nelem

         mass(j) = mass(j) + shell_totalnumberdensity(i)*volume(i)*mp/2E33*NFMAT(i,j)*massvec(j)

         IF (i <= 5) THEN

            masssynt(j) = masssynt(j) + shell_totalnumberdensity(i)*volume(i)*mp/2E33*NFMAT(i,j)*massvec(j)

         ELSE

            massprim(j) = massprim(j) +  shell_totalnumberdensity(i)*volume(i)*mp/2E33*NFMAT(i,j)*massvec(j)

         END IF

      END DO

   END DO

   DO i = 1, Nelem

      WRITE(40,'(I8,3ES10.2)') i, mass(i), masssynt(i), massprim(i)
      WRITE(41,'(I8,3ES10.2)') i, mass(i), masssynt(i), massprim(i)

   END DO

   write(40,*) 'Total', sum(mass)

   WRITE(41,*) '    Total :', sum(mass)

   CLOSE(7)
   CLOSE(40)
   CLOSE(25)
   CLOSE(50)
   CLOSE(300)
   CLOSE(41)
   CLOSE(55)

END IF

Ekin = 0

DO i = 1, Ncore  ! Does not enter if Ncore = 0

   Ekin = Ekin + ff(i)*4*pi/10.*time**3*shell_totalnumberdensity(i)*Amean(i)*mp*(velocity_out(i)**5 - velocity_in(i)**5)*(1e5)**5   ! Compute total kinetic energy

END DO

DO i = Ncore + 1, Nr

    Ekin = Ekin + 4*pi/10.*time**3*shell_totalnumberdensity(i)*Amean(i)*mp*(velocity_out(i)**5 - velocity_in(i)**5)*(1e5)**5

END DO

if (globid == 0) then
    print *, 'Ekin (macro-mixed if on)', Ekin
endif

write(56,*) 'Ekin (macro-mixed if on)', Ekin

close(22)
close(56)

END
