SUBROUTINE netheating(R_index, heatrate, N_atoms, atoms, EXCmax, EXC_MATRIX, ne, DIMS, x, T, IONRATES_COLLION, nh, cool_co, cool_sio, fv1_co, fv1_sio, PRINTT)
USE M1, ONLY : map2, R_index_str, atomm, Nr, MOLECULESWITCH, time, Nelem, ITERATIVERUN, Nstages, Nabin, Npis
USE PARAMETERS, ONLY : k, eV
USE FILES
USE INTERFACE, ONLY : createQ, makealpha, molcooling
IMPLICIT NONE
DOUBLE PRECISION, INTENT(IN) :: heatrate
INTEGER, INTENT(IN) :: R_index, N_atoms, EXCmax, DIMS
TYPE(atomm), DIMENSION(N_atoms), INTENT(IN) :: atoms
DOUBLE PRECISION, DIMENSION(N_atoms, EXCmax), INTENT(IN) :: EXC_MATRIX
DOUBLE PRECISION, INTENT(IN) :: ne
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, INTENT(IN) :: T
REAL, DIMENSION(Nabin,Npis), INTENT(IN) :: IONRATES_COLLION
DOUBLE PRECISION, INTENT(OUT) :: nh, cool_co, cool_sio, fv1_co, fv1_sio
LOGICAL, INTENT(IN) :: PRINTT
! ============================================
! Task: Computes the net heating rate (heating-cooling)
! Input :  R_inpendex         Zone index
!          heatrate        Heating rate (erg s^-1 cm^-3). Includes all heating mechanisms except Penning ionization heating, which is done here.
!          N_atoms         Number of atoms
!          EXCmax          Maximum number of levels solved for
!          EXC_MATRIX      Array with atomic level population solutions [cm^-3]
!          ne              Electron number density [cm^-3]
!          DIMS            Length of state vector x
!          x               Zone state vector
!          T               Temperature [K]
! Output : nh              Net heating [K s^-1]
!          cool_co         CO cooling [erg s^-1 cm^-3]
!          cool_sio        SiO cooling [erg s^-1 cm^-3]
!          fv1_co
!          fv1_sio
! Globals :
! ============================================
INTEGER :: N_LEVELS, i, j, m, sel, NlevO
DOUBLE PRECISION :: heatingrate, f, f2, T2, fprim, convlim, dT, lambda, rec_cooling, dielec_heating, brems_cooling, gf, coolthis, heatthis, netcoolthis, nplus, cooling, cool_co_cf, cool_sio_cf, COv1, COv2, SiOv1, SiOv2, biggestyet, biggestyet2, biggestyet3, bestlambda, bestlambda2, bestlambda3, zonebest, zonebestlambda, cool7155, cool7300, cool6300, nco_temp, nsio_temp, heatrate_penning, adiabaticcooling, ntot, collion_cooling
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: Q
LOGICAL :: CONVERGED
TYPE(atomm) :: atom
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem) :: alpha_tot
DOUBLE PRECISION, DIMENSION(N_atoms) :: coolingvec, maincooler, maincooler2, maincooler3, mainflux, mainflux2, mainflux3, maini, mainj
CHARACTER :: trash
LOGICAL, SAVE :: FIRST=.TRUE.
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: n_co, n_sio
CHARACTER(len=5) :: zonebestname
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: COOLO
! ---------------------------------------------------------------------

ALLOCATE(n_co(Nr)); n_co=0
ALLOCATE(n_sio(Nr)); n_sio=0

CONVERGED = .FALSE.

gf = 1.3  ! CARE Bremsstrahlung gaunt factor.  Taken from deKool 1998 page 863

convlim = 1E-10  ! CARE

rec_cooling = 0  ! IMP

brems_cooling = 0  ! IMP

cool_CO_CF = 0

cool_SiO_CF = 0

cool_co = 0

cool_sio = 0

cov1 = 0

cov2 = 0

siov1 = 0

siov2 = 0

fv1_co = 0

fv1_sio = 0

maincooler = 0  ! Imp

maincooler2 = 0

maincooler3 = 0

coolingvec = 0  ! Imp

mainflux = 0

dielec_heating = 0  ! Important

mainflux2 = 0

mainflux3 = 0

zonebest = 0

maini = 0

mainj = 0

cool7155 = 0 ! Imp

cool7300 = 0 ! Imp

cool6300 = 0 ! Imp

collion_cooling = 0.

! ---------------------------------------------------------------------

DO m = 1, N_atoms

   atom = atoms(m)

   ALLOCATE(Q(atom%N,atom%N)); Q = 0

   CALL createQ(T, atom, atom%N, Q, ne)    ! Get collision rates [cm^3 s^-1]

   coolthis = 0  ! Imp do not remove

   heatthis = 0   ! VERY Imp do not remove

   biggestyet = 0  ! Important

   biggestyet2 = 0

   biggestyet3 = 0

   bestlambda = 0

   bestlambda2 = 0

   bestlambda3 = 0

   !if (atom%NAME == ' OI  ') THEN

!	NlevO = atom%N_LEVELS

!	ALLOCATE(COOLO(NlevO,NlevO))

!	COOLO = 0.

 !  end if

   DO i = 1, atom%N_LEVELS-1   ! For each lower level i to all levels above it

      DO j = (i+1), atom%N_LEVELS  ! Only allow collisions to states that also emit, otherwise energy gets lost

        coolthis = EXC_MATRIX(m,i)*ne*Q(i,j)*(atom%LEVELS_E(j)-atom%LEVELS_E(i))   ! Upwards collisions --> cooling

        heatthis = EXC_MATRIX(m,j)*ne*Q(j,i)*(atom%LEVELS_E(j)-atom%LEVELS_E(i))  ! Downward collisions --> heating

        netcoolthis = coolthis - heatthis   ! Net cooling by this transition

        !IF (DOPRINT .and. (R_index == IDP)) print *, 'Cool', i, j, netcoolthis

        !if (atom%NAME == ' OI  ') COOLO(i,j) = netcoolthis

        IF (netcoolthis > biggestyet) THEN  ! Sorting..

           biggestyet3 = biggestyet2 ! Move 2d to 3d

           bestlambda3 = bestlambda2

           maincooler3(m) = maincooler2(m)

           mainflux3(m) = mainflux2(m)

           biggestyet2 = biggestyet  ! Move 1st to 2d

           bestlambda2 = bestlambda

           maincooler2(m) = maincooler(m)

           mainflux2(m) = mainflux(m)

           biggestyet = netcoolthis  ! Replace 1st

           bestlambda = atom%LAMBDA(j,i)

           maincooler(m) = bestlambda

           mainflux(m) = netcoolthis

           maini(m) = i

           mainj(m) = j


        ELSE IF (netcoolthis > biggestyet2) THEN

           biggestyet3 = biggestyet2  ! Move 2d to 3d

           bestlambda3 = bestlambda2

           maincooler3(m) = maincooler2(m)

           mainflux3(m) = mainflux2(m)

           biggestyet2 = netcoolthis  ! replace 2nd

           bestlambda2 = atom%LAMBDA(j,i)

           maincooler2(m) = bestlambda2

           mainflux2(m) = netcoolthis

        ELSE IF (netcoolthis > biggestyet3) THEN

           biggestyet3 = netcoolthis    ! replace 3d

           bestlambda3 = atom%LAMBDA(j,i)

           maincooler3(m) = bestlambda3

           mainflux3(m) = netcoolthis


        END IF

        IF (netcoolthis > zonebest) THEN  ! Best cooler in zone

           zonebest = netcoolthis

           zonebestlambda = atom%LAMBDA(j,i)

           zonebestname = atom%NAME

        END IF

        IF (atom%NAME .EQ. 'FeII ' .AND. (j == 17)) THEN

           cool7155 = cool7155 + netcoolthis*0.75*(atom%LEVELS_E(17)-atom%LEVELS_E(6))/atom%LEVELS_E(17)

        ELSE IF (atom%NAME .EQ. 'FeII ' .AND. (j == 18)) THEN

           cool7155 = cool7155 + netcoolthis*0.5*(atom%LEVELS_E(18)-atom%LEVELS_E(7))/atom%LEVELS_E(18)

        END IF

        IF (atom%NAME .EQ. 'CaII ' .and. ((j == 2) .or. (j == 3))) THEN

           cool7300 = cool7300 + netcoolthis

        ELSE IF (atom%NAME .EQ. 'CaII ' .AND. ((j==4) .OR. (j==5))) THEN

           cool7300 = cool7300 + netcoolthis*3968./7300.

        END IF

        IF (atom%NAME .EQ. ' OI  ' .and. (j == 4)) THEN

           cool6300 = cool6300 + netcoolthis

        END IF


        coolingvec(m) = coolingvec(m) + netcoolthis

      !upthis = upthis + EXC_MATRIX(m,i)*ne*DOT_PRODUCT(Q(i,(i+1):atom%N), atom%levels_E(i+1:atom%N) - atom%levels_E(i))

     END DO

 	if (i <= Npis) then

        collion_cooling = collion_cooling + EXC_MATRIX(m,i)*ne*IONRATES_COLLION(atom%ABIN,i)*(atom%E_ION_GS - atom%LEVELS_E(i))

    end if

  END DO

  DEALLOCATE(Q)

END DO

!--- RECOMBINATION AND FREE-FREE COOLING -----------------------------

CALL makealpha(T, alpha_tot)  ! Get alphas (total rec. rates) for all neutral and singly ionized elements

! Compute recombination and bremsstrahlung cooling:

DO i = 1, N_atoms

   atom = atoms(i)

   IF (atom%CHARGE >= (Nstages-1)) CYCLE  ! Exclude highest ions (I have zero abundances for the higher ions necessary)

   nplus = x(atom%POS+1)  ! If atom is max ionized, nplus will be zero

   DO j = 1, (Nstages-1)*Nelem
      
      IF (map2(j) .EQ. atom%ABIN) THEN

         sel = j

         EXIT

      END IF

   END DO

   rec_cooling = rec_cooling + nplus*ne*alpha_tot(sel)*0.8*k*T

   brems_cooling = brems_cooling + 1.42E-27*(atom%CHARGE+1)**2*SQRT(T)*ne*nplus*gf   ! Osterbrock sec 3.4

END DO

!IF (myid == IDP .AND. DOPRINT) THEN

!   PRINT *, 'T:', T

!   PRINT *, 'Heating:', heatrate

!   PRINT * ,'Penning heating', 7.5E-10*SQRT(T/300.)*EXC_MATRIX(1,1)*EXC_MATRIX(2,2)*6.2*eV

!   PRINT *, 'Recombination cooling:', rec_cooling

!   PRINT *, 'Bremsstrahlung cooling:', brems_cooling

!   PRINT *, 'Collisionsal cooling:', SUM(coolingvec)

!END IF

IF (MOLECULESWITCH == 2 .AND. ITERATIVERUN .AND. FIRST) THEN   ! Molecular cooling

  open(10,file=TRIM(file_moldens)//R_index_str(R_index),STATUS='OLD')

  DO

    READ(10,*,END=55) j, nco_temp, nsio_temp

    IF (j <= Nr) THEN

	n_co(j) = nco_temp

	n_sio(j) = nsio_temp

    END IF

  END DO

  55 close(10)

  FIRST = .FALSE.

END IF

IF (MOLECULESWITCH == 1) THEN

   IF (R_index == 3) THEN  

      cool_sio = heatrate  - (sum(coolingvec) + rec_cooling + brems_cooling)  ! CARE Put all missing cooling in SiO

   ELSEIF (R_index == 5) THEN

      cool_co =  heatrate  - (sum(coolingvec) + rec_cooling + brems_cooling)  ! CARE Put all missing cooling in CO

   END IF

END IF


!IF (n_CO(R_index) > 0) THEN

!   CALL molcoolCF(1, T, ne, n_CO(R_index), time, cool_CO_CF, COv1, COv2)

!   fv1_co = COv1/cool_CO_CF

!END IF

!IF (n_SiO(R_index) > 0) THEN

!   CALL molcoolCF(2, T, ne, n_SiO(R_index), time, cool_SiO_CF, SiOv1, SiOv2)

!   fv1_sio = SiOv1/cool_SiO_CF

!END IF

!cool_CO = cool_CO_CF  ! CARE

!cool_SiO = cool_Sio_CF ! CARE

!IF (myid == IDP .AND. DOPRINT) PRINT *, R_index, 'Cooling CO:', cool_co, cool_CO_CF, fv1_co

!IF (myid == IDP .AND. DOPRINT) PRINT *, R_index, 'Cooling SiO:', cool_sio, cool_SiO_CF, fv1_sio



cooling = sum(coolingvec) + rec_cooling + brems_cooling + collion_cooling + cool_co + cool_sio   ! Total cooling in this zone

IF (atoms(1)%NAME == ' HI  ' .AND. atoms(2)%NAME == 'HeI  ') THEN

   heatrate_penning = 7.5E-10*SQRT(T/300.)*EXC_MATRIX(1,1)*EXC_MATRIX(2,2)*6.22*eV    ! [erg s^-1 cm^-3]

ELSE

   heatrate_penning = 0

   print *, 'ERROR in penning heating (netheating.f90)'

END IF

ntot = x(DIMS-3)

!Convert nh to K/s
nh = (heatrate - cooling + heatrate_penning)/(3./2.*k*ntot)

!IF (myid == IDP .AND. DOPRINT) PRINT *, 'Total cooling:', cooling

IF (PRINTT) THEN

   OPEN(10,file='out/netheating/cooling.dat'//R_index_str(R_index),STATUS='REPLACE')

   write(10,'(F10.1, A12, ES12.2, A60, 7ES13.4)') T, 'Cooling', cooling, 'Fracs : Lines   Rec  Free-free   Collion  CO   SiO', sum(coolingvec)/cooling, rec_cooling/cooling, brems_cooling/cooling, collion_cooling/cooling, cool_co/cooling, cool_sio/cooling

   DO i = 1, N_atoms

      WRITE(10,'(A15, A10, ES12.2, F12.4, 3ES20.2, 5ES12.2)'), 'Coolin el', atoms(i)%NAME, coolingvec(i), coolingvec(i)/abs(cooling), maincooler(i), maincooler2(i), maincooler3(i), mainflux(i)/abs(cooling), mainflux2(i)/abs(cooling), mainflux3(i)/abs(cooling), maini(i), mainj(i)

   END DO

   WRITE(10,*) 'BEST COOLER:', zonebestname, zonebestlambda

   WRITE(10,*) '7155 cooling', cool7155/abs(cooling)

   WRITE(10,*) '7300 doublet cooling', cool7300/abs(cooling)

   WRITE(10,*) '6300 doublet cooling', cool6300/abs(cooling)

   adiabaticcooling = 3./2.*x(DIMS-3)*(1.+x(DIMS-2))*k*2*T/time  ! 3/2n(1+x2)k*2T/t  KF 98

   write(10,*) 'Adiabatic cooling', adiabaticcooling/abs(cooling)

   !DO i = 1, NlevO

   !	DO j = 1, NlevO

!	   write(10,*) i, j, COOLO(i,j)/abs(cooling)

 !       END DO

  ! END DO

   close(10)

ELSE

   OPEN(10,file='out/netheating/cooling.dat'//R_index_str(R_index)//'h',STATUS='REPLACE')

   write(10,'(F10.0, A12, ES12.2, A50, 7ES13.4)') T, 'Cooling', cooling, 'Fracs : Lines   Rec  Free-free Collion CO   SiO', sum(coolingvec)/cooling, rec_cooling/abs(cooling), brems_cooling/abs(cooling), collion_cooling/abs(cooling), cool_co/abs(cooling), cool_sio/abs(cooling)

   DO i = 1, N_atoms

      WRITE(10,'(A15, A10, ES12.2, F12.4, 3ES12.2, 5ES12.2)'), 'Coolin el', atoms(i)%NAME, coolingvec(i), coolingvec(i)/abs(cooling), maincooler(i), maincooler2(i), maincooler3(i), mainflux(i)/abs(cooling), mainflux2(i)/abs(cooling), mainflux3(i)/abs(cooling), maini(i), mainj(i)

   END DO

   WRITE(10,*) 'BEST COOLER:', zonebestname, zonebestlambda

   WRITE(10,*) '7155 cooling', cool7155/abs(cooling)

   WRITE(10,*) '7300 doublet cooling', cool7300/abs(cooling)

   WRITE(10,*) '6300 doublet cooling', cool6300/abs(cooling)

   adiabaticcooling = 3./2.*x(DIMS-3)*(1.+x(DIMS-2))*k*2*T/time  ! 3/2n(1+x2)k*2T/t  KF 98

   write(10,*) 'Adiabatic cooling', adiabaticcooling/abs(cooling)

   close(10)

END IF

DEALLOCATE(n_co, n_sio)

END SUBROUTINE NETHEATING
