SUBROUTINE nextline3(R_index, lambda, Nlineszone, TAUMAT, lineind, linej, linei, lambda_line, seq, FOUNDNEXTLINE, n_CO, MOLLINE, tau_mol)
USE M1, ONLY : Nabin
USE INTERFACE, ONLY : findbin2, findbin3
IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index, Nlineszone
DOUBLE PRECISION, INTENT(IN) :: lambda, n_CO
REAL, DIMENSION(:,:,:), INTENT(IN) :: TAUMAT
DOUBLE PRECISION, INTENT(OUT) :: lambda_line, tau_mol
INTEGER, INTENT(OUT) :: seq, lineind, linej, linei
LOGICAL, INTENT(OUT) :: FOUNDNEXTLINE
LOGICAL, INTENT(OUT) :: MOLLINE
! =====================================================
! Task : Returns the atomic/molecular line closest to the given input wavelengtj (on the red side)
! Input :   R_index           Zone index
!           lambda            Wavelength [A]
!           Nlineszone        Number of lines in the zone
!           n_CO              Number density of CO [cm-3]
!           TAUMAT            Array with lines , ordered?
! Output :  lineind           Index of closest line
!           linej             Upper level
!           linei             Lower level
!           lambda_line       Wavelength of line [A]
!           seq               ?
!           FOUNDNEXTLINE     Boolean, was a line found?
!           MOLLINE           Boolean, is the line a molecular line?
!           tau_mol
! ====================================================
DOUBLE PRECISION :: tday
INTEGER :: ireadph, k, bin, abin, i

! ---------------------------------------------------------------------------------------------

FOUNDNEXTLINE = .FALSE.
MOLLINE = .FALSE.
seq = 0
tau_mol = 0
linei = 0
linej = 0
lambda_line = 0
lineind = 0

! --------------------------------------------------------------------------------------------

IF (Nlineszone > 0) THEN   ! If there are any lines in this zone...

   CALL findbin3(R_index,Nlineszone, TAUMAT(:,:,5), lambda, lineind)   ! find index of line with closest lambda on the red side

   IF ((lineind .NE. 0 .AND. lineind .NE. Nlineszone) .OR. (lineind .EQ. 0 .AND. lambda < TAUMAT(R_index,1,5))) THEN   ! If such a line was found, or if lambda was smaller than the smallest wavelength (then lineind = 1, which is 0 + 1, see below)

      lineind = lineind + 1  !The line is on the right side of the bin ( if lamba= 1200 and lines are [1150,1250,1350,..] findbin returns 1 as the index)

      FOUNDNEXTLINE = .TRUE.  ! A line was found

      abin = NINT(TAUMAT(R_index,lineind,1))  ! Get element's abin code

      IF (abin > Nabin) THEN  ! it is a molecular line with no NLTE sols..

         MOLLINE = .TRUE.

      ELSE        ! the line is an atomic or NLTE molecule line 

         MOLLINE = .FALSE.

      END IF

      seq = NINT(TAUMAT(R_index,lineind,2))   ! get sequence in the atoms vector

      linei = NINT(TAUMAT(R_index,lineind,3))  ! get lower level index

      linej = NINT(TAUMAT(R_index,lineind,4))  ! get upper level index

      lambda_line = TAUMAT(R_index,lineind,5)  ! get line wavelength

   END IF


END IF
   
END

SUBROUTINE nextline2(R_index, lambda, Nlineszone, TAUMAT, lineind, linej, linei, lambda_line, seq, FOUNDNEXTLINE, n_CO, MOLLINE, tau_mol)
USE M1, ONLY : Nabin
USE INTERFACE, ONLY : findbin2, findbin3
IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index, Nlineszone
DOUBLE PRECISION, INTENT(IN) :: lambda, n_CO
REAL, DIMENSION(Nlineszone, 6), INTENT(IN) :: TAUMAT
DOUBLE PRECISION, INTENT(OUT) :: lambda_line, tau_mol
INTEGER, INTENT(OUT) :: seq, lineind, linej, linei
LOGICAL, INTENT(OUT) :: FOUNDNEXTLINE
LOGICAL, INTENT(OUT) :: MOLLINE
! =====================================================
! Task : Returns the atomic/molecular line closest to the given input wavelengtj (on the red side)
! Input :   R_index           Zone index
!           lambda            Wavelength [A]
!           Nlineszone        Number of lines in the zone
!           n_CO              Number density of CO [cm-3]
!           TAUMAT            Array with lines , ordered?
! Output :  lineind           Index of closest line
!           linej             Upper level
!           linei             Lower level
!           lambda_line       Wavelength of line [A]
!           seq               ?
!           FOUNDNEXTLINE     Boolean, was a line found?
!           MOLLINE           Boolean, is the line a molecular line?
!           tau_mol
! ====================================================
DOUBLE PRECISION :: tday
INTEGER :: ireadph, k, bin, abin, i

! ---------------------------------------------------------------------------------------------

FOUNDNEXTLINE = .FALSE.
MOLLINE = .FALSE.
seq = 0
tau_mol = 0
linei = 0
linej = 0
lambda_line = 0
lineind = 0

! --------------------------------------------------------------------------------------------

IF (Nlineszone > 0) THEN   ! If there are any lines in this zone...

   CALL findbin2(Nlineszone, DBLE(TAUMAT(1:Nlineszone,5)), lambda, lineind)   ! find index of line with closest lambda on the red side

   IF ((lineind .NE. 0 .AND. lineind .NE. Nlineszone) .OR. (lineind .EQ. 0 .AND. lambda < TAUMAT(1,5))) THEN   ! If such a line was found, or if lambda was smaller than the smallest wavelength (then lineind = 1, which is 0 + 1, see below)

      lineind = lineind + 1  !The line is on the right side of the bin ( if lamba= 1200 and lines are [1150,1250,1350,..] findbin returns 1 as the index)

      FOUNDNEXTLINE = .TRUE.  ! A line was found

      abin = NINT(TAUMAT(lineind,1))  ! Get element's abin code

      IF (abin > Nabin) THEN  

         MOLLINE = .TRUE.

      ELSE        

         MOLLINE = .FALSE.

      END IF

      seq = NINT(TAUMAT(lineind,2))   ! get sequence in the atoms vector

      linei = NINT(TAUMAT(lineind,3))  ! get lower level index

      linej = NINT(TAUMAT(lineind,4))  ! get upper level index

      lambda_line = TAUMAT(lineind,5)  ! get line wavelength

   END IF


END IF
   
END
