PROGRAM RADMAKER
use M1, ONLY : time, time_prev, TIMEDEPENDENT_ION, Nr, Nelem, Nabin, Npis, DOPRINT, IDP, map2, map3, map_abin2elem2, map_abin2elem3, Ephotoexc, myid, sharedcomm, window, globcomm, globid, taskcolor, taskcomm, R_index_str, ngvec, Nmol, ITERATIVERUN, MOLECULESWITCH, ievec, mmol, Zvec, Nstages, Nmol_NLTE
use PARAMETERS, ONLY : h, c, k, mp, pi
USE INTERFACE, ONLY : co_photodiss, findbin2, ionization, memshare
USE FILES, ONLY : file_runparameters, file_moldens, file_molres, file_ctheating, file_spectralrange, file_ionstructure, file_ionedges, file_nabspreion, file_totpreion, file_preheating, file_nabsnt, file_volume, file_EXCCUBE, file_dead, file_radmaker_nabsrad, file_radmaker_nabsfracs, file_cd, file_dep, file_Eemit, file_radmaker_ionrates, file_radmaker_radheating, file_radmaker_ionratesspecific, file_photoexcall, file_photoexc_long, file_lineabstot, file_ebalance, file_dustabs, file_edeadforeign, file_natoms, file_glevels, file_elevels, file_trec, file_radmerged, file_groundfracs, file_photoexcrates, file_numberfractionsused, file_ebalance_this, file_timedays, file_timeprev
USE MPI
IMPLICIT NONE
!INCLUDE 'mpif.h' 

! Description�:::::::::::::::::::::
!
! Following a MC run, this MPI program collects the data on photoionizations and photoexcitations, and calculates the new ionization balance,
! as well as heating rates due to photoionization, free-free absorption, charge transfer reactions.
!
! Usage: mpirun -np (Nr+1) -machinefile ~/.yourhostfile suib yourparameterfile.dat
! ::::::::::::::::::::::::::::::::::::::

DOUBLE PRECISION :: lambdamin, lambdamax, factor, lambda, E, e_eV, sig, r, tol, n_co, tday, COlinewidth, T, ndiss, dissrate, nabs, ctheating_temp, nphotoexc_temp, photoexcrate, temp, corrf, val, pop, ngrounduse, totalnumberdens, apa1, ct_heating, totpreion, Edead, Edust, Ebrems, Eemit, pi_heating, ratio, ratio2, preheating, br_heating, edeptot, Etemp
INTEGER :: Nwl, r_ind, i, j, r_bin, bin, sourcei, absorber, m, shell, R_index, abin, abinold, sourceshell, lev, lines, N, i_rf, DATA=1, tag=0, ierr, numprocs, origzone, zone, seq, ind, ind2, ind3, r1, n1, mm1, i1, j1, counter=0, ntop
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem) :: fracg2
DOUBLE PRECISION, DIMENSION(Nelem) :: numberfracvec
INTEGER, DIMENSION(:), POINTER :: abinvec
LOGICAL :: TESTRUN, ERROR, CTON, RF_EXISTS
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: wl, totalnumberdensity, popground, poptot, fracground3, fracground2, rate_dead_radtot, n_abs_nt
DOUBLE PRECISION, DIMENSION(:), POINTER :: volume, cd_heating, cd_heating_foreign, Edep, Edeadforeign, nabsco, gammaCO, n_h2disstot, ratiovec
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: NUMBERDENSITY_ALLSHELLS, NUMBERFRACTION, EXCCUBE, N_ABS_SPEC_RAD, N_ABS_SPEC_TOT, N_ABS_SPEC_PRE, N_ABS_SPEC_NT, N_ABS_SPEC_RADTOT, RATE_DEAD_SPEC
DOUBLE PRECISION, DIMENSION(:,:), POINTER :: IONEDGES, rf, Elevels, Glevels, PHOTOEXCTOT
DOUBLE PRECISION, DIMENSION(100) :: cal, ans
CHARACTER :: trashchar, trash, iterative_s
INTEGER, DIMENSION((Nstages-1)*Nelem+Nmol) :: bigmap
INTEGER :: N_atoms
INTEGER :: Nfrom, Nto
REAL :: t1, t2  ! For timing purposes!
REAL, DIMENSION(:,:,:), POINTER :: APA
DOUBLE PRECISION, DIMENSION(10000) :: lambdaradfields
DOUBLE PRECISION, DIMENSION(40) :: lambda_co, tau_co
INTEGER, DIMENSION(MPI_STATUS_SIZE) :: status
INTEGER, PARAMETER :: sizeuse=2000
! DOUBLE PRECISION, DIMENSION(Nabin,2000) ::Elevels, Glevels
INTEGER, DIMENSION(:), POINTER :: nlev
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem+Nmol) :: numberdensities_singlezone
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: NPHOTOEXCITATIONS
LOGICAL :: MOLON

TYPE(mmol), DIMENSION(maxval([1,Nmol])):: mols
INTEGER :: err_length, err_tmp
CHARACTER(len=MPI_MAX_ERROR_STRING) :: err_message
! ---------------------------------------------------------------

Nfrom = sizeuse-1    ! CARE Maximum absorbing level to compute photoexcitation rates

Nto = sizeuse    ! Maximum upper levels for photoexcitations

! --------------------
CALL MPI_INIT(ierr)   ! initialize MPI
CALL MPI_COMM_SIZE(MPI_COMM_WORLD, numprocs, ierr)  ! numprocs is the total number of processors invoked
CALL MPI_COMM_DUP(MPI_COMM_WORLD, globcomm, ierr)  ! Duplicate MPI_COMM_WORLD into a global communicator called globcomm
CALL MPI_COMM_RANK(globcomm, globid, ierr)  ! globid is the GLOBAL RANK for each processor
CALL MPI_COMM_SPLIT_TYPE(MPI_COMM_WORLD, MPI_COMM_TYPE_SHARED, 0, MPI_INFO_NULL, sharedcomm, ierr)  ! This splits the processors into communicators that envelop one (1) node
CALL MPI_COMM_RANK(sharedcomm, myid, ierr)  ! MYID is the ID number for each processor INSIDE ITS OWN NODE (so 0-31)

! Put all myid == 0 into one communicator to allow nodes to update their values to each other
if (myid == 0) then
    taskcolor = 0
else
    taskcolor = MPI_UNDEFINED
endif
CALL MPI_COMM_SPLIT(MPI_COMM_WORLD, taskcolor, 0, taskcomm, ierr)
! ---------------------------

CALL GET_COMMAND_ARGUMENT(1, file_runparameters) ! Run parameters file name (optional)

CALL GET_COMMAND_ARGUMENT(2, iterative_s)

if (iterative_s == 'y') then

	ITERATIVERUN = .TRUE.

else

	ITERATIVERUN = .FALSE.

end if

call cpu_time(t1)

if (globid == 0) print *, "ITERATIVE", ITERATIVERUN

IF (globid == 0) THEN  ! Global master reads & we global BCAST later
	OPEN(10,file=file_runparameters,STATUS='OLD');REWIND(10)  ! READ only CTION and MOLON logical parameters

	DO i = 1, 4; read(10,*) trashchar; END DO

	READ(10,*) trashchar, Nr

	DO i = 6, 19; READ(10,*) trashchar; END DO

	READ(10,*) trashchar, Npis

	DO i = 21, 25; read(10,*) trashchar; END DO

	READ(10,*) trashchar, CTON

	READ(10,*) trashchar, MOLECULESWITCH

	DO i = 28, 30; read(10,*) trash; END DO

	read(10,*) trashchar

	read(10,*) trashchar

	READ(10,*) trashchar, DOPRINT, IDP

	close(10)

	INQUIRE(file='out/makeatoms/map2.txt',Exist=RF_EXISTS)

	IF (RF_EXISTS) THEN

		open(10,file='out/makeatoms/map2.txt',STATUS='OLD')

		DO i = 1, (Nstages-1)*Nelem + Nmol
			read(10,*) trash, map2(i), ngvec(i), ievec(i)
		END DO

		close(10)

		open(10,file='out/makeatoms/map3.txt',STATUS='OLD')

		DO i = 1, Nstages*Nelem + Nmol
			read(10,*) trash, map3(i)
		END DO

		close(10)

		open(10,file='out/makeatoms/abinmaps.txt',STATUS='OLD')

		DO i = 1, Nabin + Nmol
			read(10,*) trash, map_abin2elem2(i), map_abin2elem3(i)
		END DO

		close(10)

		open(10,file='out/makeatoms/Zvec.txt',STATUS='OLD')

		DO i = 1, Nelem
			read(10,*) trash, Zvec(i)
		END DO

		close(10)
	ELSE
		print *, 'ERROR, map2 does not exist. Stopping.'
		stop 1
	END IF
END IF

! Share runparameters ------------
CALL MPI_BCAST(Nr, 1, MPI_INTEGER, 0, globcomm, ierr)
CALL MPI_BCAST(Npis, 1, MPI_INTEGER, 0, globcomm, ierr)
CALL MPI_BCAST(CTON, 1, MPI_LOGICAL, 0, globcomm, ierr)
Call MPI_BCAST(MOLECULESWITCH, 1, MPI_INTEGER, 0, globcomm, ierr)
call MPI_BCAST(ITERATIVERUN, 1, MPI_LOGICAL, 0, globcomm, ierr)
CALL MPI_BCAST(DOPRINT, 1, MPI_LOGICAL, 0, globcomm, ierr)
Call MPI_BCAST(IDP, 1, MPI_INTEGER, 0, globcomm, ierr)
call MPI_BCAST(map2, (Nstages-1)*Nelem + Nmol, MPI_INTEGER, 0, globcomm, ierr)
call MPI_BCAST(ngvec, (Nstages-1)*Nelem + Nmol, MPI_INTEGER, 0, globcomm, ierr)
call MPI_BCAST(ievec, (Nstages-1)*Nelem + Nmol, MPI_INTEGER, 0, globcomm, ierr)
call MPI_BCAST(map3, Nstages*Nelem + Nmol, MPI_INTEGER, 0, globcomm, ierr)
call MPI_BCAST(map_abin2elem2, Nabin + Nmol, MPI_INTEGER, 0, globcomm, ierr)
call MPI_BCAST(map_abin2elem3, Nabin + Nmol, MPI_INTEGER, 0, globcomm, ierr)
call MPI_BCAST(Zvec, Nelem, MPI_INTEGER, 0, globcomm, ierr)

ntop = maxval([Npis,ngvec])  ! Number of photoionization states, never smaller than number of ground levels
! ALLOCATE/memshare arrays based on runparameters ------------
! ALLOCATE(IONEDGES(2*Nelem, ntop)); IONEDGES=0
ALLOCATE(fracground3(Nstages*Nelem));fracground3=1.
ALLOCATE(popground(Nstages*Nelem)); popground=0
ALLOCATE(poptot(Nstages*Nelem)); poptot=0
ALLOCATE(fracground2((Nstages-1)*Nelem));fracground2=1.
ALLOCATE(RATE_DEAD_RADTOT((Nstages-1)*Nelem+Nmol)); RATE_DEAD_RADTOT=0
ALLOCATE(EXCCUBE(Nabin,sizeuse)); EXCCUBE=0
ALLOCATE(N_ABS_SPEC_TOT((Nstages-1)*Nelem+Nmol,ntop));N_ABS_SPEC_TOT=0
ALLOCATE(N_ABS_SPEC_RADTOT((Nstages-1)*Nelem+Nmol,ntop));N_ABS_SPEC_RADTOT=0
ALLOCATE(N_ABS_SPEC_NT((Nstages-1)*Nelem+Nmol,ntop));N_ABS_SPEC_NT=0
ALLOCATE(n_abs_nt((Nstages-1)*Nelem)); n_abs_nt = 0
ALLOCATE(N_ABS_SPEC_RAD((Nstages-1)*Nelem+Nmol,ntop));N_ABS_SPEC_RAD=0
ALLOCATE(RATE_DEAD_SPEC((Nstages-1)*Nelem+Nmol,ntop));RATE_DEAD_SPEC=0

CALL memshare(sharedcomm, myid, ratiovec, window, (/ Nr /)); ratiovec = 0
CALL memshare(sharedcomm, myid, Edep, window, (/ Nr /)); Edep = 0
CALL memshare(sharedcomm, myid, volume, window, (/ Nr /)); volume = 0
CALL memshare(sharedcomm, myid, cd_heating, window, (/ Nr /)); cd_heating = 0
CALL memshare(sharedcomm, myid, cd_heating_foreign, window, (/ Nr /)); cd_heating_foreign = 0
CALL memshare(sharedcomm, myid, Edeadforeign, window, (/ Nr /)); Edeadforeign = 0
CALL memshare(sharedcomm, myid, nabsco, window, (/ Nr /)); nabsco = 0
CALL memshare(sharedcomm, myid, gammaCO, window, (/ Nr /)); gammaCO = 0
CALL memshare(sharedcomm, myid, n_h2disstot, window, (/ Nr /)); n_h2disstot = 0
CALL memshare(sharedcomm, myid, Ephotoexc, window, (/ Nr /)); Ephotoexc = 0
CALL memshare(sharedcomm, myid, nlev, window, (/ Nabin /)); nlev = 0
CALL memshare(sharedcomm, myid, rf, window, (/ Nr, sizeuse /)); rf = 0
CALL memshare(sharedcomm, myid, Elevels, window, (/ Nabin,sizeuse /)); Elevels = 0
CALL memshare(sharedcomm, myid, Glevels, window, (/ Nabin,sizeuse /)); Glevels = 0
CALL memshare(sharedcomm, myid, IONEDGES, window, (/ (Nelem-1)*Nelem, ntop /)); IONEDGES = 0

CALL memshare(sharedcomm, myid, time_prev, window); time_prev = 0

! Master, read input files ----

TIMEDEPENDENT_ION = .FALSE.

call alloc_mol(mols) ! TEMP

IF (myid == 0) THEN  ! MYID masters read; we *globally* BCAST after for the non-shared properties

   IF (TIMEDEPENDENT_ION .EQV. .TRUE.) THEN

        open(8,file=file_timedays,STATUS='OLD')

        read(8,*) time

        close(8)

        time = time*24*3600

        open(8,file=file_timeprev,STATUS='OLD')

        read(8,*) time_prev

        close(8)

        time_prev = time_prev*24*3600

   ELSE

        time = 0.

        time_prev = 0.

   END IF

   ! Read lambdamin and lambdamax ---------------

   OPEN(8,file=file_spectralrange,STATUS='OLD')

     READ(8,*) lambdamin, lambdamax    ! Read limits of photon wavelengths used in the MC run

   CLOSE(8)

   OPEN(8, file=file_ionstructure,STATUS='OLD')

   ! Read number density -------------------------

   ALLOCATE(NUMBERDENSITY_ALLSHELLS(Nr,(Nstages-1)*Nelem+Nmol)); NUMBERDENSITY_ALLSHELLS=0

   DO

      READ(8,*,END=400) shell, i, NUMBERDENSITY_ALLSHELLS(shell,i)   ! Read number densities of all 2*Nelem+Nmol species in each zone. i is the 2*Nelem numbering.

   END DO

400 CLOSE(8)

    DO i = 1, Nr

       tag = 0
       IF (globid == 0) THEN  ! Only GLOBAL master sends
           call MPI_SEND(NUMBERDENSITY_ALLSHELLS(i,:), (Nstages-1)*Nelem+Nmol, MPI_DOUBLE_PRECISION, i, tag, globcomm, ierr)
       ENDIF
    END DO

    DEALLOCATE(NUMBERDENSITY_ALLSHELLS)

    ! Read ionedges -----------------------------------

    OPEN(8,file=file_ionedges,STATUS='OLD')

    DO i = 1, (Nstages-1)*Nelem

       READ(8,*) trash, IONEDGES(i,1:ntop)   ! Read the ionization edges (in �) for the 2*Nelem elements (molecules excluded) first Npis states

    END DO

   CLOSE(8)

   ! collisional deexcitations -----------

   OPEN(200, file=file_cd,STATUS='OLD')  ! Read heating caused by line deexcitations (this can occur also with the ray-tracing, following exc. to high levels)

   DO

      READ(200,*,END=600) r_ind, E

      cd_heating(r_ind) = cd_heating(r_ind) + E

   END DO

600 CLOSE(200)

   ! energy depositions ---------------------

   OPEN(130,file=file_dep,STATUS='OLD')   ! Read total radioactive energy deposition (used to check energy balance)

   read(130,*) trash

   DO

      READ(130,*,END=70) r_ind, Etemp

      if (r_ind <= Nr) Edep(r_ind) = Etemp

   END DO

70 CLOSE(130)
   ! Volume -----------------------

   OPEN(60,file=file_volume,STATUS='OLD')

   DO i = 1, Nr   ! Read zone volumes

      READ(60,*) trash, volume(i)

   END DO

   CLOSE(60)

   ! Line absorptions -----------------

   open(100,file=file_lineabstot,STATUS='OLD')

   DO

      read(100,*,END=741) r_ind, temp

      Ephotoexc(r_ind) = Ephotoexc(r_ind) + temp    ! all photoexcitations that have occured (irrespective of subsequent event)

   END DO

741 close(100)
   ! --------------

   open(10,file=file_natoms,STATUS='OLD')

     read(10,*) N_atoms

   close(10)

   ! ---------

   open(10,file=file_elevels,STATUS='OLD')

   DO

      read(10,*,END=52) abin, nlev(abin), Elevels(abin,1:nlev(abin))  ! read abin code, number of energy levels, the the actual level energies in eV

   END DO
   52 close(10)

   ! ---------------------------

    open(10,file=file_glevels,STATUS='OLD')

   DO
      read(10,*,END=53) abin, nlev(abin), Glevels(abin,1:nlev(abin))  ! Need to compute corrections for stimulated emission in photoexcitation rates. read abin code, number of energy levels, the the actual level energies in eV
    !print*, abin, nlev(abin)
   END DO
   53 close(10)

   ! -------------------------

   ALLOCATE(totalnumberdensity(Nr)); totalnumberdensity = 0

   ALLOCATE(NUMBERFRACTION(Nr,Nelem)); NUMBERFRACTION = 0

   OPEN(10, file=file_numberfractionsused, STATUS='OLD')

   DO    ! Read number fractions of nuclei

      READ(10,*,END=50) i, totalnumberdensity(i), NUMBERFRACTION(i,1:Nelem)

   END DO

50 CLOSE(10)

   DO i = 1, Nr

      tag = 0

      IF (globid == 0) THEN  ! Only GlobalMaster SENDS
          call MPI_SEND(totalnumberdensity(i), 1, MPI_DOUBLE_PRECISION, i, tag, globcomm, ierr)
          call MPI_SEND(NUMBERFRACTION(i,1:Nelem), Nelem, MPI_DOUBLE_PRECISION, i, tag, globcomm, ierr)
      ENDIF
   END DO

   DEALLOCATE(totalnumberdensity)

   DEALLOCATE(NUMBERFRACTION)

   IF (globid /= 0) THEN  ! To make sure that all non-master globals receive the zone values they need, these also need to MPI_RECV
       call MPI_RECV(numberdensities_singlezone, (Nstages-1)*Nelem+Nmol, MPI_DOUBLE_PRECISION, 0, MPI_ANY_TAG, globcomm, status, ierr)
       call MPI_RECV(totalnumberdens, 1, MPI_DOUBLE_PRECISION, 0, MPI_ANY_TAG, globcomm, status, ierr)
       call MPI_RECV(numberfracvec, Nelem, MPI_DOUBLE_PRECISION, 0, MPI_ANY_TAG, globcomm, status, ierr)
   ENDIF

ELSE  ! Global slaves RECV

   call MPI_RECV(numberdensities_singlezone, (Nstages-1)*Nelem+Nmol, MPI_DOUBLE_PRECISION, 0, MPI_ANY_TAG, globcomm, status, ierr)

   call MPI_RECV(totalnumberdens, 1, MPI_DOUBLE_PRECISION, 0, MPI_ANY_TAG, globcomm, status, ierr)

   call MPI_RECV(numberfracvec, Nelem, MPI_DOUBLE_PRECISION, 0, MPI_ANY_TAG, globcomm, status, ierr)

END IF

!BCAST values that are not shared, wait for all processes to arrive first
CALL MPI_BARRIER(globcomm, ierr)

call MPI_BCAST(lambdamin, 1, MPI_DOUBLE_PRECISION, 0, globcomm, ierr)
call MPI_BCAST(lambdamax, 1, MPI_DOUBLE_PRECISION, 0, globcomm, ierr)
call MPI_BCAST(N_atoms, 1, MPI_INTEGER, 0, globcomm, ierr)
call MPI_BCAST(time, 1, MPI_DOUBLE_PRECISION, 0, globcomm, ierr)

TESTRUN = .FALSE.    ! Change to true if test run is desired

ALLOCATE(NPHOTOEXCITATIONS(N_atoms, Nfrom, Nto)); NPHOTOEXCITATIONS=0
ALLOCATE(PHOTOEXCTOT(N_atoms, Nfrom)); PHOTOEXCTOT=0
! CALL memshare(sharedcomm, myid, PHOTOEXCTOT, window, (/ N_atoms, Nfrom /)); PHOTOEXCTOT=0
CALL memshare(sharedcomm, myid, APA, window, (/ N_atoms, 10, 100 /)); APA=0
CALL memshare(sharedcomm, myid, abinvec, window, (/ N_atoms /)); abinvec=0

IF (globid >= 1) THEN  ! GlobalID

  ! OPEN(10,file=TRIM(file_moldens)//R_index_str(myid),STATUS='REPLACE'); CLOSE(10)   ! Initialize file.. Multi from before

   IF (ITERATIVERUN) THEN   ! Multi from before

      OPEN(10,file=TRIM(file_ctheating)//R_index_str(globid),STATUS='OLD');  ! Read the ct-heating used in the previous run..(only used to compute energy balance in each zone)

         READ(10,*,END=30) ct_heating

30    CLOSE(10)

   ELSE

      ct_heating = 0

   END IF
    ! FIXME this is probably redudant
   bigmap(1:(Nstages-1)*Nelem) = map2(1:(Nstages-1)*Nelem)     ! map2 exists in the M1 module, it specifies the abin values of [HI (HII) HeI HeII CI CII etc]

   bigmap((Nstages-1)*Nelem+1:(Nstages-1)*Nelem+Nmol) = [(Nabin + i,i=1,Nmol)]   ! Molecules are mapped as Nabin + 1,2,3,  hmm correct?

   pi_heating = 0         ! Important
   br_heating = 0
   cd_heating_foreign = 0
   Edead = 0
   Edeadforeign = 0
   Edust = 0
   Ebrems = 0
   EXCCUBE = 0  ! IMPORTANT
   lines = 0
   totpreion = 0
   preheating = 0

   tol = 3E-2   ! Error tolerance for TEST runs

   ! --------------------- INTERNAL FLUXES --------------------------

   factor = 2E-5   ! Resolution for internal flux fields  lambda(i+1)/lambda(i) - 1

   OPEN(8,file=TRIM(file_totpreion)//R_index_str(globid),STATUS='OLD')  ! Multi from before

        READ(8,*,END=752) totpreion       ! The total pre-ionization energy (used only to check energy balance)

752 CLOSE(8)

    OPEN(8,file=TRIM(file_preheating)//R_index_str(globid),STATUS='OLD')  ! Multi from before

        READ(8,*,END=753) preheating      ! The total heating from the pre-ioizations (added to the heating computed from the MC run)

753 CLOSE(8)

    OPEN(10,file=TRIM(file_nabsnt)//R_index_str(globid),STATUS='OLD')  ! Multi from before READ NUMBER OF NON_THERMAL IONIZATIONS, species i label j

    DO

       READ(10,*,END=123) R_index, i, n_abs_nt(i) !N_ABS_SPEC_NT(i, j)

    END DO

123 CLOSE(10)

   ! EXCCUBE -------

   OPEN(70,file=TRIM(file_EXCCUBE)//R_index_str(globid),STATUS='OLD')  ! Multi from before. Read atomic populations (needed to compute photoexcitation rates)
   DO

      READ(70,*,END=201) R_index, seq, abin, i, EXCCUBE(abin,i)

      abinvec(seq) = abin

   END DO

201 CLOSE(70)

   DO i = 1, (Nstages-1)*Nelem

      IF (i == 2) CYCLE  ! HII

      abin = map2(i)

      if (abin == 0) CYCLE

      DO j = 1, ngvec(i)

         if (sum(EXCCUBE(abin,1:ngvec(i))) > 0) then

           N_ABS_SPEC_NT(i,j) = n_abs_nt(i)*EXCCUBE(abin,j)/sum(EXCCUBE(abin,1:ngvec(i)))  ! Allocate non-thermal ionization according to ground state populations
        ! IF (myid == 1 .AND. N_ABS_SPEC_NT(i,j) > 0) print *, 'apan', i, abin, j, N_ABS_SPEC_NT(i,j)

         else

           N_ABS_SPEC_NT(i,j) = 0.

         end if

      END DO

   END DO

   IF (TESTRUN .EQV. .TRUE.) THEN

      volume(1) = 1d20
      volume(2) = 1d30
      volume(3) = 1D31
      volume(4) = 1D30
      volume(5) = 1D30
      volume(6) = 1D30
      volume(10) = 1D30

   END IF

   ! WL vector for internal rad field

   Nwl = INT(LOG(lambdamax/lambdamin)/LOG(1. + factor))

   ALLOCATE(wl(Nwl)); wl=0

   wl(1) = lambdamin

   DO i = 2, Nwl           ! Create lambda vector for internal rad fields

      wl(i) = wl(i-1)*(1.+factor)

   END DO

   !  --- READ PHOTOEXCITATIONS ------------------------------------

   open(100,file=TRIM(file_photoexcall)//R_index_str(globid), FORM='UNFORMATTED',STATUS='OLD')

   DO

      read(100,END=755) i, j, m, n, nphotoexc_temp, apa1   ! i=R_index, j=abin, m=from,n=to (higher for absorption, lower for SE)

      IF ((m <= Nfrom) .AND. (n <= nto) .AND. (j <= N_atoms)) THEN

         if (isnan(NPHOTOEXCITATIONS(j,m,n))) THEN

            print *, 'Nan!', i, j, m, n

         elseif (abs(NPHOTOEXCITATIONS(j,m,n)) > huge(nphotoexc_temp)) THEN

            print *, 'Inf!', i, j, m, n

         else

            NPHOTOEXCITATIONS(j,m,n) =  nphotoexc_temp

         end if

         IF (m <= 10 .AND. n <= 100) APA(j,m,n) = apa1

      END IF

   END DO

755 close(100)

   ! Write Photoexc rates  ------------------------------------------------

   open(100,file=TRIM(file_photoexcrates)//R_index_str(globid),STATUS='REPLACE')  ! NEW OUTPUT

   DO j = 1, N_atoms

      DO m = 1, Nfrom

         DO n = (m+1), Nto   ! stim emission is stored as negative number in low-high order. low also to lower levels

            val = NPHOTOEXCITATIONS(j,m,n)

            IF (val == 0. .OR. abinvec(j) == 0) CYCLE

            IF ((val > 0) .AND. (EXCCUBE(abinvec(j),m) > 0)) THEN   ! absorption

            	IF (EXCCUBE(abinvec(j),n) .NE. 0) THEN

                	corrf =  1. - Glevels(abinvec(j),m)/Glevels(abinvec(j),n)*EXCCUBE(abinvec(j),n)/EXCCUBE(abinvec(j),m)  ! correct for stimulated emission


               	ELSE

                  	corrf = 1.

               	END IF

               	IF (corrf == 0) then

					corrf = 1

			   		print *, 'WARNING, corrf=0 in ionization.f90; globid myid', globid, myid

				END IF

               	photoexcrate = val/(EXCCUBE(abinvec(j),m)*volume(globid))/corrf

               	PHOTOEXCTOT(j,m) = PHOTOEXCTOT(j,m) + photoexcrate

               	if (isnan(photoexcrate)) print *, 'Nan!', Glevels(abinvec(j),n), EXCCUBE(abinvec(j),m), val, abinvec(j), j, volume(myid), volume(globid)

               	IF (m <= 10 .AND. n <= 100) THEN

                  	!write(100,'(4I6,2ES12.3)') myid, abinvec(j), m, n, photoexcrate, APA(j,m,n)/photoexcrate
                  	write(100,'(4I6,2ES12.3)') globid, abinvec(j), m, n, photoexcrate, corrf

               	ELSE

                 	! write(100,'(4I6,2ES12.3)') myid, abinvec(j), m, n, photoexcrate, 0d0
                 	write(100,'(4I6,2ES12.3)') globid, abinvec(j), m, n, photoexcrate, corrf

               	END IF

            	ELSE IF ((val < 0) .AND. (EXCCUBE(abinvec(j),n) > 0)) THEN  ! stimulated emission


               		corrf = 1. - Glevels(abinvec(j),n)/Glevels(abinvec(j),m)*EXCCUBE(abinvec(j),m)/EXCCUBE(abinvec(j),n) ! correct for asborptions

               		print *, 'Stim!', abinvec(j), m, n, corrf

               IF (corrf == 0) THEN

				corrf = 1.

                print *, 'WARNING,corrf=0 in ionization.f90', globid

               END IF

               IF (corrf < 0.) print *, 'WARNING negative corrf in ionization.f90'

               photoexcrate = abs(val)/(EXCCUBE(abinvec(j),n)*volume(globid))/corrf*Glevels(abinvec(j),n)/Glevels(abinvec(j),m)  ! R_up = R_down g_up / d_down

               IF (m <= 10 .AND. n <= 100) THEN

                  write(100,'(4I6,2ES12.3)') globid, abinvec(j), m, n, photoexcrate, APA(j,m,n)/photoexcrate

               ELSE

                  write(100,'(4I6,2ES12.3)') globid, abinvec(j), m, n, photoexcrate, 0d0

               END IF

            ELSE

               print *, 'ERROR 3', val, EXCCUBE(abinvec(j),m), EXCCUBE(abinvec(j),n)  ! Should not occur

            END IF

         END DO

      END DO

   END DO

   close(100)

   open(100,file='out/radmaker/photoexctot.dat'//R_index_str(globid),STATUS='REPLACE')  ! NEW OUTPUT

   DO j = 1, N_atoms

      DO m = 1, Nfrom

         IF (PHOTOEXCTOT(j,m) > 0) write(100,*) abinvec(j), m, PHOTOEXCTOT(j,m)

      END DO

    END DO

    close(100)

!   open(15,file=file_photoexc_long,POSITION='APPEND')

!   i1 = 1; j1 = 2; mm1 = 2; n1 = 4;

!   write(15,*) NPHOTOEXCITATIONS(i1,j1,mm1,n1)/(EXCCUBE(i1,abinvec(j1),mm1)*volume(i1))/(1. - Glevels(abinvec(j1),mm1)/Glevels(abinvec(j1),n1)*EXCCUBE(i1,abinvec(j1),n1)/EXCCUBE(i1,abinvec(j1),mm1))

!   close(15)

   ! N_ABS_SPEC_PRE -------------------------

   ALLOCATE(N_ABS_SPEC_PRE((Nstages-1)*Nelem+Nmol,ntop)); N_ABS_SPEC_PRE=0

!   OPEN(8,file=TRIM(file_nabspreion)//R_index_str(myid),STATUS='OLD')   ! NEW MULTI IN
!
!   DO

!      READ(8,*,END=333) R_index, i, j, N_ABS_SPEC_PRE(i,j)  ! Read number of preionations of element i, level j in shell R_index

!   END DO

333 CLOSE(8)

    ! ----- ADD PHOTOIONIZATION PACKAGES ----------------------------

   OPEN(100,file=TRIM(file_dead)//R_index_str(globid),STATUS='OLD',FORM='UNFORMATTED',POSITION='REWIND')  ! NEW MULTI IN.  PHOTOIONIZATION absorptions.  Care binary file

   lines = 0

   DO

      READ(100,ERR=128,END=500) r_ind, absorber, lev, nabs, E

      IF ((isnan(nabs) .EQV. .FALSE.) .AND. (nabs < huge(nabs)) .AND. (isnan(E) .EQV. .FALSE.) .AND. (E < huge(E))) THEN

         counter = counter + 1

         IF (E > 0) THEN

            lambda = h*c/(E/nabs)*1e8  ! lambda = h*c/Epp = h*c/(E/nabs) IN �

         ELSE

            lambda = 0

         END IF

         lines = lines + 1

         IF (absorber <= (Nstages-1)*Nelem+Nmol) THEN           ! Dont count dust or bremsstrahlung absorption

            N_ABS_SPEC_RAD(absorber, lev) = N_ABS_SPEC_RAD(absorber, lev) + nabs   ! Add number of photoionization in this shell, for this element, this level

            Edead = Edead + E  ! Add energy to the total photoionization energy in this zone

            !IF (origzone .NE. r_ind) THEN

            !    Edeadforeign(r_ind) = Edeadforeign(r_ind) + E  ! just bookkeeping

            ! END IF

            IF (absorber <= (Nstages-1)*Nelem ) THEN    ! PHOTOIONIZATION HEATING, not for molecules or dust..

               IF (IONEDGES(absorber,lev) .NE. 0) THEN

                  pi_heating = pi_heating + E*(1. - lambda/IONEDGES(absorber, lev))  ! Compute heating caused by the photoionization. E_heat = E_tot - N_tot*Xi = E_tot*(1 - Xi*N_tot/E_tot) = E_tot*(1-lambda_bar / lambda_ion)

               END IF

            END IF

         ELSE IF (absorber == (Nstages-1)*Nelem+Nmol+1) THEN  ! dust absorption

            Edust = Edust + E

         ELSE  ! Bremsstrahlung absorption

            Ebrems = Ebrems + E

         END IF

         cycle

128      print *, 'WARNING, ERROR IN READING PHOTOIONIZATION FILE'

      ELSE

         print *, 'ERROR'

          print *, nabs, E

       END IF

   END DO


500   CLOSE(100)




   OPEN(77,file=TRIM(file_radmaker_nabsrad)//R_index_str(globid), STATUS='REPLACE')  ! NEW MULTI OUT Write the number of photoionizations for each element and level and zone

   DO i = 1, (Nstages-1)*Nelem+Nmol  ! For each element...

      DO j = 1, ntop  ! For each level (participating in photoionization)

         IF (N_ABS_SPEC_RAD(i, j) > 0) WRITE(77,*) globid, i, j, N_ABS_SPEC_RAD(i, j)

      END DO

   END DO

   CLOSE(77)

   N_ABS_SPEC_TOT = N_ABS_SPEC_PRE + N_ABS_SPEC_NT + N_ABS_SPEC_RAD   ! The total number of ionizations (=preionizations + nonthermal + photoelectric)

   N_ABS_SPEC_RADTOT =  N_ABS_SPEC_PRE +  N_ABS_SPEC_RAD     ! The radiative ones (exclude non-thermal)

   OPEN(40,file=TRIM(file_radmaker_nabsfracs)//R_index_str(globid),STATUS='REPLACE')  ! Write fraction of total ionization rate that is contributed by which level (just bookkeeping)

   write(40,*) '2*Nelem   j  f(rad)   f(nt)'

   DO i = 1, (Nstages-1)*Nelem+Nmol

      DO j = 1, ntop

         IF (N_ABS_SPEC_TOT(i,j) > 0) THEN

            WRITE(40,'(3I5,2ES10.2)') globid, bigmap(i), j, N_ABS_SPEC_RAD(i,j)/SUM(N_ABS_SPEC_TOT(i,:)), N_ABS_SPEC_NT(i,j)/SUM(N_ABS_SPEC_TOT(i,:))

         END IF

      END DO

   END DO

   CLOSE(40)

   OPEN(130,file=TRIM(file_Eemit)//R_index_str(globid),STATUS='OLD') ! Multi from before. Read total emission (used to check energy balance only)

     READ(130,*) Eemit

   CLOSE(130)

   IF ((Edep(globid) + Edead + totpreion + cd_heating(globid) + ct_heating*volume(globid)) > 0) THEN  ! some sort of energy dep in the zone..

      edeptot = Edep(globid) + Edead + totpreion + cd_heating(globid) + Ephotoexc(globid)

      ratio = Eemit/(edeptot) ! dont include ctheating

      !print *, 'ebalance, shell', myid, ratio, Edep(myid)/edeptot, Edead/edeptot, totpreion/edeptot, cd_heating(myid)/edeptot, Ephotoexc(myid)/edeptot

   ELSE

      ratio = 0

   END IF

   call MPI_SEND(ratio, 1, MPI_DOUBLE_PRECISION, 0, tag, globcomm, ierr)

   pi_heating = pi_heating + preheating    ! Total heating caused by photoionizations

   OPEN(70,file=TRIM(file_radmaker_ionrates)//R_index_str(globid),STATUS='REPLACE')  ! NEW MULTI OUT

   OPEN(80,file=TRIM(file_radmaker_radheating)//R_index_str(globid),STATUS='REPLACE')  ! NEW MULTI OUT

   DO j = 1, (Nstages-1)*Nelem+Nmol

      IF (j .NE. 2) THEN    ! 2 would be ionized hydrogen (which doesnt exist)

         IF (j <= (Nstages-1)*Nelem) THEN  ! No molecular specifics

            if (bigmap(j) == 0) CYCLE

            DO m = 1, ntop  ! Compute ionization rates for specific levels = total number of ionizations divided by total number of particles in that state.

               IF (EXCCUBE(bigmap(j),m) > 0) RATE_DEAD_SPEC(j,m) = N_ABS_SPEC_TOT(j,m)/(EXCCUBE(bigmap(j),m)*volume(globid))

            END DO

         END IF

         IF (numberdensities_singlezone(j) > 0) THEN   ! Also molecules may be photoionized

            rate_dead_radtot(j) = SUM(N_ABS_SPEC_RADTOT(j,:))/(numberdensities_singlezone(j)*volume(globid))  ! The ionization rate per particle is the sum of ionizations that have occured divided by the total particle number

         END IF

      END IF

   END DO
   !print*, RATE_DEAD_SPEC(j,m), N_ABS_SPEC_TOT(j,m), EXCCUBE(bigmap(j),m), volume(myid)
   j = 51
   print*, numberdensities_singlezone(j), SUM(N_ABS_SPEC_RADTOT(j,:)), rate_dead_radtot(j)

   DO j = 1, Nelem   ! Write radiative ionization rates for neutrals and singly ionized

      WRITE(70,*) globid, j, rate_dead_radtot((Nstages-1)*(j-1)+1), rate_dead_radtot((Nstages-1)*(j-1)+2)

   END DO

   DO j = 1, Nmol  ! Radiative ionization rates of molecules

      WRITE(70,*) globid, (Nstages-1)*Nelem+j, rate_dead_radtot((Nstages-1)*Nelem+j)

   END DO

   IF (volume(globid) > 0) THEN

      pi_heating = pi_heating/volume(globid)

      cd_heating(globid) = cd_heating(globid)/volume(globid)

      br_heating = Ebrems/volume(globid)

  ELSE

     pi_heating = 0

     cd_heating(globid) = 0

     br_heating = 0

  END IF

  WRITE(80,'(I10, 3ES12.2)') globid, pi_heating, cd_heating(globid), br_heating

  CLOSE(70)

  CLOSE(80)

  OPEN(30,file=TRIM(file_radmaker_ionratesspecific)//R_index_str(globid),STATUS='REPLACE')  ! NEW MULTI OUTPUT. Further used by main.f90

  DO j = 1, (Nstages-1)*Nelem  ! Dont add molecules

     DO m = 1, ntop

        IF (RATE_DEAD_SPEC(j,m) > 0) WRITE(30,*) globid, bigmap(j), m, RATE_DEAD_SPEC(j,m)

     END DO

  END DO

  CLOSE(30)

!  INQUIRE(file=file_radmerged,Exist=RF_EXISTS)

!  IF (RF_EXISTS) THEN

!     OPEN(20,file=file_radmerged,STATUS='OLD')  ! Radfield parts produced during the MC run

!      i_rf = 0

!      DO

!         READ(20,*,END=33) lambdaradfields(i_rf+1), rf(1:Nr,i_rf+1)

!         i_rf = i_rf + 1

!      END DO

!33    CLOSE(20)

!   END IF

   ! Estimate PHOTODISSOCIATION OF CO from the radiation field

   tday = 300 ! CARE doesnt matter, taus not used

   n_co = 1e5 ! CARE doesnt matter

   T = 3000. ! CARE doesnt matter

   nabsco = 0.

  ! CALL co_photodiss(1, n_co, tday, lambda_CO, tau_CO)

  ! nabsco = 0.

  ! IF (RF_EXISTS .AND. i_rf >= 2) THEN

  !    DO R_index = 1, Nr

  !       DO i = 1, 37

  !          CALL findbin2(i_rf-1, lambdaradfields(1:i_rf-1), lambda_CO(i), bin)

  !          COlinewidth = lambda_CO(i)*SQRT(3*k*T/(28*mp))/c

  !          nabsco(R_index) = nabsco(R_index) + rf(R_index,bin)/(h*c/(lambdaradfields(i)*1E-8))*COlinewidth

  !       END DO

  !    END DO

  ! ELSE

  !    nabsco = 0

  ! END IF

   IF (Nmol > 0) then

      if (numberdensities_singlezone((Nstages-1)*Nelem+1) > 0) THEN

      gammaCO = 4*pi/c*nabsco(globid)/numberdensities_singlezone((Nstages-1)*Nelem+1)  ! Photon density / molecule density and assumotion of complete absorption gives the rate

      end if

   ELSE

      gammaCO = 0.

   END IF

   IF (TESTRUN .EQV. .TRUE.) THEN

      PRINT *, 'Checking in radmaker..'

      ! Shell 3

      IF (globid == 3) THEN

         cal(1) = N_ABS_SPEC_TOT(17,1)
         ans(1) = 1.26D45

         cal(2) = RATE_DEAD_SPEC(17,1)
         ans(2) = 1.26E6

         cal(3) = RATE_DEAD_RADTOT(17)
         ans(3) = 6.62E6

         cal(4) = pi_heating
         ans(4) = 3356.3

	 ELSEIF (globid == 2) THEN

      ! Shell 2

         cal(5) = N_ABS_SPEC_TOT(17,1)
         ans(5) = 2.61D46

         cal(6) = RATE_DEAD_SPEC(17,1)
         ANS(6) = 2.61e10

         cal(7) = RATE_DEAD_RADTOT(17)
         ans(7) = 3.09E8

         cal(8) = pi_heating
         ans(8) = 58066

	 ELSEIF (globid == 4) THEN

      !  SHELL 4    WITH NT IONIATIONS

         cal(9) = 0
         ans(9) = 0

         cal(10) = RATE_DEAD_SPEC(17,1)
         ans(10) = 6.0E10

         cal(11) = RATE_DEAD_SPEC(17,2)
         ans(11) = 5.0E9

         cal(12) = RATE_DEAD_SPEC(17,3)
         ans(12) = 2.0E9

	 ELSEIF (globid == 5) THEN

      ! SHELL 5

         cal(13) = N_ABS_SPEC_TOT(17,2)
         ans(13) = 1.0D51


         cal(14) = RATE_DEAD_RADTOT(17)
         ans(14) = 1.92E16

         cal(15) = RATE_DEAD_SPEC(17,2)
         ans(15) = 1.0E12

	 ELSEIF (globid == 6) THEN

         ! SHELL 6

         cal(16) = RATE_DEAD_RADTOT(1)
         ans(16) = 1.11E12

         cal(17) = RATE_DEAD_SPEC(1,1)
         ans(17) = 5.03E-10

         cal(18) = RATE_DEAD_SPEC(1,2)
         ans(18) = 2.41E-10

      END IF

      N = 18

      CALL checkerrors(N, cal(1:N), ans(1:N), tol, ERROR)

      PRINT *, 'radmaker TESTING SUCCESFUL.'

   END IF


   if (DOPRINT) WRITE(*,*) 'Total dust absorption:', Edust

   open(10,file=TRIM(file_dustabs)//R_index_str(globid),STATUS='REPLACE')  ! NEW MULTI OUT

     write(10,*) Edust

   close(10)

   abinold = 0

   ! Read fraction of atoms in ground state (needed for CT). For each shell..



   OPEN(10,file=TRIM(file_EXCCUBE)//R_index_str(globid),STATUS='OLD')  ! Multi from before

   DO

      READ(10,*,END=37) trash, trash, abin, lev, pop

      IF (abin .NE. abinold) THEN  ! find new positions only if new element

         ind3 = map_abin2elem3(abin)

         ind2 = map_abin2elem2(abin)

         IF (ind2 == 0) THEN  ! Highest ionization

            ngrounduse = 1.

         ELSE

            ngrounduse = ngvec(ind2)

         END IF

      END IF

      abinold = abin

      IF (lev <= ngrounduse) THEN  ! If this is the ground state population

         if (ind3 > 0) popground(ind3) = popground(ind3) + pop

      END IF

      if (ind3 > 0) poptot(ind3) = poptot(ind3) + pop  ! Add up to total population of this ion in this shell

   END DO

37 close(10)

   open(10,file=TRIM(file_groundfracs)//R_index_str(globid),STATUS='REPLACE')  ! NEW MULTI OUT write fractions in ground state

   DO j = 1, Nstages*Nelem

      IF (poptot(j) > 0) THEN

         fracground3(j) = popground(j)/poptot(j)

      ELSE

         fracground3(j) = 1.

      END IF

      if (j .gt. Nstages*(Nelem-Nmol_NLTE)) then

         fracground3(j) = 1.  ! TEMPFIX, why molecules cant have non-unity?

      end if

      abin = map3(j)

      ind = 1

      DO  ! Find the abin2 values of this element

         IF (map2(ind) == abin) EXIT

         ind = ind + 1

         IF (ind > (Nstages-1)*Nelem) EXIT

      END DO

      IF (ind <= (Nstages-1)*Nelem) fracground2(ind) = fracground3(j)

      write(10,*) globid, map3(j), ind, fracground3(j)

   END DO

   close(10)

   IF (MOLECULESWITCH == 3) THEN

      MOLON = .TRUE.

   else

      MOLON = .FALSE.

   end if

   CALL ionization(TESTRUN, nlev, sizeuse, Elevels, fracground2, fracground3, CTON, MOLON, totalnumberdens, numberfracvec, volume(globid), Edep(globid), rate_dead_radtot, 0, mols)

   DEALLOCATE(wl, N_ABS_SPEC_PRE)

ELSE

   DO i = 1, Nr

      call MPI_RECV(ratio, 1, MPI_DOUBLE_PRECISION, MPI_ANY_SOURCE, MPI_ANY_TAG, globcomm, status, ierr)

      ratiovec(STATUS(MPI_SOURCE)) = ratio

   END DO

   INQUIRE(file=file_ebalance,Exist=RF_EXISTS)

   IF (RF_EXISTS) THEN

      OPEN(75,file=file_ebalance,POSITION='APPEND')  ! NEW MULTI OUT

   ELSE

      OPEN(75,file=file_ebalance,STATUS='REPLACE')  ! NEW MULTI OUT

   END IF

   WRITE(75,'(10000ES12.3)') ratiovec

   CLOSE(75)

   open(75,file=file_ebalance_this,STATUS='REPLACE')

   DO i = 1, Nr

      write(75,*), i, ratiovec(i)

   END DO

   close(75)

END IF

DEALLOCATE(fracground3, popground, poptot, fracground2, RATE_DEAD_RADTOT, EXCCUBE, N_ABS_SPEC_TOT, N_ABS_SPEC_RADTOT, N_ABS_SPEC_NT, n_abs_nt, N_ABS_SPEC_RAD, RATE_DEAD_SPEC, NPHOTOEXCITATIONS)

CALL MPI_WIN_FREE(window, ierr)  ! Release all the shared memory!
IF (myid == 0) THEN
    CALL MPI_COMM_FREE(taskcomm, ierr)
ENDIF
CALL MPI_Comm_set_errhandler(MPI_COMM_WORLD, MPI_ERRORS_RETURN, ierr)
CALL MPI_FINALIZE(ierr)

IF (ierr /= MPI_SUCCESS) THEN
	IF (globid == 0) THEN
    	CALL MPI_Error_string(ierr, err_message, err_length, err_tmp)
		print *, "-----------------------------------------------------------------------"
		print *, "Note: MPI_Finalize returned the following error."
		print *, "      Please double check your results against single-node calculation."
		print *, "-----------------------------------------------------------------------"
		print *, err_message(1:err_length)
		print *, "-----------------------------------------------------------------------"
	END IF
END IF

if (globid == 0) print *, 'Program finished. cpu time', t2-t1

END PROGRAM RADMAKER
