SUBROUTINE readlevelsandlines(atom, T_typical)
USE M1, ONLY : atomm, myid
USE INTERFACE, ONLY : regemorter
USE PARAMETERS, ONLY : h, c
IMPLICIT NONE

TYPE(atomm), INTENT(INOUT) :: atom				! String specification of the atom to be loaded
DOUBLE PRECISION, INTENT(INOUT) :: T_typical
! ----------------------------------------------
! TASK :     Reads atomic data files and stores in the atom struct
! INPUT :    T_typical              Used by regemorter routine to estimate collision strengths
! OUTPUT :   Fills in atomic data data in the atom struct
! -----------------------------------------------
CHARACTER(len=5) :: trash, name
INTEGER :: Ntrash1, Ntrash2 
INTEGER :: i, j, m, jval, ierr
INTEGER, DIMENSION(atom%N) :: nv
CHARACTER(len=100) :: longtrash
DOUBLE PRECISION :: lambda_temp, A_temp

Ntrash1 = 2   ! Number of lines before levels start
Ntrash2 = 2   ! Number of lines between level list and line list

! ----------------------------------------------------

name = atom%NAME

IF (atom%Nlines .EQ. 0) THEN

   atom%Nlines = atom%N*(atom%N-1)/2		! Number of lines to read (note that we need lines also not involving a N_level)

END IF

atom%LAMBDA = 0		!Initialize

atom%A = 0

OPEN(10, FILE=atom%FILE_ADDRESS, STATUS='OLD', ACTION='READ')		! Open atomic data file, associate with number 10
				
DO i = 1, Ntrash1				! Skip down to data section

   READ(10,*) trash		! Each new read starts from a new line

END DO

IF (atom%RR_ALPHA_EXISTS .AND. atom%RR_BETA_EXISTS) THEN   ! If radiative recombination rates with T-dependence exist for this atom..

   DO m = 1, atom%N_LEVELS

      READ(10,*) nv(m), atom%LEVELS_E(m), atom%LEVELS_G(m), atom%RR_ALPHA0(m), atom%RR_T0(m), atom%RR_BETA(m)
                 
      IF (nv(m) .NE. m) THEN; WRITE(*,*) 'ERROR: Incorrect level reading. ABORTING.';STOP 1;END IF

   END DO

   DO m = atom%N_LEVELS+1, atom%N   ! Dont need rec. rates for levels that are not solved for, E and g enough

      READ(10,*) nv(m), atom%LEVELS_E(m), atom%LEVELS_G(m)

   END DO
		        
ELSE IF (atom%RR_ALPHA_EXISTS) THEN   ! Rad. rec rates exist but no T-dependences

   DO m = 1, atom%N_LEVELS
						
      READ(10,*) nv(m), atom%LEVELS_E(m), atom%LEVELS_G(m), atom%RR_ALPHA0(m), atom%RR_T0(m)
        
      IF (nv(m) .NE. m) THEN; WRITE(*,*) 'ERROR: Incorrect level reading. ABORTING.';STOP 1;;END IF
      
   END DO

   DO m = atom%N_LEVELS + 1, atom%N

      READ(10,*) nv(m), atom%LEVELS_E(m), atom%LEVELS_G(m)

   END DO

ELSE   ! No specific rec. rates exist

	DO m = 1, atom%N			
         
    	READ(10,*) nv(m), atom%LEVELS_E(m), atom%LEVELS_G(m)
                 
        IF (nv(m) .NE. m) THEN; WRITE(*,*) 'ERROR: Incorrect level reading. ABORTING.'; STOP 1;END IF  ! Check that were on the right line..
        
   END DO

END IF

atom%LEVELS_E = atom%LEVELS_E*h*c		! CONVERT FROM CM-1 TO ERGS

WHERE (atom%RR_BETA == 0)

   atom%RR_BETA = -0.5    ! Assume spec. rec rates scale as T^-0.5 for those levels that do not have specific dependencies

END WHERE

! ------------- LINE READING -----------------------------------------------------------------------------------

DO m = 1, Ntrash2 ! Skip trash between levels and lines 

   READ(10,*) trash

END DO

DO m = 1, atom%Nlines

   IF (atom%Z > 30) THEN

      READ(10,*) j, i, lambda_temp, A_temp

      IF (j <= atom%N_LEVELS) THEN

         atom%LAMBDA(j,i) = lambda_temp

         atom%A(j,i) = A_temp

      END IF
      
   ELSE
        
      IF (atom%UPSILON0_BETA_EXISTS) THEN      ! Temperature-dependency for collision strengths exist..
         
         READ(10,*) j, i, atom%LAMBDA(j,i), atom%A(j,i), atom%UPSILON0(j,i), atom%UPSILON0_T0(j,i), atom%UPSILON0_BETA(j,i)
         
      ELSE IF (atom%UPSILON0_EXISTS) THEN    ! Collision strengths exist (but no temp. dependencies)

         READ(10,*) j, i, atom%LAMBDA(j,i), atom%A(j,i), atom%UPSILON0(j,i)


      ELSE    ! No collision strengths exist, only A-values

         READ(10,*) j, i, atom%LAMBDA(j,i), atom%A(j,i)
                                                       
      END IF

      IF (j .LE. i) THEN

         WRITE(*,*) 'ERROR in readlevelsandlines. j and i are in wrong order.'; print *, atom%NAME, j, i; STOP 1
         
      END IF

      IF (m .EQ. 1) THEN
                
         IF (j .EQ. 2 .AND. i .EQ. 1) THEN
         
            CONTINUE    ! check that everything is as expected, 2-1 line first
         
         ELSE

            IF (myid == 1) WRITE(*,*) 'WARNING. first line is not 2 --> 1.'
                                    
         END IF

      END IF
    
   END IF

END DO

close(10)

CALL regemorter(atom%N_LEVELS, T_typical, atom)  ! CARE would be better to use atom%N but takes long time

IF (atom%species_type .ne. 'mol') THEN

	WHERE (atom%UPSILON0 .EQ. 0)

   		atom%UPSILON0 = 0.5          ! van Allen 1973 --> upsilon usually between 0.1 and 1 for forbidden lines (and allowed between 1 and 10). Another approach here is to set upsilon to 10% of the closest allowed transition value

	END WHERE

end if

IF (atom%NAME .EQ. ' HI  ') THEN   

   atom%A(2,1) = 0  ! Remove 2-photon from A  (otherwise it makes a line) 

ELSE IF (atom%NAME .EQ. 'HeI  ') THEN

   atom%A(3,1) = 0   ! Remove two-photon (otherwise it makes a line)  

END IF

END


