SUBROUTINE READRUNPARAMETERS(time_days, file_explmodel, tau_dust, vterm, Nblobs, tau_tresh, sf_tresh, REALRECCONT, FORCECAII, OLI, Dustzonemax, N_atoms, atoms)
USE M1
USE FILES, ONLY : file_runparameters
USE MPI
IMPLICIT NONE
!INCLUDE 'mpif.h'

DOUBLE PRECISION, INTENT(OUT) :: time_days
INTEGER, INTENT(IN) :: N_atoms
TYPE(atomm), DIMENSION(N_atoms), INTENT(INOUT) :: atoms
CHARACTER(len=100), INTENT(OUT) :: file_explmodel
DOUBLE PRECISION, INTENT(OUT) :: tau_dust, tau_tresh, sf_tresh
DOUBLE PRECISION, POINTER, INTENT(OUT) :: vterm
INTEGER, INTENT(OUT) :: Nblobs, Dustzonemax
LOGICAL, INTENT(OUT) :: REALRECCONT, FORCECAII, OLI
! LOGICAL, POINTER, INTENT(OUT) :: BLEND

CHARACTER :: trash
INTEGER :: ierr, nlevels, i
LOGICAL :: sends, absorbs
CHARACTER(len=5) :: atomname

! BLEND = .FALSE. ! This will be set later, in modelreader

!Other parameters in code to consider :

!depositionfuntion  :       Ntheta, Nsplit     controls the statistical noise in the gamma ray deposition
!statisticalandthermaleq :  convlimT     controls convergence limit for T
!stateq :                   exc_conv_lim  : convergence limit for excitation solutions
!ionization :               dxlim    convergence limit for ionization solutions
!allocGAMMAEXC              Tuse     temperature to use for PF
!fsige :                    Tuse
!fe2HECSreader              nlevR
!followphoton               ndoppler
!hI_recomspectrum           gauntfactor

IF (myid .EQ. 0) THEN   ! Only master reads data (transmits later to slaves)

   OPEN(10,file=file_runparameters,STATUS='OLD')   ! Read run parameters

   read(10,*) trash, time_days

   READ(10,*) trash, distance_MPc   ! distance to SN

   READ(10,*) trash, tau_dust   ! dust (radial) optical depth


   READ(10,*) trash, file_explmodel


   read(10,*) trash, Nr

   read(10,*) trash, Ncore

   read(10,*) trash, Dustzonemax

   READ(10,*) trash, Nblobs    ! Number of clumps per core zone

   READ(10,*) trash

   READ(10,*) trash ! Angstrom

   READ(10,*) trash  ! Angstrom

   READ(10,*) trash  ! resolution at 1000 Angstrom (Angstrom)

   READ(10,*) trash

   read(10,*) trash

   READ(10,*) trash

   READ(10,*) trash

   READ(10,*) trash

   READ(10,*) trash

   READ(10,*) trash, leakage    ! Positron leakage (0-1)?

   read(10,*) trash, Npis

   IF (Npis <= 0) Npis = 1          ! Always ionization from ground state

   read(10,*) trash, PHOTOEXC   ! Include photoexcitations/deexcitations? 0=no, 1=?, 2=yes for absorptions up to upper=N_levels.q

   READ(10,*) trash, betal_Lyalpha  ! escape probabilities in neighboring metal lines for Ly-alpha and Ly-beta

   READ(10,*) trash, betal_Lybeta

   READ(10,*) trash, tau_tresh   ! Minimum tau for lines to include

   READ(10,*) trash, sf_tresh  ! Minimum abundance to calculate non-thermal depositions for

   READ(10,*) trash    ! CTON

   read(10,*) trash,  MOLECULESWITCH  ! compute molecular abundances and influence on temperature?

   read(10,*) trash, OLI

   READ(10,*) trash, REALRECCONT   ! Use real hydrogen rec continua or as artifical lines?

   READ(10,*) trash, FORCECAII    ! Force Ca to be Ca II?

   READ(10,*) trash  ! Iterative run, i.e. use pre-existing files for ionization solution etc?

   read(10,*) trash

   READ(10,*) trash, DOPRINT, IDP  ! Print various output, and if so which zone? (only one zone can print at a time)

   close(10)

END IF

! Broadcast runparameters data to slaves

call MPI_BCAST(time_days, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(distance_MPc, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(tau_dust, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(file_explmodel, 100, MPI_CHARACTER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(Nr, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(Ncore, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(Dustzonemax, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(Nblobs, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

! -----------------------------

call MPI_BCAST(leakage, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(Npis, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(PHOTOEXC, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(betal_Lyalpha, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(betal_Lybeta, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(tau_tresh, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(sf_tresh, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(MOLECULESWITCH, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(OLI, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(REALRECCONT, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(FORCECAII, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(ITERATIVERUN, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(DOPRINT, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, ierr)

call MPI_BCAST(IDP, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, ierr)
! --------------------

END SUBROUTINE READRUNPARAMETERS
