SUBROUTINE RECLINES_SMOOTH(atom, T, ne, nplus, eexc, N, Nsplit, lambdarec, Erec)
USE PARAMETERS,ONLY : k, h, c, eV
USE M1, ONLY : atomm
IMPLICIT NONE

TYPE(atomm), INTENT(IN) :: atom
INTEGER, INTENT(IN) :: Nsplit
DOUBLE PRECISION, INTENT(IN) :: T, ne, nplus, eexc
INTEGER, INTENT(IN) :: N
DOUBLE PRECISION, DIMENSION(N, Nsplit), INTENT(OUT) :: lambdarec, Erec

INTEGER :: posin30, i, j
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: alpha_spec
DOUBLE PRECISION :: alfabetaconv, Eperphoton

! -----------------------------------------------------------------
! Task :    Creates free-bound emission
! Input :   atom         Type atomm (M1)
!           T            Temperature
!           ne           Electron number density [cm-3]
!           nplus        Ion number density [cm^-3]  (nplus is what recombines)
!           Nsplit       Number of lines to break free-bound continuum into
! Output    N            Number of bins
!           lambdarec    Array of wavelengths
!           Erec         Array of free-bound fluxes
! Globals : --
!----------------------------------------------------------------------

alfabetaconv = 0.8  ! CARE Osterbrock table 3.2 shows that for HI, beta is smaller than alfa, in the 0.5 - 1 range..assume 0.8 for all atoms

! ------------------------------------------------------------------

lambdarec = 0

Erec = 0

ALLOCATE(alpha_spec(atom%N_LEVELS)); alpha_spec = 0

CALL makealphavec(atom, T, atom%N_LEVELS, alpha_spec)   ! Get specific recombination rates

DO i = 1, N   ! To each level i..

   DO j = 1, Nsplit

      Eperphoton = atom%E_ION_GS - atom%LEVELS_E(i)  + (0.5+DBLE(j-1)/DBLE(Nsplit-1))*alfabetaconv*k*T + eexc ! put the photons uniformly distributed between 0.5 and 1.5 kT from the ion. edge

      lambdarec(i,j) = h*c/Eperphoton*1e8 !CARE

      Erec(i,j) = 1./DBLE(Nsplit)*alpha_spec(i)*ne*nplus*Eperphoton !CARE  to get emission we not only have to know alfa but also the average value for 1/2mv2 for the recombining electrons...the alfabetaconv factor is used for this..

   END DO

END DO

END
