SUBROUTINE STATEQ(R_index, N_atoms, atoms, EXCMAX, DIMS, x, deposition_tot_pervol, IONRATES, fracmat_exc, fground, Nlambda, lambdavec, alfa_c, T, TOTINMAT, Tuse, EXC_MATRIX, convprobs)
USE M1, ONLY : SFMAX, DOPRINT, IDP, Nabin, myid, globid, atomm, time, PHOTOEXC, popsave, R_index_str, Npis, ITERATIVERUN, ngvec, Nelem, Nstages
USE INTERFACE, ONLY : makealpha, allocGAMMAEXC, boltzmann, createQ, makealphavec, lyalphalybeta_alloc, createtau_s, createbeta_s, createbeta_c, createbeta_l, matrixer, excitation_solver
USE FILES, ONLY : file_popsave

IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index, N_atoms, EXCMAX, DIMS, Nlambda
TYPE(atomm), DIMENSION(N_atoms), INTENT(IN) :: atoms
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, INTENT(INOUT) :: deposition_tot_pervol, T, Tuse
DOUBLE PRECISION, DIMENSION(Nabin,EXCMAX) :: TOTINMAT
REAL, DIMENSION(:,:), INTENT(IN) :: IONRATES  ! Nabin*Npis
DOUBLE PRECISION, DIMENSION(Nabin, SFMAX), INTENT(IN) :: fracmat_exc
DOUBLE PRECISION, DIMENSION(Nabin), INTENT(IN) :: fground
DOUBLE PRECISION, DIMENSION(Nlambda), INTENT(IN) :: lambdavec, alfa_c
DOUBLE PRECISION, DIMENSION(N_atoms,EXCMAX), INTENT(OUT) :: EXC_MATRIX
LOGICAL, DIMENSION(N_atoms+1), INTENT(OUT), OPTIONAL :: convprobs
! ================================================================
! Task :  Computes statistical equilibrium populations [cm^-3] for all atoms at given input temperature (and ion abundances)
!         The system is non-linear because of optical depths in lines...
! Input : R_index         Shell number
!         N_atoms         Number of atoms to solve for
!         atoms           Vector with atoms, which are of struct type ATOMM (module M1)
!         EXCMAX          The largest number of levels to be solved for
!         DIMS            Dimension of the x array, which contains zone info (see modelreader.f90)
!         x               Array with data on the current shell (made in modelreader.f90)
!         deposition_tot_pervol  Total amount of energy deposited per unit time and volume in this zone [erg s^-1 cm^-3]
!         IONRATES   Matrix with radiative ionization rates
!         fracmat_exc     Matrix (i,j) containing the fractions going into excitation of channel j in atom i
!         Nlambda         Number of elements in wl vector used for continuum opacities
!         lambdavec       Vector with wl's for continuum opacities
!         alfa_c          Vector with continuum opacities [cm^-1]
!         T               Temperature
!         TOTINMAT         Charge transfer rates to excited states [s^-1]
! Output: EXC_MATRIX      Matrix (i,j) with populations [cm^-3] in level j in atom i
! =================================================================
INTEGER :: i, N_LEVELS, N_UPPER, niter, N_levels_print, FAIL, NEG=0, max_iter, Nuse, j, i_low, jj, ii, pos
DOUBLE PRECISION :: nk, nplus, ne, dx_change_max, exc_conv_lim, Ly_alpha_flow, Ly_beta_flow, damp, damp0, alphaout, nk_ground,renorm, z
TYPE(atomm) :: atom
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: gammaion, alpha_trash, alpha_trash2, alpha_spec, levels_pop, levels_pop_temp, levels_pop_new, BM_sol_norm, ctanddirection, threebodyrates
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: AMAT, TAU_S, BETA_S, BETA_C, BETA_L, BETA_EFF, QMAT, NTEXCMAT, LYMAT, GAMMAPHOTOEXC
DOUBLE PRECISION, DIMENSION(SFMAX) :: NT_excvec
LOGICAL :: CONVERGED
DOUBLE PRECISION, DIMENSION((Nstages-1)*Nelem) :: alphatot

! -------------------------

damp0 = 0.9  ! CARE. This should be 1 for normal NR step, 0 means that no step is taken

exc_conv_lim = 1E-2  ! CARE Convergence criteria, maxium change n_new/n_old

max_iter = 400   ! Limit on the number of iterations attempted.

! ----------------------------

IF (present(convprobs)) convprobs = .FALSE.

Ly_alpha_flow = 0

Ly_beta_flow = 0

ne = x(DIMS-2)*x(DIMS-3)   ! Electron number density [cm^-3]

call makealpha(T, alphatot)

DO i = 1, N_atoms

	atom = atoms(i)

	IF (atom%species_type .EQ. 'mol') THEN

    	nk = x(atom%POS)

    	nk_ground = nk  ! TEMPFIX why dont we allow fground here?

   ELSE

      	nk = x(atom%POS)      ! Total ion number density. Do not move down, used in conditional check

      	IF (atom%ABIN <= Nabin) THEN

         	nk_ground = nk*fground(atom%ABIN)

      	ELSE

         	nk_ground = nk

      	END IF

   	END IF

   	IF ((atoms(i)%ABSORBS .OR. atoms(i)%SENDS) .AND. (nk > 0) ) THEN  ! If atom has non-zero abundance and emits or absorbs we have to compute its level populations

!      IF (atom%species_type .ne. 'mol') THEN

!         nplus = x(atom%POS + 1)    ! Number density of the next higher ion. stage
!
!      ELSE

!         nplus = 0   ! for molecules ignore higher ions

!      END IF

   		nplus = x(atom%POS + 1)

      	N_levels = atom%N_LEVELS   ! Number of levels to compute NLTE populations for

      	N_upper = atom%N

      	ALLOCATE(AMAT(N_levels, N_levels)); AMAT = atom%A(1:N_levels,1:N_levels)
      	ALLOCATE(TAU_S(N_levels, N_levels)); TAU_S = 0
      	ALLOCATE(BETA_S(N_levels, N_levels)); BETA_S = 0
      	ALLOCATE(BETA_C(N_levels, N_levels)); BETA_C = 0
      	ALLOCATE(BETA_L(N_levels, N_levels)); BETA_L = 0
      	ALLOCATE(BETA_EFF(N_levels, N_levels)); BETA_EFF = 0
      	ALLOCATE(QMAT(N_levels, N_levels)); QMAT = 0
      	ALLOCATE(alpha_trash(N_levels)); alpha_trash = 0
      	ALLOCATE(alpha_trash2(N_levels)); alpha_trash2 = 0
      	ALLOCATE(alpha_spec(N_levels)); alpha_spec = 0
      	ALLOCATE(threebodyrates(N_levels)); threebodyrates = 0
	 !     ALLOCATE(alpha_diel(N_levels)); alpha_diel = 0
 	!     ALLOCATE(alpha_tot(N_levels)); alpha_tot = 0
      	ALLOCATE(levels_pop(N_levels)); levels_pop = 0
      	ALLOCATE(levels_pop_temp(N_levels)); levels_pop_temp = 0
      	ALLOCATE(levels_pop_new(N_levels)); levels_pop_new = 0
      	ALLOCATE(BM_sol_norm(N_levels)); BM_sol_norm = 0
      	ALLOCATE(NTEXCMAT(N_levels, N_levels)); NTEXCMAT = 0
      	ALLOCATE(LYMAT(N_levels,N_levels)); LYMAT = 0
      	ALLOCATE(gammaion(N_levels)); gammaion = 0
      	ALLOCATE(ctanddirection(N_levels)); ctanddirection = 0
      	ALLOCATE(GAMMAPHOTOEXC(N_levels, N_upper)); GAMMAPHOTOEXC = 0

      	gammaion(1:MINVAL([maxval([Npis,ngvec]),atom%N_LEVELS])) = IONRATES(atom%ABIN,1:MINVAL([maxval([Npis,ngvec]),atom%N_LEVELS])) ! Photoionization rates from specific levels

      	IF (ITERATIVERUN .AND. PHOTOEXC >= 1) THEN  ! If photoexcitations/deexcitations should be considered..
        	Nuse = minval([N_levels, atom%N_LEVELS_ABSORB])

         	IF (Nuse > 0) THEN   ! N_LEVELS_ABSORB can be zero..

            	DO i_low = 1, Nuse  ! For each lower level..

               		DO j = (i_low+1), atom%N ! for each to level (can be < i_low, stimulated emissions)

                  		GAMMAPHOTOEXC(i_low,j) = atom%PHOTOEXCRATES(R_index,i_low,j)  !,1:N_levels)

                  		IF (j <= atom%N_LEVELS) THEN

                     		GAMMAPHOTOEXC(j,i_low) = GAMMAPHOTOEXC(i_low,j)*atom%LEVELS_G(i_low)/atom%LEVELS_G(j)  ! Stimulated emission

                  		END IF

               		END DO

            	END DO

         	END IF

     	END IF

      	IF (atom%NAME .EQ. 'HeI  ') THEN

         	gammaion(2) = gammaion(2) + 7.5E-10*SQRT(T/300.)*x(1)  ! Penning ionization HeI(i=2) + HI(gs) --> HeI(i=1)+ HII. Since the i=1 equation for He I is replaced later anyway, its ok to put this rate as an ionization rate for i=2 for HeI. Assumes H is first element in x

      	END IF

      	NT_excvec = 0      ! Important

      	IF (atom%ABIN <= Nabin) THEN

        	NT_excvec = fracmat_exc(atom%ABIN,:)  ! Vector with nonthermal excitation fractions

         	IF (nk_ground > 0 .AND. (sum(NT_excvec) > 0)) THEN   ! If nonthermal excitations exist for this ion

            	CALL allocGAMMAEXC(atom, deposition_tot_pervol, nk_ground, Tuse, NT_excvec, NTEXCMAT)   ! allocate non-thermal excitation rates
         	ELSE

            	NTEXCMAT = 0

         	END IF

      	ELSE

         	NTEXCMAT = 0

      	END IF

      	CALL boltzmann(T, atom, N_levels, BM_sol_norm)     ! Use Boltzmann distribution as starting guess

      	levels_pop = BM_sol_norm*nk

        IF (ITERATIVERUN) THEN

        	DO j = 1, N_levels

            	if (isnan(popsave(i,j)) .EQV. .FALSE.) levels_pop(j) = popsave(i,j)  ! If a NaN has emerged, dont read in (use boltzmann value instead)

          	END DO

       	END IF

      	CALL createQ(T, atom, N_levels, QMAT, ne)		   ! Get collisional rates QMAT (function of UPSILON0 and T)

      	if (atom%species_type .ne. 'mol')  then

      		CALL makealphavec(atom, T, N_levels, alpha_spec)   ! Get specific recombination rates

      		IF (atom%CHARGE > 0) THEN    ! TEMP old version looks wrong. Unless this is a neutral atom, recombinations out of the states can happen

         		pos = (Nstages-1)*(atom%DENSITY_ID-1) + 1 + atom%CHARGE

             	alphaout = alphatot(pos-1)

			ELSE

				alphaout = 0

			END IF

      	else

			alphaout = 0
          	alpha_spec = 0

		end if

      	IF (atom%NAME .EQ. ' HI  ') THEN

         	AMAT(2,1) = 8.29   ! Two-photon radiative rate

      	ELSE IF (atom%NAME .EQ. 'HeI  ') THEN

         	AMAT(3,1) = 51.3      ! Two-photon

      	END IF

      	CONVERGED = .FALSE.     ! IMPORTANT

      	niter = 1

      	CALL lyalphalybeta_alloc(atom%NAME, Ly_alpha_flow, Ly_beta_flow, N_levels, levels_pop, LYMAT)   ! Ly alfa fluorescence

      	damp = damp0

      	DO WHILE (CONVERGED .EQV. .FALSE.)             ! Find NLTE excitation structure

        	if (present(convprobs)) convprobs(i)=.FALSE.

         	CALL createtau_s(atom, N_levels, N_levels, levels_pop, TAU_S)  ! Get sobolev optical depths

         	CALL createbeta_s(atom%name, N_levels, N_levels, TAU_S, BETA_S)   ! Get escape probabilities

         	CALL createbeta_c(atom, N_levels, TAU_S, T, Nlambda, lambdavec, alfa_c, BETA_C)  ! Get continuum destruction probabilities

         	CALL createbeta_l(atom, N_levels, TAU_S, T, BETA_L)   ! Get (other) line escape probabilities

         	BETA_EFF = (BETA_S + BETA_C + BETA_L)/(1. + BETA_C + BETA_L)   ! Effective escape probability

         	IF (DOPRINT .AND. myid == IDP) THEN

            	PRINT *, atom%NAME

            	N_levels_print = MINVAL([12,N_levels])

            	PRINT *, 'BETA_S:'

            	CALL matrixer(N_levels_print, BETA_S(1:N_levels_print,1:N_levels_print))

            	IF (MAXVAL(BETA_C(1:N_levels_print,1:N_levels_print)) > 0) PRINT *, 'BETA_C:'

            	CALL matrixer(N_levels_print, BETA_C(1:N_levels_print,1:N_levels_print))

            	PRINT *, 'BETA_EFF:'

            	CALL matrixer(N_levels_print, BETA_EFF(1:N_levels_print,1:N_levels_print))

            	PRINT *, 'BETA_L:'

            	CALL matrixer(N_levels_print, BETA_L(1:N_levels_print,1:N_levels_print))

             	PRINT *, 'LYMAT:'

            	CALL matrixer(N_levels_print, LYMAT(1:N_levels_print,1:N_levels_print))

         	END IF

         	ctanddirection = 0

         	ctanddirection(1:N_levels) = TOTINMAT(atom%ABIN,1:N_levels)  ! Charge transfer to excited states

         	CALL excitation_solver(atom%NAME, N_levels, N_upper, AMAT, QMAT, BETA_EFF, NTEXCMAT, LYMAT, gammaion, GAMMAPHOTOEXC, alpha_spec, threebodyrates, alphaout, ctanddirection, nk, nplus, ne, levels_pop_new, FAIL, NEG)   ! Solve rate equations

         	if (NEG == 1 .and. present(convprobs)) convprobs(i) = .TRUE.

         	IF (FAIL .EQ. 1) THEN

            	WRITE(*,*), 'WARNING! FAILURE in excitation_solver (from getexcitationstructure). Using last solution.'

            	CONVERGED = .TRUE.

         	ELSE

            	dx_change_max = MAXVAL( ABS(levels_pop_new - levels_pop)/(levels_pop + 1D-40))  ! Maximum change TEMP

            	IF (myid .EQ. IDP .and. DOPRINT) THEN

                	print *, 'dx_change_max', dx_change_max

                  	print *,'rel change', (levels_pop_new - levels_pop)/(levels_pop)

                  	print *, 'new sol', levels_pop_new

                  	print *, 'old sol', levels_pop

            	END IF

            	IF (dx_change_max < exc_conv_lim .AND. niter >= 3) THEN  ! do at least one iteration..stop when changes are small enough

               		IF (myid == IDP .AND. DOPRINT) PRINT *, 'Solution Convergence after ', niter, ' iterations.'
               		CONVERGED = .TRUE.

            	ELSE IF (niter .GT. max_iter) THEN

               		PRINT *, 'WARNING..Exceeding max niter for excitation structure. Using last solution.', globid, atom%NAME
               		!print *, 'Change factors',ABS(levels_pop_new - levels_pop)/levels_pop

               		CONVERGED = .TRUE.

               		!levels_pop(1) = nk

               		levels_pop_new = levels_pop

               		!levels_pop_new(1) = nk

               		IF (present(convprobs)) convprobs(i) = .true.

               		!levels_pop = popsave(i,1:N_levels) ! Dont update

               		EXIT

            	ELSE IF (niter > max_iter/2) THEN

               		damp = damp0*0.1

               		IF (niter > max_iter*0.75) THEN

                 		damp = damp0*0.01

               		END IF

            	END IF

            	levels_pop_temp = (1.-damp)*levels_pop + damp*levels_pop_new

            	renorm = sum(levels_pop_temp)/nk

            	if (DOPRINT .AND. myid == IDP) print *, 'renorm', renorm, sum(levels_pop)/nk, sum(levels_pop_new)/nk

            	levels_pop = levels_pop_temp/renorm     ! Take N-R step. Important do not move this up

            	if (DOPRINT .AND. myid == IDP) print *, 'updating', levels_pop

            	niter = niter + 1

         	END IF

      	END DO

123   	continue

      	EXC_MATRIX(i, 1:N_levels) = levels_pop  ! Store solution

        if (present(convprobs)) popsave(i,1:N_levels) = levels_pop  ! Only for first call, not for T*1.01

      	IF (atom%NAME .EQ. ' HI  ') THEN  ! HI is solved first, compute Ly_alpha flow (total energy going into metal lines from Lya per sec)

         	Ly_alpha_flow = levels_pop(3)*AMAT(3,1)*BETA_L(3,1)  ! [s^{-1} cm^{-3}] 3 is 2p level.

         	Ly_beta_flow = levels_pop(5)*AMAT(5,1)*BETA_L(5,1)

      	END IF

      	DEALLOCATE(AMAT)
      	DEALLOCATE(TAU_S)
      	DEALLOCATE(BETA_S)
      	DEALLOCATE(BETA_C)
      	DEALLOCATE(BETA_L)
      	DEALLOCATE(BETA_EFF)
      	DEALLOCATE(QMAT)
      	DEALLOCATE(alpha_trash)
      	DEALLOCATE(alpha_trash2)
      	DEALLOCATE(alpha_spec)
      	DEALLOCATE(threebodyrates)
!      	DEALLOCATE(alpha_diel)
!      	DEALLOCATE(alpha_tot)
      	DEALLOCATE(levels_pop)
      	DEALLOCATE(levels_pop_temp)
     	DEALLOCATE(levels_pop_new)
      	DEALLOCATE(BM_sol_norm)
     	DEALLOCATE(NTEXCMAT)
      	DEALLOCATE(LYMAT)
      	DEALLOCATE(gammaion)
      	DEALLOCATE(gammaphotoexc)
      	DEALLOCATE(ctanddirection)

   	end if

END DO

END SUBROUTINE STATEQ
