#!/bin/bash
#
# --- sumo_compilervars.sh ----------------------------
#
#     A simple script to set basic SUMO environment
#     variables before compilation.
#
#     Note: Using the compilervars script does not set
#     the dir of the env. variables for the shared 
#     libraries used by SUMO (lapack and blas). If 
#     these are not located automatically by your 
#     system, you must set the LD_LIBRARY_PATH 
#     environment variable.
#
#     Usage:
#        >> ./$SUMO_DIR/sumo_compilervars.sh <compiler>
#     or
#        >> source $SUMO_DIR/sumo_compilervars <compiler>
#
#     where <compiler> is either 'gfortran' or 'ifort',
#     and SUMO_DIR is set to the SUMO root dir.
#
#     Run this manualy before compilation, or put in
#     your .bashrc together with SUMO_DIR. E.g:
#
#     export SUMO_DIR=$HOME/Codes/SUMO
#     source $SUMO_DIR/sumo_compilervars.sh gfortran
#
# -----------------------------------------------------

# 1) Set libray flags:
export SUMO_LIBS="-llapack -lblas"

# 2) Set compilation variables based on chosen compiler:
if [[ $1 == 'gfortran' ]]; then
   export SUMO_FC=gfortran
   export SUMO_FC_MPI=mpifort
   export OMPI_FC=$SUMO_FC
elif [[ $1 == 'ifort' ]]; then
   export SUMO_FC=ifort
   export SUMO_FC_MPI=mpiifort
   export OMPI_FC=$SUMO_FC
else
   echo
   echo " Warning from <SUMO ROOT DIR>/sumo_compilervars.sh:"
   echo " No command line argument detected!"
   echo " Must be either 'gfortran' or 'ifort'."
   echo " E.g. in your .bashrc, add the following line:"
   echo
   echo " source <SUMO ROOT DIR>/sumo_compilervars.sh gfortran"
   echo
fi
