SUBROUTINE thinspectrum(R_index, lambdastart_A, lambdaend_A, emitend_A, stepatstart_A, DIMS, x, EMISS_MAT, Nwl, wlvec, tau_dust)
USE M1, ONLY : distance_Mpc, Nabin, v_out, R_index_str
USE INTERFACE, ONLY : findbin2, dustattenuation
USE PARAMETERS, ONLY : pi, c
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: R_index
DOUBLE PRECISION, INTENT(IN) :: lambdastart_A, lambdaend_A, stepatstart_A, emitend_A
DOUBLE PRECISION, INTENT(INOUT) :: tau_dust
INTEGER, INTENT(IN) :: DIMS
DOUBLE PRECISION, DIMENSION(DIMS), INTENT(IN) :: x
DOUBLE PRECISION, DIMENSION(:,:), INTENT(IN) :: EMISS_MAT
INTEGER, INTENT(OUT) :: Nwl
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: wlvec
! ====================================================================
! TASK :    Bins all emissivities (given in EMISS_MAT) into total emissivity vectors for each shell (written to file).
!           Both (velocity) convolved and unconvolved arrays are created.
! INPUT :   R_index               Shell number
!           lambdastart_A         Lower wavelength [A]
!           lambdaend_A           Upper wavelength [A]
!           emitend_A             Upper wavelength [A] for photons emitted in MC
!           stepatstart_A         Step size at lower end [A]
!           DIMS                  Length of the state array x
!           x                     Array with all relevant state parameters (ntot, T etc) for this shell. See modelreader.f90 for contents.
!           EMISS_MAT             All emissivities from this shell
! OUTPUT :  Nwl                   Number of bins in wavelength vectors
!           wlvec                 Wl vector
!           flux_noconv (file)    Array with total emissivities per shell and wl bin
! GLOBALS : distance_MPc          Read in main.f90 from runparameters.dat file
!           v_out                 Maximum ejecta velocity in cm/s
! ==================================================================
 DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: F_vec, flux_noconv
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: F_mat_individual, FBIG
DOUBLE PRECISION :: r_in_norm, r_out_norm, vel_inneredge, vel_outeredge, lambdashift_in, lambdashift_out, outsideflux, totflux, dL, shell_volume, leftedge, rightedge, Flambda, dlambda_out, dlambda_in, const, lambda_line, emiss_line, emiss_code, factor, DF, fuse, vOvdust, vin, vout
INTEGER :: i, j, step, start, index_low, index_high, abin, pos_center, pos_left, pos_right, aa
LOGICAL :: LYBETAFOUND=.FALSE., OI6300FOUND = .FALSE., OI6364FOUND=.FALSE., MGIFOUND=.FALSE.
DOUBLE PRECISION, DIMENSION(Nabin) :: outsidevec = 0

! ------------------------------------------------------------------------------------------------------

DF = 4*pi*(distance_Mpc*1E6*3.086D18)**2     ! The area (cm^2) of the sphere with the earth-SN distance as radius

outsideflux = 0

totflux = 0

Nwl = INT(LOG(lambdaend_A/lambdastart_A)/LOG(1.+stepatstart_A/lambdastart_A))  ! Number of bins in log. spaced wl vector

ALLOCATE(wlvec(Nwl)); wlvec = 0  ! Array to contain the wavelengths
ALLOCATE(F_vec(Nwl)); F_vec = 0   ! The convolved emissivity array
ALLOCATE(F_mat_individual(Nabin,Nwl)); F_mat_individual = 0  ! To contain convolved individual element contributions to flux
ALLOCATE(FBIG(Nwl,NABIN)); FBIG = 0  ! Unconvolved individual contributions

wlvec(1) = lambdastart_A    ! Shortest wl

DO i = 2, Nwl   ! Create log. spaced wl vector

   wlvec(i) = wlvec(i-1)*(1. + stepatstart_A/lambdastart_A)

    IF ((LYBETAFOUND .EQV. .FALSE.) .AND. (wlvec(i) > 1025.72)) THEN  ! Special for Ly beta, make a bin JUST to the right of it

       wlvec(i) = 1025.73

       LYBETAFOUND = .TRUE.

    ELSEIF ((OI6300FOUND .EQV. .FALSE.) .AND. (wlvec(i) > 6300.30)) THEN  ! Special for Ly beta, make a bin JUST to the right of it

       wlvec(i) = 6300.31

       OI6300FOUND = .TRUE.

    ELSEIF ((OI6364FOUND .EQV. .FALSE.) .AND. (wlvec(i) > 6363.78)) THEN  ! Special for Ly beta, make a bin JUST to the right of it

       wlvec(i) = 6363.79

       OI6364FOUND = .TRUE.

    ELSEIF ((MGIFOUND .EQV. .FALSE.) .AND. (wlvec(i) > 2852.13)) THEN

       wlvec(i) = 2852.14   ! Mg I 2852 can resonance into Na I 2853

       MGIFOUND = .TRUE.

    END IF

END DO

ALLOCATE(flux_noconv(Nwl)); flux_noconv = 0  ! Unconvolved total emissivity vector

r_in_norm = x(DIMS-11)   ! Inner radius (normalized to outer ejecta radius)

r_out_norm = x(DIMS-10)   ! Outer radius (normalized to outer ejecta radius)

shell_volume = x(DIMS-8)  ! Volume [cm^3]


! -----------------------------------------------
! sl: very temp - setting the velocity
!vel_inneredge = 0.
!v_out =  350000000.
! -----------------------------------------------


vel_inneredge = v_out*r_in_norm   ! Inner edge velocity [cm/s]

vel_outeredge = v_out*r_out_norm   ! Outer edge velocity [cm/s]

print*, 'vinner, vouter', vel_inneredge, vel_outeredge
! -----------------------------------------------
! sl: very temp - setting the velocity
!vel_inneredge = 180000000.
!vel_outeredge =  200000000.
! -----------------------------------------------


j = 1

DO WHILE (EMISS_MAT(j,4) .NE. 0)   ! Position 4 is lambda. Loop through all emission lines in this shell

   abin = NINT(EMISS_MAT(j,1))    ! Sender's abin code

   lambda_line = EMISS_MAT(j,4)  ! Wavelength [�]

   emiss_line = EMISS_MAT(j,5)    ! emissivity [erg s^-1 cm^-3 ] NOT per steradian

   emiss_code = EMISS_MAT(j,6)    ! line emission = 0, twophoton = 2 etc

   IF ( (lambda_line >= wlvec(1)*(1.-vel_outeredge/c)) .AND. (lambda_line <= wlvec(Nwl)*(1.+vel_outeredge/c)) ) THEN  ! Wavelength is within the boundaries of the desired output range

      totflux = totflux + emiss_line*shell_volume

      dlambda_out = lambda_line*vel_outeredge/c   ! Line broadening at outer edge of the zone

      dlambda_in = lambda_line*vel_inneredge/c    ! Line broadening at inner edge of the zone

      factor = dlambda_in*(1.-(dlambda_in/dlambda_out)**2) + 2./3.*dlambda_out - dlambda_in + 1./3.*dlambda_in**3/dlambda_out**2   ! See documentation

      const = 0.5*emiss_line*shell_volume/factor  ! See documentation

      CALL findbin2(Nwl, wlvec, lambda_line, pos_center)  ! Find the bin of the center wavelength

      CALL findbin2(Nwl, wlvec, lambda_line - dlambda_out, pos_left)  ! Find the bin of the leftmost edge of the convolved emission

      CALL findbin2(Nwl, wlvec, lambda_line + dlambda_out, pos_right) ! Find the rightmost edge of the convolved emission

      IF (pos_center-2 > 0) THEN  ! If the center wavelength is within the comp. range (pos_center-2 used becaue recombinations are shifted left, see below)

         IF (emiss_code .EQ. 4) THEN  ! recombination code is 4..special treatment, shift LEFT to make sure self-absorptions can occur

            flux_noconv(pos_center-2) = flux_noconv(pos_center-2) + emiss_line*shell_volume  ! Add flux to total lum vector

            FBIG(pos_center-2, abin) = FBIG(pos_center-2, abin) + emiss_line*shell_volume  ! Recombination, put in a few bins to the left to make sure that it is not sent off at a wavelength longer than the treshold (especially important for low temperatures) CARE

         ELSE   ! Emit in this bin (will be sent off at bin's upper wavelength)

            flux_noconv(pos_center) = flux_noconv(pos_center) + emiss_line*shell_volume  ! Add flux to total lum vector

            FBIG(pos_center, abin) = FBIG(pos_center, abin) + emiss_line*shell_volume  ! add emissivity to specific emitter matrix

         END IF

      END IF

      IF (pos_left .EQ. 0) THEN   ! Lefte edge did not fall within the comp. range

         IF (lambda_line - dlambda_out < wlvec(1)) THEN   ! Left bin is on the left side of the whole range

            IF (pos_right .EQ. 0) THEN    ! ..and also right bin is to the left side, skip the whole thing

               GOTO 100

            ELSE                  ! ..but if right bin is inside..

               GOTO 80

            END IF

         ELSE                 ! left bin is to the right of the whole range, skip the whole thing

            GOTO 100

         END IF

      ELSE IF (pos_right .EQ. 0) THEN  ! Left side is within but rigt side is not, extend to last bin

         pos_right = Nwl

      END IF

80    pos_left = pos_left + 1           ! Avoid negatives

      DO aa = pos_left, pos_right  ! For each bin within the convolved range..

         IF ( ABS( wlvec(aa) - lambda_line ) < dlambda_in) THEN   ! wl is inside the inner doppler edge

            Flambda = const*(1-(dlambda_in/dlambda_out)**2)   ! See documentation


         ELSE  ! wl is outside inner doppler edge


            Flambda = const*(1-((wlvec(aa)-lambda_line)/dlambda_out)**2)      ! See documentation


         END IF

         ! Add dust attenutation to spectrum

        ! IF (wlvec(aa) < lambda_line) THEN  ! blue side of line profile, small changes!

        !    IF (time/(3600*24.) < 400.) THEN   !  CARE

        !       fuse = 0.95 - 0.05*(wlvec(aa)-lambda_line)/dlambda_out   ! wlvec(aa) - lambda_line < 0

        !    ELSE

        !       fuse = 0.86 - 0.17*(wlvec(aa)-lambda_line)/dlambda_out

        !    END IF


         !ELSE    ! red side of line profile, large changes

         !   IF (time/(3600*24.) < 400.) THEN

         !       fuse = 0.95 - 0.3*(wlvec(aa)-lambda_line)/dlambda_out   ! wlvec(aa) - lambda_line > 0

         !   ELSE

         !      fuse = 0.86 - 0.45*(wlvec(aa)-lambda_line)/dlambda_out

         !   END IF


        !END IF
         !
       ! IF (wlvec(aa) > 5e4 .AND. wlvec(aa) < 30e4) THEN

       !    vOvdust = (wlvec(aa)-lambda_line)/lambda_line*c/(1800*1e5)

       !    vin = vel_inneredge/(1800*1e5)

       !    vout = vel_outeredge/(1800*1e5)

       !    call dustattenuation(vOvdust, vin, vout, tau_dust, fuse) !

       !ELSE

       !    fuse = 1.

       ! END IF

        fuse = 1. ! CARE, does not consider dust effects in thin spectra

        Flambda = Flambda*fuse

        F_vec(aa) = F_vec(aa) + Flambda  ! Add Convolved flux

        F_mat_individual(abin, aa) =  F_mat_individual(abin, aa) + Flambda  ! Add element-specific convolved flux

      END DO

   END IF

   IF ((lambda_line > emitend_A) .OR. (lambda_line < wlvec(1)*(1-vel_outeredge/c))) THEN

      outsideflux = outsideflux + emiss_line*shell_volume    ! Add flux to total flux that will not be tracked in the MC run

      outsidevec(abin) = outsidevec(abin) + emiss_line*shell_volume

   END IF

100 CONTINUE

   j = j + 1  ! Prepare for next line

END DO


OPEN(22, file=TRIM(file_tc)//R_index_str(R_index), STATUS='REPLACE')  ! Contains individual atomic contributions to convolved flux

OPEN(24, file=TRIM(file_tnc)//R_index_str(R_index), STATUS='REPLACE')  ! Contains individual shell contr without vel broadening

!OPEN(30, file=file_thinspecs, STATUS='REPLACE');WRITE(30,*) Nwl; CLOSE(30)

DO i = 1, Nwl-1

   WRITE(22,50) wlvec(i), F_vec(i)/DF, F_mat_individual(:,i)/DF  ! convolved received fluxes....Wl, total conv., then each ind. atom conv.

   IF (wlvec(i) < emitend_A) THEN

      WRITE(24,51) i, wlvec(i), flux_noconv(i), FBIG(i,1:Nabin)  ! unconvolved emission....index,  Wl, total luminosity in that bin (in this shell), and from individual elements

   END IF

50 FORMAT(ES16.6, 1000ES12.3E3)

51 FORMAT(I10, ES16.6, 1000ES12.3E3)


END DO

CLOSE(22)

CLOSE(24)

OPEN(10,file=TRIM(file_outside)//R_index_str(R_index))  ! write outside flux to file

  WRITE(10,*) outsideflux, outsideflux/(outsideflux + totflux)

  DO i = 1, Nabin

     IF (outsidevec(i) > 0 ) write(10,*) i, outsidevec(i)/outsideflux

  END DO

close(10)

DEALLOCATE(wlvec, F_vec, F_mat_individual, FBIG, flux_noconv)

END
