import pickle
import glob
import numpy as np

# =============================================================================
# This code makes the human readable network files. This is based on either
# 1) parent atoms defined, and then all possible combinations of molecules
# made from these atoms are found,
# 2) or from  defining all included species, that makes up all the reactions
#
# The output are a hr cvs network file, defined by nw_name  
# =============================================================================

sav_name = 'allmol3'

#-----------------------------------------------
#-----------------------------------------------
# class containing info about each reaction

class reaction(object):
    def __init__(self, number, rt):
        self.id = number
        self.fullr = rt
#       
        rtemp = rt.split(',')
        
        #reactants
        self.r1 = rtemp[0]
        self.r2 = rtemp[1]
        self.r3 = rtemp[2]
        
        #prodcuts
        self.p1 = rtemp[3]
        self.p2 = rtemp[4]
        self.p3 = rtemp[5]
        
        #all species in this reaction
        self.alls = [self.r1, self.r2, self.r3, self.p1, self.p2, self.p3]
        
        check = ['-' for i in range(len(db))]
        
        indi_all = []
        
        # finds the index of this reaction in each included data base
        
        
        for i in range(len(db)):
            if rt in tot_react_db[i]:
                check[i] = db[i]['name']
                indi_all.append(tot_react_db[i].index(rt))
            else:
                indi_all.append(-1)               
        
        self.indi_all = indi_all
        self.check = check
        
        # finds rate of this reaction in the first database of the list
        # where this reaction is included
        if rt in tot_react_db[0]:
            indi = tot_react_db[0].index(rt)
            indi2 = rtabs.index(umistr['type'][indi])
            rtype = rtabl[indi2]
        else:
            rtype = 'other'
            
        # the reaction type, only possible if the reaction is in umist
        self.rtype = rtype
        
        
        # which family of reactions
        if 'ee' in [self.r1, self.r2, self.r3]:
            family = 'recomb'
        elif 'ec' in [self.r1, self.r2, self.r3]:
            family = 'nt'
        elif 'h_nu' in [self.r1, self.r2, self.r3]:
            family = 'photo'
        else:
            family = 'coll'
        self.family = family
        
        
#        if rt in override:
#            oindex = np.where(override['rt'] == rt)[0]

#            for o in oindex:
#                if o == rt:
##                    self.rate = 
##                    self.a = 
##                    self.b =
##                    self.g = 
##                    self.tint = 
#        else:
        for i in range(len(db)):
            if indi_all[i] != -1:
                rate = db[i]['rates'][indi_all[i]]
                rate = rate.replace('x', '*')
                rate = rate.replace('10^', '1e')
                
                self.rate = rate
                
                trate = rate.split(',')
                
                self.a = eval(trate[0])
                self.b = eval(trate[1])
                self.g = eval(trate[2])
                if i == 0:
                    tint = db[i]['tint'][indi_all[i]]
                else:
                    tint = [10, 20000]
                self.tint = tint
                break




def species_finder(inp_array):
    '''
    this function will find all possible molecules
    that can form from some initial parent atoms,
    and returns all species in the network. 
    
    Parameters
    ----------
    inp_array: array of strings
              The parent species
    Returns
    -------
    temp: array of strings
              All possible species that can form
              from the parent species in inp_array.
    '''
    while True:
        temp = list(np.copy(inp_array))
        for t in test:
            rec = t.alls[:3]
            if set(rec).issubset(set(inp_array)):
                for p in t.alls[3:]:
                    temp.append(p)
    
        temp = sorted(list(set(temp)))
        if set(temp).issubset(set(inp_array)):
            break
        else:
            inp_array = list(np.copy(temp))
    return temp
    
#-----------------------------------------------
#-----------------------------------------------

# Define utput file name
nw_name = 'umistnetwork'

rtab = []
with open("RATE12.type.txt") as file: # Use file to refer to the file object
    rtab = file.readlines()

rtab = [r.split(',')[:2] for r in rtab]
rtab = [[r.strip() for r in rr] for rr in rtab]
rtabs = [r[0] for r in rtab]
rtabl = [r[1] for r in rtab]

#-----------------------------------------------
#-----------------------------------------------

# imports the override files
# this file will override the db values, if there are
# updated values available, or if errors have been found in 
# the database

fname  = 'override_react.csv'
data = []
with open(fname, 'r') as fin:
    data=fin.readlines()

data = [d.strip() for d in data]
data = [d.split(';') for d in data]
data = np.transpose(np.array(data))
override = {'rt':data[0], 'tint':data[1],'a':data[2], 'b':data[3], 'g':data[4], }


# imports the different databases

slud = pickle.load( open( "slud_full.p", "rb" ))
cher = pickle.load( open( "cher_full.p", "rb" ))
umistr = pickle.load( open( "umist_full.p", "rb" ))

db = [umistr, slud, cher]

# makes a flat list of all the uniquie species and reactions
tot_species = [d['species'] for d in db]
tot_species = [item for sublist in tot_species for item in sublist]
tot_species = sorted(list(set(tot_species)))

tot_react = [d['reactions'] for d in db]
tot_react_db = [[','.join(t) for t in tt] for tt in tot_react]

tot_react = [item for sublist in tot_react for item in sublist]
tot_react = sorted(list(set([','.join(t) for t in tot_react])))

#-----------------------------------------------
#-----------------------------------------------
# Makes a list with reaction objects

test = []


for i in range(len(tot_react)):
    tr = tot_react[i]
    test.append(reaction(i+1,tr))


#-----------------------------------------------
#-----------------------------------------------
# Two possible routes:
#1) define the parent species. Might then include really big
#molecules, or dust predecesors. 
#2) define the included species, which then can cap the size
#of the molecules.
#uncomment the wanted way of picking species
#-----------------------------------------------
#-----------------------------------------------

# # 1) Parent atoms (uncomment!)
#
# parent_species = ['C',
#  'C+', 'O',
#  'O+', 'ee',
#  'S', 'S+',
#  'Si', 'Si+',
#  'h_nu',
#  'x', 'Mg', 'Ne', 'He+']

#parent_species = ['Si', 'Si+',
#  'O', 'O+', 
#  'S', 'S+',
#  'Ar', 'Ar+',
#  'Ca', 'Ca+',
#  'ee',
#  'h_nu',
#  'x']
##
## # all the included species
#inc_species = species_finder(parent_species)

#-----------------------------------------------
#-----------------------------------------------

# 2) Defining all the included species for the
#    network. No other species will be present.

# this is for the only CO zone 
#inc_species = ['C',
# 'C+',
# 'CO',
# 'CO+',
# 'CO_2',
# 'CO_2+',
# 'C_2',
# 'C_2+',
# 'C_2O',
# 'C_2O+',
# 'O',
# 'O+',
# 'O_2',
# 'O_2+',
# 'ee',
# 'h_nu'
# ,'x']

#inc_species = ['O',
# 'O+',
# 'O_2',
# 'O_2+',
# 'Si',
# 'Si+',
# 'SiO',
# 'SiO+',
# 'ee',
# 'h_nu',
# 'x']

#inc_species = ['Ar',
# 'Ar+',
# 'Ca',
# 'Ca+',
# 'O',
# 'O+',
# 'O_2',
# 'O_2+',
# 'S',
# 'S+',
# 'SO',
# 'SO+',
# 'Si',
# 'Si+',
# 'SiO',
# 'SiO+',
# 'SiS',
# 'SiS+',
# 'ee',
# 'h_nu',
# 'x']

# all mol species included
inc_species = ['C',
 'C+',
 'CO',
 'CO+',
 'CO_2',
 'CO_2+',
 'C_2',
 'C_2+',
 'C_2O',
 'C_2O+',
 'He',
 'He+',
 'Ne',
 'O',
 'O+',
 'O_2',
 'O_2+',
 'S',
 'S+',
 'S_2',
 'S_2+',
 'SO',
 'SO+',
 'Si',
 'Si+',
 'SiO',
 'SiO+',
 'SiS',
 'SiS+',
 'ee',
 'h_nu',
 'x']


# removed complex species, for now;
# 'SO_2',
# 'SO_2+',
# 'SiO_2',
# 'Si_2',
# 'Si_2O_4',
# 'Si_3',
# 'Si_4',
# 'S_2',
# 'S_2+',
 
#-----------------------------------------------
#-----------------------------------------------
#uses the umist definitions as a default def


coll_out = []
recomb_out = []


for t in test:
    if all([s in inc_species for s in t.alls]):
#        print(t.fullr, t.check, t.rtype, t.family)
        if t.family == 'recomb':
            recomb_out.append(t)
        if t.family == 'coll':
            coll_out.append(t)



outp = []
#-----------------------------------------------
#-----------------------------------------------


# Creates header for the hr output
header = 'Reaction type,Reactant 1,Reactant 2,Product 1,Product 2,Product 3,min temp,max temp,alpha,beta,gamma,Notes,,'

outp.append(header)

# Orginzes data for output
for c in coll_out:
    rt= c.rtype + ','
    rec1 = [c.r1, c.r2, c.p1, c.p2, c.p3]
    
    rec1 = [r.replace('_', '') for r in rec1]
    rec1 = [r + 'I' if r[-1] != '+' else r[:-1] + 'II' for r in rec1]
    
    rec1 = [r if 'x' not in r else '0' for r in rec1]
    rec1 = [r if 'hnu' not in r else '0' for r in rec1]
    
    rec2 = ','.join(rec1) + ','
    
    if c.fullr in override['rt']:
        print(c.fullr)
        oindex = np.where(override['rt'] == c.fullr)[0]
        for o in oindex:
            tint = override['tint'][o]+ ','
            a = override['a'][o] + ','
            b = override['b'][o] + ','
            c = override['g'][o] + ','
            
            note = 'from override file'
            
            outp.append(rt + rec2 + tint + a + b + g + note)
            
    
    else:
        tint = '%s, %s,'%(c.tint[0], c.tint[1])
    #    mintemp = '10,'
    #    maxtemp = '10000,'
        
        a = str(c.a) + ','
        b = str(c.b) + ','
        g = str(c.g) + ','
        
        note = ','.join(c.check)
        
        outp.append(rt + rec2 + tint + a + b + g + note)


with open('hr_%s_%s.csv'%(nw_name, sav_name), mode='w') as out_file:
    [out_file.write(o + '\n') for o in outp]

#-----------------------------------------------
#-----------------------------------------------

# outputs the recombination data
outp = []

for c in recomb_out:
    rec1 = [c.r1, c.r2, c.p1, c.p2, c.p3]
    
    rec1 = [r.replace('_', '') for r in rec1]
    rec1 = [r + 'I' if r[-1] != '+' else r[:-1] + 'II' for r in rec1]
    
    rec1 = [r if 'x' not in r else '0' for r in rec1]
    rec1 = [r if 'hnu' not in r else '0' for r in rec1]
    
    rec2 = ','.join(rec1) + ','

    a = str(c.a) + ','
    b = str(c.b) + ','
    g = str(c.g) + ','
    
    
    outp.append(rec2 +  a + b + g )
    

with open('hr_alpha_%s.csv'%nw_name, mode='w') as out_file:
    [out_file.write(o + '\n') for o in outp]
















