SUBROUTINE TWOPHOTON_AY(name, lambda, ERROR)
USE M1, ONLY : idum
IMPLICIT NONE

! DESCRIPTION -------------------------
! TASK         Produces a random photon wavelength from the probability distribution for 2-photon-decay
!              for HI and HeI (which one is specified by in-variable name). 
! INPUT        name     Atom name  "HI "  or "HeI  ". Other names produces emergency stops.
! OUTPUT       lambda   wavelength [A]
!              ERROR    Error signal
! GLOBALS :    -
! ------------------------------------

CHARACTER(len=3), INTENT(IN) :: name
DOUBLE PRECISION, INTENT(OUT) :: lambda
LOGICAL, INTENT(OUT) :: ERROR

INTEGER, PARAMETER :: Ny=1000   ! CARE Resolution
INTEGER :: i
DOUBLE PRECISION, DIMENSION(Ny) :: yvec
DOUBLE PRECISION :: dj, j, z, A, y, dy, accum, d_accum
REAL, EXTERNAL :: ran1
! --------------------------------------

ERROR = .FALSE.

accum = 0   ! Important

yvec = [(DBLE(i)/DBLE(Ny),i=1,Ny)]  ! Array with normalized energies  y = nu/nu_0

z = DBLE(ran1(idum))  !call RANDOM_NUMBER(z)

i = 1

dy = yvec(2)-yvec(1)

IF (name .EQ. 'HI ') THEN   ! atom is HI

   DO WHILE (accum < z*8.22)  ! A integrates to 16.44 since there are two photons per deexcitation. Int Aydy = 0.5A since A is symmetrical --> 8.22

      y = yvec(i)

      A = 202.*(y*(1-y)*(1-(4*y*(1-y))**0.8) + 0.88*(y*(1-y))**1.53*(4*y*(1-y))**0.8)  ! Nussbaumer & Schmutz 1984. Probability per second of emitting a photon of energy y. Symmetric around 0.5

      d_accum = A*y*dy     !A*y**3*dy  ! All photons get the same energy (there is not two photons emitted whose sum equals 10.2 eV, but a single photon with 10.2 eV), so we should therefore really sample the *emissivity*...this is prop to A*y (Nussbaumer1984). The bins are also equally spaced.

      !dj = A/(2*8.22)*dy

      accum = accum + d_accum

      i = i + 1

   END DO

   lambda = MAXVAL([1215.7/yvec(i),1215.7d0])  ! lambda should never be below Ly-alpha wl

ELSE IF (name .EQ. 'HeI') THEN  ! atom is HeI

   DO WHILE (accum < z*51.785)  ! see discussion for HI above

      y = yvec(i)

      A = 145.*(1-(ABS(1-2*y))**2.5)            ! Li & McCray 1996    Eq 14

      d_accum = A*y*dy

      accum = accum + d_accum

      i = i + 1

   END DO

   lambda = MAXVAL([601.40d0, 601.40/yvec(i)])  ! lambda should never be below 601.4 

ELSE

   PRINT *, 'ERROR in twophoton_ay : atom not recognized.'

   ERROR = .TRUE.

   stop 1

END IF

END




