SUBROUTINE WRITEIONEDGES(N_atoms, atoms)
USE M1, ONLY : atomm, Npis, lambdaion, Nelem, ngvec, Nstages
USE PARAMETERS, ONLY : h, c
USE FILES
IMPLICIT NONE
INTEGER, INTENT(IN) :: N_atoms
TYPE(atomm), DIMENSION(N_atoms), INTENT(IN) :: atoms
! ----------------------------------------
! TASK :   Writes a file (file_ionedges specified in the FILES module) with the wavelengths for the first Npis ionization edges of all 2*Nelem elements. This file is used by calculatecubes.f90 to compute preionization rates. (But that is currently turned off). Also used by radmaker.f90 to compute heating rates from photoionizations.
! INPUT :  N_atoms       Number of atoms to consider (atoms with computed structures get Npis values, others get just for the ground state)
!          atoms         Array with atoms of struct type atomm (M1 module)
! OUTPUT : writes to file_ionedges (Set in FILES module)
! GLOBALS :lambdaion     Ground state ion. wavelengths [] for all neutral and singly ion. species (M1 module)
!          Npis          Number of excited states to compute (M1)
!          Nelem         Number of elements (H, He, ...)  (M1)
! ----------------------------------------
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: IONEDGES
INTEGER :: i, j, pos
TYPE(atomm) :: atom
! -----------------------------------------

ALLOCATE(ionedges((Nstages-1)*Nelem,maxval([Npis,ngvec]))); ionedges=0

IONEDGES = 0

IONEDGES(1:(Nstages-1)*Nelem,1) = lambdaion  ! Ground state wavelength *lambdaion* is used for elements that are not solved for

DO i = 1, N_atoms   ! For each atom...

   atom = atoms(i)

   IF (atom%CHARGE >= Nstages-1) CYCLE    ! Dont include ionization by doubly ionized or higher (these cannot be ionized anyway)

   pos = (Nstages-1)*(atom%DENSITY_ID-1) + 1 + atom%CHARGE  ! Position in the 2*Nelem vector ([HI (HII) HeI HeII CI CII ..])

   DO j = 1, MINVAL([maxval([Npis,ngvec]), atom%N_LEVELS])   ! For each of the first Npis states, or atom%N_LEVELS if that is smaller than Npis
!IF (atom%species_type .ne. 'mol') THEN
         IONEDGES(pos,j) = h*c/(atom%E_ION_GS - atom%LEVELS_E(j))*1E8  ! Ion. edge in    use this since emitted photons are set to wavelengths given by this formula
!end if
   END DO

END DO

OPEN(10,file=file_ionedges,STATUS='REPLACE')

DO i = 1, (Nstages-1)*Nelem

   WRITE(10,'(I5,1000F12.2)') i, IONEDGES(i,1:maxval([Npis,ngvec]))

END DO

close(10)

END SUBROUTINE WRITEIONEDGES
   
